@@ -27,28 +27,52 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int16_t)simde_mm_extract_epi16(vWH, 7);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)simde_mm_extract_epi8(vWH, 15);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int16_t)simde_mm_extract_epi16(vWH, 6);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)simde_mm_extract_epi8(vWH, 14);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int16_t)simde_mm_extract_epi16(vWH, 5);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)simde_mm_extract_epi8(vWH, 13);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int16_t)simde_mm_extract_epi16(vWH, 4);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)simde_mm_extract_epi8(vWH, 12);
     }
     if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int16_t)simde_mm_extract_epi16(vWH, 3);
+        array[1LL*(i+4)*s2Len + (j-4)] = (int8_t)simde_mm_extract_epi8(vWH, 11);
     }
     if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int16_t)simde_mm_extract_epi16(vWH, 2);
+        array[1LL*(i+5)*s2Len + (j-5)] = (int8_t)simde_mm_extract_epi8(vWH, 10);
     }
     if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int16_t)simde_mm_extract_epi16(vWH, 1);
+        array[1LL*(i+6)*s2Len + (j-6)] = (int8_t)simde_mm_extract_epi8(vWH, 9);
     }
     if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int16_t)simde_mm_extract_epi16(vWH, 0);
+        array[1LL*(i+7)*s2Len + (j-7)] = (int8_t)simde_mm_extract_epi8(vWH, 8);
+    }
+    if (0 <= i+8 && i+8 < s1Len && 0 <= j-8 && j-8 < s2Len) {
+        array[1LL*(i+8)*s2Len + (j-8)] = (int8_t)simde_mm_extract_epi8(vWH, 7);
+    }
+    if (0 <= i+9 && i+9 < s1Len && 0 <= j-9 && j-9 < s2Len) {
+        array[1LL*(i+9)*s2Len + (j-9)] = (int8_t)simde_mm_extract_epi8(vWH, 6);
+    }
+    if (0 <= i+10 && i+10 < s1Len && 0 <= j-10 && j-10 < s2Len) {
+        array[1LL*(i+10)*s2Len + (j-10)] = (int8_t)simde_mm_extract_epi8(vWH, 5);
+    }
+    if (0 <= i+11 && i+11 < s1Len && 0 <= j-11 && j-11 < s2Len) {
+        array[1LL*(i+11)*s2Len + (j-11)] = (int8_t)simde_mm_extract_epi8(vWH, 4);
+    }
+    if (0 <= i+12 && i+12 < s1Len && 0 <= j-12 && j-12 < s2Len) {
+        array[1LL*(i+12)*s2Len + (j-12)] = (int8_t)simde_mm_extract_epi8(vWH, 3);
+    }
+    if (0 <= i+13 && i+13 < s1Len && 0 <= j-13 && j-13 < s2Len) {
+        array[1LL*(i+13)*s2Len + (j-13)] = (int8_t)simde_mm_extract_epi8(vWH, 2);
+    }
+    if (0 <= i+14 && i+14 < s1Len && 0 <= j-14 && j-14 < s2Len) {
+        array[1LL*(i+14)*s2Len + (j-14)] = (int8_t)simde_mm_extract_epi8(vWH, 1);
+    }
+    if (0 <= i+15 && i+15 < s1Len && 0 <= j-15 && j-15 < s2Len) {
+        array[1LL*(i+15)*s2Len + (j-15)] = (int8_t)simde_mm_extract_epi8(vWH, 0);
     }
 }
 #endif
@@ -64,63 +88,111 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int16_t)simde_mm_extract_epi16(vWH, 7);
+        row[j-0] = (int8_t)simde_mm_extract_epi8(vWH, 15);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int16_t)simde_mm_extract_epi16(vWH, 7);
+        col[(i+0)] = (int8_t)simde_mm_extract_epi8(vWH, 15);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int16_t)simde_mm_extract_epi16(vWH, 6);
+        row[j-1] = (int8_t)simde_mm_extract_epi8(vWH, 14);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int16_t)simde_mm_extract_epi16(vWH, 6);
+        col[(i+1)] = (int8_t)simde_mm_extract_epi8(vWH, 14);
     }
     if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int16_t)simde_mm_extract_epi16(vWH, 5);
+        row[j-2] = (int8_t)simde_mm_extract_epi8(vWH, 13);
     }
     if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int16_t)simde_mm_extract_epi16(vWH, 5);
+        col[(i+2)] = (int8_t)simde_mm_extract_epi8(vWH, 13);
     }
     if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int16_t)simde_mm_extract_epi16(vWH, 4);
+        row[j-3] = (int8_t)simde_mm_extract_epi8(vWH, 12);
     }
     if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int16_t)simde_mm_extract_epi16(vWH, 4);
+        col[(i+3)] = (int8_t)simde_mm_extract_epi8(vWH, 12);
     }
     if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
-        row[j-4] = (int16_t)simde_mm_extract_epi16(vWH, 3);
+        row[j-4] = (int8_t)simde_mm_extract_epi8(vWH, 11);
     }
     if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
-        col[(i+4)] = (int16_t)simde_mm_extract_epi16(vWH, 3);
+        col[(i+4)] = (int8_t)simde_mm_extract_epi8(vWH, 11);
     }
     if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
-        row[j-5] = (int16_t)simde_mm_extract_epi16(vWH, 2);
+        row[j-5] = (int8_t)simde_mm_extract_epi8(vWH, 10);
     }
     if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
-        col[(i+5)] = (int16_t)simde_mm_extract_epi16(vWH, 2);
+        col[(i+5)] = (int8_t)simde_mm_extract_epi8(vWH, 10);
     }
     if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
-        row[j-6] = (int16_t)simde_mm_extract_epi16(vWH, 1);
+        row[j-6] = (int8_t)simde_mm_extract_epi8(vWH, 9);
     }
     if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
-        col[(i+6)] = (int16_t)simde_mm_extract_epi16(vWH, 1);
+        col[(i+6)] = (int8_t)simde_mm_extract_epi8(vWH, 9);
     }
     if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
-        row[j-7] = (int16_t)simde_mm_extract_epi16(vWH, 0);
+        row[j-7] = (int8_t)simde_mm_extract_epi8(vWH, 8);
     }
     if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
-        col[(i+7)] = (int16_t)simde_mm_extract_epi16(vWH, 0);
+        col[(i+7)] = (int8_t)simde_mm_extract_epi8(vWH, 8);
+    }
+    if (i+8 == s1Len-1 && 0 <= j-8 && j-8 < s2Len) {
+        row[j-8] = (int8_t)simde_mm_extract_epi8(vWH, 7);
+    }
+    if (j-8 == s2Len-1 && 0 <= i+8 && i+8 < s1Len) {
+        col[(i+8)] = (int8_t)simde_mm_extract_epi8(vWH, 7);
+    }
+    if (i+9 == s1Len-1 && 0 <= j-9 && j-9 < s2Len) {
+        row[j-9] = (int8_t)simde_mm_extract_epi8(vWH, 6);
+    }
+    if (j-9 == s2Len-1 && 0 <= i+9 && i+9 < s1Len) {
+        col[(i+9)] = (int8_t)simde_mm_extract_epi8(vWH, 6);
+    }
+    if (i+10 == s1Len-1 && 0 <= j-10 && j-10 < s2Len) {
+        row[j-10] = (int8_t)simde_mm_extract_epi8(vWH, 5);
+    }
+    if (j-10 == s2Len-1 && 0 <= i+10 && i+10 < s1Len) {
+        col[(i+10)] = (int8_t)simde_mm_extract_epi8(vWH, 5);
+    }
+    if (i+11 == s1Len-1 && 0 <= j-11 && j-11 < s2Len) {
+        row[j-11] = (int8_t)simde_mm_extract_epi8(vWH, 4);
+    }
+    if (j-11 == s2Len-1 && 0 <= i+11 && i+11 < s1Len) {
+        col[(i+11)] = (int8_t)simde_mm_extract_epi8(vWH, 4);
+    }
+    if (i+12 == s1Len-1 && 0 <= j-12 && j-12 < s2Len) {
+        row[j-12] = (int8_t)simde_mm_extract_epi8(vWH, 3);
+    }
+    if (j-12 == s2Len-1 && 0 <= i+12 && i+12 < s1Len) {
+        col[(i+12)] = (int8_t)simde_mm_extract_epi8(vWH, 3);
+    }
+    if (i+13 == s1Len-1 && 0 <= j-13 && j-13 < s2Len) {
+        row[j-13] = (int8_t)simde_mm_extract_epi8(vWH, 2);
+    }
+    if (j-13 == s2Len-1 && 0 <= i+13 && i+13 < s1Len) {
+        col[(i+13)] = (int8_t)simde_mm_extract_epi8(vWH, 2);
+    }
+    if (i+14 == s1Len-1 && 0 <= j-14 && j-14 < s2Len) {
+        row[j-14] = (int8_t)simde_mm_extract_epi8(vWH, 1);
+    }
+    if (j-14 == s2Len-1 && 0 <= i+14 && i+14 < s1Len) {
+        col[(i+14)] = (int8_t)simde_mm_extract_epi8(vWH, 1);
+    }
+    if (i+15 == s1Len-1 && 0 <= j-15 && j-15 < s2Len) {
+        row[j-15] = (int8_t)simde_mm_extract_epi8(vWH, 0);
+    }
+    if (j-15 == s2Len-1 && 0 <= i+15 && i+15 < s1Len) {
+        col[(i+15)] = (int8_t)simde_mm_extract_epi8(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_table_diag_neon_128_16
+#define FNAME parasail_nw_table_diag_neon_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_rowcol_diag_neon_128_16
+#define FNAME parasail_nw_rowcol_diag_neon_128_8
 #else
-#define FNAME parasail_nw_diag_neon_128_16
+#define FNAME parasail_nw_diag_neon_128_8
 #endif
 #endif
 
@@ -136,21 +208,21 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int16_t * restrict s1 = NULL;
-    int16_t * restrict s2B = NULL;
-    int16_t * restrict _H_pr = NULL;
-    int16_t * restrict _F_pr = NULL;
-    int16_t * restrict s2 = NULL;
-    int16_t * restrict H_pr = NULL;
-    int16_t * restrict F_pr = NULL;
+    int8_t * restrict s1 = NULL;
+    int8_t * restrict s2B = NULL;
+    int8_t * restrict _H_pr = NULL;
+    int8_t * restrict _F_pr = NULL;
+    int8_t * restrict s2 = NULL;
+    int8_t * restrict H_pr = NULL;
+    int8_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
+    int8_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -183,7 +255,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 8; /* number of values in vector */
+    N = 16; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -193,28 +265,28 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi16(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = simde_mm_set1_epi16(NEG_LIMIT);
-    vOpen = simde_mm_set1_epi16(open);
-    vGap  = simde_mm_set1_epi16(gap);
-    vOne = simde_mm_set1_epi16(1);
-    vN = simde_mm_set1_epi16(N);
-    vGapN = simde_mm_set1_epi16(gap*N);
-    vNegOne = simde_mm_set1_epi16(-1);
-    vI = simde_mm_set_epi16(0,1,2,3,4,5,6,7);
-    vJreset = simde_mm_set_epi16(0,-1,-2,-3,-4,-5,-6,-7);
+    vNegInf = simde_mm_set1_epi8(NEG_LIMIT);
+    vOpen = simde_mm_set1_epi8(open);
+    vGap  = simde_mm_set1_epi8(gap);
+    vOne = simde_mm_set1_epi8(1);
+    vN = simde_mm_set1_epi8(N);
+    vGapN = simde_mm_set1_epi8(gap*N);
+    vNegOne = simde_mm_set1_epi8(-1);
+    vI = simde_mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
+    vJreset = simde_mm_set_epi8(0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15);
     vMax = vNegInf;
-    vILimit = simde_mm_set1_epi16(s1Len);
-    vILimit1 = simde_mm_subs_epi16(vILimit, vOne);
-    vJLimit = simde_mm_set1_epi16(s2Len);
-    vJLimit1 = simde_mm_subs_epi16(vJLimit, vOne);
-    vIBoundary = simde_mm_set_epi16(
+    vILimit = simde_mm_set1_epi8(s1Len);
+    vILimit1 = simde_mm_subs_epi8(vILimit, vOne);
+    vJLimit = simde_mm_set1_epi8(s2Len);
+    vJLimit1 = simde_mm_subs_epi8(vJLimit, vOne);
+    vIBoundary = simde_mm_set_epi8(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
@@ -222,7 +294,15 @@ parasail_result_t* FNAME(
             -open-4*gap,
             -open-5*gap,
             -open-6*gap,
-            -open-7*gap);
+            -open-7*gap,
+            -open-8*gap,
+            -open-9*gap,
+            -open-10*gap,
+            -open-11*gap,
+            -open-12*gap,
+            -open-13*gap,
+            -open-14*gap,
+            -open-15*gap);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -238,7 +318,7 @@ parasail_result_t* FNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -247,9 +327,9 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int16_t(16, s2Len+PAD2);
-    _H_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _F_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    s2B= parasail_memalign_int8_t(16, s2Len+PAD2);
+    _H_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
+    _F_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -261,7 +341,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int16_t(16, s1Len+PAD);
+        s1 = parasail_memalign_int8_t(16, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -317,26 +397,34 @@ parasail_result_t* FNAME(
         const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
         const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
         const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        vNH = simde_mm_srli_si128(vNH, 2);
-        vNH = simde_mm_insert_epi16(vNH, H_pr[-1], 7);
-        vWH = simde_mm_srli_si128(vWH, 2);
-        vWH = simde_mm_insert_epi16(vWH, -open - i*gap, 7);
+        const int * const restrict matrow8 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+8] : ((i+8 >= s1Len) ? s1Len-1 : i+8))];
+        const int * const restrict matrow9 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+9] : ((i+9 >= s1Len) ? s1Len-1 : i+9))];
+        const int * const restrict matrow10 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+10] : ((i+10 >= s1Len) ? s1Len-1 : i+10))];
+        const int * const restrict matrow11 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+11] : ((i+11 >= s1Len) ? s1Len-1 : i+11))];
+        const int * const restrict matrow12 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+12] : ((i+12 >= s1Len) ? s1Len-1 : i+12))];
+        const int * const restrict matrow13 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+13] : ((i+13 >= s1Len) ? s1Len-1 : i+13))];
+        const int * const restrict matrow14 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+14] : ((i+14 >= s1Len) ? s1Len-1 : i+14))];
+        const int * const restrict matrow15 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+15] : ((i+15 >= s1Len) ? s1Len-1 : i+15))];
+        vNH = simde_mm_srli_si128(vNH, 1);
+        vNH = simde_mm_insert_epi8(vNH, H_pr[-1], 15);
+        vWH = simde_mm_srli_si128(vWH, 1);
+        vWH = simde_mm_insert_epi8(vWH, -open - i*gap, 15);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             simde__m128i vMat;
             simde__m128i vNWH = vNH;
-            vNH = simde_mm_srli_si128(vWH, 2);
-            vNH = simde_mm_insert_epi16(vNH, H_pr[j], 7);
-            vF = simde_mm_srli_si128(vF, 2);
-            vF = simde_mm_insert_epi16(vF, F_pr[j], 7);
-            vF = simde_mm_max_epi16(
-                    simde_mm_subs_epi16(vNH, vOpen),
-                    simde_mm_subs_epi16(vF, vGap));
-            vE = simde_mm_max_epi16(
-                    simde_mm_subs_epi16(vWH, vOpen),
-                    simde_mm_subs_epi16(vE, vGap));
-            vMat = simde_mm_set_epi16(
+            vNH = simde_mm_srli_si128(vWH, 1);
+            vNH = simde_mm_insert_epi8(vNH, H_pr[j], 15);
+            vF = simde_mm_srli_si128(vF, 1);
+            vF = simde_mm_insert_epi8(vF, F_pr[j], 15);
+            vF = simde_mm_max_epi8(
+                    simde_mm_subs_epi8(vNH, vOpen),
+                    simde_mm_subs_epi8(vF, vGap));
+            vE = simde_mm_max_epi8(
+                    simde_mm_subs_epi8(vWH, vOpen),
+                    simde_mm_subs_epi8(vE, vGap));
+            vMat = simde_mm_set_epi8(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
@@ -344,23 +432,31 @@ parasail_result_t* FNAME(
                     matrow4[s2[j-4]],
                     matrow5[s2[j-5]],
                     matrow6[s2[j-6]],
-                    matrow7[s2[j-7]]
+                    matrow7[s2[j-7]],
+                    matrow8[s2[j-8]],
+                    matrow9[s2[j-9]],
+                    matrow10[s2[j-10]],
+                    matrow11[s2[j-11]],
+                    matrow12[s2[j-12]],
+                    matrow13[s2[j-13]],
+                    matrow14[s2[j-14]],
+                    matrow15[s2[j-15]]
                     );
-            vNWH = simde_mm_adds_epi16(vNWH, vMat);
-            vWH = simde_mm_max_epi16(vNWH, vE);
-            vWH = simde_mm_max_epi16(vWH, vF);
+            vNWH = simde_mm_adds_epi8(vNWH, vMat);
+            vWH = simde_mm_max_epi8(vNWH, vE);
+            vWH = simde_mm_max_epi8(vWH, vF);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                simde__m128i cond = simde_mm_cmpeq_epi16(vJ,vNegOne);
+                simde__m128i cond = simde_mm_cmpeq_epi8(vJ,vNegOne);
                 vWH = simde_mm_blendv_epi8(vWH, vIBoundary, cond);
                 vF = simde_mm_blendv_epi8(vF, vNegInf, cond);
                 vE = simde_mm_blendv_epi8(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vWH);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -368,35 +464,35 @@ parasail_result_t* FNAME(
 #ifdef PARASAIL_ROWCOL
             arr_store_rowcol(result->rowcols->score_row, result->rowcols->score_col, vWH, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-7] = (int16_t)simde_mm_extract_epi16(vWH,0);
-            F_pr[j-7] = (int16_t)simde_mm_extract_epi16(vF,0);
+            H_pr[j-15] = (int8_t)simde_mm_extract_epi8(vWH,0);
+            F_pr[j-15] = (int8_t)simde_mm_extract_epi8(vF,0);
             /* as minor diagonal vector passes across table, extract
                last table value at the i,j bound */
             {
-                simde__m128i cond_valid_I = simde_mm_cmpeq_epi16(vI, vILimit1);
-                simde__m128i cond_valid_J = simde_mm_cmpeq_epi16(vJ, vJLimit1);
+                simde__m128i cond_valid_I = simde_mm_cmpeq_epi8(vI, vILimit1);
+                simde__m128i cond_valid_J = simde_mm_cmpeq_epi8(vJ, vJLimit1);
                 simde__m128i cond_all = simde_mm_and_si128(cond_valid_I, cond_valid_J);
                 vMax = simde_mm_blendv_epi8(vMax, vWH, cond_all);
             }
-            vJ = simde_mm_adds_epi16(vJ, vOne);
+            vJ = simde_mm_adds_epi8(vJ, vOne);
         }
-        vI = simde_mm_adds_epi16(vI, vN);
-        vIBoundary = simde_mm_subs_epi16(vIBoundary, vGapN);
+        vI = simde_mm_adds_epi8(vI, vN);
+        vIBoundary = simde_mm_subs_epi8(vIBoundary, vGapN);
     }
 
     /* max in vMax */
     for (i=0; i<N; ++i) {
-        int16_t value;
-        value = (int16_t) simde_mm_extract_epi16(vMax, 7);
+        int8_t value;
+        value = (int8_t) simde_mm_extract_epi8(vMax, 15);
         if (value > score) {
             score = value;
         }
-        vMax = simde_mm_slli_si128(vMax, 2);
+        vMax = simde_mm_slli_si128(vMax, 1);
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
