@@ -27,8 +27,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -39,21 +53,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    col[ 0*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_table_scan_neon_128_64
-#define PNAME parasail_nw_table_scan_profile_neon_128_64
+#define FNAME parasail_nw_table_scan_neon_128_8
+#define PNAME parasail_nw_table_scan_profile_neon_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_rowcol_scan_neon_128_64
-#define PNAME parasail_nw_rowcol_scan_profile_neon_128_64
+#define FNAME parasail_nw_rowcol_scan_neon_128_8
+#define PNAME parasail_nw_rowcol_scan_profile_neon_128_8
 #else
-#define FNAME parasail_nw_scan_neon_128_64
-#define PNAME parasail_nw_scan_profile_neon_128_64
+#define FNAME parasail_nw_scan_neon_128_8
+#define PNAME parasail_nw_scan_profile_neon_128_8
 #endif
 #endif
 
@@ -78,7 +106,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -106,16 +134,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     simde__m128i* restrict pvP = NULL;
     simde__m128i* restrict pvE = NULL;
-    int64_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     simde__m128i* restrict pvHt = NULL;
     simde__m128i* restrict pvH = NULL;
     simde__m128i* restrict pvGapper = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     simde__m128i vZero;
-    int64_t score = 0;
+    int8_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -126,7 +154,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -142,25 +170,25 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (simde__m128i*)profile->profile64.score;
-    vGapO = simde_mm_set1_epi64x(open);
-    vGapE = simde_mm_set1_epi64x(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (simde__m128i*)profile->profile8.score;
+    vGapO = simde_mm_set1_epi8(open);
+    vGapE = simde_mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vNegInfFront = vZero;
-    vNegInfFront = simde_mm_insert_epi64(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = simde_mm_add_epi64(vNegInfFront,
-            simde_mm_slli_si128(simde_mm_set1_epi64x(-segLen*gap), 8));
+    vNegInfFront = simde_mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = simde_mm_adds_epi8(vNegInfFront,
+            simde_mm_slli_si128(simde_mm_set1_epi8(-segLen*gap), 1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -176,7 +204,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -186,7 +214,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign_simde__m128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
     pvHt= parasail_memalign_simde__m128i(16, segLen);
     pvH = parasail_memalign_simde__m128i(16, segLen);
     pvGapper = parasail_memalign_simde__m128i(16, segLen);
@@ -207,9 +235,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             simde_mm_store_si128(&pvH[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -222,15 +250,15 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
     {
-        simde__m128i vGapper = simde_mm_sub_epi64(vZero,vGapO);
+        simde__m128i vGapper = simde_mm_subs_epi8(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             simde_mm_store_si128(pvGapper+i, vGapper);
-            vGapper = simde_mm_sub_epi64(vGapper, vGapE);
+            vGapper = simde_mm_subs_epi8(vGapper, vGapE);
         }
     }
 
@@ -248,49 +276,49 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = simde_mm_load_si128(pvH+(segLen-1));
-        vHp = simde_mm_slli_si128(vHp, 8);
-        vHp = simde_mm_insert_epi64(vHp, boundary[j], 0);
+        vHp = simde_mm_slli_si128(vHp, 1);
+        vHp = simde_mm_insert_epi8(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = simde_mm_sub_epi64(vNegLimit, pvGapper[0]);
+        vHt = simde_mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = simde_mm_load_si128(pvH+i);
             vE = simde_mm_load_si128(pvE+i);
             vW = simde_mm_load_si128(pvW+i);
-            vE = simde_mm_max_epi64(
-                    simde_mm_sub_epi64(vE, vGapE),
-                    simde_mm_sub_epi64(vH, vGapO));
-            vHp = simde_mm_add_epi64(vHp, vW);
-            vF = simde_mm_max_epi64(vF, simde_mm_add_epi64(vHt, pvGapper[i]));
-            vHt = simde_mm_max_epi64(vE, vHp);
+            vE = simde_mm_max_epi8(
+                    simde_mm_subs_epi8(vE, vGapE),
+                    simde_mm_subs_epi8(vH, vGapO));
+            vHp = simde_mm_adds_epi8(vHp, vW);
+            vF = simde_mm_max_epi8(vF, simde_mm_adds_epi8(vHt, pvGapper[i]));
+            vHt = simde_mm_max_epi8(vE, vHp);
             simde_mm_store_si128(pvE+i, vE);
             simde_mm_store_si128(pvHt+i, vHt);
             vHp = vH;
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = simde_mm_slli_si128(vHt, 8);
-        vHt = simde_mm_insert_epi64(vHt, boundary[j+1], 0);
-        vF = simde_mm_max_epi64(vF, simde_mm_add_epi64(vHt, pvGapper[0]));
+        vHt = simde_mm_slli_si128(vHt, 1);
+        vHt = simde_mm_insert_epi8(vHt, boundary[j+1], 0);
+        vF = simde_mm_max_epi8(vF, simde_mm_adds_epi8(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
-            simde__m128i vFt = simde_mm_slli_si128(vF, 8);
-            vFt = simde_mm_add_epi64(vFt, vSegLenXgap);
-            vF = simde_mm_max_epi64(vF, vFt);
+            simde__m128i vFt = simde_mm_slli_si128(vF, 1);
+            vFt = simde_mm_adds_epi8(vFt, vSegLenXgap);
+            vF = simde_mm_max_epi8(vF, vFt);
         }
 
         /* calculate final H */
-        vF = simde_mm_slli_si128(vF, 8);
-        vF = simde_mm_add_epi64(vF, vNegInfFront);
-        vH = simde_mm_max_epi64(vHt, vF);
+        vF = simde_mm_slli_si128(vF, 1);
+        vF = simde_mm_adds_epi8(vF, vNegInfFront);
+        vH = simde_mm_max_epi8(vHt, vF);
         for (i=0; i<segLen; ++i) {
             vHt = simde_mm_load_si128(pvHt+i);
-            vF = simde_mm_max_epi64(
-                    simde_mm_sub_epi64(vF, vGapE),
-                    simde_mm_sub_epi64(vH, vGapO));
-            vH = simde_mm_max_epi64(vHt, vF);
+            vF = simde_mm_max_epi8(
+                    simde_mm_subs_epi8(vF, vGapE),
+                    simde_mm_subs_epi8(vH, vGapO));
+            vH = simde_mm_max_epi8(vHt, vF);
             simde_mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
@@ -302,9 +330,9 @@ parasail_result_t* PNAME(
         {
             vH = simde_mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 8);
+                vH = simde_mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
+            result->rowcols->score_row[j] = (int8_t) simde_mm_extract_epi8 (vH, 15);
         }
 #endif
     }
@@ -320,14 +348,14 @@ parasail_result_t* PNAME(
     {
         simde__m128i vH = simde_mm_load_si128(pvH + offset);
         for (k=0; k<position; ++k) {
-            vH = simde_mm_slli_si128(vH, 8);
+            vH = simde_mm_slli_si128(vH, 1);
         }
-        score = (int64_t) simde_mm_extract_epi64 (vH, 1);
+        score = (int8_t) simde_mm_extract_epi8 (vH, 15);
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
