@@ -13,6 +13,7 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
+#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -20,12 +21,6 @@
 #include "parasail/internal_sse.h"
 
 
-static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
-    a = _mm_andnot_si128(mask, a);
-    a = _mm_or_si128(a, _mm_and_si128(mask, b));
-    return a;
-}
-
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -125,12 +120,12 @@ static inline void arr_store_rowcol(
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_diag_sse2_128_16
+#define FNAME parasail_nw_stats_table_diag_sse41_128_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_diag_sse2_128_16
+#define FNAME parasail_nw_stats_rowcol_diag_sse41_128_16
 #else
-#define FNAME parasail_nw_stats_diag_sse2_128_16
+#define FNAME parasail_nw_stats_diag_sse41_128_16
 #endif
 #endif
 
@@ -451,17 +446,17 @@ parasail_result_t* FNAME(
             vF_ext = _mm_subs_epi16(vF, vGap);
             vF = _mm_max_epi16(vF_opn, vF_ext);
             case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
-            vFM = _mm_blendv_epi8_rpl(vFM, vNM, case1);
-            vFS = _mm_blendv_epi8_rpl(vFS, vNS, case1);
-            vFL = _mm_blendv_epi8_rpl(vFL, vNL, case1);
+            vFM = _mm_blendv_epi8(vFM, vNM, case1);
+            vFS = _mm_blendv_epi8(vFS, vNS, case1);
+            vFL = _mm_blendv_epi8(vFL, vNL, case1);
             vFL = _mm_adds_epi16(vFL, vOne);
             vE_opn = _mm_subs_epi16(vWH, vOpen);
             vE_ext = _mm_subs_epi16(vE, vGap);
             vE = _mm_max_epi16(vE_opn, vE_ext);
             case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
-            vEM = _mm_blendv_epi8_rpl(vEM, vWM, case1);
-            vES = _mm_blendv_epi8_rpl(vES, vWS, case1);
-            vEL = _mm_blendv_epi8_rpl(vEL, vWL, case1);
+            vEM = _mm_blendv_epi8(vEM, vWM, case1);
+            vES = _mm_blendv_epi8(vES, vWS, case1);
+            vEL = _mm_blendv_epi8(vEL, vWL, case1);
             vEL = _mm_adds_epi16(vEL, vOne);
             vs2 = _mm_srli_si128(vs2, 2);
             vs2 = _mm_insert_epi16(vs2, s2[j], 7);
@@ -480,32 +475,32 @@ parasail_result_t* FNAME(
             vWH = _mm_max_epi16(vWH, vF);
             case1 = _mm_cmpeq_epi16(vWH, vNWH);
             case2 = _mm_cmpeq_epi16(vWH, vF);
-            vWM = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
+            vWM = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vEM, vFM, case2),
                     _mm_adds_epi16(vNWM,
                         _mm_and_si128(
                             _mm_cmpeq_epi16(vs1,vs2),
                             vOne)),
                     case1);
-            vWS = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vES, vFS, case2),
+            vWS = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vES, vFS, case2),
                     _mm_adds_epi16(vNWS,
                         _mm_and_si128(
                             _mm_cmpgt_epi16(vMat,vZero),
                             vOne)),
                     case1);
-            vWL = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
+            vWL = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vEL, vFL, case2),
                     _mm_adds_epi16(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
                 __m128i cond = _mm_cmpeq_epi16(vJ,vNegOne);
-                vWH = _mm_blendv_epi8_rpl(vWH, vIBoundary, cond);
+                vWH = _mm_blendv_epi8(vWH, vIBoundary, cond);
                 vWM = _mm_andnot_si128(cond, vWM);
                 vWS = _mm_andnot_si128(cond, vWS);
                 vWL = _mm_andnot_si128(cond, vWL);
-                vE = _mm_blendv_epi8_rpl(vE, vNegInf, cond);
+                vE = _mm_blendv_epi8(vE, vNegInf, cond);
                 vEM = _mm_andnot_si128(cond, vEM);
                 vES = _mm_andnot_si128(cond, vES);
                 vEL = _mm_andnot_si128(cond, vEL);
@@ -545,10 +540,10 @@ parasail_result_t* FNAME(
                 __m128i cond_valid_I = _mm_cmpeq_epi16(vI, vILimit1);
                 __m128i cond_valid_J = _mm_cmpeq_epi16(vJ, vJLimit1);
                 __m128i cond_all = _mm_and_si128(cond_valid_I, cond_valid_J);
-                vMaxH = _mm_blendv_epi8_rpl(vMaxH, vWH, cond_all);
-                vMaxM = _mm_blendv_epi8_rpl(vMaxM, vWM, cond_all);
-                vMaxS = _mm_blendv_epi8_rpl(vMaxS, vWS, cond_all);
-                vMaxL = _mm_blendv_epi8_rpl(vMaxL, vWL, cond_all);
+                vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
+                vMaxM = _mm_blendv_epi8(vMaxM, vWM, cond_all);
+                vMaxS = _mm_blendv_epi8(vMaxS, vWS, cond_all);
+                vMaxL = _mm_blendv_epi8(vMaxL, vWL, cond_all);
             }
             vJ = _mm_adds_epi16(vJ, vOne);
         }
