@@ -14,6 +14,7 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
+#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -21,12 +22,6 @@
 #include "parasail/internal_sse.h"
 
 
-static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
-    a = _mm_andnot_si128(mask, a);
-    a = _mm_or_si128(a, _mm_and_si128(mask, b));
-    return a;
-}
-
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -37,14 +32,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -55,27 +58,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_scan_sse2_128_16
-#define PNAME parasail_nw_stats_table_scan_profile_sse2_128_16
+#define FNAME parasail_nw_stats_table_scan_sse41_128_8
+#define PNAME parasail_nw_stats_table_scan_profile_sse41_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_scan_sse2_128_16
-#define PNAME parasail_nw_stats_rowcol_scan_profile_sse2_128_16
+#define FNAME parasail_nw_stats_rowcol_scan_sse41_128_8
+#define PNAME parasail_nw_stats_rowcol_scan_profile_sse41_128_8
 #else
-#define FNAME parasail_nw_stats_scan_sse2_128_16
-#define PNAME parasail_nw_stats_scan_profile_sse2_128_16
+#define FNAME parasail_nw_stats_scan_sse41_128_8
+#define PNAME parasail_nw_stats_scan_profile_sse41_128_8
 #endif
 #endif
 
@@ -103,7 +114,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_sse_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_sse_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -146,17 +157,17 @@ parasail_result_t* PNAME(
     __m128i* restrict pvHLMax = NULL;
     __m128i* restrict pvGapper = NULL;
     __m128i* restrict pvGapperL = NULL;
-    int16_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     __m128i vZero;
     __m128i vOne;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -168,7 +179,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -184,32 +195,32 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP  = (__m128i*)profile->profile16.score;
-    pvPm = (__m128i*)profile->profile16.matches;
-    pvPs = (__m128i*)profile->profile16.similar;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    pvP  = (__m128i*)profile->profile8.score;
+    pvPm = (__m128i*)profile->profile8.matches;
+    pvPs = (__m128i*)profile->profile8.similar;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi16(1);
+    vOne = _mm_set1_epi8(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vNegInfFront = vZero;
-    vSegLen = _mm_slli_si128(_mm_set1_epi16(segLen), 2);
-    vNegInfFront = _mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_adds_epi16(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi16(-segLen*gap), 2));
+    vSegLen = _mm_slli_si128(_mm_set1_epi8(segLen), 1);
+    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -226,7 +237,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -249,7 +260,7 @@ parasail_result_t* PNAME(
     pvHLMax = parasail_memalign___m128i(16, segLen);
     pvGapper = parasail_memalign___m128i(16, segLen);
     pvGapperL = parasail_memalign___m128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvE) return NULL;
@@ -276,13 +287,13 @@ parasail_result_t* PNAME(
     parasail_memset___m128i(pvES, vZero, segLen);
     parasail_memset___m128i(pvEL, vZero, segLen);
     {
-        __m128i vGapper = _mm_subs_epi16(vZero,vGapO);
+        __m128i vGapper = _mm_subs_epi8(vZero,vGapO);
         __m128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
             _mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = _mm_subs_epi16(vGapper, vGapE);
-            vGapperL = _mm_adds_epi16(vGapperL, vOne);
+            vGapper = _mm_subs_epi8(vGapper, vGapE);
+            vGapperL = _mm_adds_epi8(vGapperL, vOne);
         }
     }
 
@@ -291,10 +302,10 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_16_t h;
+            __m128i_8_t h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             ++index;
@@ -306,7 +317,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -354,15 +365,15 @@ parasail_result_t* PNAME(
         vHpM = _mm_load_si128(pvHM+(segLen-1));
         vHpS = _mm_load_si128(pvHS+(segLen-1));
         vHpL = _mm_load_si128(pvHL+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 2);
-        vHpM = _mm_slli_si128(vHpM, 2);
-        vHpS = _mm_slli_si128(vHpS, 2);
-        vHpL = _mm_slli_si128(vHpL, 2);
-        vHp = _mm_insert_epi16(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 1);
+        vHpM = _mm_slli_si128(vHpM, 1);
+        vHpS = _mm_slli_si128(vHpS, 1);
+        vHpL = _mm_slli_si128(vHpL, 1);
+        vHp = _mm_insert_epi8(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_subs_epi16(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -381,33 +392,33 @@ parasail_result_t* PNAME(
             vWS = _mm_load_si128(pvWS+i);
             vGapper = _mm_load_si128(pvGapper+i);
             vGapperL = _mm_load_si128(pvGapperL+i);
-            vE_opn = _mm_subs_epi16(vH, vGapO);
-            vE_ext = _mm_subs_epi16(vE, vGapE);
-            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
-            vE = _mm_max_epi16(vE_opn, vE_ext);
-            vEM = _mm_blendv_epi8_rpl(vEM, vHM, case1);
-            vES = _mm_blendv_epi8_rpl(vES, vHS, case1);
-            vEL = _mm_blendv_epi8_rpl(vEL, vHL, case1);
-            vEL = _mm_adds_epi16(vEL, vOne);
-            vGapper = _mm_adds_epi16(vHt, vGapper);
+            vE_opn = _mm_subs_epi8(vH, vGapO);
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
+            vE = _mm_max_epi8(vE_opn, vE_ext);
+            vEM = _mm_blendv_epi8(vEM, vHM, case1);
+            vES = _mm_blendv_epi8(vES, vHS, case1);
+            vEL = _mm_blendv_epi8(vEL, vHL, case1);
+            vEL = _mm_adds_epi8(vEL, vOne);
+            vGapper = _mm_adds_epi8(vHt, vGapper);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi16(vF, vGapper),
-                    _mm_cmpeq_epi16(vF, vGapper));
-            vF = _mm_max_epi16(vF, vGapper);
-            vFM = _mm_blendv_epi8_rpl(vHtM, vFM, case1);
-            vFS = _mm_blendv_epi8_rpl(vHtS, vFS, case1);
-            vFL = _mm_blendv_epi8_rpl(
-                    _mm_adds_epi16(vHtL, vGapperL),
+                    _mm_cmpgt_epi8(vF, vGapper),
+                    _mm_cmpeq_epi8(vF, vGapper));
+            vF = _mm_max_epi8(vF, vGapper);
+            vFM = _mm_blendv_epi8(vHtM, vFM, case1);
+            vFS = _mm_blendv_epi8(vHtS, vFS, case1);
+            vFL = _mm_blendv_epi8(
+                    _mm_adds_epi8(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm_adds_epi16(vHp, vW);
-            vHpM = _mm_adds_epi16(vHpM, vWM);
-            vHpS = _mm_adds_epi16(vHpS, vWS);
-            vHpL = _mm_adds_epi16(vHpL, vOne);
-            case1 = _mm_cmpgt_epi16(vE, vHp);
-            vHt = _mm_max_epi16(vE, vHp);
-            vHtM = _mm_blendv_epi8_rpl(vHpM, vEM, case1);
-            vHtS = _mm_blendv_epi8_rpl(vHpS, vES, case1);
-            vHtL = _mm_blendv_epi8_rpl(vHpL, vEL, case1);
+            vHp = _mm_adds_epi8(vHp, vW);
+            vHpM = _mm_adds_epi8(vHpM, vWM);
+            vHpS = _mm_adds_epi8(vHpS, vWS);
+            vHpL = _mm_adds_epi8(vHpL, vOne);
+            case1 = _mm_cmpgt_epi8(vE, vHp);
+            vHt = _mm_max_epi8(vE, vHp);
+            vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
+            vHtS = _mm_blendv_epi8(vHpS, vES, case1);
+            vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvEM+i, vEM);
             _mm_store_si128(pvES+i, vES);
@@ -423,53 +434,53 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 2);
-        vHtM = _mm_slli_si128(vHtM, 2);
-        vHtS = _mm_slli_si128(vHtS, 2);
-        vHtL = _mm_slli_si128(vHtL, 2);
-        vHt = _mm_insert_epi16(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 1);
+        vHtM = _mm_slli_si128(vHtM, 1);
+        vHtS = _mm_slli_si128(vHtS, 1);
+        vHtL = _mm_slli_si128(vHtL, 1);
+        vHt = _mm_insert_epi8(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
-        vGapper = _mm_adds_epi16(vHt, vGapper);
+        vGapper = _mm_adds_epi8(vHt, vGapper);
         case1 = _mm_or_si128(
-                _mm_cmpgt_epi16(vGapper, vF),
-                _mm_cmpeq_epi16(vGapper, vF));
-        vF = _mm_max_epi16(vF, vGapper);
-        vFM = _mm_blendv_epi8_rpl(vFM, vHtM, case1);
-        vFS = _mm_blendv_epi8_rpl(vFS, vHtS, case1);
-        vFL = _mm_blendv_epi8_rpl(
+                _mm_cmpgt_epi8(vGapper, vF),
+                _mm_cmpeq_epi8(vGapper, vF));
+        vF = _mm_max_epi8(vF, vGapper);
+        vFM = _mm_blendv_epi8(vFM, vHtM, case1);
+        vFS = _mm_blendv_epi8(vFS, vHtS, case1);
+        vFL = _mm_blendv_epi8(
                 vFL,
-                _mm_adds_epi16(vHtL, vGapperL),
+                _mm_adds_epi8(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            __m128i vFt = _mm_slli_si128(vF, 2);
-            __m128i vFtM = _mm_slli_si128(vFM, 2);
-            __m128i vFtS = _mm_slli_si128(vFS, 2);
-            __m128i vFtL = _mm_slli_si128(vFL, 2);
-            vFt = _mm_adds_epi16(vFt, vSegLenXgap);
+            __m128i vFt = _mm_slli_si128(vF, 1);
+            __m128i vFtM = _mm_slli_si128(vFM, 1);
+            __m128i vFtS = _mm_slli_si128(vFS, 1);
+            __m128i vFtL = _mm_slli_si128(vFL, 1);
+            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi16(vFt, vF),
-                    _mm_cmpeq_epi16(vFt, vF));
-            vF = _mm_max_epi16(vF, vFt);
-            vFM = _mm_blendv_epi8_rpl(vFM, vFtM, case1);
-            vFS = _mm_blendv_epi8_rpl(vFS, vFtS, case1);
-            vFL = _mm_blendv_epi8_rpl(
+                    _mm_cmpgt_epi8(vFt, vF),
+                    _mm_cmpeq_epi8(vFt, vF));
+            vF = _mm_max_epi8(vF, vFt);
+            vFM = _mm_blendv_epi8(vFM, vFtM, case1);
+            vFS = _mm_blendv_epi8(vFS, vFtS, case1);
+            vFL = _mm_blendv_epi8(
                     vFL,
-                    _mm_adds_epi16(vFtL, vSegLen),
+                    _mm_adds_epi8(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 2);
-        vFM = _mm_slli_si128(vFM, 2);
-        vFS = _mm_slli_si128(vFS, 2);
-        vFL = _mm_slli_si128(vFL, 2);
-        vF = _mm_adds_epi16(vF, vNegInfFront);
-        case1 = _mm_cmpgt_epi16(vF, vHt);
-        vH = _mm_max_epi16(vF, vHt);
-        vHM = _mm_blendv_epi8_rpl(vHtM, vFM, case1);
-        vHS = _mm_blendv_epi8_rpl(vHtS, vFS, case1);
-        vHL = _mm_blendv_epi8_rpl(vHtL, vFL, case1);
+        vF = _mm_slli_si128(vF, 1);
+        vFM = _mm_slli_si128(vFM, 1);
+        vFS = _mm_slli_si128(vFS, 1);
+        vFL = _mm_slli_si128(vFL, 1);
+        vF = _mm_adds_epi8(vF, vNegInfFront);
+        case1 = _mm_cmpgt_epi8(vF, vHt);
+        vH = _mm_max_epi8(vF, vHt);
+        vHM = _mm_blendv_epi8(vHtM, vFM, case1);
+        vHS = _mm_blendv_epi8(vHtS, vFS, case1);
+        vHL = _mm_blendv_epi8(vHtL, vFL, case1);
         for (i=0; i<segLen; ++i) {
             vHp = _mm_load_si128(pvH+i);
             vHpM = _mm_load_si128(pvHM+i);
@@ -479,36 +490,36 @@ parasail_result_t* PNAME(
             vEM = _mm_load_si128(pvEM+i);
             vES = _mm_load_si128(pvES+i);
             vEL = _mm_load_si128(pvEL+i);
-            vF_opn = _mm_subs_epi16(vH, vGapO);
-            vF_ext = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
-            vFM = _mm_blendv_epi8_rpl(vFM, vHM, case1);
-            vFS = _mm_blendv_epi8_rpl(vFS, vHS, case1);
-            vFL = _mm_blendv_epi8_rpl(vFL, vHL, case1);
-            vFL = _mm_adds_epi16(vFL, vOne);
-            vH = _mm_max_epi16(vHp, vE);
-            vH = _mm_max_epi16(vH, vF);
-            case1 = _mm_cmpeq_epi16(vH, vHp);
-            case2 = _mm_cmpeq_epi16(vH, vF);
-            vHM = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
+            vF_opn = _mm_subs_epi8(vH, vGapO);
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
+            vFM = _mm_blendv_epi8(vFM, vHM, case1);
+            vFS = _mm_blendv_epi8(vFS, vHS, case1);
+            vFL = _mm_blendv_epi8(vFL, vHL, case1);
+            vFL = _mm_adds_epi8(vFL, vOne);
+            vH = _mm_max_epi8(vHp, vE);
+            vH = _mm_max_epi8(vH, vF);
+            case1 = _mm_cmpeq_epi8(vH, vHp);
+            case2 = _mm_cmpeq_epi8(vH, vF);
+            vHM = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
-            vHS = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vES, vFS, case2),
+            vHS = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vES, vFS, case2),
                     vHpS, case1);
-            vHL = _mm_blendv_epi8_rpl(
-                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
+            vHL = _mm_blendv_epi8(
+                    _mm_blendv_epi8(vEL, vFL, case2),
                     vHpL, case1);
             _mm_store_si128(pvH+i, vH);
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -525,15 +536,15 @@ parasail_result_t* PNAME(
             vHS = _mm_load_si128(pvHS + offset);
             vHL = _mm_load_si128(pvHL + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
-                vHM = _mm_slli_si128(vHM, 2);
-                vHS = _mm_slli_si128(vHS, 2);
-                vHL = _mm_slli_si128(vHL, 2);
+                vH = _mm_slli_si128(vH, 1);
+                vHM = _mm_slli_si128(vHM, 1);
+                vHS = _mm_slli_si128(vHS, 1);
+                vHL = _mm_slli_si128(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
-            result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7);
-            result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7);
-            result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7);
+            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8 (vHM, 15);
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8 (vHS, 15);
+            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8 (vHL, 15);
         }
 #endif
     }
@@ -558,20 +569,20 @@ parasail_result_t* PNAME(
         __m128i vHS = _mm_load_si128(pvHS + offset);
         __m128i vHL = _mm_load_si128(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 2);
-            vHM = _mm_slli_si128(vHM, 2);
-            vHS = _mm_slli_si128(vHS, 2);
-            vHL = _mm_slli_si128(vHL, 2);
+            vH = _mm_slli_si128(vH, 1);
+            vHM = _mm_slli_si128(vHM, 1);
+            vHS = _mm_slli_si128(vHS, 1);
+            vHL = _mm_slli_si128(vHL, 1);
         }
-        score = (int16_t) _mm_extract_epi16 (vH, 7);
-        matches = (int16_t) _mm_extract_epi16 (vHM, 7);
-        similar = (int16_t) _mm_extract_epi16 (vHS, 7);
-        length = (int16_t) _mm_extract_epi16 (vHL, 7);
+        score = (int8_t) _mm_extract_epi8 (vH, 15);
+        matches = (int8_t) _mm_extract_epi8 (vHM, 15);
+        similar = (int8_t) _mm_extract_epi8 (vHS, 15);
+        length = (int8_t) _mm_extract_epi8 (vHL, 15);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
