@@ -16,78 +16,28 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI64
-#define _mm256_insert_epi64_rpl _mm256_insert_epi64
+#if HAVE_AVX2_MM256_INSERT_EPI32
+#define _mm256_insert_epi32_rpl _mm256_insert_epi32
 #else
-static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
-    __m256i_64_t A;
+static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
+    __m256i_32_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_SET1_EPI64X
-#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
+#if HAVE_AVX2_MM256_EXTRACT_EPI32
+#define _mm256_extract_epi32_rpl _mm256_extract_epi32
 #else
-static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
-    __m256i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    A.v[2] = i;
-    A.v[3] = i;
-    return A.m;
-}
-#endif
-
-static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#if HAVE_AVX2_MM256_SET_EPI64X
-#define _mm256_set_epi64x_rpl _mm256_set_epi64x
-#else
-static inline __m256i _mm256_set_epi64x_rpl(int64_t e3, int64_t e2, int64_t e1, int64_t e0) {
-    __m256i_64_t A;
-    A.v[0] = e0;
-    A.v[1] = e1;
-    A.v[2] = e2;
-    A.v[3] = e3;
-    return A.m;
-}
-#endif
-
-#if HAVE_AVX2_MM256_EXTRACT_EPI64
-#define _mm256_extract_epi64_rpl _mm256_extract_epi64
-#else
-static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
-    __m256i_64_t A;
+static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
+    __m256i_32_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
-    __m256i_64_t A;
-    __m256i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
-    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
-    return A.m;
-}
-
-#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
+#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
 
 #define _mm256_srli_si256_rpl(a,imm) _mm256_or_si256(_mm256_slli_si256(_mm256_permute2x128_si256(a, a, _MM_SHUFFLE(3,0,0,1)), 16-imm), _mm256_srli_si256(a, imm))
 
@@ -103,20 +53,32 @@ static inline void arr_store_si256(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm256_extract_epi64_rpl(vWH, 3);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 7);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm256_extract_epi64_rpl(vWH, 2);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 6);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)_mm256_extract_epi64_rpl(vWH, 1);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 5);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)_mm256_extract_epi64_rpl(vWH, 0);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 4);
+    }
+    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
+        array[1LL*(i+4)*s2Len + (j-4)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 3);
+    }
+    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
+        array[1LL*(i+5)*s2Len + (j-5)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 2);
+    }
+    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
+        array[1LL*(i+6)*s2Len + (j-6)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 1);
+    }
+    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
+        array[1LL*(i+7)*s2Len + (j-7)] = (int8_t)_mm256_extract_epi32_rpl(vWH, 0);
     }
 }
 
-#define FNAME parasail_nw_trace_diag_avx2_256_64
+#define FNAME parasail_nw_trace_diag_avx2_256_32
 
 parasail_result_t* FNAME(
         const char * const restrict _s1, const int _s1Len,
@@ -129,22 +91,22 @@ parasail_result_t* FNAME(
     int32_t PAD2 = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int64_t * restrict s1 = NULL;
-    int64_t * restrict s2B = NULL;
-    int64_t * restrict _H_pr = NULL;
-    int64_t * restrict _F_pr = NULL;
-    int64_t * restrict s2 = NULL;
-    int64_t * restrict H_pr = NULL;
-    int64_t * restrict F_pr = NULL;
+    int32_t * restrict s1 = NULL;
+    int32_t * restrict s2B = NULL;
+    int32_t * restrict _H_pr = NULL;
+    int32_t * restrict _F_pr = NULL;
+    int32_t * restrict s2 = NULL;
+    int32_t * restrict H_pr = NULL;
+    int32_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t s1Len = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
-    int64_t score = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -184,7 +146,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 4; /* number of values in vector */
+    N = 8; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -194,39 +156,43 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    vOpen = _mm256_set1_epi64x_rpl(open);
-    vGap  = _mm256_set1_epi64x_rpl(gap);
-    vOne = _mm256_set1_epi64x_rpl(1);
-    vN = _mm256_set1_epi64x_rpl(N);
-    vGapN = _mm256_set1_epi64x_rpl(gap*N);
-    vNegOne = _mm256_set1_epi64x_rpl(-1);
-    vI = _mm256_set_epi64x_rpl(0,1,2,3);
-    vJreset = _mm256_set_epi64x_rpl(0,-1,-2,-3);
+    vNegInf = _mm256_set1_epi32(NEG_LIMIT);
+    vOpen = _mm256_set1_epi32(open);
+    vGap  = _mm256_set1_epi32(gap);
+    vOne = _mm256_set1_epi32(1);
+    vN = _mm256_set1_epi32(N);
+    vGapN = _mm256_set1_epi32(gap*N);
+    vNegOne = _mm256_set1_epi32(-1);
+    vI = _mm256_set_epi32(0,1,2,3,4,5,6,7);
+    vJreset = _mm256_set_epi32(0,-1,-2,-3,-4,-5,-6,-7);
     vMax = vNegInf;
-    vILimit = _mm256_set1_epi64x_rpl(s1Len);
-    vILimit1 = _mm256_sub_epi64(vILimit, vOne);
-    vJLimit = _mm256_set1_epi64x_rpl(s2Len);
-    vJLimit1 = _mm256_sub_epi64(vJLimit, vOne);
-    vIBoundary = _mm256_set_epi64x_rpl(
+    vILimit = _mm256_set1_epi32(s1Len);
+    vILimit1 = _mm256_sub_epi32(vILimit, vOne);
+    vJLimit = _mm256_set1_epi32(s2Len);
+    vJLimit1 = _mm256_sub_epi32(vJLimit, vOne);
+    vIBoundary = _mm256_set_epi32(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
-            -open-3*gap);
-    vTDiag = _mm256_set1_epi64x_rpl(PARASAIL_DIAG);
-    vTIns = _mm256_set1_epi64x_rpl(PARASAIL_INS);
-    vTDel = _mm256_set1_epi64x_rpl(PARASAIL_DEL);
-    vTDiagE = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_E);
-    vTInsE = _mm256_set1_epi64x_rpl(PARASAIL_INS_E);
-    vTDiagF = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_F);
-    vTDelF = _mm256_set1_epi64x_rpl(PARASAIL_DEL_F);
+            -open-3*gap,
+            -open-4*gap,
+            -open-5*gap,
+            -open-6*gap,
+            -open-7*gap);
+    vTDiag = _mm256_set1_epi32(PARASAIL_DIAG);
+    vTIns = _mm256_set1_epi32(PARASAIL_INS);
+    vTDel = _mm256_set1_epi32(PARASAIL_DEL);
+    vTDiagE = _mm256_set1_epi32(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi32(PARASAIL_INS_E);
+    vTDiagF = _mm256_set1_epi32(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi32(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(s1Len, s2Len, 32, sizeof(int8_t));
@@ -235,12 +201,12 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int64_t(32, s2Len+PAD2);
-    _H_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    _F_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    s2B= parasail_memalign_int32_t(32, s2Len+PAD2);
+    _H_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _F_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -252,7 +218,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int64_t(32, s1Len+PAD);
+        s1 = parasail_memalign_int32_t(32, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -308,92 +274,100 @@ parasail_result_t* FNAME(
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
         const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
         const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        vNH = _mm256_srli_si256_rpl(vNH, 8);
-        vNH = _mm256_insert_epi64_rpl(vNH, H_pr[-1], 3);
-        vWH = _mm256_srli_si256_rpl(vWH, 8);
-        vWH = _mm256_insert_epi64_rpl(vWH, -open - i*gap, 3);
+        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
+        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
+        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
+        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
+        vNH = _mm256_srli_si256_rpl(vNH, 4);
+        vNH = _mm256_insert_epi32_rpl(vNH, H_pr[-1], 7);
+        vWH = _mm256_srli_si256_rpl(vWH, 4);
+        vWH = _mm256_insert_epi32_rpl(vWH, -open - i*gap, 7);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             __m256i vMat;
             __m256i vNWH = vNH;
-            vNH = _mm256_srli_si256_rpl(vWH, 8);
-            vNH = _mm256_insert_epi64_rpl(vNH, H_pr[j], 3);
-            vF = _mm256_srli_si256_rpl(vF, 8);
-            vF = _mm256_insert_epi64_rpl(vF, F_pr[j], 3);
-            vF_opn = _mm256_sub_epi64(vNH, vOpen);
-            vF_ext = _mm256_sub_epi64(vF, vGap);
-            vF = _mm256_max_epi64_rpl(vF_opn, vF_ext);
-            vE_opn = _mm256_sub_epi64(vWH, vOpen);
-            vE_ext = _mm256_sub_epi64(vE, vGap);
-            vE = _mm256_max_epi64_rpl(vE_opn, vE_ext);
-            vMat = _mm256_set_epi64x_rpl(
+            vNH = _mm256_srli_si256_rpl(vWH, 4);
+            vNH = _mm256_insert_epi32_rpl(vNH, H_pr[j], 7);
+            vF = _mm256_srli_si256_rpl(vF, 4);
+            vF = _mm256_insert_epi32_rpl(vF, F_pr[j], 7);
+            vF_opn = _mm256_sub_epi32(vNH, vOpen);
+            vF_ext = _mm256_sub_epi32(vF, vGap);
+            vF = _mm256_max_epi32(vF_opn, vF_ext);
+            vE_opn = _mm256_sub_epi32(vWH, vOpen);
+            vE_ext = _mm256_sub_epi32(vE, vGap);
+            vE = _mm256_max_epi32(vE_opn, vE_ext);
+            vMat = _mm256_set_epi32(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
-                    matrow3[s2[j-3]]
+                    matrow3[s2[j-3]],
+                    matrow4[s2[j-4]],
+                    matrow5[s2[j-5]],
+                    matrow6[s2[j-6]],
+                    matrow7[s2[j-7]]
                     );
-            vNWH = _mm256_add_epi64(vNWH, vMat);
-            vWH = _mm256_max_epi64_rpl(vNWH, vE);
-            vWH = _mm256_max_epi64_rpl(vWH, vF);
+            vNWH = _mm256_add_epi32(vNWH, vMat);
+            vWH = _mm256_max_epi32(vNWH, vE);
+            vWH = _mm256_max_epi32(vWH, vF);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m256i cond = _mm256_cmpeq_epi64(vJ,vNegOne);
+                __m256i cond = _mm256_cmpeq_epi32(vJ,vNegOne);
                 vWH = _mm256_blendv_epi8(vWH, vIBoundary, cond);
                 vF = _mm256_blendv_epi8(vF, vNegInf, cond);
                 vE = _mm256_blendv_epi8(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWH);
             }
             /* trace table */
             {
-                __m256i case1 = _mm256_cmpeq_epi64(vWH, vNWH);
-                __m256i case2 = _mm256_cmpeq_epi64(vWH, vF);
+                __m256i case1 = _mm256_cmpeq_epi32(vWH, vNWH);
+                __m256i case2 = _mm256_cmpeq_epi32(vWH, vF);
                 __m256i vT = _mm256_blendv_epi8(
                         _mm256_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag,
                         case1);
-                __m256i condE = _mm256_cmpgt_epi64(vE_opn, vE_ext);
-                __m256i condF = _mm256_cmpgt_epi64(vF_opn, vF_ext);
+                __m256i condE = _mm256_cmpgt_epi32(vE_opn, vE_ext);
+                __m256i condF = _mm256_cmpgt_epi32(vF_opn, vF_ext);
                 __m256i vET = _mm256_blendv_epi8(vTInsE, vTDiagE, condE);
                 __m256i vFT = _mm256_blendv_epi8(vTDelF, vTDiagF, condF);
                 vT = _mm256_or_si256(vT, vET);
                 vT = _mm256_or_si256(vT, vFT);
                 arr_store_si256(result->trace->trace_table, vT, i, s1Len, j, s2Len);
             }
-            H_pr[j-3] = (int64_t)_mm256_extract_epi64_rpl(vWH,0);
-            F_pr[j-3] = (int64_t)_mm256_extract_epi64_rpl(vF,0);
+            H_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWH,0);
+            F_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vF,0);
             /* as minor diagonal vector passes across table, extract
                last table value at the i,j bound */
             {
-                __m256i cond_valid_I = _mm256_cmpeq_epi64(vI, vILimit1);
-                __m256i cond_valid_J = _mm256_cmpeq_epi64(vJ, vJLimit1);
+                __m256i cond_valid_I = _mm256_cmpeq_epi32(vI, vILimit1);
+                __m256i cond_valid_J = _mm256_cmpeq_epi32(vJ, vJLimit1);
                 __m256i cond_all = _mm256_and_si256(cond_valid_I, cond_valid_J);
                 vMax = _mm256_blendv_epi8(vMax, vWH, cond_all);
             }
-            vJ = _mm256_add_epi64(vJ, vOne);
+            vJ = _mm256_add_epi32(vJ, vOne);
         }
-        vI = _mm256_add_epi64(vI, vN);
-        vIBoundary = _mm256_sub_epi64(vIBoundary, vGapN);
+        vI = _mm256_add_epi32(vI, vN);
+        vIBoundary = _mm256_sub_epi32(vIBoundary, vGapN);
     }
 
     /* max in vMax */
     for (i=0; i<N; ++i) {
-        int64_t value;
-        value = (int64_t) _mm256_extract_epi64_rpl(vMax, 3);
+        int32_t value;
+        value = (int32_t) _mm256_extract_epi32_rpl(vMax, 7);
         if (value > score) {
             score = value;
         }
-        vMax = _mm256_slli_si256_rpl(vMax, 8);
+        vMax = _mm256_slli_si256_rpl(vMax, 4);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
