@@ -37,8 +37,8 @@ static inline simde__m128i arr_load(
     return simde_mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_nw_trace_scan_neon_128_64
-#define PNAME parasail_nw_trace_scan_profile_neon_128_64
+#define FNAME parasail_nw_trace_scan_neon_128_8
+#define PNAME parasail_nw_trace_scan_profile_neon_128_8
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -61,7 +61,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -89,16 +89,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     simde__m128i* restrict pvP = NULL;
     simde__m128i* restrict pvE = NULL;
-    int64_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     simde__m128i* restrict pvHt = NULL;
     simde__m128i* restrict pvH = NULL;
     simde__m128i* restrict pvGapper = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     simde__m128i vZero;
-    int64_t score = 0;
+    int8_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -116,7 +116,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -132,32 +132,32 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (simde__m128i*)profile->profile64.score;
-    vGapO = simde_mm_set1_epi64x(open);
-    vGapE = simde_mm_set1_epi64x(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    pvP = (simde__m128i*)profile->profile8.score;
+    vGapO = simde_mm_set1_epi8(open);
+    vGapE = simde_mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vNegInfFront = vZero;
-    vTIns  = simde_mm_set1_epi64x(PARASAIL_INS);
-    vTDel  = simde_mm_set1_epi64x(PARASAIL_DEL);
-    vTDiag = simde_mm_set1_epi64x(PARASAIL_DIAG);
-    vTDiagE = simde_mm_set1_epi64x(PARASAIL_DIAG_E);
-    vTInsE = simde_mm_set1_epi64x(PARASAIL_INS_E);
-    vTDiagF = simde_mm_set1_epi64x(PARASAIL_DIAG_F);
-    vTDelF = simde_mm_set1_epi64x(PARASAIL_DEL_F);
-    vNegInfFront = simde_mm_insert_epi64(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = simde_mm_add_epi64(vNegInfFront,
-            simde_mm_slli_si128(simde_mm_set1_epi64x(-segLen*gap), 8));
+    vTIns  = simde_mm_set1_epi8(PARASAIL_INS);
+    vTDel  = simde_mm_set1_epi8(PARASAIL_DEL);
+    vTDiag = simde_mm_set1_epi8(PARASAIL_DIAG);
+    vTDiagE = simde_mm_set1_epi8(PARASAIL_DIAG_E);
+    vTInsE = simde_mm_set1_epi8(PARASAIL_INS_E);
+    vTDiagF = simde_mm_set1_epi8(PARASAIL_DIAG_F);
+    vTDelF = simde_mm_set1_epi8(PARASAIL_DEL_F);
+    vNegInfFront = simde_mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = simde_mm_adds_epi8(vNegInfFront,
+            simde_mm_slli_si128(simde_mm_set1_epi8(-segLen*gap), 1));
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(simde__m128i));
@@ -166,11 +166,11 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 
     /* initialize heap variables */
     pvE = parasail_memalign_simde__m128i(16, segLen);
-    boundary = parasail_memalign_int64_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
     pvHt= parasail_memalign_simde__m128i(16, segLen);
     pvH = parasail_memalign_simde__m128i(16, segLen);
     pvGapper = parasail_memalign_simde__m128i(16, segLen);
@@ -191,9 +191,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             simde_mm_store_si128(&pvH[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -206,15 +206,15 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
     {
-        simde__m128i vGapper = simde_mm_sub_epi64(vZero,vGapO);
+        simde__m128i vGapper = simde_mm_subs_epi8(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             simde_mm_store_si128(pvGapper+i, vGapper);
-            vGapper = simde_mm_sub_epi64(vGapper, vGapE);
+            vGapper = simde_mm_subs_epi8(vGapper, vGapE);
         }
     }
 
@@ -242,27 +242,27 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = simde_mm_load_si128(pvH+(segLen-1));
-        vHp = simde_mm_slli_si128(vHp, 8);
-        vHp = simde_mm_insert_epi64(vHp, boundary[j], 0);
+        vHp = simde_mm_slli_si128(vHp, 1);
+        vHp = simde_mm_insert_epi8(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = simde_mm_sub_epi64(vNegLimit, pvGapper[0]);
+        vHt = simde_mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = simde_mm_load_si128(pvH+i);
             vE = simde_mm_load_si128(pvE+i);
             vW = simde_mm_load_si128(pvW+i);
             vGapper = simde_mm_load_si128(pvGapper+i);
-            vE_opn = simde_mm_sub_epi64(vH, vGapO);
-            vE_ext = simde_mm_sub_epi64(vE, vGapE);
-            case1 = simde_mm_cmpgt_epi64(vE_opn, vE_ext);
+            vE_opn = simde_mm_subs_epi8(vH, vGapO);
+            vE_ext = simde_mm_subs_epi8(vE, vGapE);
+            case1 = simde_mm_cmpgt_epi8(vE_opn, vE_ext);
             vET = simde_mm_blendv_epi8(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = simde_mm_max_epi64(vE_opn, vE_ext);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vE);
-            vGapper = simde_mm_add_epi64(vHt, vGapper);
-            vF = simde_mm_max_epi64(vF, vGapper);
-            vHp = simde_mm_add_epi64(vHp, vW);
-            vHt = simde_mm_max_epi64(vE, vHp);
+            vE = simde_mm_max_epi8(vE_opn, vE_ext);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vE);
+            vGapper = simde_mm_adds_epi8(vHt, vGapper);
+            vF = simde_mm_max_epi8(vF, vGapper);
+            vHp = simde_mm_adds_epi8(vHp, vW);
+            vHt = simde_mm_max_epi8(vE, vHp);
             simde_mm_store_si128(pvE+i, vE);
             simde_mm_store_si128(pvHt+i, vHt);
             simde_mm_store_si128(pvH+i, vHp);
@@ -270,33 +270,33 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = simde_mm_slli_si128(vHt, 8);
-        vHt = simde_mm_insert_epi64(vHt, boundary[j+1], 0);
+        vHt = simde_mm_slli_si128(vHt, 1);
+        vHt = simde_mm_insert_epi8(vHt, boundary[j+1], 0);
         vGapper = simde_mm_load_si128(pvGapper);
-        vGapper = simde_mm_add_epi64(vHt, vGapper);
-        vF = simde_mm_max_epi64(vF, vGapper);
+        vGapper = simde_mm_adds_epi8(vHt, vGapper);
+        vF = simde_mm_max_epi8(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            simde__m128i vFt = simde_mm_slli_si128(vF, 8);
-            vFt = simde_mm_add_epi64(vFt, vSegLenXgap);
-            vF = simde_mm_max_epi64(vF, vFt);
+            simde__m128i vFt = simde_mm_slli_si128(vF, 1);
+            vFt = simde_mm_adds_epi8(vFt, vSegLenXgap);
+            vF = simde_mm_max_epi8(vF, vFt);
         }
 
         /* calculate final H */
-        vF = simde_mm_slli_si128(vF, 8);
-        vF = simde_mm_add_epi64(vF, vNegInfFront);
-        vH = simde_mm_max_epi64(vF, vHt);
+        vF = simde_mm_slli_si128(vF, 1);
+        vF = simde_mm_adds_epi8(vF, vNegInfFront);
+        vH = simde_mm_max_epi8(vF, vHt);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = simde_mm_load_si128(pvH+i);
             vHt = simde_mm_load_si128(pvHt+i);
-            vF_opn = simde_mm_sub_epi64(vH, vGapO);
-            vF_ext = simde_mm_sub_epi64(vF, vGapE);
-            vF = simde_mm_max_epi64(vF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi64(vF_opn, vF_ext);
+            vF_opn = simde_mm_subs_epi8(vH, vGapO);
+            vF_ext = simde_mm_subs_epi8(vF, vGapE);
+            vF = simde_mm_max_epi8(vF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi8(vF_opn, vF_ext);
             vFT = simde_mm_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = simde_mm_max_epi64(vHt, vF);
-            case1 = simde_mm_cmpeq_epi64(vH, vHp);
-            case2 = simde_mm_cmpeq_epi64(vH, vF);
+            vH = simde_mm_max_epi8(vHt, vF);
+            case1 = simde_mm_cmpeq_epi8(vH, vHp);
+            case2 = simde_mm_cmpeq_epi8(vH, vF);
             vT = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vTIns, vTDel, case2),
                     vTDiag, case1);
@@ -304,9 +304,9 @@ parasail_result_t* PNAME(
             vT = simde_mm_or_si128(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             simde_mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
-            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vF);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vF);
+            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vH);
         }
     }
 
@@ -314,14 +314,14 @@ parasail_result_t* PNAME(
     {
         simde__m128i vH = simde_mm_load_si128(pvH + offset);
         for (k=0; k<position; ++k) {
-            vH = simde_mm_slli_si128(vH, 8);
+            vH = simde_mm_slli_si128(vH, 1);
         }
-        score = (int64_t) simde_mm_extract_epi64 (vH, 1);
+        score = (int8_t) simde_mm_extract_epi8 (vH, 15);
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
