@@ -39,8 +39,8 @@ static inline vec128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_nw_trace_striped_altivec_128_32
-#define PNAME parasail_nw_trace_striped_profile_altivec_128_32
+#define FNAME parasail_nw_trace_striped_altivec_128_8
+#define PNAME parasail_nw_trace_striped_profile_altivec_128_8
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -63,7 +63,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_altivec_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_altivec_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -96,12 +96,12 @@ parasail_result_t* PNAME(
     vec128i* restrict pvEaStore = NULL;
     vec128i* restrict pvEaLoad = NULL;
     vec128i* restrict pvHT = NULL;
-    int32_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
+    int8_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -119,7 +119,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -135,29 +135,29 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (vec128i*)profile->profile32.score;
-    vGapO = _mm_set1_epi32(open);
-    vGapE = _mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile = (vec128i*)profile->profile8.score;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vTIns  = _mm_set1_epi32(PARASAIL_INS);
-    vTDel  = _mm_set1_epi32(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi32(PARASAIL_DIAG);
-    vTDiagE = _mm_set1_epi32(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi32(PARASAIL_INS_E);
-    vTDiagF = _mm_set1_epi32(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi32(PARASAIL_DEL_F);
-    vTMask = _mm_set1_epi32(PARASAIL_ZERO_MASK);
-    vFTMask = _mm_set1_epi32(PARASAIL_F_MASK);
+    vTIns  = _mm_set1_epi8(PARASAIL_INS);
+    vTDel  = _mm_set1_epi8(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi8(PARASAIL_DIAG);
+    vTDiagE = _mm_set1_epi8(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi8(PARASAIL_INS_E);
+    vTDiagF = _mm_set1_epi8(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi8(PARASAIL_DEL_F);
+    vTMask = _mm_set1_epi8(PARASAIL_ZERO_MASK);
+    vFTMask = _mm_set1_epi8(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(vec128i));
@@ -166,7 +166,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 
     /* initialize heap variables */
     pvHStore = parasail_memalign_vec128i(16, segLen);
@@ -175,7 +175,7 @@ parasail_result_t* PNAME(
     pvEaStore = parasail_memalign_vec128i(16, segLen);
     pvEaLoad = parasail_memalign_vec128i(16, segLen);
     pvHT = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int32_t(16, s2Len+1);
+    boundary = parasail_memalign_int8_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -191,13 +191,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_32_t h;
-            vec128i_32_t e;
+            vec128i_8_t h;
+            vec128i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = -open-gap*(segNum*segLen+i);
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -211,7 +211,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = -open-gap*(i-1);
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -236,12 +236,12 @@ parasail_result_t* PNAME(
          * corrected in the Lazy_F loop.  */
         vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 4 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 4);
+        vH = _mm_slli_si128(vH, 1);
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi32(vH, boundary[j], 0);
+        vH = _mm_insert_epi8(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -255,20 +255,20 @@ parasail_result_t* PNAME(
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_add_epi32(vH, _mm_load_si128(vP + i));
-            vH = _mm_max_epi32(vH_dag, vE);
-            vH = _mm_max_epi32(vH, vF);
+            vH_dag = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
+            vH = _mm_max_epi8(vH_dag, vE);
+            vH = _mm_max_epi8(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vE);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vE);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vF);
 
             {
                 vec128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                vec128i case1 = _mm_cmpeq_epi32(vH, vH_dag);
-                vec128i case2 = _mm_cmpeq_epi32(vH, vF);
+                vec128i case1 = _mm_cmpeq_epi8(vH, vH_dag);
+                vec128i case2 = _mm_cmpeq_epi8(vH, vF);
                 vec128i vT = _mm_blendv_epi8(
                         _mm_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
@@ -277,30 +277,30 @@ parasail_result_t* PNAME(
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = _mm_sub_epi32(vH, vGapO);
+            vEF_opn = _mm_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_sub_epi32(vE, vGapE);
-            vE = _mm_max_epi32(vEF_opn, vE_ext);
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            vE = _mm_max_epi8(vEF_opn, vE_ext);
             _mm_store_si128(pvE + i, vE);
             {
                 vec128i vEa = _mm_load_si128(pvEaLoad + i);
-                vec128i vEa_ext = _mm_sub_epi32(vEa, vGapE);
-                vEa = _mm_max_epi32(vEF_opn, vEa_ext);
+                vec128i vEa_ext = _mm_subs_epi8(vEa, vGapE);
+                vEa = _mm_max_epi8(vEF_opn, vEa_ext);
                 _mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    vec128i cond = _mm_cmpgt_epi32(vEF_opn, vEa_ext);
+                    vec128i cond = _mm_cmpgt_epi8(vEF_opn, vEa_ext);
                     vec128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vEF_opn, vF_ext);
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 vec128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                vec128i cond = _mm_cmpgt_epi32(vEF_opn, vF_ext);
+                vec128i cond = _mm_cmpgt_epi8(vEF_opn, vF_ext);
                 vec128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = _mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -317,35 +317,35 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = boundary[j+1]-open;
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
+            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
             vec128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = _mm_slli_si128(vHp, 4);
-            vHp = _mm_insert_epi32(vHp, boundary[j], 0);
-            vEF_opn = _mm_slli_si128(vEF_opn, 4);
-            vEF_opn = _mm_insert_epi32(vEF_opn, tmp2, 0);
-            vF_ext = _mm_slli_si128(vF_ext, 4);
-            vF_ext = _mm_insert_epi32(vF_ext, NEG_LIMIT, 0);
-            vF = _mm_slli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, tmp2, 0);
-            vFa_ext = _mm_slli_si128(vFa_ext, 4);
-            vFa_ext = _mm_insert_epi32(vFa_ext, NEG_LIMIT, 0);
-            vFa = _mm_slli_si128(vFa, 4);
-            vFa = _mm_insert_epi32(vFa, tmp2, 0);
+            vHp = _mm_slli_si128(vHp, 1);
+            vHp = _mm_insert_epi8(vHp, boundary[j], 0);
+            vEF_opn = _mm_slli_si128(vEF_opn, 1);
+            vEF_opn = _mm_insert_epi8(vEF_opn, tmp2, 0);
+            vF_ext = _mm_slli_si128(vF_ext, 1);
+            vF_ext = _mm_insert_epi8(vF_ext, NEG_LIMIT, 0);
+            vF = _mm_slli_si128(vF, 1);
+            vF = _mm_insert_epi8(vF, tmp2, 0);
+            vFa_ext = _mm_slli_si128(vFa_ext, 1);
+            vFa_ext = _mm_insert_epi8(vFa_ext, NEG_LIMIT, 0);
+            vFa = _mm_slli_si128(vFa, 1);
+            vFa = _mm_insert_epi8(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi32(vH,vF);
+                vH = _mm_max_epi8(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
+                vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
                 {
                     vec128i vTAll;
                     vec128i vT;
                     vec128i case1;
                     vec128i case2;
                     vec128i cond;
-                    vHp = _mm_add_epi32(vHp, _mm_load_si128(vP + i));
-                    case1 = _mm_cmpeq_epi32(vH, vHp);
-                    case2 = _mm_cmpeq_epi32(vH, vF);
+                    vHp = _mm_adds_epi8(vHp, _mm_load_si128(vP + i));
+                    case1 = _mm_cmpeq_epi8(vH, vHp);
+                    case2 = _mm_cmpeq_epi8(vH, vF);
                     cond = _mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm_load_si128(pvHT + i);
@@ -358,34 +358,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     vec128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    vec128i cond = _mm_cmpgt_epi32(vEF_opn, vFa_ext);
+                    vec128i cond = _mm_cmpgt_epi8(vEF_opn, vFa_ext);
                     vec128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = _mm_and_si128(vTAll, vFTMask);
                     vTAll = _mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm_sub_epi32(vH, vGapO);
-                vF_ext = _mm_sub_epi32(vF, vGapE);
+                vEF_opn = _mm_subs_epi8(vH, vGapO);
+                vF_ext = _mm_subs_epi8(vF, vGapE);
                 {
                     vec128i vEa = _mm_load_si128(pvEaLoad + i);
-                    vec128i vEa_ext = _mm_sub_epi32(vEa, vGapE);
-                    vEa = _mm_max_epi32(vEF_opn, vEa_ext);
+                    vec128i vEa_ext = _mm_subs_epi8(vEa, vGapE);
+                    vEa = _mm_max_epi8(vEF_opn, vEa_ext);
                     _mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        vec128i cond = _mm_cmpgt_epi32(vEF_opn, vEa_ext);
+                        vec128i cond = _mm_cmpgt_epi8(vEF_opn, vEa_ext);
                         vec128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi32(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi32(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi8(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi8(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi32(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi8(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm_sub_epi32(vFa, vGapE);
-                vFa = _mm_max_epi32(vEF_opn, vFa_ext);
+                vFa_ext = _mm_subs_epi8(vFa, vGapE);
+                vFa = _mm_max_epi8(vEF_opn, vFa_ext);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
         }
@@ -398,14 +398,14 @@ end:
     {
         vec128i vH = _mm_load_si128(pvHStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128 (vH, 4);
+            vH = _mm_slli_si128 (vH, 1);
         }
-        score = (int32_t) _mm_extract_epi32 (vH, 3);
+        score = (int8_t) _mm_extract_epi8 (vH, 15);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
