@@ -16,33 +16,33 @@
 #include "parasail/internal_avx.h"
 
 #define SG_STATS
-#define SG_SUFFIX _diag_avx2_256_16
+#define SG_SUFFIX _diag_avx2_256_32
 #include "sg_helper.h"
 
 
-#define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
-
-#if HAVE_AVX2_MM256_INSERT_EPI16
-#define _mm256_insert_epi16_rpl _mm256_insert_epi16
+#if HAVE_AVX2_MM256_INSERT_EPI32
+#define _mm256_insert_epi32_rpl _mm256_insert_epi32
 #else
-static inline __m256i _mm256_insert_epi16_rpl(__m256i a, int16_t i, int imm) {
-    __m256i_16_t A;
+static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
+    __m256i_32_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI16
-#define _mm256_extract_epi16_rpl _mm256_extract_epi16
+#if HAVE_AVX2_MM256_EXTRACT_EPI32
+#define _mm256_extract_epi32_rpl _mm256_extract_epi32
 #else
-static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
-    __m256i_16_t A;
+static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
+    __m256i_32_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
+#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+
 #define _mm256_srli_si256_rpl(a,imm) _mm256_or_si256(_mm256_slli_si256(_mm256_permute2x128_si256(a, a, _MM_SHUFFLE(3,0,0,1)), 16-imm), _mm256_srli_si256(a, imm))
 
 
@@ -56,52 +56,28 @@ static inline void arr_store_si256(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 15);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 7);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 14);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 6);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 13);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 5);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 12);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 4);
     }
     if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 11);
+        array[1LL*(i+4)*s2Len + (j-4)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 3);
     }
     if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 10);
+        array[1LL*(i+5)*s2Len + (j-5)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 2);
     }
     if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 9);
+        array[1LL*(i+6)*s2Len + (j-6)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 1);
     }
     if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 8);
-    }
-    if (0 <= i+8 && i+8 < s1Len && 0 <= j-8 && j-8 < s2Len) {
-        array[1LL*(i+8)*s2Len + (j-8)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 7);
-    }
-    if (0 <= i+9 && i+9 < s1Len && 0 <= j-9 && j-9 < s2Len) {
-        array[1LL*(i+9)*s2Len + (j-9)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 6);
-    }
-    if (0 <= i+10 && i+10 < s1Len && 0 <= j-10 && j-10 < s2Len) {
-        array[1LL*(i+10)*s2Len + (j-10)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 5);
-    }
-    if (0 <= i+11 && i+11 < s1Len && 0 <= j-11 && j-11 < s2Len) {
-        array[1LL*(i+11)*s2Len + (j-11)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 4);
-    }
-    if (0 <= i+12 && i+12 < s1Len && 0 <= j-12 && j-12 < s2Len) {
-        array[1LL*(i+12)*s2Len + (j-12)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 3);
-    }
-    if (0 <= i+13 && i+13 < s1Len && 0 <= j-13 && j-13 < s2Len) {
-        array[1LL*(i+13)*s2Len + (j-13)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 2);
-    }
-    if (0 <= i+14 && i+14 < s1Len && 0 <= j-14 && j-14 < s2Len) {
-        array[1LL*(i+14)*s2Len + (j-14)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 1);
-    }
-    if (0 <= i+15 && i+15 < s1Len && 0 <= j-15 && j-15 < s2Len) {
-        array[1LL*(i+15)*s2Len + (j-15)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 0);
+        array[1LL*(i+7)*s2Len + (j-7)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 0);
     }
 }
 #endif
@@ -117,111 +93,63 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int16_t)_mm256_extract_epi16_rpl(vWH, 15);
+        row[j-0] = (int32_t)_mm256_extract_epi32_rpl(vWH, 7);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 15);
+        col[(i+0)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 7);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int16_t)_mm256_extract_epi16_rpl(vWH, 14);
+        row[j-1] = (int32_t)_mm256_extract_epi32_rpl(vWH, 6);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 14);
+        col[(i+1)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 6);
     }
     if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int16_t)_mm256_extract_epi16_rpl(vWH, 13);
+        row[j-2] = (int32_t)_mm256_extract_epi32_rpl(vWH, 5);
     }
     if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 13);
+        col[(i+2)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 5);
     }
     if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int16_t)_mm256_extract_epi16_rpl(vWH, 12);
+        row[j-3] = (int32_t)_mm256_extract_epi32_rpl(vWH, 4);
     }
     if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 12);
+        col[(i+3)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 4);
     }
     if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
-        row[j-4] = (int16_t)_mm256_extract_epi16_rpl(vWH, 11);
+        row[j-4] = (int32_t)_mm256_extract_epi32_rpl(vWH, 3);
     }
     if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
-        col[(i+4)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 11);
+        col[(i+4)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 3);
     }
     if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
-        row[j-5] = (int16_t)_mm256_extract_epi16_rpl(vWH, 10);
+        row[j-5] = (int32_t)_mm256_extract_epi32_rpl(vWH, 2);
     }
     if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
-        col[(i+5)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 10);
+        col[(i+5)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 2);
     }
     if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
-        row[j-6] = (int16_t)_mm256_extract_epi16_rpl(vWH, 9);
+        row[j-6] = (int32_t)_mm256_extract_epi32_rpl(vWH, 1);
     }
     if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
-        col[(i+6)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 9);
+        col[(i+6)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 1);
     }
     if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
-        row[j-7] = (int16_t)_mm256_extract_epi16_rpl(vWH, 8);
+        row[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWH, 0);
     }
     if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
-        col[(i+7)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 8);
-    }
-    if (i+8 == s1Len-1 && 0 <= j-8 && j-8 < s2Len) {
-        row[j-8] = (int16_t)_mm256_extract_epi16_rpl(vWH, 7);
-    }
-    if (j-8 == s2Len-1 && 0 <= i+8 && i+8 < s1Len) {
-        col[(i+8)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 7);
-    }
-    if (i+9 == s1Len-1 && 0 <= j-9 && j-9 < s2Len) {
-        row[j-9] = (int16_t)_mm256_extract_epi16_rpl(vWH, 6);
-    }
-    if (j-9 == s2Len-1 && 0 <= i+9 && i+9 < s1Len) {
-        col[(i+9)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 6);
-    }
-    if (i+10 == s1Len-1 && 0 <= j-10 && j-10 < s2Len) {
-        row[j-10] = (int16_t)_mm256_extract_epi16_rpl(vWH, 5);
-    }
-    if (j-10 == s2Len-1 && 0 <= i+10 && i+10 < s1Len) {
-        col[(i+10)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 5);
-    }
-    if (i+11 == s1Len-1 && 0 <= j-11 && j-11 < s2Len) {
-        row[j-11] = (int16_t)_mm256_extract_epi16_rpl(vWH, 4);
-    }
-    if (j-11 == s2Len-1 && 0 <= i+11 && i+11 < s1Len) {
-        col[(i+11)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 4);
-    }
-    if (i+12 == s1Len-1 && 0 <= j-12 && j-12 < s2Len) {
-        row[j-12] = (int16_t)_mm256_extract_epi16_rpl(vWH, 3);
-    }
-    if (j-12 == s2Len-1 && 0 <= i+12 && i+12 < s1Len) {
-        col[(i+12)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 3);
-    }
-    if (i+13 == s1Len-1 && 0 <= j-13 && j-13 < s2Len) {
-        row[j-13] = (int16_t)_mm256_extract_epi16_rpl(vWH, 2);
-    }
-    if (j-13 == s2Len-1 && 0 <= i+13 && i+13 < s1Len) {
-        col[(i+13)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 2);
-    }
-    if (i+14 == s1Len-1 && 0 <= j-14 && j-14 < s2Len) {
-        row[j-14] = (int16_t)_mm256_extract_epi16_rpl(vWH, 1);
-    }
-    if (j-14 == s2Len-1 && 0 <= i+14 && i+14 < s1Len) {
-        col[(i+14)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 1);
-    }
-    if (i+15 == s1Len-1 && 0 <= j-15 && j-15 < s2Len) {
-        row[j-15] = (int16_t)_mm256_extract_epi16_rpl(vWH, 0);
-    }
-    if (j-15 == s2Len-1 && 0 <= i+15 && i+15 < s1Len) {
-        col[(i+15)] = (int16_t)_mm256_extract_epi16_rpl(vWH, 0);
+        col[(i+7)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_diag_avx2_256_16
+#define FNAME parasail_sg_flags_stats_table_diag_avx2_256_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_diag_avx2_256_16
+#define FNAME parasail_sg_flags_stats_rowcol_diag_avx2_256_32
 #else
-#define FNAME parasail_sg_flags_stats_diag_avx2_256_16
+#define FNAME parasail_sg_flags_stats_diag_avx2_256_32
 #endif
 #endif
 
@@ -238,36 +166,36 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int16_t * restrict s1 = NULL;
-    int16_t * restrict s2B = NULL;
-    int16_t * restrict _H_pr = NULL;
-    int16_t * restrict _HM_pr = NULL;
-    int16_t * restrict _HS_pr = NULL;
-    int16_t * restrict _HL_pr = NULL;
-    int16_t * restrict _F_pr = NULL;
-    int16_t * restrict _FM_pr = NULL;
-    int16_t * restrict _FS_pr = NULL;
-    int16_t * restrict _FL_pr = NULL;
-    int16_t * restrict s2 = NULL;
-    int16_t * restrict H_pr = NULL;
-    int16_t * restrict HM_pr = NULL;
-    int16_t * restrict HS_pr = NULL;
-    int16_t * restrict HL_pr = NULL;
-    int16_t * restrict F_pr = NULL;
-    int16_t * restrict FM_pr = NULL;
-    int16_t * restrict FS_pr = NULL;
-    int16_t * restrict FL_pr = NULL;
+    int32_t * restrict s1 = NULL;
+    int32_t * restrict s2B = NULL;
+    int32_t * restrict _H_pr = NULL;
+    int32_t * restrict _HM_pr = NULL;
+    int32_t * restrict _HS_pr = NULL;
+    int32_t * restrict _HL_pr = NULL;
+    int32_t * restrict _F_pr = NULL;
+    int32_t * restrict _FM_pr = NULL;
+    int32_t * restrict _FS_pr = NULL;
+    int32_t * restrict _FL_pr = NULL;
+    int32_t * restrict s2 = NULL;
+    int32_t * restrict H_pr = NULL;
+    int32_t * restrict HM_pr = NULL;
+    int32_t * restrict HS_pr = NULL;
+    int32_t * restrict HL_pr = NULL;
+    int32_t * restrict F_pr = NULL;
+    int32_t * restrict FM_pr = NULL;
+    int32_t * restrict FS_pr = NULL;
+    int32_t * restrict FL_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -317,7 +245,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 16; /* number of values in vector */
+    N = 8; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -327,26 +255,26 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     score = NEG_LIMIT;
     matches = NEG_LIMIT;
     similar = NEG_LIMIT;
     length = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm256_set1_epi16(NEG_LIMIT);
-    vOpen = _mm256_set1_epi16(open);
-    vGap  = _mm256_set1_epi16(gap);
-    vZero = _mm256_set1_epi16(0);
-    vOne = _mm256_set1_epi16(1);
-    vN = _mm256_set1_epi16(N);
-    vGapN = s1_beg ? _mm256_set1_epi16(0) : _mm256_set1_epi16(gap*N);
-    vNegOne = _mm256_set1_epi16(-1);
-    vI = _mm256_set_epi16(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
-    vJreset = _mm256_set_epi16(0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15);
+    vNegInf = _mm256_set1_epi32(NEG_LIMIT);
+    vOpen = _mm256_set1_epi32(open);
+    vGap  = _mm256_set1_epi32(gap);
+    vZero = _mm256_set1_epi32(0);
+    vOne = _mm256_set1_epi32(1);
+    vN = _mm256_set1_epi32(N);
+    vGapN = s1_beg ? _mm256_set1_epi32(0) : _mm256_set1_epi32(gap*N);
+    vNegOne = _mm256_set1_epi32(-1);
+    vI = _mm256_set_epi32(0,1,2,3,4,5,6,7);
+    vJreset = _mm256_set_epi32(0,-1,-2,-3,-4,-5,-6,-7);
     vMaxHRow = vNegInf;
     vMaxMRow = vNegInf;
     vMaxSRow = vNegInf;
@@ -361,11 +289,11 @@ parasail_result_t* FNAME(
     vLastValL = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm256_set1_epi16(s1Len);
-    vILimit1 = _mm256_subs_epi16(vILimit, vOne);
-    vJLimit = _mm256_set1_epi16(s2Len);
-    vJLimit1 = _mm256_subs_epi16(vJLimit, vOne);
-    vIBoundary = s1_beg ? _mm256_set1_epi16(0) : _mm256_set_epi16(
+    vILimit = _mm256_set1_epi32(s1Len);
+    vILimit1 = _mm256_sub_epi32(vILimit, vOne);
+    vJLimit = _mm256_set1_epi32(s2Len);
+    vJLimit1 = _mm256_sub_epi32(vJLimit, vOne);
+    vIBoundary = s1_beg ? _mm256_set1_epi32(0) : _mm256_set_epi32(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
@@ -373,15 +301,7 @@ parasail_result_t* FNAME(
             -open-4*gap,
             -open-5*gap,
             -open-6*gap,
-            -open-7*gap,
-            -open-8*gap,
-            -open-9*gap,
-            -open-10*gap,
-            -open-11*gap,
-            -open-12*gap,
-            -open-13*gap,
-            -open-14*gap,
-            -open-15*gap);
+            -open-7*gap);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -398,7 +318,7 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -411,16 +331,16 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s1     = parasail_memalign_int16_t(32, s1Len+PAD);
-    s2B    = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _H_pr  = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _HM_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _HS_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _HL_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _F_pr  = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _FM_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _FS_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    _FL_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    s1     = parasail_memalign_int32_t(32, s1Len+PAD);
+    s2B    = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     HM_pr = _HM_pr+PAD;
@@ -539,7 +459,7 @@ parasail_result_t* FNAME(
         __m256i vFS = vZero;
         __m256i vFL = vZero;
         __m256i vJ = vJreset;
-        __m256i vs1 = _mm256_set_epi16(
+        __m256i vs1 = _mm256_set_epi32(
                 s1[i+0],
                 s1[i+1],
                 s1[i+2],
@@ -547,15 +467,7 @@ parasail_result_t* FNAME(
                 s1[i+4],
                 s1[i+5],
                 s1[i+6],
-                s1[i+7],
-                s1[i+8],
-                s1[i+9],
-                s1[i+10],
-                s1[i+11],
-                s1[i+12],
-                s1[i+13],
-                s1[i+14],
-                s1[i+15]);
+                s1[i+7]);
         __m256i vs2 = vNegInf;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
@@ -565,20 +477,12 @@ parasail_result_t* FNAME(
         const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
         const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
         const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        const int * const restrict matrow8 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+8] : ((i+8 >= s1Len) ? s1Len-1 : i+8))];
-        const int * const restrict matrow9 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+9] : ((i+9 >= s1Len) ? s1Len-1 : i+9))];
-        const int * const restrict matrow10 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+10] : ((i+10 >= s1Len) ? s1Len-1 : i+10))];
-        const int * const restrict matrow11 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+11] : ((i+11 >= s1Len) ? s1Len-1 : i+11))];
-        const int * const restrict matrow12 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+12] : ((i+12 >= s1Len) ? s1Len-1 : i+12))];
-        const int * const restrict matrow13 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+13] : ((i+13 >= s1Len) ? s1Len-1 : i+13))];
-        const int * const restrict matrow14 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+14] : ((i+14 >= s1Len) ? s1Len-1 : i+14))];
-        const int * const restrict matrow15 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+15] : ((i+15 >= s1Len) ? s1Len-1 : i+15))];
-        __m256i vIltLimit = _mm256_cmplt_epi16_rpl(vI, vILimit);
-        __m256i vIeqLimit1 = _mm256_cmpeq_epi16(vI, vILimit1);
-        vNH = _mm256_srli_si256_rpl(vNH, 2);
-        vNH = _mm256_insert_epi16_rpl(vNH, H_pr[-1], 15);
-        vWH = _mm256_srli_si256_rpl(vWH, 2);
-        vWH = _mm256_insert_epi16_rpl(vWH, s1_beg ? 0 : (-open - i*gap), 15);
+        __m256i vIltLimit = _mm256_cmplt_epi32_rpl(vI, vILimit);
+        __m256i vIeqLimit1 = _mm256_cmpeq_epi32(vI, vILimit1);
+        vNH = _mm256_srli_si256_rpl(vNH, 4);
+        vNH = _mm256_insert_epi32_rpl(vNH, H_pr[-1], 7);
+        vWH = _mm256_srli_si256_rpl(vWH, 4);
+        vWH = _mm256_insert_epi32_rpl(vWH, s1_beg ? 0 : (-open - i*gap), 7);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
@@ -587,41 +491,41 @@ parasail_result_t* FNAME(
             __m256i vNWM = vNM;
             __m256i vNWS = vNS;
             __m256i vNWL = vNL;
-            vNH = _mm256_srli_si256_rpl(vWH, 2);
-            vNH = _mm256_insert_epi16_rpl(vNH, H_pr[j], 15);
-            vNM = _mm256_srli_si256_rpl(vWM, 2);
-            vNM = _mm256_insert_epi16_rpl(vNM, HM_pr[j], 15);
-            vNS = _mm256_srli_si256_rpl(vWS, 2);
-            vNS = _mm256_insert_epi16_rpl(vNS, HS_pr[j], 15);
-            vNL = _mm256_srli_si256_rpl(vWL, 2);
-            vNL = _mm256_insert_epi16_rpl(vNL, HL_pr[j], 15);
-            vF = _mm256_srli_si256_rpl(vF, 2);
-            vF = _mm256_insert_epi16_rpl(vF, F_pr[j], 15);
-            vFM = _mm256_srli_si256_rpl(vFM, 2);
-            vFM = _mm256_insert_epi16_rpl(vFM, FM_pr[j], 15);
-            vFS = _mm256_srli_si256_rpl(vFS, 2);
-            vFS = _mm256_insert_epi16_rpl(vFS, FS_pr[j], 15);
-            vFL = _mm256_srli_si256_rpl(vFL, 2);
-            vFL = _mm256_insert_epi16_rpl(vFL, FL_pr[j], 15);
-            vF_opn = _mm256_subs_epi16(vNH, vOpen);
-            vF_ext = _mm256_subs_epi16(vF, vGap);
-            vF = _mm256_max_epi16(vF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi16(vF_opn, vF_ext);
+            vNH = _mm256_srli_si256_rpl(vWH, 4);
+            vNH = _mm256_insert_epi32_rpl(vNH, H_pr[j], 7);
+            vNM = _mm256_srli_si256_rpl(vWM, 4);
+            vNM = _mm256_insert_epi32_rpl(vNM, HM_pr[j], 7);
+            vNS = _mm256_srli_si256_rpl(vWS, 4);
+            vNS = _mm256_insert_epi32_rpl(vNS, HS_pr[j], 7);
+            vNL = _mm256_srli_si256_rpl(vWL, 4);
+            vNL = _mm256_insert_epi32_rpl(vNL, HL_pr[j], 7);
+            vF = _mm256_srli_si256_rpl(vF, 4);
+            vF = _mm256_insert_epi32_rpl(vF, F_pr[j], 7);
+            vFM = _mm256_srli_si256_rpl(vFM, 4);
+            vFM = _mm256_insert_epi32_rpl(vFM, FM_pr[j], 7);
+            vFS = _mm256_srli_si256_rpl(vFS, 4);
+            vFS = _mm256_insert_epi32_rpl(vFS, FS_pr[j], 7);
+            vFL = _mm256_srli_si256_rpl(vFL, 4);
+            vFL = _mm256_insert_epi32_rpl(vFL, FL_pr[j], 7);
+            vF_opn = _mm256_sub_epi32(vNH, vOpen);
+            vF_ext = _mm256_sub_epi32(vF, vGap);
+            vF = _mm256_max_epi32(vF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi32(vF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vNM, case1);
             vFS = _mm256_blendv_epi8(vFS, vNS, case1);
             vFL = _mm256_blendv_epi8(vFL, vNL, case1);
-            vFL = _mm256_adds_epi16(vFL, vOne);
-            vE_opn = _mm256_subs_epi16(vWH, vOpen);
-            vE_ext = _mm256_subs_epi16(vE, vGap);
-            vE = _mm256_max_epi16(vE_opn, vE_ext);
-            case1 = _mm256_cmpgt_epi16(vE_opn, vE_ext);
+            vFL = _mm256_add_epi32(vFL, vOne);
+            vE_opn = _mm256_sub_epi32(vWH, vOpen);
+            vE_ext = _mm256_sub_epi32(vE, vGap);
+            vE = _mm256_max_epi32(vE_opn, vE_ext);
+            case1 = _mm256_cmpgt_epi32(vE_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vWM, case1);
             vES = _mm256_blendv_epi8(vES, vWS, case1);
             vEL = _mm256_blendv_epi8(vEL, vWL, case1);
-            vEL = _mm256_adds_epi16(vEL, vOne);
-            vs2 = _mm256_srli_si256_rpl(vs2, 2);
-            vs2 = _mm256_insert_epi16_rpl(vs2, s2[j], 15);
-            vMat = _mm256_set_epi16(
+            vEL = _mm256_add_epi32(vEL, vOne);
+            vs2 = _mm256_srli_si256_rpl(vs2, 4);
+            vs2 = _mm256_insert_epi32_rpl(vs2, s2[j], 7);
+            vMat = _mm256_set_epi32(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
@@ -629,42 +533,34 @@ parasail_result_t* FNAME(
                     matrow4[s2[j-4]],
                     matrow5[s2[j-5]],
                     matrow6[s2[j-6]],
-                    matrow7[s2[j-7]],
-                    matrow8[s2[j-8]],
-                    matrow9[s2[j-9]],
-                    matrow10[s2[j-10]],
-                    matrow11[s2[j-11]],
-                    matrow12[s2[j-12]],
-                    matrow13[s2[j-13]],
-                    matrow14[s2[j-14]],
-                    matrow15[s2[j-15]]
+                    matrow7[s2[j-7]]
                     );
-            vNWH = _mm256_adds_epi16(vNWH, vMat);
-            vWH = _mm256_max_epi16(vNWH, vE);
-            vWH = _mm256_max_epi16(vWH, vF);
-            case1 = _mm256_cmpeq_epi16(vWH, vNWH);
-            case2 = _mm256_cmpeq_epi16(vWH, vF);
+            vNWH = _mm256_add_epi32(vNWH, vMat);
+            vWH = _mm256_max_epi32(vNWH, vE);
+            vWH = _mm256_max_epi32(vWH, vF);
+            case1 = _mm256_cmpeq_epi32(vWH, vNWH);
+            case2 = _mm256_cmpeq_epi32(vWH, vF);
             vWM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
-                    _mm256_adds_epi16(vNWM,
+                    _mm256_add_epi32(vNWM,
                         _mm256_and_si256(
-                            _mm256_cmpeq_epi16(vs1,vs2),
+                            _mm256_cmpeq_epi32(vs1,vs2),
                             vOne)),
                     case1);
             vWS = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vES, vFS, case2),
-                    _mm256_adds_epi16(vNWS,
+                    _mm256_add_epi32(vNWS,
                         _mm256_and_si256(
-                            _mm256_cmpgt_epi16(vMat,vZero),
+                            _mm256_cmpgt_epi32(vMat,vZero),
                             vOne)),
                     case1);
             vWL = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEL, vFL, case2),
-                    _mm256_adds_epi16(vNWL, vOne), case1);
+                    _mm256_add_epi32(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m256i cond = _mm256_cmpeq_epi16(vJ,vNegOne);
+                __m256i cond = _mm256_cmpeq_epi32(vJ,vNegOne);
                 vWH = _mm256_blendv_epi8(vWH, vIBoundary, cond);
                 vWM = _mm256_andnot_si256(cond, vWM);
                 vWS = _mm256_andnot_si256(cond, vWS);
@@ -676,12 +572,12 @@ parasail_result_t* FNAME(
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWH);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWM);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWS);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWL);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWL);
+                vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWL);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -695,25 +591,25 @@ parasail_result_t* FNAME(
             arr_store_rowcol(result->stats->rowcols->similar_row, result->stats->rowcols->similar_col, vWS, i, s1Len, j, s2Len);
             arr_store_rowcol(result->stats->rowcols->length_row,  result->stats->rowcols->length_col, vWL, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vWH,0);
-            HM_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vWM,0);
-            HS_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vWS,0);
-            HL_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vWL,0);
-            F_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vF,0);
-            FM_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vFM,0);
-            FS_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vFS,0);
-            FL_pr[j-15] = (int16_t)_mm256_extract_epi16_rpl(vFL,0);
+            H_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWH,0);
+            HM_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWM,0);
+            HS_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWS,0);
+            HL_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWL,0);
+            F_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vF,0);
+            FM_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vFM,0);
+            FS_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vFS,0);
+            FL_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vFL,0);
             /* as minor diagonal vector passes across the i or j limit
              * boundary, extract the last value of the column or row */
             {
-                __m256i vJeqLimit1 = _mm256_cmpeq_epi16(vJ, vJLimit1);
-                __m256i vJgtNegOne = _mm256_cmpgt_epi16(vJ, vNegOne);
-                __m256i vJltLimit = _mm256_cmplt_epi16_rpl(vJ, vJLimit);
+                __m256i vJeqLimit1 = _mm256_cmpeq_epi32(vJ, vJLimit1);
+                __m256i vJgtNegOne = _mm256_cmpgt_epi32(vJ, vNegOne);
+                __m256i vJltLimit = _mm256_cmplt_epi32_rpl(vJ, vJLimit);
                 __m256i cond_j = _mm256_and_si256(vIltLimit, vJeqLimit1);
                 __m256i cond_i = _mm256_and_si256(vIeqLimit1,
                         _mm256_and_si256(vJgtNegOne, vJltLimit));
-                __m256i cond_max_row = _mm256_cmpgt_epi16(vWH, vMaxHRow);
-                __m256i cond_max_col = _mm256_cmpgt_epi16(vWH, vMaxHCol);
+                __m256i cond_max_row = _mm256_cmpgt_epi32(vWH, vMaxHRow);
+                __m256i cond_max_col = _mm256_cmpgt_epi32(vWH, vMaxHCol);
                 __m256i cond_last_val = _mm256_and_si256(vIeqLimit1, vJeqLimit1);
                 __m256i cond_all_row = _mm256_and_si256(cond_max_row, cond_i);
                 __m256i cond_all_col = _mm256_and_si256(cond_max_col, cond_j);
@@ -732,40 +628,40 @@ parasail_result_t* FNAME(
                 vEndI = _mm256_blendv_epi8(vEndI, vI, cond_all_col);
                 vEndJ = _mm256_blendv_epi8(vEndJ, vJ, cond_all_row);
             }
-            vJ = _mm256_adds_epi16(vJ, vOne);
+            vJ = _mm256_add_epi32(vJ, vOne);
         }
-        vI = _mm256_adds_epi16(vI, vN);
-        vIBoundary = _mm256_subs_epi16(vIBoundary, vGapN);
+        vI = _mm256_add_epi32(vI, vN);
+        vIBoundary = _mm256_sub_epi32(vIBoundary, vGapN);
     }
 
     /* alignment ending position */
     {
-        int16_t max_rowh = NEG_LIMIT;
-        int16_t max_rowm = NEG_LIMIT;
-        int16_t max_rows = NEG_LIMIT;
-        int16_t max_rowl = NEG_LIMIT;
-        int16_t max_colh = NEG_LIMIT;
-        int16_t max_colm = NEG_LIMIT;
-        int16_t max_cols = NEG_LIMIT;
-        int16_t max_coll = NEG_LIMIT;
-        int16_t last_valh = NEG_LIMIT;
-        int16_t last_valm = NEG_LIMIT;
-        int16_t last_vals = NEG_LIMIT;
-        int16_t last_vall = NEG_LIMIT;
-        int16_t *rh = (int16_t*)&vMaxHRow;
-        int16_t *rm = (int16_t*)&vMaxMRow;
-        int16_t *rs = (int16_t*)&vMaxSRow;
-        int16_t *rl = (int16_t*)&vMaxLRow;
-        int16_t *ch = (int16_t*)&vMaxHCol;
-        int16_t *cm = (int16_t*)&vMaxMCol;
-        int16_t *cs = (int16_t*)&vMaxSCol;
-        int16_t *cl = (int16_t*)&vMaxLCol;
-        int16_t *lh = (int16_t*)&vLastValH;
-        int16_t *lm = (int16_t*)&vLastValM;
-        int16_t *ls = (int16_t*)&vLastValS;
-        int16_t *ll = (int16_t*)&vLastValL;
-        int16_t *i = (int16_t*)&vEndI;
-        int16_t *j = (int16_t*)&vEndJ;
+        int32_t max_rowh = NEG_LIMIT;
+        int32_t max_rowm = NEG_LIMIT;
+        int32_t max_rows = NEG_LIMIT;
+        int32_t max_rowl = NEG_LIMIT;
+        int32_t max_colh = NEG_LIMIT;
+        int32_t max_colm = NEG_LIMIT;
+        int32_t max_cols = NEG_LIMIT;
+        int32_t max_coll = NEG_LIMIT;
+        int32_t last_valh = NEG_LIMIT;
+        int32_t last_valm = NEG_LIMIT;
+        int32_t last_vals = NEG_LIMIT;
+        int32_t last_vall = NEG_LIMIT;
+        int32_t *rh = (int32_t*)&vMaxHRow;
+        int32_t *rm = (int32_t*)&vMaxMRow;
+        int32_t *rs = (int32_t*)&vMaxSRow;
+        int32_t *rl = (int32_t*)&vMaxLRow;
+        int32_t *ch = (int32_t*)&vMaxHCol;
+        int32_t *cm = (int32_t*)&vMaxMCol;
+        int32_t *cs = (int32_t*)&vMaxSCol;
+        int32_t *cl = (int32_t*)&vMaxLCol;
+        int32_t *lh = (int32_t*)&vLastValH;
+        int32_t *lm = (int32_t*)&vLastValM;
+        int32_t *ls = (int32_t*)&vLastValS;
+        int32_t *ll = (int32_t*)&vLastValL;
+        int32_t *i = (int32_t*)&vEndI;
+        int32_t *j = (int32_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++rh, ++rm, ++rs, ++rl, ++ch, ++cm, ++cs, ++cl, ++lh, ++lm, ++ls, ++ll, ++i, ++j) {
             if (*ch > max_colh || (*ch == max_colh && *i < end_query)) {
@@ -830,8 +726,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi16_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
