@@ -20,16 +20,101 @@
 #include "parasail/internal_sse.h"
 
 #define SG_STATS
-#define SG_SUFFIX _diag_sse2_128_16
+#define SG_SUFFIX _diag_sse2_128_64
 #include "sg_helper.h"
 
 
+static inline __m128i _mm_cmpeq_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]==B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]==B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
 static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
     a = _mm_andnot_si128(mask, a);
     a = _mm_or_si128(a, _mm_and_si128(mask, b));
     return a;
 }
 
+static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, const int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+#if HAVE_SSE2_MM_SET_EPI64X
+#define _mm_set_epi64x_rpl _mm_set_epi64x
+#else
+static inline __m128i _mm_set_epi64x_rpl(int64_t e1, int64_t e0) {
+    __m128i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    return A.m;
+}
+#endif
+
+static inline int64_t _mm_extract_epi64_rpl(__m128i a, const int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+#if HAVE_SSE2_MM_SET1_EPI64X
+#define _mm_set1_epi64x_rpl _mm_set1_epi64x
+#else
+static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
+    __m128i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    return A.m;
+}
+#endif
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -41,28 +126,10 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int16_t)_mm_extract_epi16(vWH, 7);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int64_t)_mm_extract_epi64_rpl(vWH, 1);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int16_t)_mm_extract_epi16(vWH, 6);
-    }
-    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int16_t)_mm_extract_epi16(vWH, 5);
-    }
-    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int16_t)_mm_extract_epi16(vWH, 4);
-    }
-    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int16_t)_mm_extract_epi16(vWH, 3);
-    }
-    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int16_t)_mm_extract_epi16(vWH, 2);
-    }
-    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int16_t)_mm_extract_epi16(vWH, 1);
-    }
-    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int16_t)_mm_extract_epi16(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int64_t)_mm_extract_epi64_rpl(vWH, 0);
     }
 }
 #endif
@@ -78,63 +145,27 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int16_t)_mm_extract_epi16(vWH, 7);
+        row[j-0] = (int64_t)_mm_extract_epi64_rpl(vWH, 1);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int16_t)_mm_extract_epi16(vWH, 7);
+        col[(i+0)] = (int64_t)_mm_extract_epi64_rpl(vWH, 1);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int16_t)_mm_extract_epi16(vWH, 6);
+        row[j-1] = (int64_t)_mm_extract_epi64_rpl(vWH, 0);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int16_t)_mm_extract_epi16(vWH, 6);
-    }
-    if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int16_t)_mm_extract_epi16(vWH, 5);
-    }
-    if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int16_t)_mm_extract_epi16(vWH, 5);
-    }
-    if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int16_t)_mm_extract_epi16(vWH, 4);
-    }
-    if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int16_t)_mm_extract_epi16(vWH, 4);
-    }
-    if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
-        row[j-4] = (int16_t)_mm_extract_epi16(vWH, 3);
-    }
-    if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
-        col[(i+4)] = (int16_t)_mm_extract_epi16(vWH, 3);
-    }
-    if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
-        row[j-5] = (int16_t)_mm_extract_epi16(vWH, 2);
-    }
-    if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
-        col[(i+5)] = (int16_t)_mm_extract_epi16(vWH, 2);
-    }
-    if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
-        row[j-6] = (int16_t)_mm_extract_epi16(vWH, 1);
-    }
-    if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
-        col[(i+6)] = (int16_t)_mm_extract_epi16(vWH, 1);
-    }
-    if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
-        row[j-7] = (int16_t)_mm_extract_epi16(vWH, 0);
-    }
-    if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
-        col[(i+7)] = (int16_t)_mm_extract_epi16(vWH, 0);
+        col[(i+1)] = (int64_t)_mm_extract_epi64_rpl(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_diag_sse2_128_16
+#define FNAME parasail_sg_flags_stats_table_diag_sse2_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_diag_sse2_128_16
+#define FNAME parasail_sg_flags_stats_rowcol_diag_sse2_128_64
 #else
-#define FNAME parasail_sg_flags_stats_diag_sse2_128_16
+#define FNAME parasail_sg_flags_stats_diag_sse2_128_64
 #endif
 #endif
 
@@ -151,36 +182,36 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int16_t * restrict s1 = NULL;
-    int16_t * restrict s2B = NULL;
-    int16_t * restrict _H_pr = NULL;
-    int16_t * restrict _HM_pr = NULL;
-    int16_t * restrict _HS_pr = NULL;
-    int16_t * restrict _HL_pr = NULL;
-    int16_t * restrict _F_pr = NULL;
-    int16_t * restrict _FM_pr = NULL;
-    int16_t * restrict _FS_pr = NULL;
-    int16_t * restrict _FL_pr = NULL;
-    int16_t * restrict s2 = NULL;
-    int16_t * restrict H_pr = NULL;
-    int16_t * restrict HM_pr = NULL;
-    int16_t * restrict HS_pr = NULL;
-    int16_t * restrict HL_pr = NULL;
-    int16_t * restrict F_pr = NULL;
-    int16_t * restrict FM_pr = NULL;
-    int16_t * restrict FS_pr = NULL;
-    int16_t * restrict FL_pr = NULL;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _HM_pr = NULL;
+    int64_t * restrict _HS_pr = NULL;
+    int64_t * restrict _HL_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict _FM_pr = NULL;
+    int64_t * restrict _FS_pr = NULL;
+    int64_t * restrict _FL_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict HM_pr = NULL;
+    int64_t * restrict HS_pr = NULL;
+    int64_t * restrict HL_pr = NULL;
+    int64_t * restrict F_pr = NULL;
+    int64_t * restrict FM_pr = NULL;
+    int64_t * restrict FS_pr = NULL;
+    int64_t * restrict FL_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -230,7 +261,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 8; /* number of values in vector */
+    N = 2; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -240,26 +271,26 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
     matches = NEG_LIMIT;
     similar = NEG_LIMIT;
     length = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi16(NEG_LIMIT);
-    vOpen = _mm_set1_epi16(open);
-    vGap  = _mm_set1_epi16(gap);
-    vZero = _mm_set1_epi16(0);
-    vOne = _mm_set1_epi16(1);
-    vN = _mm_set1_epi16(N);
-    vGapN = s1_beg ? _mm_set1_epi16(0) : _mm_set1_epi16(gap*N);
-    vNegOne = _mm_set1_epi16(-1);
-    vI = _mm_set_epi16(0,1,2,3,4,5,6,7);
-    vJreset = _mm_set_epi16(0,-1,-2,-3,-4,-5,-6,-7);
+    vNegInf = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vOpen = _mm_set1_epi64x_rpl(open);
+    vGap  = _mm_set1_epi64x_rpl(gap);
+    vZero = _mm_set1_epi64x_rpl(0);
+    vOne = _mm_set1_epi64x_rpl(1);
+    vN = _mm_set1_epi64x_rpl(N);
+    vGapN = s1_beg ? _mm_set1_epi64x_rpl(0) : _mm_set1_epi64x_rpl(gap*N);
+    vNegOne = _mm_set1_epi64x_rpl(-1);
+    vI = _mm_set_epi64x_rpl(0,1);
+    vJreset = _mm_set_epi64x_rpl(0,-1);
     vMaxHRow = vNegInf;
     vMaxMRow = vNegInf;
     vMaxSRow = vNegInf;
@@ -274,19 +305,13 @@ parasail_result_t* FNAME(
     vLastValL = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm_set1_epi16(s1Len);
-    vILimit1 = _mm_subs_epi16(vILimit, vOne);
-    vJLimit = _mm_set1_epi16(s2Len);
-    vJLimit1 = _mm_subs_epi16(vJLimit, vOne);
-    vIBoundary = s1_beg ? _mm_set1_epi16(0) : _mm_set_epi16(
+    vILimit = _mm_set1_epi64x_rpl(s1Len);
+    vILimit1 = _mm_sub_epi64(vILimit, vOne);
+    vJLimit = _mm_set1_epi64x_rpl(s2Len);
+    vJLimit1 = _mm_sub_epi64(vJLimit, vOne);
+    vIBoundary = s1_beg ? _mm_set1_epi64x_rpl(0) : _mm_set_epi64x_rpl(
             -open-0*gap,
-            -open-1*gap,
-            -open-2*gap,
-            -open-3*gap,
-            -open-4*gap,
-            -open-5*gap,
-            -open-6*gap,
-            -open-7*gap);
+            -open-1*gap);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -303,7 +328,7 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -316,16 +341,16 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s1     = parasail_memalign_int16_t(16, s1Len+PAD);
-    s2B    = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _H_pr  = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _HM_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _HS_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _HL_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _F_pr  = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _FM_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _FS_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _FL_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    s1     = parasail_memalign_int64_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     HM_pr = _HM_pr+PAD;
@@ -444,30 +469,18 @@ parasail_result_t* FNAME(
         __m128i vFS = vZero;
         __m128i vFL = vZero;
         __m128i vJ = vJreset;
-        __m128i vs1 = _mm_set_epi16(
+        __m128i vs1 = _mm_set_epi64x_rpl(
                 s1[i+0],
-                s1[i+1],
-                s1[i+2],
-                s1[i+3],
-                s1[i+4],
-                s1[i+5],
-                s1[i+6],
-                s1[i+7]);
+                s1[i+1]);
         __m128i vs2 = vNegInf;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
-        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
-        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
-        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        __m128i vIltLimit = _mm_cmplt_epi16(vI, vILimit);
-        __m128i vIeqLimit1 = _mm_cmpeq_epi16(vI, vILimit1);
-        vNH = _mm_srli_si128(vNH, 2);
-        vNH = _mm_insert_epi16(vNH, H_pr[-1], 7);
-        vWH = _mm_srli_si128(vWH, 2);
-        vWH = _mm_insert_epi16(vWH, s1_beg ? 0 : (-open - i*gap), 7);
+        __m128i vIltLimit = _mm_cmplt_epi64_rpl(vI, vILimit);
+        __m128i vIeqLimit1 = _mm_cmpeq_epi64_rpl(vI, vILimit1);
+        vNH = _mm_srli_si128(vNH, 8);
+        vNH = _mm_insert_epi64_rpl(vNH, H_pr[-1], 1);
+        vWH = _mm_srli_si128(vWH, 8);
+        vWH = _mm_insert_epi64_rpl(vWH, s1_beg ? 0 : (-open - i*gap), 1);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
@@ -476,76 +489,70 @@ parasail_result_t* FNAME(
             __m128i vNWM = vNM;
             __m128i vNWS = vNS;
             __m128i vNWL = vNL;
-            vNH = _mm_srli_si128(vWH, 2);
-            vNH = _mm_insert_epi16(vNH, H_pr[j], 7);
-            vNM = _mm_srli_si128(vWM, 2);
-            vNM = _mm_insert_epi16(vNM, HM_pr[j], 7);
-            vNS = _mm_srli_si128(vWS, 2);
-            vNS = _mm_insert_epi16(vNS, HS_pr[j], 7);
-            vNL = _mm_srli_si128(vWL, 2);
-            vNL = _mm_insert_epi16(vNL, HL_pr[j], 7);
-            vF = _mm_srli_si128(vF, 2);
-            vF = _mm_insert_epi16(vF, F_pr[j], 7);
-            vFM = _mm_srli_si128(vFM, 2);
-            vFM = _mm_insert_epi16(vFM, FM_pr[j], 7);
-            vFS = _mm_srli_si128(vFS, 2);
-            vFS = _mm_insert_epi16(vFS, FS_pr[j], 7);
-            vFL = _mm_srli_si128(vFL, 2);
-            vFL = _mm_insert_epi16(vFL, FL_pr[j], 7);
-            vF_opn = _mm_subs_epi16(vNH, vOpen);
-            vF_ext = _mm_subs_epi16(vF, vGap);
-            vF = _mm_max_epi16(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
+            vNH = _mm_srli_si128(vWH, 8);
+            vNH = _mm_insert_epi64_rpl(vNH, H_pr[j], 1);
+            vNM = _mm_srli_si128(vWM, 8);
+            vNM = _mm_insert_epi64_rpl(vNM, HM_pr[j], 1);
+            vNS = _mm_srli_si128(vWS, 8);
+            vNS = _mm_insert_epi64_rpl(vNS, HS_pr[j], 1);
+            vNL = _mm_srli_si128(vWL, 8);
+            vNL = _mm_insert_epi64_rpl(vNL, HL_pr[j], 1);
+            vF = _mm_srli_si128(vF, 8);
+            vF = _mm_insert_epi64_rpl(vF, F_pr[j], 1);
+            vFM = _mm_srli_si128(vFM, 8);
+            vFM = _mm_insert_epi64_rpl(vFM, FM_pr[j], 1);
+            vFS = _mm_srli_si128(vFS, 8);
+            vFS = _mm_insert_epi64_rpl(vFS, FS_pr[j], 1);
+            vFL = _mm_srli_si128(vFL, 8);
+            vFL = _mm_insert_epi64_rpl(vFL, FL_pr[j], 1);
+            vF_opn = _mm_sub_epi64(vNH, vOpen);
+            vF_ext = _mm_sub_epi64(vF, vGap);
+            vF = _mm_max_epi64_rpl(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi64_rpl(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8_rpl(vFM, vNM, case1);
             vFS = _mm_blendv_epi8_rpl(vFS, vNS, case1);
             vFL = _mm_blendv_epi8_rpl(vFL, vNL, case1);
-            vFL = _mm_adds_epi16(vFL, vOne);
-            vE_opn = _mm_subs_epi16(vWH, vOpen);
-            vE_ext = _mm_subs_epi16(vE, vGap);
-            vE = _mm_max_epi16(vE_opn, vE_ext);
-            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
+            vFL = _mm_add_epi64(vFL, vOne);
+            vE_opn = _mm_sub_epi64(vWH, vOpen);
+            vE_ext = _mm_sub_epi64(vE, vGap);
+            vE = _mm_max_epi64_rpl(vE_opn, vE_ext);
+            case1 = _mm_cmpgt_epi64_rpl(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8_rpl(vEM, vWM, case1);
             vES = _mm_blendv_epi8_rpl(vES, vWS, case1);
             vEL = _mm_blendv_epi8_rpl(vEL, vWL, case1);
-            vEL = _mm_adds_epi16(vEL, vOne);
-            vs2 = _mm_srli_si128(vs2, 2);
-            vs2 = _mm_insert_epi16(vs2, s2[j], 7);
-            vMat = _mm_set_epi16(
+            vEL = _mm_add_epi64(vEL, vOne);
+            vs2 = _mm_srli_si128(vs2, 8);
+            vs2 = _mm_insert_epi64_rpl(vs2, s2[j], 1);
+            vMat = _mm_set_epi64x_rpl(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]],
-                    matrow2[s2[j-2]],
-                    matrow3[s2[j-3]],
-                    matrow4[s2[j-4]],
-                    matrow5[s2[j-5]],
-                    matrow6[s2[j-6]],
-                    matrow7[s2[j-7]]
+                    matrow1[s2[j-1]]
                     );
-            vNWH = _mm_adds_epi16(vNWH, vMat);
-            vWH = _mm_max_epi16(vNWH, vE);
-            vWH = _mm_max_epi16(vWH, vF);
-            case1 = _mm_cmpeq_epi16(vWH, vNWH);
-            case2 = _mm_cmpeq_epi16(vWH, vF);
+            vNWH = _mm_add_epi64(vNWH, vMat);
+            vWH = _mm_max_epi64_rpl(vNWH, vE);
+            vWH = _mm_max_epi64_rpl(vWH, vF);
+            case1 = _mm_cmpeq_epi64_rpl(vWH, vNWH);
+            case2 = _mm_cmpeq_epi64_rpl(vWH, vF);
             vWM = _mm_blendv_epi8_rpl(
                     _mm_blendv_epi8_rpl(vEM, vFM, case2),
-                    _mm_adds_epi16(vNWM,
+                    _mm_add_epi64(vNWM,
                         _mm_and_si128(
-                            _mm_cmpeq_epi16(vs1,vs2),
+                            _mm_cmpeq_epi64_rpl(vs1,vs2),
                             vOne)),
                     case1);
             vWS = _mm_blendv_epi8_rpl(
                     _mm_blendv_epi8_rpl(vES, vFS, case2),
-                    _mm_adds_epi16(vNWS,
+                    _mm_add_epi64(vNWS,
                         _mm_and_si128(
-                            _mm_cmpgt_epi16(vMat,vZero),
+                            _mm_cmpgt_epi64_rpl(vMat,vZero),
                             vOne)),
                     case1);
             vWL = _mm_blendv_epi8_rpl(
                     _mm_blendv_epi8_rpl(vEL, vFL, case2),
-                    _mm_adds_epi16(vNWL, vOne), case1);
+                    _mm_add_epi64(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m128i cond = _mm_cmpeq_epi16(vJ,vNegOne);
+                __m128i cond = _mm_cmpeq_epi64_rpl(vJ,vNegOne);
                 vWH = _mm_blendv_epi8_rpl(vWH, vIBoundary, cond);
                 vWM = _mm_andnot_si128(cond, vWM);
                 vWS = _mm_andnot_si128(cond, vWS);
@@ -557,12 +564,12 @@ parasail_result_t* FNAME(
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWM);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWS);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWL);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWL);
+                vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWL);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -576,25 +583,25 @@ parasail_result_t* FNAME(
             arr_store_rowcol(result->stats->rowcols->similar_row, result->stats->rowcols->similar_col, vWS, i, s1Len, j, s2Len);
             arr_store_rowcol(result->stats->rowcols->length_row,  result->stats->rowcols->length_col, vWL, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-7] = (int16_t)_mm_extract_epi16(vWH,0);
-            HM_pr[j-7] = (int16_t)_mm_extract_epi16(vWM,0);
-            HS_pr[j-7] = (int16_t)_mm_extract_epi16(vWS,0);
-            HL_pr[j-7] = (int16_t)_mm_extract_epi16(vWL,0);
-            F_pr[j-7] = (int16_t)_mm_extract_epi16(vF,0);
-            FM_pr[j-7] = (int16_t)_mm_extract_epi16(vFM,0);
-            FS_pr[j-7] = (int16_t)_mm_extract_epi16(vFS,0);
-            FL_pr[j-7] = (int16_t)_mm_extract_epi16(vFL,0);
+            H_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vWH,0);
+            HM_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vWM,0);
+            HS_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vWS,0);
+            HL_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vWL,0);
+            F_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vF,0);
+            FM_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vFM,0);
+            FS_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vFS,0);
+            FL_pr[j-1] = (int64_t)_mm_extract_epi64_rpl(vFL,0);
             /* as minor diagonal vector passes across the i or j limit
              * boundary, extract the last value of the column or row */
             {
-                __m128i vJeqLimit1 = _mm_cmpeq_epi16(vJ, vJLimit1);
-                __m128i vJgtNegOne = _mm_cmpgt_epi16(vJ, vNegOne);
-                __m128i vJltLimit = _mm_cmplt_epi16(vJ, vJLimit);
+                __m128i vJeqLimit1 = _mm_cmpeq_epi64_rpl(vJ, vJLimit1);
+                __m128i vJgtNegOne = _mm_cmpgt_epi64_rpl(vJ, vNegOne);
+                __m128i vJltLimit = _mm_cmplt_epi64_rpl(vJ, vJLimit);
                 __m128i cond_j = _mm_and_si128(vIltLimit, vJeqLimit1);
                 __m128i cond_i = _mm_and_si128(vIeqLimit1,
                         _mm_and_si128(vJgtNegOne, vJltLimit));
-                __m128i cond_max_row = _mm_cmpgt_epi16(vWH, vMaxHRow);
-                __m128i cond_max_col = _mm_cmpgt_epi16(vWH, vMaxHCol);
+                __m128i cond_max_row = _mm_cmpgt_epi64_rpl(vWH, vMaxHRow);
+                __m128i cond_max_col = _mm_cmpgt_epi64_rpl(vWH, vMaxHCol);
                 __m128i cond_last_val = _mm_and_si128(vIeqLimit1, vJeqLimit1);
                 __m128i cond_all_row = _mm_and_si128(cond_max_row, cond_i);
                 __m128i cond_all_col = _mm_and_si128(cond_max_col, cond_j);
@@ -613,40 +620,40 @@ parasail_result_t* FNAME(
                 vEndI = _mm_blendv_epi8_rpl(vEndI, vI, cond_all_col);
                 vEndJ = _mm_blendv_epi8_rpl(vEndJ, vJ, cond_all_row);
             }
-            vJ = _mm_adds_epi16(vJ, vOne);
+            vJ = _mm_add_epi64(vJ, vOne);
         }
-        vI = _mm_adds_epi16(vI, vN);
-        vIBoundary = _mm_subs_epi16(vIBoundary, vGapN);
+        vI = _mm_add_epi64(vI, vN);
+        vIBoundary = _mm_sub_epi64(vIBoundary, vGapN);
     }
 
     /* alignment ending position */
     {
-        int16_t max_rowh = NEG_LIMIT;
-        int16_t max_rowm = NEG_LIMIT;
-        int16_t max_rows = NEG_LIMIT;
-        int16_t max_rowl = NEG_LIMIT;
-        int16_t max_colh = NEG_LIMIT;
-        int16_t max_colm = NEG_LIMIT;
-        int16_t max_cols = NEG_LIMIT;
-        int16_t max_coll = NEG_LIMIT;
-        int16_t last_valh = NEG_LIMIT;
-        int16_t last_valm = NEG_LIMIT;
-        int16_t last_vals = NEG_LIMIT;
-        int16_t last_vall = NEG_LIMIT;
-        int16_t *rh = (int16_t*)&vMaxHRow;
-        int16_t *rm = (int16_t*)&vMaxMRow;
-        int16_t *rs = (int16_t*)&vMaxSRow;
-        int16_t *rl = (int16_t*)&vMaxLRow;
-        int16_t *ch = (int16_t*)&vMaxHCol;
-        int16_t *cm = (int16_t*)&vMaxMCol;
-        int16_t *cs = (int16_t*)&vMaxSCol;
-        int16_t *cl = (int16_t*)&vMaxLCol;
-        int16_t *lh = (int16_t*)&vLastValH;
-        int16_t *lm = (int16_t*)&vLastValM;
-        int16_t *ls = (int16_t*)&vLastValS;
-        int16_t *ll = (int16_t*)&vLastValL;
-        int16_t *i = (int16_t*)&vEndI;
-        int16_t *j = (int16_t*)&vEndJ;
+        int64_t max_rowh = NEG_LIMIT;
+        int64_t max_rowm = NEG_LIMIT;
+        int64_t max_rows = NEG_LIMIT;
+        int64_t max_rowl = NEG_LIMIT;
+        int64_t max_colh = NEG_LIMIT;
+        int64_t max_colm = NEG_LIMIT;
+        int64_t max_cols = NEG_LIMIT;
+        int64_t max_coll = NEG_LIMIT;
+        int64_t last_valh = NEG_LIMIT;
+        int64_t last_valm = NEG_LIMIT;
+        int64_t last_vals = NEG_LIMIT;
+        int64_t last_vall = NEG_LIMIT;
+        int64_t *rh = (int64_t*)&vMaxHRow;
+        int64_t *rm = (int64_t*)&vMaxMRow;
+        int64_t *rs = (int64_t*)&vMaxSRow;
+        int64_t *rl = (int64_t*)&vMaxLRow;
+        int64_t *ch = (int64_t*)&vMaxHCol;
+        int64_t *cm = (int64_t*)&vMaxMCol;
+        int64_t *cs = (int64_t*)&vMaxSCol;
+        int64_t *cl = (int64_t*)&vMaxLCol;
+        int64_t *lh = (int64_t*)&vLastValH;
+        int64_t *lm = (int64_t*)&vLastValM;
+        int64_t *ls = (int64_t*)&vLastValS;
+        int64_t *ll = (int64_t*)&vLastValL;
+        int64_t *i = (int64_t*)&vEndI;
+        int64_t *j = (int64_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++rh, ++rm, ++rs, ++rl, ++ch, ++cm, ++cs, ++cl, ++lh, ++lm, ++ls, ++ll, ++i, ++j) {
             if (*ch > max_colh || (*ch == max_colh && *i < end_query)) {
@@ -711,8 +718,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
