@@ -17,8 +17,8 @@
 #include "parasail/internal_altivec.h"
 
 #define SG_STATS
-#define SG_SUFFIX _scan_altivec_128_16
-#define SG_SUFFIX_PROF _scan_profile_altivec_128_16
+#define SG_SUFFIX _scan_altivec_128_64
+#define SG_SUFFIX_PROF _scan_profile_altivec_128_64
 #include "sg_helper.h"
 
 
@@ -32,14 +32,8 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64(vH, 1);
 }
 #endif
 
@@ -50,27 +44,21 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
+    col[0*seglen+t] = (int64_t)_mm_extract_epi64(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm_extract_epi64(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_scan_altivec_128_16
-#define PNAME parasail_sg_flags_stats_table_scan_profile_altivec_128_16
+#define FNAME parasail_sg_flags_stats_table_scan_altivec_128_64
+#define PNAME parasail_sg_flags_stats_table_scan_profile_altivec_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_scan_altivec_128_16
-#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_altivec_128_16
+#define FNAME parasail_sg_flags_stats_rowcol_scan_altivec_128_64
+#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_altivec_128_64
 #else
-#define FNAME parasail_sg_flags_stats_scan_altivec_128_16
-#define PNAME parasail_sg_flags_stats_scan_profile_altivec_128_16
+#define FNAME parasail_sg_flags_stats_scan_altivec_128_64
+#define PNAME parasail_sg_flags_stats_scan_profile_altivec_128_64
 #endif
 #endif
 
@@ -99,7 +87,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_altivec_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_altivec_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -143,17 +131,17 @@ parasail_result_t* PNAME(
     vec128i* restrict pvHLMax = NULL;
     vec128i* restrict pvGapper = NULL;
     vec128i* restrict pvGapperL = NULL;
-    int16_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     vec128i vGapO;
     vec128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     vec128i vZero;
     vec128i vOne;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -170,7 +158,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -186,38 +174,38 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP  = (vec128i*)profile->profile16.score;
-    pvPm = (vec128i*)profile->profile16.matches;
-    pvPs = (vec128i*)profile->profile16.similar;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    pvP  = (vec128i*)profile->profile64.score;
+    pvPm = (vec128i*)profile->profile64.matches;
+    pvPs = (vec128i*)profile->profile64.similar;
+    vGapO = _mm_set1_epi64(open);
+    vGapE = _mm_set1_epi64(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi16(1);
+    vOne = _mm_set1_epi64(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxM = vNegLimit;
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
-    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
-            _mm_set_epi16(0,1,2,3,4,5,6,7));
+    vPosMask = _mm_cmpeq_epi64(_mm_set1_epi64(position),
+            _mm_set_epi64(0,1));
     vNegInfFront = vZero;
-    vSegLen = _mm_slli_si128(_mm_set1_epi16(segLen), 2);
-    vNegInfFront = _mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_adds_epi16(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi16(-segLen*gap), 2));
+    vSegLen = _mm_slli_si128(_mm_set1_epi64(segLen), 8);
+    vNegInfFront = _mm_insert_epi64(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_add_epi64(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi64(-segLen*gap), 8));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -234,7 +222,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -261,7 +249,7 @@ parasail_result_t* PNAME(
     pvHLMax = parasail_memalign_vec128i(16, segLen);
     pvGapper = parasail_memalign_vec128i(16, segLen);
     pvGapperL = parasail_memalign_vec128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int64_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvE) return NULL;
@@ -288,13 +276,13 @@ parasail_result_t* PNAME(
     parasail_memset_vec128i(pvES, vZero, segLen);
     parasail_memset_vec128i(pvEL, vZero, segLen);
     {
-        vec128i vGapper = _mm_subs_epi16(vZero,vGapO);
+        vec128i vGapper = _mm_sub_epi64(vZero,vGapO);
         vec128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
             _mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = _mm_subs_epi16(vGapper, vGapE);
-            vGapperL = _mm_adds_epi16(vGapperL, vOne);
+            vGapper = _mm_sub_epi64(vGapper, vGapE);
+            vGapperL = _mm_add_epi64(vGapperL, vOne);
         }
     }
 
@@ -303,10 +291,10 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            vec128i_16_t h;
+            vec128i_64_t h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             ++index;
@@ -318,7 +306,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -366,15 +354,15 @@ parasail_result_t* PNAME(
         vHpM = _mm_load_si128(pvHM+(segLen-1));
         vHpS = _mm_load_si128(pvHS+(segLen-1));
         vHpL = _mm_load_si128(pvHL+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 2);
-        vHpM = _mm_slli_si128(vHpM, 2);
-        vHpS = _mm_slli_si128(vHpS, 2);
-        vHpL = _mm_slli_si128(vHpL, 2);
-        vHp = _mm_insert_epi16(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 8);
+        vHpM = _mm_slli_si128(vHpM, 8);
+        vHpS = _mm_slli_si128(vHpS, 8);
+        vHpL = _mm_slli_si128(vHpL, 8);
+        vHp = _mm_insert_epi64(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_subs_epi16(vNegLimit, pvGapper[0]);
+        vHt = _mm_sub_epi64(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -393,30 +381,30 @@ parasail_result_t* PNAME(
             vWS = _mm_load_si128(pvWS+i);
             vGapper = _mm_load_si128(pvGapper+i);
             vGapperL = _mm_load_si128(pvGapperL+i);
-            vE_opn = _mm_subs_epi16(vH, vGapO);
-            vE_ext = _mm_subs_epi16(vE, vGapE);
-            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
-            vE = _mm_max_epi16(vE_opn, vE_ext);
+            vE_opn = _mm_sub_epi64(vH, vGapO);
+            vE_ext = _mm_sub_epi64(vE, vGapE);
+            case1 = _mm_cmpgt_epi64(vE_opn, vE_ext);
+            vE = _mm_max_epi64(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm_adds_epi16(vEL, vOne);
-            vGapper = _mm_adds_epi16(vHt, vGapper);
+            vEL = _mm_add_epi64(vEL, vOne);
+            vGapper = _mm_add_epi64(vHt, vGapper);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi16(vF, vGapper),
-                    _mm_cmpeq_epi16(vF, vGapper));
-            vF = _mm_max_epi16(vF, vGapper);
+                    _mm_cmpgt_epi64(vF, vGapper),
+                    _mm_cmpeq_epi64(vF, vGapper));
+            vF = _mm_max_epi64(vF, vGapper);
             vFM = _mm_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_adds_epi16(vHtL, vGapperL),
+                    _mm_add_epi64(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm_adds_epi16(vHp, vW);
-            vHpM = _mm_adds_epi16(vHpM, vWM);
-            vHpS = _mm_adds_epi16(vHpS, vWS);
-            vHpL = _mm_adds_epi16(vHpL, vOne);
-            case1 = _mm_cmpgt_epi16(vE, vHp);
-            vHt = _mm_max_epi16(vE, vHp);
+            vHp = _mm_add_epi64(vHp, vW);
+            vHpM = _mm_add_epi64(vHpM, vWM);
+            vHpS = _mm_add_epi64(vHpS, vWS);
+            vHpL = _mm_add_epi64(vHpL, vOne);
+            case1 = _mm_cmpgt_epi64(vE, vHp);
+            vHt = _mm_max_epi64(vE, vHp);
             vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
@@ -435,50 +423,50 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 2);
-        vHtM = _mm_slli_si128(vHtM, 2);
-        vHtS = _mm_slli_si128(vHtS, 2);
-        vHtL = _mm_slli_si128(vHtL, 2);
-        vHt = _mm_insert_epi16(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 8);
+        vHtM = _mm_slli_si128(vHtM, 8);
+        vHtS = _mm_slli_si128(vHtS, 8);
+        vHtL = _mm_slli_si128(vHtL, 8);
+        vHt = _mm_insert_epi64(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
-        vGapper = _mm_adds_epi16(vHt, vGapper);
+        vGapper = _mm_add_epi64(vHt, vGapper);
         case1 = _mm_or_si128(
-                _mm_cmpgt_epi16(vGapper, vF),
-                _mm_cmpeq_epi16(vGapper, vF));
-        vF = _mm_max_epi16(vF, vGapper);
+                _mm_cmpgt_epi64(vGapper, vF),
+                _mm_cmpeq_epi64(vGapper, vF));
+        vF = _mm_max_epi64(vF, vGapper);
         vFM = _mm_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm_blendv_epi8(
                 vFL,
-                _mm_adds_epi16(vHtL, vGapperL),
+                _mm_add_epi64(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            vec128i vFt = _mm_slli_si128(vF, 2);
-            vec128i vFtM = _mm_slli_si128(vFM, 2);
-            vec128i vFtS = _mm_slli_si128(vFS, 2);
-            vec128i vFtL = _mm_slli_si128(vFL, 2);
-            vFt = _mm_adds_epi16(vFt, vSegLenXgap);
+            vec128i vFt = _mm_slli_si128(vF, 8);
+            vec128i vFtM = _mm_slli_si128(vFM, 8);
+            vec128i vFtS = _mm_slli_si128(vFS, 8);
+            vec128i vFtL = _mm_slli_si128(vFL, 8);
+            vFt = _mm_add_epi64(vFt, vSegLenXgap);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi16(vFt, vF),
-                    _mm_cmpeq_epi16(vFt, vF));
-            vF = _mm_max_epi16(vF, vFt);
+                    _mm_cmpgt_epi64(vFt, vF),
+                    _mm_cmpeq_epi64(vFt, vF));
+            vF = _mm_max_epi64(vF, vFt);
             vFM = _mm_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm_blendv_epi8(
                     vFL,
-                    _mm_adds_epi16(vFtL, vSegLen),
+                    _mm_add_epi64(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 2);
-        vFM = _mm_slli_si128(vFM, 2);
-        vFS = _mm_slli_si128(vFS, 2);
-        vFL = _mm_slli_si128(vFL, 2);
-        vF = _mm_adds_epi16(vF, vNegInfFront);
-        case1 = _mm_cmpgt_epi16(vF, vHt);
-        vH = _mm_max_epi16(vF, vHt);
+        vF = _mm_slli_si128(vF, 8);
+        vFM = _mm_slli_si128(vFM, 8);
+        vFS = _mm_slli_si128(vFS, 8);
+        vFL = _mm_slli_si128(vFL, 8);
+        vF = _mm_add_epi64(vF, vNegInfFront);
+        case1 = _mm_cmpgt_epi64(vF, vHt);
+        vH = _mm_max_epi64(vF, vHt);
         vHM = _mm_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm_blendv_epi8(vHtL, vFL, case1);
@@ -491,18 +479,18 @@ parasail_result_t* PNAME(
             vEM = _mm_load_si128(pvEM+i);
             vES = _mm_load_si128(pvES+i);
             vEL = _mm_load_si128(pvEL+i);
-            vF_opn = _mm_subs_epi16(vH, vGapO);
-            vF_ext = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
+            vF_opn = _mm_sub_epi64(vH, vGapO);
+            vF_ext = _mm_sub_epi64(vF, vGapE);
+            vF = _mm_max_epi64(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi64(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm_adds_epi16(vFL, vOne);
-            vH = _mm_max_epi16(vHp, vE);
-            vH = _mm_max_epi16(vH, vF);
-            case1 = _mm_cmpeq_epi16(vH, vHp);
-            case2 = _mm_cmpeq_epi16(vH, vF);
+            vFL = _mm_add_epi64(vFL, vOne);
+            vH = _mm_max_epi64(vHp, vE);
+            vH = _mm_max_epi64(vH, vF);
+            case1 = _mm_cmpeq_epi64(vH, vHp);
+            case2 = _mm_cmpeq_epi64(vH, vF);
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -516,11 +504,11 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -535,8 +523,8 @@ parasail_result_t* PNAME(
             vHM = _mm_load_si128(pvHM + offset);
             vHS = _mm_load_si128(pvHS + offset);
             vHL = _mm_load_si128(pvHL + offset);
-            cond_max = _mm_cmpgt_epi16(vH, vMaxH);
-            vMaxH = _mm_max_epi16(vMaxH, vH);
+            cond_max = _mm_cmpgt_epi64(vH, vMaxH);
+            vMaxH = _mm_max_epi64(vMaxH, vH);
             vMaxM = _mm_blendv_epi8(vMaxM, vHM, cond_max);
             vMaxS = _mm_blendv_epi8(vMaxS, vHS, cond_max);
             vMaxL = _mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -545,15 +533,15 @@ parasail_result_t* PNAME(
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
-                vHM = _mm_slli_si128(vHM, 2);
-                vHS = _mm_slli_si128(vHS, 2);
-                vHL = _mm_slli_si128(vHL, 2);
+                vH = _mm_slli_si128(vH, 8);
+                vHM = _mm_slli_si128(vHM, 8);
+                vHS = _mm_slli_si128(vHS, 8);
+                vHL = _mm_slli_si128(vHL, 8);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
-            result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7);
-            result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7);
-            result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7);
+            result->stats->rowcols->score_row[j] = (int64_t) _mm_extract_epi64 (vH, 1);
+            result->stats->rowcols->matches_row[j] = (int64_t) _mm_extract_epi64 (vHM, 1);
+            result->stats->rowcols->similar_row[j] = (int64_t) _mm_extract_epi64 (vHS, 1);
+            result->stats->rowcols->length_row[j] = (int64_t) _mm_extract_epi64 (vHL, 1);
 #endif
         }
     }
@@ -575,26 +563,26 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 2);
-            vMaxM = _mm_slli_si128(vMaxM, 2);
-            vMaxS = _mm_slli_si128(vMaxS, 2);
-            vMaxL = _mm_slli_si128(vMaxL, 2);
+            vMaxH = _mm_slli_si128(vMaxH, 8);
+            vMaxM = _mm_slli_si128(vMaxM, 8);
+            vMaxS = _mm_slli_si128(vMaxS, 8);
+            vMaxL = _mm_slli_si128(vMaxL, 8);
         }
         end_query = s1Len-1;
-        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
-        matches = (int16_t) _mm_extract_epi16(vMaxM, 7);
-        similar = (int16_t) _mm_extract_epi16(vMaxS, 7);
-        length = (int16_t) _mm_extract_epi16(vMaxL, 7);
+        score = (int64_t) _mm_extract_epi64(vMaxH, 1);
+        matches = (int64_t) _mm_extract_epi64(vMaxM, 1);
+        similar = (int64_t) _mm_extract_epi64(vMaxS, 1);
+        length = (int64_t) _mm_extract_epi64(vMaxL, 1);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvH;
-        int16_t *m = (int16_t*)pvHM;
-        int16_t *s = (int16_t*)pvHS;
-        int16_t *l = (int16_t*)pvHL;
+        int64_t *t = (int64_t*)pvH;
+        int64_t *m = (int64_t*)pvHM;
+        int64_t *s = (int64_t*)pvHS;
+        int64_t *l = (int64_t*)pvHL;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -624,22 +612,22 @@ parasail_result_t* PNAME(
         vec128i vHS = _mm_load_si128(pvHS + offset);
         vec128i vHL = _mm_load_si128(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 2);
-            vHM = _mm_slli_si128(vHM, 2);
-            vHS = _mm_slli_si128(vHS, 2);
-            vHL = _mm_slli_si128(vHL, 2);
+            vH = _mm_slli_si128(vH, 8);
+            vHM = _mm_slli_si128(vHM, 8);
+            vHS = _mm_slli_si128(vHS, 8);
+            vHL = _mm_slli_si128(vHL, 8);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int16_t) _mm_extract_epi16 (vH, 7);
-        matches = (int16_t) _mm_extract_epi16 (vHM, 7);
-        similar = (int16_t) _mm_extract_epi16 (vHS, 7);
-        length = (int16_t) _mm_extract_epi16 (vHL, 7);
+        score = (int64_t) _mm_extract_epi64 (vH, 1);
+        matches = (int64_t) _mm_extract_epi64 (vHM, 1);
+        similar = (int64_t) _mm_extract_epi64 (vHS, 1);
+        length = (int64_t) _mm_extract_epi64 (vHL, 1);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
