@@ -17,8 +17,8 @@
 #include "parasail/internal_neon.h"
 
 #define SG_STATS
-#define SG_SUFFIX _scan_neon_128_16
-#define SG_SUFFIX_PROF _scan_profile_neon_128_16
+#define SG_SUFFIX _scan_neon_128_32
+#define SG_SUFFIX_PROF _scan_profile_neon_128_32
 #include "sg_helper.h"
 
 
@@ -32,14 +32,10 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
 }
 #endif
 
@@ -50,27 +46,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 7);
+    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
+    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
+    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
+    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_scan_neon_128_16
-#define PNAME parasail_sg_flags_stats_table_scan_profile_neon_128_16
+#define FNAME parasail_sg_flags_stats_table_scan_neon_128_32
+#define PNAME parasail_sg_flags_stats_table_scan_profile_neon_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_scan_neon_128_16
-#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_neon_128_16
+#define FNAME parasail_sg_flags_stats_rowcol_scan_neon_128_32
+#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_neon_128_32
 #else
-#define FNAME parasail_sg_flags_stats_scan_neon_128_16
-#define PNAME parasail_sg_flags_stats_scan_profile_neon_128_16
+#define FNAME parasail_sg_flags_stats_scan_neon_128_32
+#define PNAME parasail_sg_flags_stats_scan_profile_neon_128_32
 #endif
 #endif
 
@@ -99,7 +91,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_neon_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_neon_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -143,17 +135,17 @@ parasail_result_t* PNAME(
     simde__m128i* restrict pvHLMax = NULL;
     simde__m128i* restrict pvGapper = NULL;
     simde__m128i* restrict pvGapperL = NULL;
-    int16_t* restrict boundary = NULL;
+    int32_t* restrict boundary = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
     simde__m128i vZero;
     simde__m128i vOne;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -170,7 +162,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -186,38 +178,38 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP  = (simde__m128i*)profile->profile16.score;
-    pvPm = (simde__m128i*)profile->profile16.matches;
-    pvPs = (simde__m128i*)profile->profile16.similar;
-    vGapO = simde_mm_set1_epi16(open);
-    vGapE = simde_mm_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    pvP  = (simde__m128i*)profile->profile32.score;
+    pvPm = (simde__m128i*)profile->profile32.matches;
+    pvPs = (simde__m128i*)profile->profile32.similar;
+    vGapO = simde_mm_set1_epi32(open);
+    vGapE = simde_mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
-    vOne = simde_mm_set1_epi16(1);
+    vOne = simde_mm_set1_epi32(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = simde_mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi16(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxM = vNegLimit;
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
-    vPosMask = simde_mm_cmpeq_epi16(simde_mm_set1_epi16(position),
-            simde_mm_set_epi16(0,1,2,3,4,5,6,7));
+    vPosMask = simde_mm_cmpeq_epi32(simde_mm_set1_epi32(position),
+            simde_mm_set_epi32(0,1,2,3));
     vNegInfFront = vZero;
-    vSegLen = simde_mm_slli_si128(simde_mm_set1_epi16(segLen), 2);
-    vNegInfFront = simde_mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = simde_mm_adds_epi16(vNegInfFront,
-            simde_mm_slli_si128(simde_mm_set1_epi16(-segLen*gap), 2));
+    vSegLen = simde_mm_slli_si128(simde_mm_set1_epi32(segLen), 4);
+    vNegInfFront = simde_mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = simde_mm_add_epi32(vNegInfFront,
+            simde_mm_slli_si128(simde_mm_set1_epi32(-segLen*gap), 4));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -234,7 +226,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -261,7 +253,7 @@ parasail_result_t* PNAME(
     pvHLMax = parasail_memalign_simde__m128i(16, segLen);
     pvGapper = parasail_memalign_simde__m128i(16, segLen);
     pvGapperL = parasail_memalign_simde__m128i(16, segLen);
-    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    boundary = parasail_memalign_int32_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvE) return NULL;
@@ -288,13 +280,13 @@ parasail_result_t* PNAME(
     parasail_memset_simde__m128i(pvES, vZero, segLen);
     parasail_memset_simde__m128i(pvEL, vZero, segLen);
     {
-        simde__m128i vGapper = simde_mm_subs_epi16(vZero,vGapO);
+        simde__m128i vGapper = simde_mm_sub_epi32(vZero,vGapO);
         simde__m128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             simde_mm_store_si128(pvGapper+i, vGapper);
             simde_mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = simde_mm_subs_epi16(vGapper, vGapE);
-            vGapperL = simde_mm_adds_epi16(vGapperL, vOne);
+            vGapper = simde_mm_sub_epi32(vGapper, vGapE);
+            vGapperL = simde_mm_add_epi32(vGapperL, vOne);
         }
     }
 
@@ -306,7 +298,7 @@ parasail_result_t* PNAME(
             simde__m128i h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             simde_mm_store_si128(&pvH[index], h);
             ++index;
@@ -318,7 +310,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
@@ -366,15 +358,15 @@ parasail_result_t* PNAME(
         vHpM = simde_mm_load_si128(pvHM+(segLen-1));
         vHpS = simde_mm_load_si128(pvHS+(segLen-1));
         vHpL = simde_mm_load_si128(pvHL+(segLen-1));
-        vHp = simde_mm_slli_si128(vHp, 2);
-        vHpM = simde_mm_slli_si128(vHpM, 2);
-        vHpS = simde_mm_slli_si128(vHpS, 2);
-        vHpL = simde_mm_slli_si128(vHpL, 2);
-        vHp = simde_mm_insert_epi16(vHp, boundary[j], 0);
+        vHp = simde_mm_slli_si128(vHp, 4);
+        vHpM = simde_mm_slli_si128(vHpM, 4);
+        vHpS = simde_mm_slli_si128(vHpS, 4);
+        vHpL = simde_mm_slli_si128(vHpL, 4);
+        vHp = simde_mm_insert_epi32(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = simde_mm_subs_epi16(vNegLimit, pvGapper[0]);
+        vHt = simde_mm_sub_epi32(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -393,30 +385,30 @@ parasail_result_t* PNAME(
             vWS = simde_mm_load_si128(pvWS+i);
             vGapper = simde_mm_load_si128(pvGapper+i);
             vGapperL = simde_mm_load_si128(pvGapperL+i);
-            vE_opn = simde_mm_subs_epi16(vH, vGapO);
-            vE_ext = simde_mm_subs_epi16(vE, vGapE);
-            case1 = simde_mm_cmpgt_epi16(vE_opn, vE_ext);
-            vE = simde_mm_max_epi16(vE_opn, vE_ext);
+            vE_opn = simde_mm_sub_epi32(vH, vGapO);
+            vE_ext = simde_mm_sub_epi32(vE, vGapE);
+            case1 = simde_mm_cmpgt_epi32(vE_opn, vE_ext);
+            vE = simde_mm_max_epi32(vE_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vHM, case1);
             vES = simde_mm_blendv_epi8(vES, vHS, case1);
             vEL = simde_mm_blendv_epi8(vEL, vHL, case1);
-            vEL = simde_mm_adds_epi16(vEL, vOne);
-            vGapper = simde_mm_adds_epi16(vHt, vGapper);
+            vEL = simde_mm_add_epi32(vEL, vOne);
+            vGapper = simde_mm_add_epi32(vHt, vGapper);
             case1 = simde_mm_or_si128(
-                    simde_mm_cmpgt_epi16(vF, vGapper),
-                    simde_mm_cmpeq_epi16(vF, vGapper));
-            vF = simde_mm_max_epi16(vF, vGapper);
+                    simde_mm_cmpgt_epi32(vF, vGapper),
+                    simde_mm_cmpeq_epi32(vF, vGapper));
+            vF = simde_mm_max_epi32(vF, vGapper);
             vFM = simde_mm_blendv_epi8(vHtM, vFM, case1);
             vFS = simde_mm_blendv_epi8(vHtS, vFS, case1);
             vFL = simde_mm_blendv_epi8(
-                    simde_mm_adds_epi16(vHtL, vGapperL),
+                    simde_mm_add_epi32(vHtL, vGapperL),
                     vFL, case1);
-            vHp = simde_mm_adds_epi16(vHp, vW);
-            vHpM = simde_mm_adds_epi16(vHpM, vWM);
-            vHpS = simde_mm_adds_epi16(vHpS, vWS);
-            vHpL = simde_mm_adds_epi16(vHpL, vOne);
-            case1 = simde_mm_cmpgt_epi16(vE, vHp);
-            vHt = simde_mm_max_epi16(vE, vHp);
+            vHp = simde_mm_add_epi32(vHp, vW);
+            vHpM = simde_mm_add_epi32(vHpM, vWM);
+            vHpS = simde_mm_add_epi32(vHpS, vWS);
+            vHpL = simde_mm_add_epi32(vHpL, vOne);
+            case1 = simde_mm_cmpgt_epi32(vE, vHp);
+            vHt = simde_mm_max_epi32(vE, vHp);
             vHtM = simde_mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = simde_mm_blendv_epi8(vHpS, vES, case1);
             vHtL = simde_mm_blendv_epi8(vHpL, vEL, case1);
@@ -435,50 +427,50 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = simde_mm_slli_si128(vHt, 2);
-        vHtM = simde_mm_slli_si128(vHtM, 2);
-        vHtS = simde_mm_slli_si128(vHtS, 2);
-        vHtL = simde_mm_slli_si128(vHtL, 2);
-        vHt = simde_mm_insert_epi16(vHt, boundary[j+1], 0);
+        vHt = simde_mm_slli_si128(vHt, 4);
+        vHtM = simde_mm_slli_si128(vHtM, 4);
+        vHtS = simde_mm_slli_si128(vHtS, 4);
+        vHtL = simde_mm_slli_si128(vHtL, 4);
+        vHt = simde_mm_insert_epi32(vHt, boundary[j+1], 0);
         vGapper = simde_mm_load_si128(pvGapper);
         vGapperL = simde_mm_load_si128(pvGapperL);
-        vGapper = simde_mm_adds_epi16(vHt, vGapper);
+        vGapper = simde_mm_add_epi32(vHt, vGapper);
         case1 = simde_mm_or_si128(
-                simde_mm_cmpgt_epi16(vGapper, vF),
-                simde_mm_cmpeq_epi16(vGapper, vF));
-        vF = simde_mm_max_epi16(vF, vGapper);
+                simde_mm_cmpgt_epi32(vGapper, vF),
+                simde_mm_cmpeq_epi32(vGapper, vF));
+        vF = simde_mm_max_epi32(vF, vGapper);
         vFM = simde_mm_blendv_epi8(vFM, vHtM, case1);
         vFS = simde_mm_blendv_epi8(vFS, vHtS, case1);
         vFL = simde_mm_blendv_epi8(
                 vFL,
-                simde_mm_adds_epi16(vHtL, vGapperL),
+                simde_mm_add_epi32(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            simde__m128i vFt = simde_mm_slli_si128(vF, 2);
-            simde__m128i vFtM = simde_mm_slli_si128(vFM, 2);
-            simde__m128i vFtS = simde_mm_slli_si128(vFS, 2);
-            simde__m128i vFtL = simde_mm_slli_si128(vFL, 2);
-            vFt = simde_mm_adds_epi16(vFt, vSegLenXgap);
+            simde__m128i vFt = simde_mm_slli_si128(vF, 4);
+            simde__m128i vFtM = simde_mm_slli_si128(vFM, 4);
+            simde__m128i vFtS = simde_mm_slli_si128(vFS, 4);
+            simde__m128i vFtL = simde_mm_slli_si128(vFL, 4);
+            vFt = simde_mm_add_epi32(vFt, vSegLenXgap);
             case1 = simde_mm_or_si128(
-                    simde_mm_cmpgt_epi16(vFt, vF),
-                    simde_mm_cmpeq_epi16(vFt, vF));
-            vF = simde_mm_max_epi16(vF, vFt);
+                    simde_mm_cmpgt_epi32(vFt, vF),
+                    simde_mm_cmpeq_epi32(vFt, vF));
+            vF = simde_mm_max_epi32(vF, vFt);
             vFM = simde_mm_blendv_epi8(vFM, vFtM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vFtS, case1);
             vFL = simde_mm_blendv_epi8(
                     vFL,
-                    simde_mm_adds_epi16(vFtL, vSegLen),
+                    simde_mm_add_epi32(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = simde_mm_slli_si128(vF, 2);
-        vFM = simde_mm_slli_si128(vFM, 2);
-        vFS = simde_mm_slli_si128(vFS, 2);
-        vFL = simde_mm_slli_si128(vFL, 2);
-        vF = simde_mm_adds_epi16(vF, vNegInfFront);
-        case1 = simde_mm_cmpgt_epi16(vF, vHt);
-        vH = simde_mm_max_epi16(vF, vHt);
+        vF = simde_mm_slli_si128(vF, 4);
+        vFM = simde_mm_slli_si128(vFM, 4);
+        vFS = simde_mm_slli_si128(vFS, 4);
+        vFL = simde_mm_slli_si128(vFL, 4);
+        vF = simde_mm_add_epi32(vF, vNegInfFront);
+        case1 = simde_mm_cmpgt_epi32(vF, vHt);
+        vH = simde_mm_max_epi32(vF, vHt);
         vHM = simde_mm_blendv_epi8(vHtM, vFM, case1);
         vHS = simde_mm_blendv_epi8(vHtS, vFS, case1);
         vHL = simde_mm_blendv_epi8(vHtL, vFL, case1);
@@ -491,18 +483,18 @@ parasail_result_t* PNAME(
             vEM = simde_mm_load_si128(pvEM+i);
             vES = simde_mm_load_si128(pvES+i);
             vEL = simde_mm_load_si128(pvEL+i);
-            vF_opn = simde_mm_subs_epi16(vH, vGapO);
-            vF_ext = simde_mm_subs_epi16(vF, vGapE);
-            vF = simde_mm_max_epi16(vF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi16(vF_opn, vF_ext);
+            vF_opn = simde_mm_sub_epi32(vH, vGapO);
+            vF_ext = simde_mm_sub_epi32(vF, vGapE);
+            vF = simde_mm_max_epi32(vF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi32(vF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vHM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vHS, case1);
             vFL = simde_mm_blendv_epi8(vFL, vHL, case1);
-            vFL = simde_mm_adds_epi16(vFL, vOne);
-            vH = simde_mm_max_epi16(vHp, vE);
-            vH = simde_mm_max_epi16(vH, vF);
-            case1 = simde_mm_cmpeq_epi16(vH, vHp);
-            case2 = simde_mm_cmpeq_epi16(vH, vF);
+            vFL = simde_mm_add_epi32(vFL, vOne);
+            vH = simde_mm_max_epi32(vHp, vE);
+            vH = simde_mm_max_epi32(vH, vF);
+            case1 = simde_mm_cmpeq_epi32(vH, vHp);
+            case2 = simde_mm_cmpeq_epi32(vH, vF);
             vHM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -516,11 +508,11 @@ parasail_result_t* PNAME(
             simde_mm_store_si128(pvHM+i, vHM);
             simde_mm_store_si128(pvHS+i, vHS);
             simde_mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -535,8 +527,8 @@ parasail_result_t* PNAME(
             vHM = simde_mm_load_si128(pvHM + offset);
             vHS = simde_mm_load_si128(pvHS + offset);
             vHL = simde_mm_load_si128(pvHL + offset);
-            cond_max = simde_mm_cmpgt_epi16(vH, vMaxH);
-            vMaxH = simde_mm_max_epi16(vMaxH, vH);
+            cond_max = simde_mm_cmpgt_epi32(vH, vMaxH);
+            vMaxH = simde_mm_max_epi32(vMaxH, vH);
             vMaxM = simde_mm_blendv_epi8(vMaxM, vHM, cond_max);
             vMaxS = simde_mm_blendv_epi8(vMaxS, vHS, cond_max);
             vMaxL = simde_mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -545,15 +537,15 @@ parasail_result_t* PNAME(
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 2);
-                vHM = simde_mm_slli_si128(vHM, 2);
-                vHS = simde_mm_slli_si128(vHS, 2);
-                vHL = simde_mm_slli_si128(vHL, 2);
+                vH = simde_mm_slli_si128(vH, 4);
+                vHM = simde_mm_slli_si128(vHM, 4);
+                vHS = simde_mm_slli_si128(vHS, 4);
+                vHL = simde_mm_slli_si128(vHL, 4);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) simde_mm_extract_epi16 (vH, 7);
-            result->stats->rowcols->matches_row[j] = (int16_t) simde_mm_extract_epi16 (vHM, 7);
-            result->stats->rowcols->similar_row[j] = (int16_t) simde_mm_extract_epi16 (vHS, 7);
-            result->stats->rowcols->length_row[j] = (int16_t) simde_mm_extract_epi16 (vHL, 7);
+            result->stats->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            result->stats->rowcols->matches_row[j] = (int32_t) simde_mm_extract_epi32 (vHM, 3);
+            result->stats->rowcols->similar_row[j] = (int32_t) simde_mm_extract_epi32 (vHS, 3);
+            result->stats->rowcols->length_row[j] = (int32_t) simde_mm_extract_epi32 (vHL, 3);
 #endif
         }
     }
@@ -575,26 +567,26 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = simde_mm_slli_si128(vMaxH, 2);
-            vMaxM = simde_mm_slli_si128(vMaxM, 2);
-            vMaxS = simde_mm_slli_si128(vMaxS, 2);
-            vMaxL = simde_mm_slli_si128(vMaxL, 2);
+            vMaxH = simde_mm_slli_si128(vMaxH, 4);
+            vMaxM = simde_mm_slli_si128(vMaxM, 4);
+            vMaxS = simde_mm_slli_si128(vMaxS, 4);
+            vMaxL = simde_mm_slli_si128(vMaxL, 4);
         }
         end_query = s1Len-1;
-        score = (int16_t) simde_mm_extract_epi16(vMaxH, 7);
-        matches = (int16_t) simde_mm_extract_epi16(vMaxM, 7);
-        similar = (int16_t) simde_mm_extract_epi16(vMaxS, 7);
-        length = (int16_t) simde_mm_extract_epi16(vMaxL, 7);
+        score = (int32_t) simde_mm_extract_epi32(vMaxH, 3);
+        matches = (int32_t) simde_mm_extract_epi32(vMaxM, 3);
+        similar = (int32_t) simde_mm_extract_epi32(vMaxS, 3);
+        length = (int32_t) simde_mm_extract_epi32(vMaxL, 3);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvH;
-        int16_t *m = (int16_t*)pvHM;
-        int16_t *s = (int16_t*)pvHS;
-        int16_t *l = (int16_t*)pvHL;
+        int32_t *t = (int32_t*)pvH;
+        int32_t *m = (int32_t*)pvHM;
+        int32_t *s = (int32_t*)pvHS;
+        int32_t *l = (int32_t*)pvHL;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -624,22 +616,22 @@ parasail_result_t* PNAME(
         simde__m128i vHS = simde_mm_load_si128(pvHS + offset);
         simde__m128i vHL = simde_mm_load_si128(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = simde_mm_slli_si128(vH, 2);
-            vHM = simde_mm_slli_si128(vHM, 2);
-            vHS = simde_mm_slli_si128(vHS, 2);
-            vHL = simde_mm_slli_si128(vHL, 2);
+            vH = simde_mm_slli_si128(vH, 4);
+            vHM = simde_mm_slli_si128(vHM, 4);
+            vHS = simde_mm_slli_si128(vHS, 4);
+            vHL = simde_mm_slli_si128(vHL, 4);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int16_t) simde_mm_extract_epi16 (vH, 7);
-        matches = (int16_t) simde_mm_extract_epi16 (vHM, 7);
-        similar = (int16_t) simde_mm_extract_epi16 (vHS, 7);
-        length = (int16_t) simde_mm_extract_epi16 (vHL, 7);
+        score = (int32_t) simde_mm_extract_epi32 (vH, 3);
+        matches = (int32_t) simde_mm_extract_epi32 (vHM, 3);
+        similar = (int32_t) simde_mm_extract_epi32 (vHS, 3);
+        length = (int32_t) simde_mm_extract_epi32 (vHL, 3);
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
