@@ -22,11 +22,94 @@
 #include "parasail/internal_sse.h"
 
 #define SG_STATS
-#define SG_SUFFIX _scan_sse41_128_32
-#define SG_SUFFIX_PROF _scan_profile_sse41_128_32
+#define SG_SUFFIX _scan_sse41_128_64
+#define SG_SUFFIX_PROF _scan_profile_sse41_128_64
 #include "sg_helper.h"
 
 
+static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+#if HAVE_SSE41_MM_INSERT_EPI64
+#define _mm_insert_epi64_rpl _mm_insert_epi64
+#else
+static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+#endif
+
+static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+#if HAVE_SSE2_MM_SET_EPI64X
+#define _mm_set_epi64x_rpl _mm_set_epi64x
+#else
+static inline __m128i _mm_set_epi64x_rpl(int64_t e1, int64_t e0) {
+    __m128i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    return A.m;
+}
+#endif
+
+#if HAVE_SSE41_MM_EXTRACT_EPI64
+#define _mm_extract_epi64_rpl _mm_extract_epi64
+#else
+static inline int64_t _mm_extract_epi64_rpl(__m128i a, int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    return A.v[imm];
+}
+#endif
+
+static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+#if HAVE_SSE2_MM_SET1_EPI64X
+#define _mm_set1_epi64x_rpl _mm_set1_epi64x
+#else
+static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
+    __m128i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    return A.m;
+}
+#endif
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -37,10 +120,8 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
 }
 #endif
 
@@ -51,23 +132,21 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
+    col[0*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_scan_sse41_128_32
-#define PNAME parasail_sg_flags_stats_table_scan_profile_sse41_128_32
+#define FNAME parasail_sg_flags_stats_table_scan_sse41_128_64
+#define PNAME parasail_sg_flags_stats_table_scan_profile_sse41_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_scan_sse41_128_32
-#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_sse41_128_32
+#define FNAME parasail_sg_flags_stats_rowcol_scan_sse41_128_64
+#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_sse41_128_64
 #else
-#define FNAME parasail_sg_flags_stats_scan_sse41_128_32
-#define PNAME parasail_sg_flags_stats_scan_profile_sse41_128_32
+#define FNAME parasail_sg_flags_stats_scan_sse41_128_64
+#define PNAME parasail_sg_flags_stats_scan_profile_sse41_128_64
 #endif
 #endif
 
@@ -96,7 +175,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_sse_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_sse_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -140,17 +219,17 @@ parasail_result_t* PNAME(
     __m128i* restrict pvHLMax = NULL;
     __m128i* restrict pvGapper = NULL;
     __m128i* restrict pvGapperL = NULL;
-    int32_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     __m128i vZero;
     __m128i vOne;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -167,7 +246,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -183,38 +262,38 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP  = (__m128i*)profile->profile32.score;
-    pvPm = (__m128i*)profile->profile32.matches;
-    pvPs = (__m128i*)profile->profile32.similar;
-    vGapO = _mm_set1_epi32(open);
-    vGapE = _mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    pvP  = (__m128i*)profile->profile64.score;
+    pvPm = (__m128i*)profile->profile64.matches;
+    pvPs = (__m128i*)profile->profile64.similar;
+    vGapO = _mm_set1_epi64x_rpl(open);
+    vGapE = _mm_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi32(1);
+    vOne = _mm_set1_epi64x_rpl(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxM = vNegLimit;
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
-    vPosMask = _mm_cmpeq_epi32(_mm_set1_epi32(position),
-            _mm_set_epi32(0,1,2,3));
+    vPosMask = _mm_cmpeq_epi64(_mm_set1_epi64x_rpl(position),
+            _mm_set_epi64x_rpl(0,1));
     vNegInfFront = vZero;
-    vSegLen = _mm_slli_si128(_mm_set1_epi32(segLen), 4);
-    vNegInfFront = _mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi32(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi32(-segLen*gap), 4));
+    vSegLen = _mm_slli_si128(_mm_set1_epi64x_rpl(segLen), 8);
+    vNegInfFront = _mm_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_add_epi64(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi64x_rpl(-segLen*gap), 8));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -231,7 +310,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -258,7 +337,7 @@ parasail_result_t* PNAME(
     pvHLMax = parasail_memalign___m128i(16, segLen);
     pvGapper = parasail_memalign___m128i(16, segLen);
     pvGapperL = parasail_memalign___m128i(16, segLen);
-    boundary = parasail_memalign_int32_t(16, s2Len+1);
+    boundary = parasail_memalign_int64_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvE) return NULL;
@@ -285,13 +364,13 @@ parasail_result_t* PNAME(
     parasail_memset___m128i(pvES, vZero, segLen);
     parasail_memset___m128i(pvEL, vZero, segLen);
     {
-        __m128i vGapper = _mm_sub_epi32(vZero,vGapO);
+        __m128i vGapper = _mm_sub_epi64(vZero,vGapO);
         __m128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
             _mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = _mm_sub_epi32(vGapper, vGapE);
-            vGapperL = _mm_add_epi32(vGapperL, vOne);
+            vGapper = _mm_sub_epi64(vGapper, vGapE);
+            vGapperL = _mm_add_epi64(vGapperL, vOne);
         }
     }
 
@@ -300,10 +379,10 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_32_t h;
+            __m128i_64_t h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             ++index;
@@ -315,7 +394,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -363,15 +442,15 @@ parasail_result_t* PNAME(
         vHpM = _mm_load_si128(pvHM+(segLen-1));
         vHpS = _mm_load_si128(pvHS+(segLen-1));
         vHpL = _mm_load_si128(pvHL+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 4);
-        vHpM = _mm_slli_si128(vHpM, 4);
-        vHpS = _mm_slli_si128(vHpS, 4);
-        vHpL = _mm_slli_si128(vHpL, 4);
-        vHp = _mm_insert_epi32(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 8);
+        vHpM = _mm_slli_si128(vHpM, 8);
+        vHpS = _mm_slli_si128(vHpS, 8);
+        vHpL = _mm_slli_si128(vHpL, 8);
+        vHp = _mm_insert_epi64_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi32(vNegLimit, pvGapper[0]);
+        vHt = _mm_sub_epi64(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -390,30 +469,30 @@ parasail_result_t* PNAME(
             vWS = _mm_load_si128(pvWS+i);
             vGapper = _mm_load_si128(pvGapper+i);
             vGapperL = _mm_load_si128(pvGapperL+i);
-            vE_opn = _mm_sub_epi32(vH, vGapO);
-            vE_ext = _mm_sub_epi32(vE, vGapE);
-            case1 = _mm_cmpgt_epi32(vE_opn, vE_ext);
-            vE = _mm_max_epi32(vE_opn, vE_ext);
+            vE_opn = _mm_sub_epi64(vH, vGapO);
+            vE_ext = _mm_sub_epi64(vE, vGapE);
+            case1 = _mm_cmpgt_epi64_rpl(vE_opn, vE_ext);
+            vE = _mm_max_epi64_rpl(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm_add_epi32(vEL, vOne);
-            vGapper = _mm_add_epi32(vHt, vGapper);
+            vEL = _mm_add_epi64(vEL, vOne);
+            vGapper = _mm_add_epi64(vHt, vGapper);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi32(vF, vGapper),
-                    _mm_cmpeq_epi32(vF, vGapper));
-            vF = _mm_max_epi32(vF, vGapper);
+                    _mm_cmpgt_epi64_rpl(vF, vGapper),
+                    _mm_cmpeq_epi64(vF, vGapper));
+            vF = _mm_max_epi64_rpl(vF, vGapper);
             vFM = _mm_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_add_epi32(vHtL, vGapperL),
+                    _mm_add_epi64(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm_add_epi32(vHp, vW);
-            vHpM = _mm_add_epi32(vHpM, vWM);
-            vHpS = _mm_add_epi32(vHpS, vWS);
-            vHpL = _mm_add_epi32(vHpL, vOne);
-            case1 = _mm_cmpgt_epi32(vE, vHp);
-            vHt = _mm_max_epi32(vE, vHp);
+            vHp = _mm_add_epi64(vHp, vW);
+            vHpM = _mm_add_epi64(vHpM, vWM);
+            vHpS = _mm_add_epi64(vHpS, vWS);
+            vHpL = _mm_add_epi64(vHpL, vOne);
+            case1 = _mm_cmpgt_epi64_rpl(vE, vHp);
+            vHt = _mm_max_epi64_rpl(vE, vHp);
             vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
@@ -432,50 +511,50 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 4);
-        vHtM = _mm_slli_si128(vHtM, 4);
-        vHtS = _mm_slli_si128(vHtS, 4);
-        vHtL = _mm_slli_si128(vHtL, 4);
-        vHt = _mm_insert_epi32(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 8);
+        vHtM = _mm_slli_si128(vHtM, 8);
+        vHtS = _mm_slli_si128(vHtS, 8);
+        vHtL = _mm_slli_si128(vHtL, 8);
+        vHt = _mm_insert_epi64_rpl(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
-        vGapper = _mm_add_epi32(vHt, vGapper);
+        vGapper = _mm_add_epi64(vHt, vGapper);
         case1 = _mm_or_si128(
-                _mm_cmpgt_epi32(vGapper, vF),
-                _mm_cmpeq_epi32(vGapper, vF));
-        vF = _mm_max_epi32(vF, vGapper);
+                _mm_cmpgt_epi64_rpl(vGapper, vF),
+                _mm_cmpeq_epi64(vGapper, vF));
+        vF = _mm_max_epi64_rpl(vF, vGapper);
         vFM = _mm_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm_blendv_epi8(
                 vFL,
-                _mm_add_epi32(vHtL, vGapperL),
+                _mm_add_epi64(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            __m128i vFt = _mm_slli_si128(vF, 4);
-            __m128i vFtM = _mm_slli_si128(vFM, 4);
-            __m128i vFtS = _mm_slli_si128(vFS, 4);
-            __m128i vFtL = _mm_slli_si128(vFL, 4);
-            vFt = _mm_add_epi32(vFt, vSegLenXgap);
+            __m128i vFt = _mm_slli_si128(vF, 8);
+            __m128i vFtM = _mm_slli_si128(vFM, 8);
+            __m128i vFtS = _mm_slli_si128(vFS, 8);
+            __m128i vFtL = _mm_slli_si128(vFL, 8);
+            vFt = _mm_add_epi64(vFt, vSegLenXgap);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi32(vFt, vF),
-                    _mm_cmpeq_epi32(vFt, vF));
-            vF = _mm_max_epi32(vF, vFt);
+                    _mm_cmpgt_epi64_rpl(vFt, vF),
+                    _mm_cmpeq_epi64(vFt, vF));
+            vF = _mm_max_epi64_rpl(vF, vFt);
             vFM = _mm_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm_blendv_epi8(
                     vFL,
-                    _mm_add_epi32(vFtL, vSegLen),
+                    _mm_add_epi64(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 4);
-        vFM = _mm_slli_si128(vFM, 4);
-        vFS = _mm_slli_si128(vFS, 4);
-        vFL = _mm_slli_si128(vFL, 4);
-        vF = _mm_add_epi32(vF, vNegInfFront);
-        case1 = _mm_cmpgt_epi32(vF, vHt);
-        vH = _mm_max_epi32(vF, vHt);
+        vF = _mm_slli_si128(vF, 8);
+        vFM = _mm_slli_si128(vFM, 8);
+        vFS = _mm_slli_si128(vFS, 8);
+        vFL = _mm_slli_si128(vFL, 8);
+        vF = _mm_add_epi64(vF, vNegInfFront);
+        case1 = _mm_cmpgt_epi64_rpl(vF, vHt);
+        vH = _mm_max_epi64_rpl(vF, vHt);
         vHM = _mm_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm_blendv_epi8(vHtL, vFL, case1);
@@ -488,18 +567,18 @@ parasail_result_t* PNAME(
             vEM = _mm_load_si128(pvEM+i);
             vES = _mm_load_si128(pvES+i);
             vEL = _mm_load_si128(pvEL+i);
-            vF_opn = _mm_sub_epi32(vH, vGapO);
-            vF_ext = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi32(vF_opn, vF_ext);
+            vF_opn = _mm_sub_epi64(vH, vGapO);
+            vF_ext = _mm_sub_epi64(vF, vGapE);
+            vF = _mm_max_epi64_rpl(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi64_rpl(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm_add_epi32(vFL, vOne);
-            vH = _mm_max_epi32(vHp, vE);
-            vH = _mm_max_epi32(vH, vF);
-            case1 = _mm_cmpeq_epi32(vH, vHp);
-            case2 = _mm_cmpeq_epi32(vH, vF);
+            vFL = _mm_add_epi64(vFL, vOne);
+            vH = _mm_max_epi64_rpl(vHp, vE);
+            vH = _mm_max_epi64_rpl(vH, vF);
+            case1 = _mm_cmpeq_epi64(vH, vHp);
+            case2 = _mm_cmpeq_epi64(vH, vF);
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -513,11 +592,11 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -532,8 +611,8 @@ parasail_result_t* PNAME(
             vHM = _mm_load_si128(pvHM + offset);
             vHS = _mm_load_si128(pvHS + offset);
             vHL = _mm_load_si128(pvHL + offset);
-            cond_max = _mm_cmpgt_epi32(vH, vMaxH);
-            vMaxH = _mm_max_epi32(vMaxH, vH);
+            cond_max = _mm_cmpgt_epi64_rpl(vH, vMaxH);
+            vMaxH = _mm_max_epi64_rpl(vMaxH, vH);
             vMaxM = _mm_blendv_epi8(vMaxM, vHM, cond_max);
             vMaxS = _mm_blendv_epi8(vMaxS, vHS, cond_max);
             vMaxL = _mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -542,15 +621,15 @@ parasail_result_t* PNAME(
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 4);
-                vHM = _mm_slli_si128(vHM, 4);
-                vHS = _mm_slli_si128(vHS, 4);
-                vHL = _mm_slli_si128(vHL, 4);
+                vH = _mm_slli_si128(vH, 8);
+                vHM = _mm_slli_si128(vHM, 8);
+                vHS = _mm_slli_si128(vHS, 8);
+                vHL = _mm_slli_si128(vHL, 8);
             }
-            result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
-            result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32 (vHM, 3);
-            result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32 (vHS, 3);
-            result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32 (vHL, 3);
+            result->stats->rowcols->score_row[j] = (int64_t) _mm_extract_epi64_rpl (vH, 1);
+            result->stats->rowcols->matches_row[j] = (int64_t) _mm_extract_epi64_rpl (vHM, 1);
+            result->stats->rowcols->similar_row[j] = (int64_t) _mm_extract_epi64_rpl (vHS, 1);
+            result->stats->rowcols->length_row[j] = (int64_t) _mm_extract_epi64_rpl (vHL, 1);
 #endif
         }
     }
@@ -572,26 +651,26 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 4);
-            vMaxM = _mm_slli_si128(vMaxM, 4);
-            vMaxS = _mm_slli_si128(vMaxS, 4);
-            vMaxL = _mm_slli_si128(vMaxL, 4);
+            vMaxH = _mm_slli_si128(vMaxH, 8);
+            vMaxM = _mm_slli_si128(vMaxM, 8);
+            vMaxS = _mm_slli_si128(vMaxS, 8);
+            vMaxL = _mm_slli_si128(vMaxL, 8);
         }
         end_query = s1Len-1;
-        score = (int32_t) _mm_extract_epi32(vMaxH, 3);
-        matches = (int32_t) _mm_extract_epi32(vMaxM, 3);
-        similar = (int32_t) _mm_extract_epi32(vMaxS, 3);
-        length = (int32_t) _mm_extract_epi32(vMaxL, 3);
+        score = (int64_t) _mm_extract_epi64_rpl(vMaxH, 1);
+        matches = (int64_t) _mm_extract_epi64_rpl(vMaxM, 1);
+        similar = (int64_t) _mm_extract_epi64_rpl(vMaxS, 1);
+        length = (int64_t) _mm_extract_epi64_rpl(vMaxL, 1);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int32_t *t = (int32_t*)pvH;
-        int32_t *m = (int32_t*)pvHM;
-        int32_t *s = (int32_t*)pvHS;
-        int32_t *l = (int32_t*)pvHL;
+        int64_t *t = (int64_t*)pvH;
+        int64_t *m = (int64_t*)pvHM;
+        int64_t *s = (int64_t*)pvHS;
+        int64_t *l = (int64_t*)pvHL;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -621,22 +700,22 @@ parasail_result_t* PNAME(
         __m128i vHS = _mm_load_si128(pvHS + offset);
         __m128i vHL = _mm_load_si128(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 4);
-            vHM = _mm_slli_si128(vHM, 4);
-            vHS = _mm_slli_si128(vHS, 4);
-            vHL = _mm_slli_si128(vHL, 4);
+            vH = _mm_slli_si128(vH, 8);
+            vHM = _mm_slli_si128(vHM, 8);
+            vHS = _mm_slli_si128(vHS, 8);
+            vHL = _mm_slli_si128(vHL, 8);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int32_t) _mm_extract_epi32 (vH, 3);
-        matches = (int32_t) _mm_extract_epi32 (vHM, 3);
-        similar = (int32_t) _mm_extract_epi32 (vHS, 3);
-        length = (int32_t) _mm_extract_epi32 (vHL, 3);
+        score = (int64_t) _mm_extract_epi64_rpl (vH, 1);
+        matches = (int64_t) _mm_extract_epi64_rpl (vHM, 1);
+        similar = (int64_t) _mm_extract_epi64_rpl (vHS, 1);
+        length = (int64_t) _mm_extract_epi64_rpl (vHL, 1);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
