@@ -11,8 +11,8 @@
 #include <stdlib.h>
 
 #define SG_STATS
-#define SG_SUFFIX _striped_avx2_256_32
-#define SG_SUFFIX_PROF _striped_profile_avx2_256_32
+#define SG_SUFFIX _striped_avx2_256_64
+#define SG_SUFFIX_PROF _striped_profile_avx2_256_64
 #include "sg_helper.h"
 
 #include <immintrin.h>
@@ -24,28 +24,78 @@
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI32
-#define _mm256_insert_epi32_rpl _mm256_insert_epi32
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI32
-#define _mm256_extract_epi32_rpl _mm256_extract_epi32
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
 #else
-static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#if HAVE_AVX2_MM256_SET_EPI64X
+#define _mm256_set_epi64x_rpl _mm256_set_epi64x
+#else
+static inline __m256i _mm256_set_epi64x_rpl(int64_t e3, int64_t e2, int64_t e1, int64_t e0) {
+    __m256i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    A.v[2] = e2;
+    A.v[3] = e3;
+    return A.m;
+}
+#endif
+
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
+#else
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
@@ -59,14 +109,10 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
@@ -77,27 +123,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
-    col[4*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
-    col[5*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
-    col[6*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
-    col[7*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
+    col[0*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 1);
+    col[2*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 2);
+    col[3*seglen+t] = (int64_t)_mm256_extract_epi64_rpl(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_striped_avx2_256_32
-#define PNAME parasail_sg_flags_stats_table_striped_profile_avx2_256_32
+#define FNAME parasail_sg_flags_stats_table_striped_avx2_256_64
+#define PNAME parasail_sg_flags_stats_table_striped_profile_avx2_256_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_striped_avx2_256_32
-#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_avx2_256_32
+#define FNAME parasail_sg_flags_stats_rowcol_striped_avx2_256_64
+#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_avx2_256_64
 #else
-#define FNAME parasail_sg_flags_stats_striped_avx2_256_32
-#define PNAME parasail_sg_flags_stats_striped_profile_avx2_256_32
+#define FNAME parasail_sg_flags_stats_striped_avx2_256_64
+#define PNAME parasail_sg_flags_stats_striped_profile_avx2_256_64
 #endif
 #endif
 
@@ -126,7 +168,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_avx_256_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_avx_256_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -168,17 +210,17 @@ parasail_result_t* PNAME(
     __m256i* restrict pvEM = NULL;
     __m256i* restrict pvES = NULL;
     __m256i* restrict pvEL = NULL;
-    int32_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     __m256i vZero;
     __m256i vOne;
-    int32_t score = 0;
-    int32_t matches = 0;
-    int32_t similar = 0;
-    int32_t length = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -192,7 +234,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -208,33 +250,33 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile  = (__m256i*)profile->profile32.score;
-    vProfileM = (__m256i*)profile->profile32.matches;
-    vProfileS = (__m256i*)profile->profile32.similar;
-    vGapO = _mm256_set1_epi32(open);
-    vGapE = _mm256_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile  = (__m256i*)profile->profile64.score;
+    vProfileM = (__m256i*)profile->profile64.matches;
+    vProfileS = (__m256i*)profile->profile64.similar;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
-    vOne = _mm256_set1_epi32(1);
+    vOne = _mm256_set1_epi64x_rpl(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHM = vNegLimit;
     vMaxHS = vNegLimit;
     vMaxHL = vNegLimit;
-    vPosMask = _mm256_cmpeq_epi32(_mm256_set1_epi32(position),
-            _mm256_set_epi32(0,1,2,3,4,5,6,7));
+    vPosMask = _mm256_cmpeq_epi64(_mm256_set1_epi64x_rpl(position),
+            _mm256_set_epi64x_rpl(0,1,2,3));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -251,7 +293,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -276,7 +318,7 @@ parasail_result_t* PNAME(
     pvEM      = parasail_memalign___m256i(32, segLen);
     pvES      = parasail_memalign___m256i(32, segLen);
     pvEL      = parasail_memalign___m256i(32, segLen);
-    boundary  = parasail_memalign_int32_t(32, s2Len+1);
+    boundary  = parasail_memalign_int64_t(32, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -305,13 +347,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_32_t h;
-            __m256i_32_t e;
+            __m256i_64_t h;
+            __m256i_64_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             _mm256_store_si256(&pvHStore[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -324,7 +366,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -357,18 +399,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 4 bytes */
+        /* load final segment of pvHStore and shift left by 8 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
         vHM = _mm256_load_si256(&pvHMStore[segLen - 1]);
         vHS = _mm256_load_si256(&pvHSStore[segLen - 1]);
         vHL = _mm256_load_si256(&pvHLStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 4);
-        vHM = _mm256_slli_si256_rpl(vHM, 4);
-        vHS = _mm256_slli_si256_rpl(vHS, 4);
-        vHL = _mm256_slli_si256_rpl(vHL, 4);
+        vH = _mm256_slli_si256_rpl(vH, 8);
+        vHM = _mm256_slli_si256_rpl(vHM, 8);
+        vHS = _mm256_slli_si256_rpl(vHS, 8);
+        vHL = _mm256_slli_si256_rpl(vHL, 8);
 
         /* insert upper boundary condition */
-        vH = _mm256_insert_epi32_rpl(vH, boundary[j], 0);
+        vH = _mm256_insert_epi64_rpl(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -392,58 +434,58 @@ parasail_result_t* PNAME(
             vEL = _mm256_load_si256(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_add_epi32(vH, _mm256_load_si256(vP + i));
-            vH = _mm256_max_epi32(vH_dag, vE);
-            vH = _mm256_max_epi32(vH, vF);
+            vH_dag = _mm256_add_epi64(vH, _mm256_load_si256(vP + i));
+            vH = _mm256_max_epi64_rpl(vH_dag, vE);
+            vH = _mm256_max_epi64_rpl(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
 
-            case1 = _mm256_cmpeq_epi32(vH, vH_dag);
-            case2 = _mm256_cmpeq_epi32(vH, vF);
+            case1 = _mm256_cmpeq_epi64(vH, vH_dag);
+            case2 = _mm256_cmpeq_epi64(vH, vF);
 
             /* calculate vM */
             vHM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
-                    _mm256_add_epi32(vHM, _mm256_load_si256(vPM + i)),
+                    _mm256_add_epi64(vHM, _mm256_load_si256(vPM + i)),
                     case1);
             _mm256_store_si256(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vES, vFS, case2),
-                    _mm256_add_epi32(vHS, _mm256_load_si256(vPS + i)),
+                    _mm256_add_epi64(vHS, _mm256_load_si256(vPS + i)),
                     case1);
             _mm256_store_si256(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEL, vFL, case2),
-                    _mm256_add_epi32(vHL, vOne),
+                    _mm256_add_epi64(vHL, vOne),
                     case1);
             _mm256_store_si256(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = _mm256_sub_epi32(vH, vGapO);
+            vEF_opn = _mm256_sub_epi64(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_sub_epi32(vE, vGapE);
-            vE = _mm256_max_epi32(vEF_opn, vE_ext);
-            case1 = _mm256_cmpgt_epi32(vEF_opn, vE_ext);
+            vE_ext = _mm256_sub_epi64(vE, vGapE);
+            vE = _mm256_max_epi64_rpl(vEF_opn, vE_ext);
+            case1 = _mm256_cmpgt_epi64(vEF_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vHM, case1);
             vES = _mm256_blendv_epi8(vES, vHS, case1);
             vEL = _mm256_blendv_epi8(
-                    _mm256_add_epi32(vEL, vOne),
-                    _mm256_add_epi32(vHL, vOne),
+                    _mm256_add_epi64(vEL, vOne),
+                    _mm256_add_epi64(vHL, vOne),
                     case1);
             _mm256_store_si256(pvE + i, vE);
             _mm256_store_si256(pvEM + i, vEM);
@@ -451,14 +493,14 @@ parasail_result_t* PNAME(
             _mm256_store_si256(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm256_sub_epi32(vF, vGapE);
-            vF = _mm256_max_epi32(vEF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi32(vEF_opn, vF_ext);
+            vF_ext = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi64(vEF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vHM, case1);
             vFS = _mm256_blendv_epi8(vFS, vHS, case1);
             vFL = _mm256_blendv_epi8(
-                    _mm256_add_epi32(vFL, vOne),
-                    _mm256_add_epi32(vHL, vOne),
+                    _mm256_add_epi64(vFL, vOne),
+                    _mm256_add_epi64(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -473,26 +515,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
-            vHp = _mm256_slli_si256_rpl(vHp, 4);
-            vF = _mm256_slli_si256_rpl(vF, 4);
-            vFM = _mm256_slli_si256_rpl(vFM, 4);
-            vFS = _mm256_slli_si256_rpl(vFS, 4);
-            vFL = _mm256_slli_si256_rpl(vFL, 4);
-            vHp = _mm256_insert_epi32_rpl(vHp, boundary[j], 0);
-            vF = _mm256_insert_epi32_rpl(vF, tmp2, 0);
-            vFL = _mm256_insert_epi32_rpl(vFL, 1, 0);
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            vHp = _mm256_slli_si256_rpl(vHp, 8);
+            vF = _mm256_slli_si256_rpl(vF, 8);
+            vFM = _mm256_slli_si256_rpl(vFM, 8);
+            vFS = _mm256_slli_si256_rpl(vFS, 8);
+            vFL = _mm256_slli_si256_rpl(vFL, 8);
+            vHp = _mm256_insert_epi64_rpl(vHp, boundary[j], 0);
+            vF = _mm256_insert_epi64_rpl(vF, tmp2, 0);
+            vFL = _mm256_insert_epi64_rpl(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 __m256i case1;
                 __m256i case2;
                 __m256i cond;
 
-                vHp = _mm256_add_epi32(vHp, _mm256_load_si256(vP + i));
+                vHp = _mm256_add_epi64(vHp, _mm256_load_si256(vP + i));
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi32(vH,vF);
+                vH = _mm256_max_epi64_rpl(vH,vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                case1 = _mm256_cmpeq_epi32(vH, vHp);
-                case2 = _mm256_cmpeq_epi32(vH, vF);
+                case1 = _mm256_cmpeq_epi64(vH, vHp);
+                case2 = _mm256_cmpeq_epi64(vH, vF);
                 cond = _mm256_andnot_si256(case1, case2);
 
                 /* calculate vM */
@@ -510,11 +552,11 @@ parasail_result_t* PNAME(
                 vHL = _mm256_blendv_epi8(vHL, vFL, cond);
                 _mm256_store_si256(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
-                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si256(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -522,21 +564,21 @@ parasail_result_t* PNAME(
                 arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = _mm256_sub_epi32(vH, vGapO);
-                vF_ext = _mm256_sub_epi32(vF, vGapE);
+                vEF_opn = _mm256_sub_epi64(vH, vGapO);
+                vF_ext = _mm256_sub_epi64(vF, vGapE);
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi32(vF_ext, vEF_opn),
-                                _mm256_cmpeq_epi32(vF_ext, vEF_opn))))
+                                _mm256_cmpgt_epi64(vF_ext, vEF_opn),
+                                _mm256_cmpeq_epi64(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm256_max_epi32(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi64_rpl(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm256_cmpgt_epi32(vEF_opn, vF_ext);
+                cond = _mm256_cmpgt_epi64(vEF_opn, vF_ext);
                 vFM = _mm256_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm256_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm256_blendv_epi8(
-                        _mm256_add_epi32(vFL, vOne),
-                        _mm256_add_epi32(vHL, vOne),
+                        _mm256_add_epi64(vFL, vOne),
+                        _mm256_add_epi64(vHL, vOne),
                         cond);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
@@ -552,7 +594,7 @@ end:
             vHM = _mm256_load_si256(pvHMStore + offset);
             vHS = _mm256_load_si256(pvHSStore + offset);
             vHL = _mm256_load_si256(pvHLStore + offset);
-            cond_max = _mm256_cmpgt_epi32(vH, vMaxH);
+            cond_max = _mm256_cmpgt_epi64(vH, vMaxH);
             vMaxH = _mm256_blendv_epi8(vMaxH, vH, cond_max);
             vMaxHM = _mm256_blendv_epi8(vMaxHM, vHM, cond_max);
             vMaxHS = _mm256_blendv_epi8(vMaxHS, vHS, cond_max);
@@ -563,15 +605,15 @@ end:
         }
 #ifdef PARASAIL_ROWCOL
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl(vH, 4);
-            vHM = _mm256_slli_si256_rpl(vHM, 4);
-            vHS = _mm256_slli_si256_rpl(vHS, 4);
-            vHL = _mm256_slli_si256_rpl(vHL, 4);
+            vH = _mm256_slli_si256_rpl(vH, 8);
+            vHM = _mm256_slli_si256_rpl(vHM, 8);
+            vHS = _mm256_slli_si256_rpl(vHS, 8);
+            vHL = _mm256_slli_si256_rpl(vHL, 8);
         }
-        result->stats->rowcols->score_row[j] = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
-        result->stats->rowcols->matches_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
-        result->stats->rowcols->similar_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
-        result->stats->rowcols->length_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
+        result->stats->rowcols->score_row[j] = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
+        result->stats->rowcols->matches_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHM, 3);
+        result->stats->rowcols->similar_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHS, 3);
+        result->stats->rowcols->length_row[j] = (int64_t) _mm256_extract_epi64_rpl (vHL, 3);
 #endif
     }
 
@@ -592,26 +634,26 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm256_slli_si256_rpl(vMaxH, 4);
-            vMaxHM = _mm256_slli_si256_rpl(vMaxHM, 4);
-            vMaxHS = _mm256_slli_si256_rpl(vMaxHS, 4);
-            vMaxHL = _mm256_slli_si256_rpl(vMaxHL, 4);
+            vMaxH = _mm256_slli_si256_rpl(vMaxH, 8);
+            vMaxHM = _mm256_slli_si256_rpl(vMaxHM, 8);
+            vMaxHS = _mm256_slli_si256_rpl(vMaxHS, 8);
+            vMaxHL = _mm256_slli_si256_rpl(vMaxHL, 8);
         }
         end_query = s1Len-1;
-        score = (int32_t) _mm256_extract_epi32_rpl(vMaxH, 7);
-        matches = (int32_t) _mm256_extract_epi32_rpl(vMaxHM, 7);
-        similar = (int32_t) _mm256_extract_epi32_rpl(vMaxHS, 7);
-        length = (int32_t) _mm256_extract_epi32_rpl(vMaxHL, 7);
+        score = (int64_t) _mm256_extract_epi64_rpl(vMaxH, 3);
+        matches = (int64_t) _mm256_extract_epi64_rpl(vMaxHM, 3);
+        similar = (int64_t) _mm256_extract_epi64_rpl(vMaxHS, 3);
+        length = (int64_t) _mm256_extract_epi64_rpl(vMaxHL, 3);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int32_t *t = (int32_t*)pvHStore;
-        int32_t *m = (int32_t*)pvHMStore;
-        int32_t *s = (int32_t*)pvHSStore;
-        int32_t *l = (int32_t*)pvHLStore;
+        int64_t *t = (int64_t*)pvHStore;
+        int64_t *m = (int64_t*)pvHMStore;
+        int64_t *s = (int64_t*)pvHSStore;
+        int64_t *l = (int64_t*)pvHLStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -641,22 +683,22 @@ end:
         __m256i vHS = _mm256_load_si256(pvHSStore + offset);
         __m256i vHL = _mm256_load_si256(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm256_slli_si256_rpl(vH, 4);
-            vHM = _mm256_slli_si256_rpl(vHM, 4);
-            vHS = _mm256_slli_si256_rpl(vHS, 4);
-            vHL = _mm256_slli_si256_rpl(vHL, 4);
+            vH = _mm256_slli_si256_rpl(vH, 8);
+            vHM = _mm256_slli_si256_rpl(vHM, 8);
+            vHS = _mm256_slli_si256_rpl(vHS, 8);
+            vHL = _mm256_slli_si256_rpl(vHL, 8);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
-        matches = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
-        similar = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
-        length = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
+        score = (int64_t) _mm256_extract_epi64_rpl (vH, 3);
+        matches = (int64_t) _mm256_extract_epi64_rpl (vHM, 3);
+        similar = (int64_t) _mm256_extract_epi64_rpl (vHS, 3);
+        length = (int64_t) _mm256_extract_epi64_rpl (vHL, 3);
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
