@@ -11,8 +11,8 @@
 #include <stdlib.h>
 
 #define SG_STATS
-#define SG_SUFFIX _striped_neon_128_8
-#define SG_SUFFIX_PROF _striped_profile_neon_128_8
+#define SG_SUFFIX _striped_neon_128_64
+#define SG_SUFFIX_PROF _striped_profile_neon_128_64
 #include "sg_helper.h"
 
 
@@ -34,22 +34,8 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 15);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
 }
 #endif
 
@@ -60,35 +46,21 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  0);
-    col[ 1*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  1);
-    col[ 2*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  2);
-    col[ 3*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  3);
-    col[ 4*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  4);
-    col[ 5*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  5);
-    col[ 6*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  6);
-    col[ 7*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  7);
-    col[ 8*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  8);
-    col[ 9*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  9);
-    col[10*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 10);
-    col[11*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 11);
-    col[12*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 12);
-    col[13*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 13);
-    col[14*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 14);
-    col[15*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 15);
+    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
+    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_striped_neon_128_8
-#define PNAME parasail_sg_flags_stats_table_striped_profile_neon_128_8
+#define FNAME parasail_sg_flags_stats_table_striped_neon_128_64
+#define PNAME parasail_sg_flags_stats_table_striped_profile_neon_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_striped_neon_128_8
-#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_neon_128_8
+#define FNAME parasail_sg_flags_stats_rowcol_striped_neon_128_64
+#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_neon_128_64
 #else
-#define FNAME parasail_sg_flags_stats_striped_neon_128_8
-#define PNAME parasail_sg_flags_stats_striped_profile_neon_128_8
+#define FNAME parasail_sg_flags_stats_striped_neon_128_64
+#define PNAME parasail_sg_flags_stats_striped_profile_neon_128_64
 #endif
 #endif
 
@@ -117,7 +89,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_neon_128_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_neon_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -159,17 +131,17 @@ parasail_result_t* PNAME(
     simde__m128i* restrict pvEM = NULL;
     simde__m128i* restrict pvES = NULL;
     simde__m128i* restrict pvEL = NULL;
-    int8_t* restrict boundary = NULL;
+    int64_t* restrict boundary = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     simde__m128i vZero;
     simde__m128i vOne;
-    int8_t score = 0;
-    int8_t matches = 0;
-    int8_t similar = 0;
-    int8_t length = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -183,7 +155,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -199,33 +171,33 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile  = (simde__m128i*)profile->profile8.score;
-    vProfileM = (simde__m128i*)profile->profile8.matches;
-    vProfileS = (simde__m128i*)profile->profile8.similar;
-    vGapO = simde_mm_set1_epi8(open);
-    vGapE = simde_mm_set1_epi8(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    vProfile  = (simde__m128i*)profile->profile64.score;
+    vProfileM = (simde__m128i*)profile->profile64.matches;
+    vProfileS = (simde__m128i*)profile->profile64.similar;
+    vGapO = simde_mm_set1_epi64x(open);
+    vGapE = simde_mm_set1_epi64x(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
-    vOne = simde_mm_set1_epi8(1);
+    vOne = simde_mm_set1_epi64x(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = simde_mm_set1_epi8(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi8(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHM = vNegLimit;
     vMaxHS = vNegLimit;
     vMaxHL = vNegLimit;
-    vPosMask = simde_mm_cmpeq_epi8(simde_mm_set1_epi8(position),
-            simde_mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
+    vPosMask = simde_mm_cmpeq_epi64(simde_mm_set1_epi64x(position),
+            simde_mm_set_epi64x(0,1));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -242,7 +214,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -267,7 +239,7 @@ parasail_result_t* PNAME(
     pvEM      = parasail_memalign_simde__m128i(16, segLen);
     pvES      = parasail_memalign_simde__m128i(16, segLen);
     pvEL      = parasail_memalign_simde__m128i(16, segLen);
-    boundary  = parasail_memalign_int8_t(16, s2Len+1);
+    boundary  = parasail_memalign_int64_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -300,9 +272,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                h.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.i8[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
+                e.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             simde_mm_store_si128(&pvHStore[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -315,7 +287,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -348,18 +320,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 1 bytes */
+        /* load final segment of pvHStore and shift left by 8 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
         vHM = simde_mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = simde_mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = simde_mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 1);
-        vHM = simde_mm_slli_si128(vHM, 1);
-        vHS = simde_mm_slli_si128(vHS, 1);
-        vHL = simde_mm_slli_si128(vHL, 1);
+        vH = simde_mm_slli_si128(vH, 8);
+        vHM = simde_mm_slli_si128(vHM, 8);
+        vHS = simde_mm_slli_si128(vHS, 8);
+        vHL = simde_mm_slli_si128(vHL, 8);
 
         /* insert upper boundary condition */
-        vH = simde_mm_insert_epi8(vH, boundary[j], 0);
+        vH = simde_mm_insert_epi64(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -383,58 +355,58 @@ parasail_result_t* PNAME(
             vEL = simde_mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_adds_epi8(vH, simde_mm_load_si128(vP + i));
-            vH = simde_mm_max_epi8(vH_dag, vE);
-            vH = simde_mm_max_epi8(vH, vF);
+            vH_dag = simde_mm_add_epi64(vH, simde_mm_load_si128(vP + i));
+            vH = simde_mm_max_epi64(vH_dag, vE);
+            vH = simde_mm_max_epi64(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
 
-            case1 = simde_mm_cmpeq_epi8(vH, vH_dag);
-            case2 = simde_mm_cmpeq_epi8(vH, vF);
+            case1 = simde_mm_cmpeq_epi64(vH, vH_dag);
+            case2 = simde_mm_cmpeq_epi64(vH, vF);
 
             /* calculate vM */
             vHM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
-                    simde_mm_adds_epi8(vHM, simde_mm_load_si128(vPM + i)),
+                    simde_mm_add_epi64(vHM, simde_mm_load_si128(vPM + i)),
                     case1);
             simde_mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vES, vFS, case2),
-                    simde_mm_adds_epi8(vHS, simde_mm_load_si128(vPS + i)),
+                    simde_mm_add_epi64(vHS, simde_mm_load_si128(vPS + i)),
                     case1);
             simde_mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEL, vFL, case2),
-                    simde_mm_adds_epi8(vHL, vOne),
+                    simde_mm_add_epi64(vHL, vOne),
                     case1);
             simde_mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vH);
-            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = simde_mm_subs_epi8(vH, vGapO);
+            vEF_opn = simde_mm_sub_epi64(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_subs_epi8(vE, vGapE);
-            vE = simde_mm_max_epi8(vEF_opn, vE_ext);
-            case1 = simde_mm_cmpgt_epi8(vEF_opn, vE_ext);
+            vE_ext = simde_mm_sub_epi64(vE, vGapE);
+            vE = simde_mm_max_epi64(vEF_opn, vE_ext);
+            case1 = simde_mm_cmpgt_epi64(vEF_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vHM, case1);
             vES = simde_mm_blendv_epi8(vES, vHS, case1);
             vEL = simde_mm_blendv_epi8(
-                    simde_mm_adds_epi8(vEL, vOne),
-                    simde_mm_adds_epi8(vHL, vOne),
+                    simde_mm_add_epi64(vEL, vOne),
+                    simde_mm_add_epi64(vHL, vOne),
                     case1);
             simde_mm_store_si128(pvE + i, vE);
             simde_mm_store_si128(pvEM + i, vEM);
@@ -442,14 +414,14 @@ parasail_result_t* PNAME(
             simde_mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = simde_mm_subs_epi8(vF, vGapE);
-            vF = simde_mm_max_epi8(vEF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi8(vEF_opn, vF_ext);
+            vF_ext = simde_mm_sub_epi64(vF, vGapE);
+            vF = simde_mm_max_epi64(vEF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vHM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vHS, case1);
             vFL = simde_mm_blendv_epi8(
-                    simde_mm_adds_epi8(vFL, vOne),
-                    simde_mm_adds_epi8(vHL, vOne),
+                    simde_mm_add_epi64(vFL, vOne),
+                    simde_mm_add_epi64(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -464,26 +436,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int8_t tmp2 = tmp < INT8_MIN ? INT8_MIN : tmp;
-            vHp = simde_mm_slli_si128(vHp, 1);
-            vF = simde_mm_slli_si128(vF, 1);
-            vFM = simde_mm_slli_si128(vFM, 1);
-            vFS = simde_mm_slli_si128(vFS, 1);
-            vFL = simde_mm_slli_si128(vFL, 1);
-            vHp = simde_mm_insert_epi8(vHp, boundary[j], 0);
-            vF = simde_mm_insert_epi8(vF, tmp2, 0);
-            vFL = simde_mm_insert_epi8(vFL, 1, 0);
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
+            vHp = simde_mm_slli_si128(vHp, 8);
+            vF = simde_mm_slli_si128(vF, 8);
+            vFM = simde_mm_slli_si128(vFM, 8);
+            vFS = simde_mm_slli_si128(vFS, 8);
+            vFL = simde_mm_slli_si128(vFL, 8);
+            vHp = simde_mm_insert_epi64(vHp, boundary[j], 0);
+            vF = simde_mm_insert_epi64(vF, tmp2, 0);
+            vFL = simde_mm_insert_epi64(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 simde__m128i case1;
                 simde__m128i case2;
                 simde__m128i cond;
 
-                vHp = simde_mm_adds_epi8(vHp, simde_mm_load_si128(vP + i));
+                vHp = simde_mm_add_epi64(vHp, simde_mm_load_si128(vP + i));
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi8(vH,vF);
+                vH = simde_mm_max_epi64(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                case1 = simde_mm_cmpeq_epi8(vH, vHp);
-                case2 = simde_mm_cmpeq_epi8(vH, vF);
+                case1 = simde_mm_cmpeq_epi64(vH, vHp);
+                case2 = simde_mm_cmpeq_epi64(vH, vF);
                 cond = simde_mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -501,11 +473,11 @@ parasail_result_t* PNAME(
                 vHL = simde_mm_blendv_epi8(vHL, vFL, cond);
                 simde_mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = simde_mm_min_epi8(vSaturationCheckMin, vH);
-                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vH);
-                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -513,21 +485,21 @@ parasail_result_t* PNAME(
                 arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = simde_mm_subs_epi8(vH, vGapO);
-                vF_ext = simde_mm_subs_epi8(vF, vGapE);
+                vEF_opn = simde_mm_sub_epi64(vH, vGapO);
+                vF_ext = simde_mm_sub_epi64(vF, vGapE);
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi8(vF_ext, vEF_opn),
-                                simde_mm_cmpeq_epi8(vF_ext, vEF_opn))))
+                                simde_mm_cmpgt_epi64(vF_ext, vEF_opn),
+                                simde_mm_cmpeq_epi64(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = simde_mm_max_epi8(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi64(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = simde_mm_cmpgt_epi8(vEF_opn, vF_ext);
+                cond = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
                 vFM = simde_mm_blendv_epi8(vFM, vHM, cond);
                 vFS = simde_mm_blendv_epi8(vFS, vHS, cond);
                 vFL = simde_mm_blendv_epi8(
-                        simde_mm_adds_epi8(vFL, vOne),
-                        simde_mm_adds_epi8(vHL, vOne),
+                        simde_mm_add_epi64(vFL, vOne),
+                        simde_mm_add_epi64(vHL, vOne),
                         cond);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
@@ -543,7 +515,7 @@ end:
             vHM = simde_mm_load_si128(pvHMStore + offset);
             vHS = simde_mm_load_si128(pvHSStore + offset);
             vHL = simde_mm_load_si128(pvHLStore + offset);
-            cond_max = simde_mm_cmpgt_epi8(vH, vMaxH);
+            cond_max = simde_mm_cmpgt_epi64(vH, vMaxH);
             vMaxH = simde_mm_blendv_epi8(vMaxH, vH, cond_max);
             vMaxHM = simde_mm_blendv_epi8(vMaxHM, vHM, cond_max);
             vMaxHS = simde_mm_blendv_epi8(vMaxHS, vHS, cond_max);
@@ -554,15 +526,15 @@ end:
         }
 #ifdef PARASAIL_ROWCOL
         for (k=0; k<position; ++k) {
-            vH = simde_mm_slli_si128(vH, 1);
-            vHM = simde_mm_slli_si128(vHM, 1);
-            vHS = simde_mm_slli_si128(vHS, 1);
-            vHL = simde_mm_slli_si128(vHL, 1);
+            vH = simde_mm_slli_si128(vH, 8);
+            vHM = simde_mm_slli_si128(vHM, 8);
+            vHS = simde_mm_slli_si128(vHS, 8);
+            vHL = simde_mm_slli_si128(vHL, 8);
         }
-        result->stats->rowcols->score_row[j] = (int8_t) simde_mm_extract_epi8 (vH, 15);
-        result->stats->rowcols->matches_row[j] = (int8_t) simde_mm_extract_epi8 (vHM, 15);
-        result->stats->rowcols->similar_row[j] = (int8_t) simde_mm_extract_epi8 (vHS, 15);
-        result->stats->rowcols->length_row[j] = (int8_t) simde_mm_extract_epi8 (vHL, 15);
+        result->stats->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
+        result->stats->rowcols->matches_row[j] = (int64_t) simde_mm_extract_epi64 (vHM, 1);
+        result->stats->rowcols->similar_row[j] = (int64_t) simde_mm_extract_epi64 (vHS, 1);
+        result->stats->rowcols->length_row[j] = (int64_t) simde_mm_extract_epi64 (vHL, 1);
 #endif
     }
 
@@ -583,26 +555,26 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = simde_mm_slli_si128(vMaxH, 1);
-            vMaxHM = simde_mm_slli_si128(vMaxHM, 1);
-            vMaxHS = simde_mm_slli_si128(vMaxHS, 1);
-            vMaxHL = simde_mm_slli_si128(vMaxHL, 1);
+            vMaxH = simde_mm_slli_si128(vMaxH, 8);
+            vMaxHM = simde_mm_slli_si128(vMaxHM, 8);
+            vMaxHS = simde_mm_slli_si128(vMaxHS, 8);
+            vMaxHL = simde_mm_slli_si128(vMaxHL, 8);
         }
         end_query = s1Len-1;
-        score = (int8_t) simde_mm_extract_epi8(vMaxH, 15);
-        matches = (int8_t) simde_mm_extract_epi8(vMaxHM, 15);
-        similar = (int8_t) simde_mm_extract_epi8(vMaxHS, 15);
-        length = (int8_t) simde_mm_extract_epi8(vMaxHL, 15);
+        score = (int64_t) simde_mm_extract_epi64(vMaxH, 1);
+        matches = (int64_t) simde_mm_extract_epi64(vMaxHM, 1);
+        similar = (int64_t) simde_mm_extract_epi64(vMaxHS, 1);
+        length = (int64_t) simde_mm_extract_epi64(vMaxHL, 1);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int8_t *t = (int8_t*)pvHStore;
-        int8_t *m = (int8_t*)pvHMStore;
-        int8_t *s = (int8_t*)pvHSStore;
-        int8_t *l = (int8_t*)pvHLStore;
+        int64_t *t = (int64_t*)pvHStore;
+        int64_t *m = (int64_t*)pvHMStore;
+        int64_t *s = (int64_t*)pvHSStore;
+        int64_t *l = (int64_t*)pvHLStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -632,22 +604,22 @@ end:
         simde__m128i vHS = simde_mm_load_si128(pvHSStore + offset);
         simde__m128i vHL = simde_mm_load_si128(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = simde_mm_slli_si128(vH, 1);
-            vHM = simde_mm_slli_si128(vHM, 1);
-            vHS = simde_mm_slli_si128(vHS, 1);
-            vHL = simde_mm_slli_si128(vHL, 1);
+            vH = simde_mm_slli_si128(vH, 8);
+            vHM = simde_mm_slli_si128(vHM, 8);
+            vHS = simde_mm_slli_si128(vHS, 8);
+            vHL = simde_mm_slli_si128(vHL, 8);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int8_t) simde_mm_extract_epi8 (vH, 15);
-        matches = (int8_t) simde_mm_extract_epi8 (vHM, 15);
-        similar = (int8_t) simde_mm_extract_epi8 (vHS, 15);
-        length = (int8_t) simde_mm_extract_epi8 (vHL, 15);
+        score = (int64_t) simde_mm_extract_epi64 (vH, 1);
+        matches = (int64_t) simde_mm_extract_epi64 (vHM, 1);
+        similar = (int64_t) simde_mm_extract_epi64 (vHS, 1);
+        length = (int64_t) simde_mm_extract_epi64 (vHL, 1);
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
