@@ -11,8 +11,8 @@
 #include <stdlib.h>
 
 #define SG_STATS
-#define SG_SUFFIX _striped_sse41_128_64
-#define SG_SUFFIX_PROF _striped_profile_sse41_128_64
+#define SG_SUFFIX _striped_sse41_128_16
+#define SG_SUFFIX_PROF _striped_profile_sse41_128_16
 #include "sg_helper.h"
 
 #if defined(_MSC_VER)
@@ -29,89 +29,6 @@
 #define SWAP(A,B) { __m128i* tmp = A; A = B; B = tmp; }
 
 
-static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-#if HAVE_SSE41_MM_INSERT_EPI64
-#define _mm_insert_epi64_rpl _mm_insert_epi64
-#else
-static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, int imm) {
-    __m128i_64_t A;
-    A.m = a;
-    A.v[imm] = i;
-    return A.m;
-}
-#endif
-
-static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
-}
-
-#if HAVE_SSE2_MM_SET_EPI64X
-#define _mm_set_epi64x_rpl _mm_set_epi64x
-#else
-static inline __m128i _mm_set_epi64x_rpl(int64_t e1, int64_t e0) {
-    __m128i_64_t A;
-    A.v[0] = e0;
-    A.v[1] = e1;
-    return A.m;
-}
-#endif
-
-#if HAVE_SSE41_MM_EXTRACT_EPI64
-#define _mm_extract_epi64_rpl _mm_extract_epi64
-#else
-static inline int64_t _mm_extract_epi64_rpl(__m128i a, int imm) {
-    __m128i_64_t A;
-    A.m = a;
-    return A.v[imm];
-}
-#endif
-
-static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
-    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
-    return A.m;
-}
-
-static inline __m128i _mm_cmplt_epi64_rpl(__m128i a, __m128i b) {
-    __m128i_64_t A;
-    __m128i_64_t B;
-    A.m = a;
-    B.m = b;
-    A.v[0] = (A.v[0]<B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    A.v[1] = (A.v[1]<B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
-    return A.m;
-}
-
-#if HAVE_SSE2_MM_SET1_EPI64X
-#define _mm_set1_epi64x_rpl _mm_set1_epi64x
-#else
-static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
-    __m128i_64_t A;
-    A.v[0] = i;
-    A.v[1] = i;
-    return A.m;
-}
-#endif
-
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -122,8 +39,14 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
+    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
+    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
+    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
+    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
+    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
 }
 #endif
 
@@ -134,21 +57,27 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
-    col[1*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
+    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
+    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
+    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
+    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
+    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
+    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
+    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
+    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_striped_sse41_128_64
-#define PNAME parasail_sg_flags_stats_table_striped_profile_sse41_128_64
+#define FNAME parasail_sg_flags_stats_table_striped_sse41_128_16
+#define PNAME parasail_sg_flags_stats_table_striped_profile_sse41_128_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_striped_sse41_128_64
-#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_sse41_128_64
+#define FNAME parasail_sg_flags_stats_rowcol_striped_sse41_128_16
+#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_sse41_128_16
 #else
-#define FNAME parasail_sg_flags_stats_striped_sse41_128_64
-#define PNAME parasail_sg_flags_stats_striped_profile_sse41_128_64
+#define FNAME parasail_sg_flags_stats_striped_sse41_128_16
+#define PNAME parasail_sg_flags_stats_striped_profile_sse41_128_16
 #endif
 #endif
 
@@ -177,7 +106,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_sse_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_sse_128_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -219,17 +148,17 @@ parasail_result_t* PNAME(
     __m128i* restrict pvEM = NULL;
     __m128i* restrict pvES = NULL;
     __m128i* restrict pvEL = NULL;
-    int64_t* restrict boundary = NULL;
+    int16_t* restrict boundary = NULL;
     __m128i vGapO;
     __m128i vGapE;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
     __m128i vZero;
     __m128i vOne;
-    int64_t score = 0;
-    int64_t matches = 0;
-    int64_t similar = 0;
-    int64_t length = 0;
+    int16_t score = 0;
+    int16_t matches = 0;
+    int16_t similar = 0;
+    int16_t length = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -243,7 +172,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -259,33 +188,33 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile  = (__m128i*)profile->profile64.score;
-    vProfileM = (__m128i*)profile->profile64.matches;
-    vProfileS = (__m128i*)profile->profile64.similar;
-    vGapO = _mm_set1_epi64x_rpl(open);
-    vGapE = _mm_set1_epi64x_rpl(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    vProfile  = (__m128i*)profile->profile16.score;
+    vProfileM = (__m128i*)profile->profile16.matches;
+    vProfileS = (__m128i*)profile->profile16.similar;
+    vGapO = _mm_set1_epi16(open);
+    vGapE = _mm_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi64x_rpl(1);
+    vOne = _mm_set1_epi16(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHM = vNegLimit;
     vMaxHS = vNegLimit;
     vMaxHL = vNegLimit;
-    vPosMask = _mm_cmpeq_epi64(_mm_set1_epi64x_rpl(position),
-            _mm_set_epi64x_rpl(0,1));
+    vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
+            _mm_set_epi16(0,1,2,3,4,5,6,7));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -302,7 +231,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -327,7 +256,7 @@ parasail_result_t* PNAME(
     pvEM      = parasail_memalign___m128i(16, segLen);
     pvES      = parasail_memalign___m128i(16, segLen);
     pvEL      = parasail_memalign___m128i(16, segLen);
-    boundary  = parasail_memalign_int64_t(16, s2Len+1);
+    boundary  = parasail_memalign_int16_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -356,13 +285,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_64_t h;
-            __m128i_64_t e;
+            __m128i_16_t h;
+            __m128i_16_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
+                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -375,7 +304,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
+            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
         }
     }
 
@@ -408,18 +337,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 8 bytes */
+        /* load final segment of pvHStore and shift left by 2 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
         vHM = _mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = _mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = _mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 8);
-        vHM = _mm_slli_si128(vHM, 8);
-        vHS = _mm_slli_si128(vHS, 8);
-        vHL = _mm_slli_si128(vHL, 8);
+        vH = _mm_slli_si128(vH, 2);
+        vHM = _mm_slli_si128(vHM, 2);
+        vHS = _mm_slli_si128(vHS, 2);
+        vHL = _mm_slli_si128(vHL, 2);
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi64_rpl(vH, boundary[j], 0);
+        vH = _mm_insert_epi16(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -443,58 +372,58 @@ parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_add_epi64(vH, _mm_load_si128(vP + i));
-            vH = _mm_max_epi64_rpl(vH_dag, vE);
-            vH = _mm_max_epi64_rpl(vH, vF);
+            vH_dag = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
+            vH = _mm_max_epi16(vH_dag, vE);
+            vH = _mm_max_epi16(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
 
-            case1 = _mm_cmpeq_epi64(vH, vH_dag);
-            case2 = _mm_cmpeq_epi64(vH, vF);
+            case1 = _mm_cmpeq_epi16(vH, vH_dag);
+            case2 = _mm_cmpeq_epi16(vH, vF);
 
             /* calculate vM */
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_add_epi64(vHM, _mm_load_si128(vPM + i)),
+                    _mm_adds_epi16(vHM, _mm_load_si128(vPM + i)),
                     case1);
             _mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm_blendv_epi8(
                     _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_add_epi64(vHS, _mm_load_si128(vPS + i)),
+                    _mm_adds_epi16(vHS, _mm_load_si128(vPS + i)),
                     case1);
             _mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_add_epi64(vHL, vOne),
+                    _mm_adds_epi16(vHL, vOne),
                     case1);
             _mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = _mm_sub_epi64(vH, vGapO);
+            vEF_opn = _mm_subs_epi16(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_sub_epi64(vE, vGapE);
-            vE = _mm_max_epi64_rpl(vEF_opn, vE_ext);
-            case1 = _mm_cmpgt_epi64_rpl(vEF_opn, vE_ext);
+            vE_ext = _mm_subs_epi16(vE, vGapE);
+            vE = _mm_max_epi16(vEF_opn, vE_ext);
+            case1 = _mm_cmpgt_epi16(vEF_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(
-                    _mm_add_epi64(vEL, vOne),
-                    _mm_add_epi64(vHL, vOne),
+                    _mm_adds_epi16(vEL, vOne),
+                    _mm_adds_epi16(vHL, vOne),
                     case1);
             _mm_store_si128(pvE + i, vE);
             _mm_store_si128(pvEM + i, vEM);
@@ -502,14 +431,14 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm_sub_epi64(vF, vGapE);
-            vF = _mm_max_epi64_rpl(vEF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi64_rpl(vEF_opn, vF_ext);
+            vF_ext = _mm_subs_epi16(vF, vGapE);
+            vF = _mm_max_epi16(vEF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi16(vEF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_add_epi64(vFL, vOne),
-                    _mm_add_epi64(vHL, vOne),
+                    _mm_adds_epi16(vFL, vOne),
+                    _mm_adds_epi16(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -524,26 +453,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             __m128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
-            vHp = _mm_slli_si128(vHp, 8);
-            vF = _mm_slli_si128(vF, 8);
-            vFM = _mm_slli_si128(vFM, 8);
-            vFS = _mm_slli_si128(vFS, 8);
-            vFL = _mm_slli_si128(vFL, 8);
-            vHp = _mm_insert_epi64_rpl(vHp, boundary[j], 0);
-            vF = _mm_insert_epi64_rpl(vF, tmp2, 0);
-            vFL = _mm_insert_epi64_rpl(vFL, 1, 0);
+            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
+            vHp = _mm_slli_si128(vHp, 2);
+            vF = _mm_slli_si128(vF, 2);
+            vFM = _mm_slli_si128(vFM, 2);
+            vFS = _mm_slli_si128(vFS, 2);
+            vFL = _mm_slli_si128(vFL, 2);
+            vHp = _mm_insert_epi16(vHp, boundary[j], 0);
+            vF = _mm_insert_epi16(vF, tmp2, 0);
+            vFL = _mm_insert_epi16(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 __m128i case1;
                 __m128i case2;
                 __m128i cond;
 
-                vHp = _mm_add_epi64(vHp, _mm_load_si128(vP + i));
+                vHp = _mm_adds_epi16(vHp, _mm_load_si128(vP + i));
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi64_rpl(vH,vF);
+                vH = _mm_max_epi16(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                case1 = _mm_cmpeq_epi64(vH, vHp);
-                case2 = _mm_cmpeq_epi64(vH, vF);
+                case1 = _mm_cmpeq_epi16(vH, vHp);
+                case2 = _mm_cmpeq_epi16(vH, vF);
                 cond = _mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -561,11 +490,11 @@ parasail_result_t* PNAME(
                 vHL = _mm_blendv_epi8(vHL, vFL, cond);
                 _mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vH);
-                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -573,21 +502,21 @@ parasail_result_t* PNAME(
                 arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = _mm_sub_epi64(vH, vGapO);
-                vF_ext = _mm_sub_epi64(vF, vGapE);
+                vEF_opn = _mm_subs_epi16(vH, vGapO);
+                vF_ext = _mm_subs_epi16(vF, vGapE);
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi64_rpl(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi64(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi16(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi64_rpl(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm_cmpgt_epi64_rpl(vEF_opn, vF_ext);
+                cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
                 vFM = _mm_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm_blendv_epi8(
-                        _mm_add_epi64(vFL, vOne),
-                        _mm_add_epi64(vHL, vOne),
+                        _mm_adds_epi16(vFL, vOne),
+                        _mm_adds_epi16(vHL, vOne),
                         cond);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
@@ -603,7 +532,7 @@ end:
             vHM = _mm_load_si128(pvHMStore + offset);
             vHS = _mm_load_si128(pvHSStore + offset);
             vHL = _mm_load_si128(pvHLStore + offset);
-            cond_max = _mm_cmpgt_epi64_rpl(vH, vMaxH);
+            cond_max = _mm_cmpgt_epi16(vH, vMaxH);
             vMaxH = _mm_blendv_epi8(vMaxH, vH, cond_max);
             vMaxHM = _mm_blendv_epi8(vMaxHM, vHM, cond_max);
             vMaxHS = _mm_blendv_epi8(vMaxHS, vHS, cond_max);
@@ -614,15 +543,15 @@ end:
         }
 #ifdef PARASAIL_ROWCOL
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 8);
-            vHM = _mm_slli_si128(vHM, 8);
-            vHS = _mm_slli_si128(vHS, 8);
-            vHL = _mm_slli_si128(vHL, 8);
+            vH = _mm_slli_si128(vH, 2);
+            vHM = _mm_slli_si128(vHM, 2);
+            vHS = _mm_slli_si128(vHS, 2);
+            vHL = _mm_slli_si128(vHL, 2);
         }
-        result->stats->rowcols->score_row[j] = (int64_t) _mm_extract_epi64_rpl (vH, 1);
-        result->stats->rowcols->matches_row[j] = (int64_t) _mm_extract_epi64_rpl (vHM, 1);
-        result->stats->rowcols->similar_row[j] = (int64_t) _mm_extract_epi64_rpl (vHS, 1);
-        result->stats->rowcols->length_row[j] = (int64_t) _mm_extract_epi64_rpl (vHL, 1);
+        result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
+        result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7);
+        result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7);
+        result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7);
 #endif
     }
 
@@ -643,26 +572,26 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 8);
-            vMaxHM = _mm_slli_si128(vMaxHM, 8);
-            vMaxHS = _mm_slli_si128(vMaxHS, 8);
-            vMaxHL = _mm_slli_si128(vMaxHL, 8);
+            vMaxH = _mm_slli_si128(vMaxH, 2);
+            vMaxHM = _mm_slli_si128(vMaxHM, 2);
+            vMaxHS = _mm_slli_si128(vMaxHS, 2);
+            vMaxHL = _mm_slli_si128(vMaxHL, 2);
         }
         end_query = s1Len-1;
-        score = (int64_t) _mm_extract_epi64_rpl(vMaxH, 1);
-        matches = (int64_t) _mm_extract_epi64_rpl(vMaxHM, 1);
-        similar = (int64_t) _mm_extract_epi64_rpl(vMaxHS, 1);
-        length = (int64_t) _mm_extract_epi64_rpl(vMaxHL, 1);
+        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
+        matches = (int16_t) _mm_extract_epi16(vMaxHM, 7);
+        similar = (int16_t) _mm_extract_epi16(vMaxHS, 7);
+        length = (int16_t) _mm_extract_epi16(vMaxHL, 7);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int64_t *t = (int64_t*)pvHStore;
-        int64_t *m = (int64_t*)pvHMStore;
-        int64_t *s = (int64_t*)pvHSStore;
-        int64_t *l = (int64_t*)pvHLStore;
+        int16_t *t = (int16_t*)pvHStore;
+        int16_t *m = (int16_t*)pvHMStore;
+        int16_t *s = (int16_t*)pvHSStore;
+        int16_t *l = (int16_t*)pvHLStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -692,22 +621,22 @@ end:
         __m128i vHS = _mm_load_si128(pvHSStore + offset);
         __m128i vHL = _mm_load_si128(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 8);
-            vHM = _mm_slli_si128(vHM, 8);
-            vHS = _mm_slli_si128(vHS, 8);
-            vHL = _mm_slli_si128(vHL, 8);
+            vH = _mm_slli_si128(vH, 2);
+            vHM = _mm_slli_si128(vHM, 2);
+            vHS = _mm_slli_si128(vHS, 2);
+            vHL = _mm_slli_si128(vHL, 2);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int64_t) _mm_extract_epi64_rpl (vH, 1);
-        matches = (int64_t) _mm_extract_epi64_rpl (vHM, 1);
-        similar = (int64_t) _mm_extract_epi64_rpl (vHS, 1);
-        length = (int64_t) _mm_extract_epi64_rpl (vHL, 1);
+        score = (int16_t) _mm_extract_epi16 (vH, 7);
+        matches = (int16_t) _mm_extract_epi16 (vHM, 7);
+        similar = (int16_t) _mm_extract_epi16 (vHS, 7);
+        length = (int16_t) _mm_extract_epi16 (vHL, 7);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi64_rpl(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
