@@ -16,8 +16,8 @@
 #include "parasail/memory.h"
 #include "parasail/internal_neon.h"
 
-#define SG_SUFFIX _striped_neon_128_32
-#define SG_SUFFIX_PROF _striped_profile_neon_128_32
+#define SG_SUFFIX _striped_neon_128_16
+#define SG_SUFFIX_PROF _striped_profile_neon_128_16
 #include "sg_helper.h"
 
 
@@ -31,10 +31,14 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 3);
+    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 4);
+    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 5);
+    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 6);
+    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)simde_mm_extract_epi16(vH, 7);
 }
 #endif
 
@@ -45,23 +49,27 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    col[0*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 0);
+    col[1*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 1);
+    col[2*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 2);
+    col[3*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 3);
+    col[4*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 4);
+    col[5*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 5);
+    col[6*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 6);
+    col[7*seglen+t] = (int16_t)simde_mm_extract_epi16(vH, 7);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_table_striped_neon_128_32
-#define PNAME parasail_sg_flags_table_striped_profile_neon_128_32
+#define FNAME parasail_sg_flags_table_striped_neon_128_16
+#define PNAME parasail_sg_flags_table_striped_profile_neon_128_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_rowcol_striped_neon_128_32
-#define PNAME parasail_sg_flags_rowcol_striped_profile_neon_128_32
+#define FNAME parasail_sg_flags_rowcol_striped_neon_128_16
+#define PNAME parasail_sg_flags_rowcol_striped_profile_neon_128_16
 #else
-#define FNAME parasail_sg_flags_striped_neon_128_32
-#define PNAME parasail_sg_flags_striped_profile_neon_128_32
+#define FNAME parasail_sg_flags_striped_neon_128_16
+#define PNAME parasail_sg_flags_striped_profile_neon_128_16
 #endif
 #endif
 
@@ -87,7 +95,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -118,12 +126,12 @@ parasail_result_t* PNAME(
     simde__m128i* restrict pvHStore = NULL;
     simde__m128i* restrict pvHLoad = NULL;
     simde__m128i* restrict pvE = NULL;
-    int32_t* restrict boundary = NULL;
+    int16_t* restrict boundary = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -134,7 +142,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -150,23 +158,23 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    vProfile = (simde__m128i*)profile->profile32.score;
-    vGapO = simde_mm_set1_epi32(open);
-    vGapE = simde_mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vProfile = (simde__m128i*)profile->profile16.score;
+    vGapO = simde_mm_set1_epi16(open);
+    vGapE = simde_mm_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = simde_mm_cmpeq_epi32(simde_mm_set1_epi32(position),
-            simde_mm_set_epi32(0,1,2,3));
+    vPosMask = simde_mm_cmpeq_epi16(simde_mm_set1_epi16(position),
+            simde_mm_set_epi16(0,1,2,3,4,5,6,7));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -182,7 +190,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -198,7 +206,7 @@ parasail_result_t* PNAME(
     pvHStore = parasail_memalign_simde__m128i(16, segLen);
     pvHLoad =  parasail_memalign_simde__m128i(16, segLen);
     pvE      = parasail_memalign_simde__m128i(16, segLen);
-    boundary = parasail_memalign_int32_t(16, s2Len+1);
+    boundary = parasail_memalign_int16_t(16, s2Len+1);
 
     /* validate heap variables */
     if (!pvHStore) return NULL;
@@ -215,9 +223,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                h.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
                 tmp = tmp - open;
-                e.i32[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
+                e.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
             }
             simde_mm_store_si128(&pvHStore[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -230,7 +238,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
+            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
         }
     }
 
@@ -242,7 +250,7 @@ parasail_result_t* PNAME(
         simde__m128i vF = vNegLimit;
 
         /* load final segment of pvHStore and shift left by 2 bytes */
-        simde__m128i vH = simde_mm_slli_si128(pvHStore[segLen - 1], 4);
+        simde__m128i vH = simde_mm_slli_si128(pvHStore[segLen - 1], 2);
 
         /* Correct part of the vProfile */
         const simde__m128i* vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -253,35 +261,35 @@ parasail_result_t* PNAME(
         pvHStore = pv;
 
         /* insert upper boundary condition */
-        vH = simde_mm_insert_epi32(vH, boundary[j], 0);
+        vH = simde_mm_insert_epi16(vH, boundary[j], 0);
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = simde_mm_add_epi32(vH, simde_mm_load_si128(vP + i));
+            vH = simde_mm_adds_epi16(vH, simde_mm_load_si128(vP + i));
             vE = simde_mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = simde_mm_max_epi32(vH, vE);
-            vH = simde_mm_max_epi32(vH, vF);
+            vH = simde_mm_max_epi16(vH, vE);
+            vH = simde_mm_max_epi16(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
-            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vE);
-            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vF);
+            vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vH);
+            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vE);
+            vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vF);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
 
             /* Update vE value. */
-            vH = simde_mm_sub_epi32(vH, vGapO);
-            vE = simde_mm_sub_epi32(vE, vGapE);
-            vE = simde_mm_max_epi32(vE, vH);
+            vH = simde_mm_subs_epi16(vH, vGapO);
+            vE = simde_mm_subs_epi16(vE, vGapE);
+            vE = simde_mm_max_epi16(vE, vH);
             simde_mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = simde_mm_sub_epi32(vF, vGapE);
-            vF = simde_mm_max_epi32(vF, vH);
+            vF = simde_mm_subs_epi16(vF, vGapE);
+            vF = simde_mm_max_epi16(vF, vH);
 
             /* Load the next vH. */
             vH = simde_mm_load_si128(pvHLoad + i);
@@ -291,22 +299,22 @@ parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
-            vF = simde_mm_slli_si128(vF, 4);
-            vF = simde_mm_insert_epi32(vF, tmp2, 0);
+            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
+            vF = simde_mm_slli_si128(vF, 2);
+            vF = simde_mm_insert_epi16(vF, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi32(vH,vF);
+                vH = simde_mm_max_epi16(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
-                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
+                vSaturationCheckMin = simde_mm_min_epi16(vSaturationCheckMin, vH);
+                vSaturationCheckMax = simde_mm_max_epi16(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vH = simde_mm_sub_epi32(vH, vGapO);
-                vF = simde_mm_sub_epi32(vF, vGapE);
-                if (! simde_mm_movemask_epi8(simde_mm_cmpgt_epi32(vF, vH))) goto end;
-                /*vF = simde_mm_max_epi32(vF, vH);*/
+                vH = simde_mm_subs_epi16(vH, vGapO);
+                vF = simde_mm_subs_epi16(vF, vGapE);
+                if (! simde_mm_movemask_epi8(simde_mm_cmpgt_epi16(vF, vH))) goto end;
+                /*vF = simde_mm_max_epi16(vF, vH);*/
             }
         }
 end:
@@ -314,16 +322,16 @@ end:
             /* extract vector containing last value from the column */
             simde__m128i vCompare;
             vH = simde_mm_load_si128(pvHStore + offset);
-            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi32(vH, vMaxH));
-            vMaxH = simde_mm_max_epi32(vH, vMaxH);
+            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi16(vH, vMaxH));
+            vMaxH = simde_mm_max_epi16(vH, vMaxH);
             if (simde_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 4);
+                vH = simde_mm_slli_si128(vH, 2);
             }
-            result->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            result->rowcols->score_row[j] = (int16_t) simde_mm_extract_epi16 (vH, 7);
 #endif
         }
     }
@@ -339,9 +347,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = simde_mm_slli_si128(vMaxH, 4);
+            vMaxH = simde_mm_slli_si128(vMaxH, 2);
         }
-        score = (int32_t) simde_mm_extract_epi32(vMaxH, 3);
+        score = (int16_t) simde_mm_extract_epi16(vMaxH, 7);
         end_query = s1Len-1;
     }
 
@@ -349,7 +357,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int32_t *t = (int32_t*)pvHStore;
+        int16_t *t = (int16_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -370,17 +378,17 @@ end:
         {
             simde__m128i vH = simde_mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 4);
+                vH = simde_mm_slli_si128(vH, 2);
             }
-            score = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            score = (int16_t) simde_mm_extract_epi16 (vH, 7);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
