@@ -20,7 +20,7 @@
 #include "parasail/internal_sse.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _diag_sse2_128_8
+#define SG_SUFFIX _diag_sse2_128_32
 #include "sg_helper.h"
 
 
@@ -30,28 +30,28 @@ static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
     return a;
 }
 
-static inline __m128i _mm_insert_epi8_rpl(__m128i a, int8_t i, const int imm) {
-    __m128i_8_t A;
+static inline __m128i _mm_insert_epi32_rpl(__m128i a, int32_t i, const int imm) {
+    __m128i_32_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 
-static inline __m128i _mm_max_epi8_rpl(__m128i a, __m128i b) {
-    __m128i mask = _mm_cmpgt_epi8(a, b);
+static inline __m128i _mm_max_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(a, b);
     a = _mm_and_si128(a, mask);
     b = _mm_andnot_si128(mask, b);
     return _mm_or_si128(a, b);
 }
 
-static inline int8_t _mm_extract_epi8_rpl(__m128i a, const int imm) {
-    __m128i_8_t A;
+static inline int32_t _mm_extract_epi32_rpl(__m128i a, const int imm) {
+    __m128i_32_t A;
     A.m = a;
     return A.v[imm];
 }
 
-static inline __m128i _mm_min_epi8_rpl(__m128i a, __m128i b) {
-    __m128i mask = _mm_cmpgt_epi8(b, a);
+static inline __m128i _mm_min_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(b, a);
     a = _mm_and_si128(a, mask);
     b = _mm_andnot_si128(mask, b);
     return _mm_or_si128(a, b);
@@ -67,56 +67,20 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm_extract_epi8_rpl(vWH, 15);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int8_t)_mm_extract_epi32_rpl(vWH, 3);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm_extract_epi8_rpl(vWH, 14);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int8_t)_mm_extract_epi32_rpl(vWH, 2);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)_mm_extract_epi8_rpl(vWH, 13);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int8_t)_mm_extract_epi32_rpl(vWH, 1);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)_mm_extract_epi8_rpl(vWH, 12);
-    }
-    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int8_t)_mm_extract_epi8_rpl(vWH, 11);
-    }
-    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int8_t)_mm_extract_epi8_rpl(vWH, 10);
-    }
-    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int8_t)_mm_extract_epi8_rpl(vWH, 9);
-    }
-    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int8_t)_mm_extract_epi8_rpl(vWH, 8);
-    }
-    if (0 <= i+8 && i+8 < s1Len && 0 <= j-8 && j-8 < s2Len) {
-        array[1LL*(i+8)*s2Len + (j-8)] = (int8_t)_mm_extract_epi8_rpl(vWH, 7);
-    }
-    if (0 <= i+9 && i+9 < s1Len && 0 <= j-9 && j-9 < s2Len) {
-        array[1LL*(i+9)*s2Len + (j-9)] = (int8_t)_mm_extract_epi8_rpl(vWH, 6);
-    }
-    if (0 <= i+10 && i+10 < s1Len && 0 <= j-10 && j-10 < s2Len) {
-        array[1LL*(i+10)*s2Len + (j-10)] = (int8_t)_mm_extract_epi8_rpl(vWH, 5);
-    }
-    if (0 <= i+11 && i+11 < s1Len && 0 <= j-11 && j-11 < s2Len) {
-        array[1LL*(i+11)*s2Len + (j-11)] = (int8_t)_mm_extract_epi8_rpl(vWH, 4);
-    }
-    if (0 <= i+12 && i+12 < s1Len && 0 <= j-12 && j-12 < s2Len) {
-        array[1LL*(i+12)*s2Len + (j-12)] = (int8_t)_mm_extract_epi8_rpl(vWH, 3);
-    }
-    if (0 <= i+13 && i+13 < s1Len && 0 <= j-13 && j-13 < s2Len) {
-        array[1LL*(i+13)*s2Len + (j-13)] = (int8_t)_mm_extract_epi8_rpl(vWH, 2);
-    }
-    if (0 <= i+14 && i+14 < s1Len && 0 <= j-14 && j-14 < s2Len) {
-        array[1LL*(i+14)*s2Len + (j-14)] = (int8_t)_mm_extract_epi8_rpl(vWH, 1);
-    }
-    if (0 <= i+15 && i+15 < s1Len && 0 <= j-15 && j-15 < s2Len) {
-        array[1LL*(i+15)*s2Len + (j-15)] = (int8_t)_mm_extract_epi8_rpl(vWH, 0);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int8_t)_mm_extract_epi32_rpl(vWH, 0);
     }
 }
 
-#define FNAME parasail_sg_flags_trace_diag_sse2_128_8
+#define FNAME parasail_sg_flags_trace_diag_sse2_128_32
 
 parasail_result_t* FNAME(
         const char * const restrict _s1, const int _s1Len,
@@ -130,22 +94,22 @@ parasail_result_t* FNAME(
     int32_t PAD2 = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int8_t * restrict s1 = NULL;
-    int8_t * restrict s2B = NULL;
-    int8_t * restrict _H_pr = NULL;
-    int8_t * restrict _F_pr = NULL;
-    int8_t * restrict s2 = NULL;
-    int8_t * restrict H_pr = NULL;
-    int8_t * restrict F_pr = NULL;
+    int32_t * restrict s1 = NULL;
+    int32_t * restrict s2B = NULL;
+    int32_t * restrict _H_pr = NULL;
+    int32_t * restrict _F_pr = NULL;
+    int32_t * restrict s2 = NULL;
+    int32_t * restrict H_pr = NULL;
+    int32_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t s1Len = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
-    int8_t score = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
     __m128i vNegLimit;
     __m128i vPosLimit;
     __m128i vSaturationCheckMin;
@@ -189,7 +153,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 16; /* number of values in vector */
+    N = 4; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -199,55 +163,43 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = s1Len-1;
     end_ref = s2Len-1;
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi8(NEG_LIMIT);
-    vOpen = _mm_set1_epi8(open);
-    vGap  = _mm_set1_epi8(gap);
-    vOne = _mm_set1_epi8(1);
-    vN = _mm_set1_epi8(N);
-    vGapN = s1_beg ? _mm_set1_epi8(0) : _mm_set1_epi8(gap*N);
-    vNegOne = _mm_set1_epi8(-1);
-    vI = _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
-    vJreset = _mm_set_epi8(0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15);
+    vNegInf = _mm_set1_epi32(NEG_LIMIT);
+    vOpen = _mm_set1_epi32(open);
+    vGap  = _mm_set1_epi32(gap);
+    vOne = _mm_set1_epi32(1);
+    vN = _mm_set1_epi32(N);
+    vGapN = s1_beg ? _mm_set1_epi32(0) : _mm_set1_epi32(gap*N);
+    vNegOne = _mm_set1_epi32(-1);
+    vI = _mm_set_epi32(0,1,2,3);
+    vJreset = _mm_set_epi32(0,-1,-2,-3);
     vMaxHRow = vNegInf;
     vMaxHCol = vNegInf;
     vLastVal = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm_set1_epi8(s1Len);
-    vILimit1 = _mm_subs_epi8(vILimit, vOne);
-    vJLimit = _mm_set1_epi8(s2Len);
-    vJLimit1 = _mm_subs_epi8(vJLimit, vOne);
-    vIBoundary = s1_beg ? _mm_set1_epi8(0) : _mm_set_epi8(
+    vILimit = _mm_set1_epi32(s1Len);
+    vILimit1 = _mm_sub_epi32(vILimit, vOne);
+    vJLimit = _mm_set1_epi32(s2Len);
+    vJLimit1 = _mm_sub_epi32(vJLimit, vOne);
+    vIBoundary = s1_beg ? _mm_set1_epi32(0) : _mm_set_epi32(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
-            -open-3*gap,
-            -open-4*gap,
-            -open-5*gap,
-            -open-6*gap,
-            -open-7*gap,
-            -open-8*gap,
-            -open-9*gap,
-            -open-10*gap,
-            -open-11*gap,
-            -open-12*gap,
-            -open-13*gap,
-            -open-14*gap,
-            -open-15*gap);
-    vTDiag = _mm_set1_epi8(PARASAIL_DIAG);
-    vTIns = _mm_set1_epi8(PARASAIL_INS);
-    vTDel = _mm_set1_epi8(PARASAIL_DEL);
-    vTDiagE = _mm_set1_epi8(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi8(PARASAIL_INS_E);
-    vTDiagF = _mm_set1_epi8(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi8(PARASAIL_DEL_F);
+            -open-3*gap);
+    vTDiag = _mm_set1_epi32(PARASAIL_DIAG);
+    vTIns = _mm_set1_epi32(PARASAIL_INS);
+    vTDel = _mm_set1_epi32(PARASAIL_DEL);
+    vTDiagE = _mm_set1_epi32(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi32(PARASAIL_INS_E);
+    vTDiagF = _mm_set1_epi32(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi32(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(s1Len, s2Len, 16, sizeof(int8_t));
@@ -256,16 +208,16 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
     result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int8_t(16, s2Len+PAD2);
-    _H_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
-    _F_pr = parasail_memalign_int8_t(16, s2Len+PAD2);
+    s2B= parasail_memalign_int32_t(16, s2Len+PAD2);
+    _H_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _F_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -277,7 +229,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int8_t(16, s1Len+PAD);
+        s1 = parasail_memalign_int32_t(16, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -341,102 +293,78 @@ parasail_result_t* FNAME(
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
         const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
         const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
-        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
-        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
-        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        const int * const restrict matrow8 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+8] : ((i+8 >= s1Len) ? s1Len-1 : i+8))];
-        const int * const restrict matrow9 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+9] : ((i+9 >= s1Len) ? s1Len-1 : i+9))];
-        const int * const restrict matrow10 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+10] : ((i+10 >= s1Len) ? s1Len-1 : i+10))];
-        const int * const restrict matrow11 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+11] : ((i+11 >= s1Len) ? s1Len-1 : i+11))];
-        const int * const restrict matrow12 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+12] : ((i+12 >= s1Len) ? s1Len-1 : i+12))];
-        const int * const restrict matrow13 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+13] : ((i+13 >= s1Len) ? s1Len-1 : i+13))];
-        const int * const restrict matrow14 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+14] : ((i+14 >= s1Len) ? s1Len-1 : i+14))];
-        const int * const restrict matrow15 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+15] : ((i+15 >= s1Len) ? s1Len-1 : i+15))];
-        __m128i vIltLimit = _mm_cmplt_epi8(vI, vILimit);
-        __m128i vIeqLimit1 = _mm_cmpeq_epi8(vI, vILimit1);
-        vNH = _mm_srli_si128(vNH, 1);
-        vNH = _mm_insert_epi8_rpl(vNH, H_pr[-1], 15);
-        vWH = _mm_srli_si128(vWH, 1);
-        vWH = _mm_insert_epi8_rpl(vWH, s1_beg ? 0 : (-open - i*gap), 15);
+        __m128i vIltLimit = _mm_cmplt_epi32(vI, vILimit);
+        __m128i vIeqLimit1 = _mm_cmpeq_epi32(vI, vILimit1);
+        vNH = _mm_srli_si128(vNH, 4);
+        vNH = _mm_insert_epi32_rpl(vNH, H_pr[-1], 3);
+        vWH = _mm_srli_si128(vWH, 4);
+        vWH = _mm_insert_epi32_rpl(vWH, s1_beg ? 0 : (-open - i*gap), 3);
         H_pr[-1] = -open - (i+N)*gap;
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             __m128i vMat;
             __m128i vNWH = vNH;
-            vNH = _mm_srli_si128(vWH, 1);
-            vNH = _mm_insert_epi8_rpl(vNH, H_pr[j], 15);
-            vF = _mm_srli_si128(vF, 1);
-            vF = _mm_insert_epi8_rpl(vF, F_pr[j], 15);
-            vF_opn = _mm_subs_epi8(vNH, vOpen);
-            vF_ext = _mm_subs_epi8(vF, vGap);
-            vF = _mm_max_epi8_rpl(vF_opn, vF_ext);
-            vE_opn = _mm_subs_epi8(vWH, vOpen);
-            vE_ext = _mm_subs_epi8(vE, vGap);
-            vE = _mm_max_epi8_rpl(vE_opn, vE_ext);
-            vMat = _mm_set_epi8(
+            vNH = _mm_srli_si128(vWH, 4);
+            vNH = _mm_insert_epi32_rpl(vNH, H_pr[j], 3);
+            vF = _mm_srli_si128(vF, 4);
+            vF = _mm_insert_epi32_rpl(vF, F_pr[j], 3);
+            vF_opn = _mm_sub_epi32(vNH, vOpen);
+            vF_ext = _mm_sub_epi32(vF, vGap);
+            vF = _mm_max_epi32_rpl(vF_opn, vF_ext);
+            vE_opn = _mm_sub_epi32(vWH, vOpen);
+            vE_ext = _mm_sub_epi32(vE, vGap);
+            vE = _mm_max_epi32_rpl(vE_opn, vE_ext);
+            vMat = _mm_set_epi32(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
-                    matrow3[s2[j-3]],
-                    matrow4[s2[j-4]],
-                    matrow5[s2[j-5]],
-                    matrow6[s2[j-6]],
-                    matrow7[s2[j-7]],
-                    matrow8[s2[j-8]],
-                    matrow9[s2[j-9]],
-                    matrow10[s2[j-10]],
-                    matrow11[s2[j-11]],
-                    matrow12[s2[j-12]],
-                    matrow13[s2[j-13]],
-                    matrow14[s2[j-14]],
-                    matrow15[s2[j-15]]
+                    matrow3[s2[j-3]]
                     );
-            vNWH = _mm_adds_epi8(vNWH, vMat);
-            vWH = _mm_max_epi8_rpl(vNWH, vE);
-            vWH = _mm_max_epi8_rpl(vWH, vF);
+            vNWH = _mm_add_epi32(vNWH, vMat);
+            vWH = _mm_max_epi32_rpl(vNWH, vE);
+            vWH = _mm_max_epi32_rpl(vWH, vF);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m128i cond = _mm_cmpeq_epi8(vJ,vNegOne);
+                __m128i cond = _mm_cmpeq_epi32(vJ,vNegOne);
                 vWH = _mm_blendv_epi8_rpl(vWH, vIBoundary, cond);
                 vF = _mm_blendv_epi8_rpl(vF, vNegInf, cond);
                 vE = _mm_blendv_epi8_rpl(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi8_rpl(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm_min_epi32_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vWH);
             }
             /* trace table */
             {
-                __m128i case1 = _mm_cmpeq_epi8(vWH, vNWH);
-                __m128i case2 = _mm_cmpeq_epi8(vWH, vF);
+                __m128i case1 = _mm_cmpeq_epi32(vWH, vNWH);
+                __m128i case2 = _mm_cmpeq_epi32(vWH, vF);
                 __m128i vT = _mm_blendv_epi8_rpl(
                         _mm_blendv_epi8_rpl(vTIns, vTDel, case2),
                         vTDiag,
                         case1);
-                __m128i condE = _mm_cmpgt_epi8(vE_opn, vE_ext);
-                __m128i condF = _mm_cmpgt_epi8(vF_opn, vF_ext);
+                __m128i condE = _mm_cmpgt_epi32(vE_opn, vE_ext);
+                __m128i condF = _mm_cmpgt_epi32(vF_opn, vF_ext);
                 __m128i vET = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, condE);
                 __m128i vFT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, condF);
                 vT = _mm_or_si128(vT, vET);
                 vT = _mm_or_si128(vT, vFT);
                 arr_store_si128(result->trace->trace_table, vT, i, s1Len, j, s2Len);
             }
-            H_pr[j-15] = (int8_t)_mm_extract_epi8_rpl(vWH,0);
-            F_pr[j-15] = (int8_t)_mm_extract_epi8_rpl(vF,0);
+            H_pr[j-3] = (int32_t)_mm_extract_epi32_rpl(vWH,0);
+            F_pr[j-3] = (int32_t)_mm_extract_epi32_rpl(vF,0);
             /* as minor diagonal vector passes across the i or j limit
              * boundary, extract the last value of the column or row */
             {
-                __m128i vJeqLimit1 = _mm_cmpeq_epi8(vJ, vJLimit1);
-                __m128i vJgtNegOne = _mm_cmpgt_epi8(vJ, vNegOne);
-                __m128i vJltLimit = _mm_cmplt_epi8(vJ, vJLimit);
+                __m128i vJeqLimit1 = _mm_cmpeq_epi32(vJ, vJLimit1);
+                __m128i vJgtNegOne = _mm_cmpgt_epi32(vJ, vNegOne);
+                __m128i vJltLimit = _mm_cmplt_epi32(vJ, vJLimit);
                 __m128i cond_j = _mm_and_si128(vIltLimit, vJeqLimit1);
                 __m128i cond_i = _mm_and_si128(vIeqLimit1,
                         _mm_and_si128(vJgtNegOne, vJltLimit));
-                __m128i cond_max_row = _mm_cmpgt_epi8(vWH, vMaxHRow);
-                __m128i cond_max_col = _mm_cmpgt_epi8(vWH, vMaxHCol);
+                __m128i cond_max_row = _mm_cmpgt_epi32(vWH, vMaxHRow);
+                __m128i cond_max_col = _mm_cmpgt_epi32(vWH, vMaxHCol);
                 __m128i cond_last_val = _mm_and_si128(vIeqLimit1, vJeqLimit1);
                 __m128i cond_all_row = _mm_and_si128(cond_max_row, cond_i);
                 __m128i cond_all_col = _mm_and_si128(cond_max_col, cond_j);
@@ -446,22 +374,22 @@ parasail_result_t* FNAME(
                 vEndI = _mm_blendv_epi8_rpl(vEndI, vI, cond_all_col);
                 vEndJ = _mm_blendv_epi8_rpl(vEndJ, vJ, cond_all_row);
             }
-            vJ = _mm_adds_epi8(vJ, vOne);
+            vJ = _mm_add_epi32(vJ, vOne);
         }
-        vI = _mm_adds_epi8(vI, vN);
-        vIBoundary = _mm_subs_epi8(vIBoundary, vGapN);
+        vI = _mm_add_epi32(vI, vN);
+        vIBoundary = _mm_sub_epi32(vIBoundary, vGapN);
     }
 
     /* alignment ending position */
     {
-        int8_t max_row = NEG_LIMIT;
-        int8_t max_col = NEG_LIMIT;
-        int8_t last_val = NEG_LIMIT;
-        int8_t *s = (int8_t*)&vMaxHRow;
-        int8_t *t = (int8_t*)&vMaxHCol;
-        int8_t *u = (int8_t*)&vLastVal;
-        int8_t *i = (int8_t*)&vEndI;
-        int8_t *j = (int8_t*)&vEndJ;
+        int32_t max_row = NEG_LIMIT;
+        int32_t max_col = NEG_LIMIT;
+        int32_t last_val = NEG_LIMIT;
+        int32_t *s = (int32_t*)&vMaxHRow;
+        int32_t *t = (int32_t*)&vMaxHCol;
+        int32_t *u = (int32_t*)&vLastVal;
+        int32_t *i = (int32_t*)&vEndI;
+        int32_t *j = (int32_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++s, ++t, ++u, ++i, ++j) {
             if (*t > max_col || (*t == max_col && *i < end_query)) {
@@ -502,8 +430,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
