@@ -17,34 +17,34 @@
 #include "parasail/internal_avx.h"
 
 #define SG_TRACE
-#define SG_SUFFIX _scan_avx2_256_16
-#define SG_SUFFIX_PROF _scan_profile_avx2_256_16
+#define SG_SUFFIX _scan_avx2_256_8
+#define SG_SUFFIX_PROF _scan_profile_avx2_256_8
 #include "sg_helper.h"
 
 
-#define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
-
-#if HAVE_AVX2_MM256_INSERT_EPI16
-#define _mm256_insert_epi16_rpl _mm256_insert_epi16
+#if HAVE_AVX2_MM256_INSERT_EPI8
+#define _mm256_insert_epi8_rpl _mm256_insert_epi8
 #else
-static inline __m256i _mm256_insert_epi16_rpl(__m256i a, int16_t i, int imm) {
-    __m256i_16_t A;
+static inline __m256i _mm256_insert_epi8_rpl(__m256i a, int8_t i, int imm) {
+    __m256i_8_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI16
-#define _mm256_extract_epi16_rpl _mm256_extract_epi16
+#if HAVE_AVX2_MM256_EXTRACT_EPI8
+#define _mm256_extract_epi8_rpl _mm256_extract_epi8
 #else
-static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
-    __m256i_16_t A;
+static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
+    __m256i_8_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
+#define _mm256_cmplt_epi8_rpl(a,b) _mm256_cmpgt_epi8(b,a)
+
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
 
@@ -67,8 +67,8 @@ static inline __m256i arr_load(
     return _mm256_load_si256(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_scan_avx2_256_16
-#define PNAME parasail_sg_flags_trace_scan_profile_avx2_256_16
+#define FNAME parasail_sg_flags_trace_scan_avx2_256_8
+#define PNAME parasail_sg_flags_trace_scan_profile_avx2_256_8
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -92,7 +92,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
 
@@ -121,16 +121,16 @@ parasail_result_t* PNAME(
     int32_t position = 0;
     __m256i* restrict pvP = NULL;
     __m256i* restrict pvE = NULL;
-    int16_t* restrict boundary = NULL;
+    int8_t* restrict boundary = NULL;
     __m256i* restrict pvHt = NULL;
     __m256i* restrict pvH = NULL;
     __m256i* restrict pvGapper = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
+    int8_t NEG_LIMIT = 0;
+    int8_t POS_LIMIT = 0;
     __m256i vZero;
-    int16_t score = 0;
+    int8_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -150,7 +150,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -166,35 +166,35 @@ parasail_result_t* PNAME(
     end_query = s1Len-1;
     end_ref = s2Len-1;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 32; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
-    pvP = (__m256i*)profile->profile16.score;
-    vGapO = _mm256_set1_epi16(open);
-    vGapE = _mm256_set1_epi16(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    pvP = (__m256i*)profile->profile8.score;
+    vGapO = _mm256_set1_epi8(open);
+    vGapE = _mm256_set1_epi8(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    POS_LIMIT = INT8_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi8(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi8(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
-    vPosMask = _mm256_cmpeq_epi16(_mm256_set1_epi16(position),
-            _mm256_set_epi16(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
+    vPosMask = _mm256_cmpeq_epi8(_mm256_set1_epi8(position),
+            _mm256_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31));
     vNegInfFront = vZero;
-    vNegInfFront = _mm256_insert_epi16_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm256_adds_epi16(vNegInfFront,
-            _mm256_slli_si256_rpl(_mm256_set1_epi16(-segLen*gap), 2));
-    vTIns  = _mm256_set1_epi16(PARASAIL_INS);
-    vTDel  = _mm256_set1_epi16(PARASAIL_DEL);
-    vTDiag = _mm256_set1_epi16(PARASAIL_DIAG);
-    vTDiagE= _mm256_set1_epi16(PARASAIL_DIAG_E);
-    vTInsE = _mm256_set1_epi16(PARASAIL_INS_E);
-    vTDiagF= _mm256_set1_epi16(PARASAIL_DIAG_F);
-    vTDelF = _mm256_set1_epi16(PARASAIL_DEL_F);
+    vNegInfFront = _mm256_insert_epi8_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm256_adds_epi8(vNegInfFront,
+            _mm256_slli_si256_rpl(_mm256_set1_epi8(-segLen*gap), 1));
+    vTIns  = _mm256_set1_epi8(PARASAIL_INS);
+    vTDel  = _mm256_set1_epi8(PARASAIL_DEL);
+    vTDiag = _mm256_set1_epi8(PARASAIL_DIAG);
+    vTDiagE= _mm256_set1_epi8(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi8(PARASAIL_INS_E);
+    vTDiagF= _mm256_set1_epi8(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi8(PARASAIL_DEL_F);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 32, sizeof(__m256i));
@@ -203,7 +203,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
@@ -211,7 +211,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvE = parasail_memalign___m256i(32, segLen);
-    boundary = parasail_memalign_int16_t(32, s2Len+1);
+    boundary = parasail_memalign_int8_t(32, s2Len+1);
     pvHt= parasail_memalign___m256i(32, segLen);
     pvH = parasail_memalign___m256i(32, segLen);
     pvGapper = parasail_memalign___m256i(32, segLen);
@@ -228,13 +228,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m256i_16_t h;
-            __m256i_16_t e;
+            __m256i_8_t h;
+            __m256i_8_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm256_store_si256(&pvH[index], h.m);
             _mm256_store_si256(&pvE[index], e.m);
@@ -247,17 +247,17 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
     {
-        __m256i vGapper = _mm256_subs_epi16(vZero,vGapO);
+        __m256i vGapper = _mm256_subs_epi8(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm256_store_si256(pvGapper+i, vGapper);
-            vGapper = _mm256_subs_epi16(vGapper, vGapE);
+            vGapper = _mm256_subs_epi8(vGapper, vGapE);
             /* long queries and/or large penalties will break the pseudo prefix scan */
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vGapper);
+            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vGapper);
         }
     }
 
@@ -285,27 +285,27 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm256_load_si256(pvH+(segLen-1));
-        vHp = _mm256_slli_si256_rpl(vHp, 2);
-        vHp = _mm256_insert_epi16_rpl(vHp, boundary[j], 0);
+        vHp = _mm256_slli_si256_rpl(vHp, 1);
+        vHp = _mm256_insert_epi8_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm256_subs_epi16(vNegLimit, pvGapper[0]);
+        vHt = _mm256_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm256_load_si256(pvH+i);
             vE = _mm256_load_si256(pvE+i);
             vW = _mm256_load_si256(pvW+i);
             vGapper = _mm256_load_si256(pvGapper+i);
-            vE_opn = _mm256_subs_epi16(vH, vGapO);
-            vE_ext = _mm256_subs_epi16(vE, vGapE);
-            case1 = _mm256_cmpgt_epi16(vE_opn, vE_ext);
+            vE_opn = _mm256_subs_epi8(vH, vGapO);
+            vE_ext = _mm256_subs_epi8(vE, vGapE);
+            case1 = _mm256_cmpgt_epi8(vE_opn, vE_ext);
             vET = _mm256_blendv_epi8(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = _mm256_max_epi16(vE_opn, vE_ext);
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vE);
-            vGapper = _mm256_adds_epi16(vHt, vGapper);
-            vF = _mm256_max_epi16(vF, vGapper);
-            vHp = _mm256_adds_epi16(vHp, vW);
-            vHt = _mm256_max_epi16(vE, vHp);
+            vE = _mm256_max_epi8(vE_opn, vE_ext);
+            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vE);
+            vGapper = _mm256_adds_epi8(vHt, vGapper);
+            vF = _mm256_max_epi8(vF, vGapper);
+            vHp = _mm256_adds_epi8(vHp, vW);
+            vHt = _mm256_max_epi8(vE, vHp);
             _mm256_store_si256(pvE+i, vE);
             _mm256_store_si256(pvHt+i, vHt);
             _mm256_store_si256(pvH+i, vHp);
@@ -313,33 +313,33 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm256_slli_si256_rpl(vHt, 2);
-        vHt = _mm256_insert_epi16_rpl(vHt, boundary[j+1], 0);
+        vHt = _mm256_slli_si256_rpl(vHt, 1);
+        vHt = _mm256_insert_epi8_rpl(vHt, boundary[j+1], 0);
         vGapper = _mm256_load_si256(pvGapper);
-        vGapper = _mm256_adds_epi16(vHt, vGapper);
-        vF = _mm256_max_epi16(vF, vGapper);
+        vGapper = _mm256_adds_epi8(vHt, vGapper);
+        vF = _mm256_max_epi8(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            __m256i vFt = _mm256_slli_si256_rpl(vF, 2);
-            vFt = _mm256_adds_epi16(vFt, vSegLenXgap);
-            vF = _mm256_max_epi16(vF, vFt);
+            __m256i vFt = _mm256_slli_si256_rpl(vF, 1);
+            vFt = _mm256_adds_epi8(vFt, vSegLenXgap);
+            vF = _mm256_max_epi8(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm256_slli_si256_rpl(vF, 2);
-        vF = _mm256_adds_epi16(vF, vNegInfFront);
-        vH = _mm256_max_epi16(vF, vHt);
+        vF = _mm256_slli_si256_rpl(vF, 1);
+        vF = _mm256_adds_epi8(vF, vNegInfFront);
+        vH = _mm256_max_epi8(vF, vHt);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm256_load_si256(pvH+i);
             vHt = _mm256_load_si256(pvHt+i);
-            vF_opn = _mm256_subs_epi16(vH, vGapO);
-            vF_ext = _mm256_subs_epi16(vF, vGapE);
-            vF = _mm256_max_epi16(vF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi16(vF_opn, vF_ext);
+            vF_opn = _mm256_subs_epi8(vH, vGapO);
+            vF_ext = _mm256_subs_epi8(vF, vGapE);
+            vF = _mm256_max_epi8(vF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi8(vF_opn, vF_ext);
             vFT = _mm256_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = _mm256_max_epi16(vHt, vF);
-            case1 = _mm256_cmpeq_epi16(vH, vHp);
-            case2 = _mm256_cmpeq_epi16(vH, vF);
+            vH = _mm256_max_epi8(vHt, vF);
+            case1 = _mm256_cmpeq_epi8(vH, vHp);
+            case2 = _mm256_cmpeq_epi8(vH, vF);
             vT = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vTIns, vTDel, case2),
                     vTDiag, case1);
@@ -347,17 +347,17 @@ parasail_result_t* PNAME(
             vT = _mm256_or_si256(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             _mm256_store_si256(pvH+i, vH);
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vF);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vF);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
         }
 
         /* extract vector containing last value from column */
         {
             __m256i vCompare;
             vH = _mm256_load_si256(pvH + offset);
-            vCompare = _mm256_and_si256(vPosMask, _mm256_cmpgt_epi16(vH, vMaxH));
-            vMaxH = _mm256_max_epi16(vH, vMaxH);
+            vCompare = _mm256_and_si256(vPosMask, _mm256_cmpgt_epi8(vH, vMaxH));
+            vMaxH = _mm256_max_epi8(vH, vMaxH);
             if (_mm256_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -368,9 +368,9 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm256_slli_si256_rpl(vMaxH, 2);
+            vMaxH = _mm256_slli_si256_rpl(vMaxH, 1);
         }
-        score = (int16_t) _mm256_extract_epi16_rpl(vMaxH, 15);
+        score = (int8_t) _mm256_extract_epi8_rpl(vMaxH, 31);
         end_query = s1Len-1;
     }
 
@@ -378,7 +378,7 @@ parasail_result_t* PNAME(
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvH;
+        int8_t *t = (int8_t*)pvH;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -399,17 +399,17 @@ parasail_result_t* PNAME(
         {
             __m256i vH = _mm256_load_si256(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 2);
+                vH = _mm256_slli_si256_rpl(vH, 1);
             }
-            score = (int16_t) _mm256_extract_epi16_rpl (vH, 15);
+            score = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi16_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi8_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
