@@ -27,16 +27,28 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int16_t)_mm_extract_epi16(vWH, 7);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm_extract_epi32(vWH, 2);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int16_t)_mm_extract_epi16(vWH, 6);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm_extract_epi32(vWH, 1);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int16_t)_mm_extract_epi16(vWH, 5);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int16_t)_mm_extract_epi16(vWH, 4);
+    }
+    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
+        array[1LL*(i+4)*s2Len + (j-4)] = (int16_t)_mm_extract_epi16(vWH, 3);
+    }
+    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
+        array[1LL*(i+5)*s2Len + (j-5)] = (int16_t)_mm_extract_epi16(vWH, 2);
+    }
+    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
+        array[1LL*(i+6)*s2Len + (j-6)] = (int16_t)_mm_extract_epi16(vWH, 1);
+    }
+    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
+        array[1LL*(i+7)*s2Len + (j-7)] = (int16_t)_mm_extract_epi16(vWH, 0);
     }
 }
 #endif
@@ -52,39 +64,63 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int32_t)_mm_extract_epi32(vWH, 3);
+        row[j-0] = (int16_t)_mm_extract_epi16(vWH, 7);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        col[(i+0)] = (int16_t)_mm_extract_epi16(vWH, 7);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int32_t)_mm_extract_epi32(vWH, 2);
+        row[j-1] = (int16_t)_mm_extract_epi16(vWH, 6);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int32_t)_mm_extract_epi32(vWH, 2);
+        col[(i+1)] = (int16_t)_mm_extract_epi16(vWH, 6);
     }
     if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int32_t)_mm_extract_epi32(vWH, 1);
+        row[j-2] = (int16_t)_mm_extract_epi16(vWH, 5);
     }
     if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int32_t)_mm_extract_epi32(vWH, 1);
+        col[(i+2)] = (int16_t)_mm_extract_epi16(vWH, 5);
     }
     if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int32_t)_mm_extract_epi32(vWH, 0);
+        row[j-3] = (int16_t)_mm_extract_epi16(vWH, 4);
     }
     if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        col[(i+3)] = (int16_t)_mm_extract_epi16(vWH, 4);
+    }
+    if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
+        row[j-4] = (int16_t)_mm_extract_epi16(vWH, 3);
+    }
+    if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
+        col[(i+4)] = (int16_t)_mm_extract_epi16(vWH, 3);
+    }
+    if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
+        row[j-5] = (int16_t)_mm_extract_epi16(vWH, 2);
+    }
+    if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
+        col[(i+5)] = (int16_t)_mm_extract_epi16(vWH, 2);
+    }
+    if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
+        row[j-6] = (int16_t)_mm_extract_epi16(vWH, 1);
+    }
+    if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
+        col[(i+6)] = (int16_t)_mm_extract_epi16(vWH, 1);
+    }
+    if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
+        row[j-7] = (int16_t)_mm_extract_epi16(vWH, 0);
+    }
+    if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
+        col[(i+7)] = (int16_t)_mm_extract_epi16(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_diag_altivec_128_32
+#define FNAME parasail_sw_table_diag_altivec_128_16
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_diag_altivec_128_32
+#define FNAME parasail_sw_rowcol_diag_altivec_128_16
 #else
-#define FNAME parasail_sw_diag_altivec_128_32
+#define FNAME parasail_sw_diag_altivec_128_16
 #endif
 #endif
 
@@ -100,21 +136,21 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int32_t * restrict s1 = NULL;
-    int32_t * restrict s2B = NULL;
-    int32_t * restrict _H_pr = NULL;
-    int32_t * restrict _F_pr = NULL;
-    int32_t * restrict s2 = NULL;
-    int32_t * restrict H_pr = NULL;
-    int32_t * restrict F_pr = NULL;
+    int16_t * restrict s1 = NULL;
+    int16_t * restrict s2B = NULL;
+    int16_t * restrict _H_pr = NULL;
+    int16_t * restrict _F_pr = NULL;
+    int16_t * restrict s2 = NULL;
+    int16_t * restrict H_pr = NULL;
+    int16_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -147,7 +183,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 4; /* number of values in vector */
+    N = 8; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -157,28 +193,28 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = 0;
     end_ref = 0;
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi32(NEG_LIMIT);
-    vNegInf0 = _mm_srli_si128(vNegInf, 4); /* shift in a 0 */
-    vOpen = _mm_set1_epi32(open);
-    vGap  = _mm_set1_epi32(gap);
-    vZero = _mm_set1_epi32(0);
-    vOne = _mm_set1_epi32(1);
-    vN = _mm_set1_epi32(N);
-    vNegOne = _mm_set1_epi32(-1);
-    vI = _mm_set_epi32(0,1,2,3);
-    vJreset = _mm_set_epi32(0,-1,-2,-3);
+    vNegInf = _mm_set1_epi16(NEG_LIMIT);
+    vNegInf0 = _mm_srli_si128(vNegInf, 2); /* shift in a 0 */
+    vOpen = _mm_set1_epi16(open);
+    vGap  = _mm_set1_epi16(gap);
+    vZero = _mm_set1_epi16(0);
+    vOne = _mm_set1_epi16(1);
+    vN = _mm_set1_epi16(N);
+    vNegOne = _mm_set1_epi16(-1);
+    vI = _mm_set_epi16(0,1,2,3,4,5,6,7);
+    vJreset = _mm_set_epi16(0,-1,-2,-3,-4,-5,-6,-7);
     vMaxH = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm_set1_epi32(s1Len);
-    vJLimit = _mm_set1_epi32(s2Len);
+    vILimit = _mm_set1_epi16(s1Len);
+    vJLimit = _mm_set1_epi16(s2Len);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -194,7 +230,7 @@ parasail_result_t* FNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -203,9 +239,9 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int32_t(16, s2Len+PAD2);
-    _H_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    _F_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    s2B= parasail_memalign_int16_t(16, s2Len+PAD2);
+    _H_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    _F_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -217,7 +253,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int32_t(16, s1Len+PAD);
+        s1 = parasail_memalign_int16_t(16, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -268,43 +304,51 @@ parasail_result_t* FNAME(
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
         const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
         const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        vec128i vIltLimit = _mm_cmplt_epi32(vI, vILimit);
+        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
+        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
+        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
+        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
+        vec128i vIltLimit = _mm_cmplt_epi16(vI, vILimit);
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             vec128i vMat;
             vec128i vNWH = vNH;
-            vNH = _mm_srli_si128(vWH, 4);
-            vNH = _mm_insert_epi32(vNH, H_pr[j], 3);
-            vF = _mm_srli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, F_pr[j], 3);
-            vF = _mm_max_epi32(
-                    _mm_sub_epi32(vNH, vOpen),
-                    _mm_sub_epi32(vF, vGap));
-            vE = _mm_max_epi32(
-                    _mm_sub_epi32(vWH, vOpen),
-                    _mm_sub_epi32(vE, vGap));
-            vMat = _mm_set_epi32(
+            vNH = _mm_srli_si128(vWH, 2);
+            vNH = _mm_insert_epi16(vNH, H_pr[j], 7);
+            vF = _mm_srli_si128(vF, 2);
+            vF = _mm_insert_epi16(vF, F_pr[j], 7);
+            vF = _mm_max_epi16(
+                    _mm_subs_epi16(vNH, vOpen),
+                    _mm_subs_epi16(vF, vGap));
+            vE = _mm_max_epi16(
+                    _mm_subs_epi16(vWH, vOpen),
+                    _mm_subs_epi16(vE, vGap));
+            vMat = _mm_set_epi16(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
-                    matrow3[s2[j-3]]
+                    matrow3[s2[j-3]],
+                    matrow4[s2[j-4]],
+                    matrow5[s2[j-5]],
+                    matrow6[s2[j-6]],
+                    matrow7[s2[j-7]]
                     );
-            vNWH = _mm_add_epi32(vNWH, vMat);
-            vWH = _mm_max_epi32(vNWH, vE);
-            vWH = _mm_max_epi32(vWH, vF);
-            vWH = _mm_max_epi32(vWH, vZero);
+            vNWH = _mm_adds_epi16(vNWH, vMat);
+            vWH = _mm_max_epi16(vNWH, vE);
+            vWH = _mm_max_epi16(vWH, vF);
+            vWH = _mm_max_epi16(vWH, vZero);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                vec128i cond = _mm_cmpeq_epi32(vJ,vNegOne);
+                vec128i cond = _mm_cmpeq_epi16(vJ,vNegOne);
                 vWH = _mm_andnot_si128(cond, vWH);
                 vF = _mm_blendv_epi8(vF, vNegInf, cond);
                 vE = _mm_blendv_epi8(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWH);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -312,19 +356,19 @@ parasail_result_t* FNAME(
 #ifdef PARASAIL_ROWCOL
             arr_store_rowcol(result->rowcols->score_row, result->rowcols->score_col, vWH, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-3] = (int32_t)_mm_extract_epi32(vWH,0);
-            F_pr[j-3] = (int32_t)_mm_extract_epi32(vF,0);
+            H_pr[j-7] = (int16_t)_mm_extract_epi16(vWH,0);
+            F_pr[j-7] = (int16_t)_mm_extract_epi16(vF,0);
             /* as minor diagonal vector passes across table, extract
              * max values within the i,j bounds */
             {
                 vec128i cond_valid_J = _mm_and_si128(
-                        _mm_cmpgt_epi32(vJ, vNegOne),
-                        _mm_cmplt_epi32(vJ, vJLimit));
+                        _mm_cmpgt_epi16(vJ, vNegOne),
+                        _mm_cmplt_epi16(vJ, vJLimit));
                 vec128i cond_valid_IJ = _mm_and_si128(cond_valid_J, vIltLimit);
-                vec128i cond_eq = _mm_cmpeq_epi32(vWH, vMaxH);
-                vec128i cond_max = _mm_cmpgt_epi32(vWH, vMaxH);
+                vec128i cond_eq = _mm_cmpeq_epi16(vWH, vMaxH);
+                vec128i cond_max = _mm_cmpgt_epi16(vWH, vMaxH);
                 vec128i cond_all = _mm_and_si128(cond_max, cond_valid_IJ);
-                vec128i cond_Jlt = _mm_cmplt_epi32(vJ, vEndJ);
+                vec128i cond_Jlt = _mm_cmplt_epi16(vJ, vEndJ);
                 vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
                 vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
@@ -333,16 +377,16 @@ parasail_result_t* FNAME(
                 vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
             }
-            vJ = _mm_add_epi32(vJ, vOne);
+            vJ = _mm_adds_epi16(vJ, vOne);
         }
-        vI = _mm_add_epi32(vI, vN);
+        vI = _mm_adds_epi16(vI, vN);
     }
 
     /* alignment ending position */
     {
-        int32_t *t = (int32_t*)&vMaxH;
-        int32_t *i = (int32_t*)&vEndI;
-        int32_t *j = (int32_t*)&vEndJ;
+        int16_t *t = (int16_t*)&vMaxH;
+        int16_t *i = (int16_t*)&vEndI;
+        int16_t *j = (int16_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++t, ++i, ++j) {
             if (*t > score) {
@@ -364,8 +408,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
