@@ -16,28 +16,78 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI32
-#define _mm256_insert_epi32_rpl _mm256_insert_epi32
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI32
-#define _mm256_extract_epi32_rpl _mm256_extract_epi32
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
 #else
-static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#if HAVE_AVX2_MM256_SET_EPI64X
+#define _mm256_set_epi64x_rpl _mm256_set_epi64x
+#else
+static inline __m256i _mm256_set_epi64x_rpl(int64_t e3, int64_t e2, int64_t e1, int64_t e0) {
+    __m256i_64_t A;
+    A.v[0] = e0;
+    A.v[1] = e1;
+    A.v[2] = e2;
+    A.v[3] = e3;
+    return A.m;
+}
+#endif
+
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
+#else
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
 
 #define _mm256_srli_si256_rpl(a,imm) _mm256_or_si256(_mm256_slli_si256(_mm256_permute2x128_si256(a, a, _MM_SHUFFLE(3,0,0,1)), 16-imm), _mm256_srli_si256(a, imm))
 
@@ -52,28 +102,16 @@ static inline void arr_store_si256(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 7);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 3);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 6);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 2);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 5);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 1);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 4);
-    }
-    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 3);
-    }
-    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 2);
-    }
-    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 1);
-    }
-    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 0);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 0);
     }
 }
 #endif
@@ -89,63 +127,39 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int32_t)_mm256_extract_epi32_rpl(vWH, 7);
+        row[j-0] = (int64_t)_mm256_extract_epi64_rpl(vWH, 3);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 7);
+        col[(i+0)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 3);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int32_t)_mm256_extract_epi32_rpl(vWH, 6);
+        row[j-1] = (int64_t)_mm256_extract_epi64_rpl(vWH, 2);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 6);
+        col[(i+1)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 2);
     }
     if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int32_t)_mm256_extract_epi32_rpl(vWH, 5);
+        row[j-2] = (int64_t)_mm256_extract_epi64_rpl(vWH, 1);
     }
     if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 5);
+        col[(i+2)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 1);
     }
     if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int32_t)_mm256_extract_epi32_rpl(vWH, 4);
+        row[j-3] = (int64_t)_mm256_extract_epi64_rpl(vWH, 0);
     }
     if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 4);
-    }
-    if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
-        row[j-4] = (int32_t)_mm256_extract_epi32_rpl(vWH, 3);
-    }
-    if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
-        col[(i+4)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 3);
-    }
-    if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
-        row[j-5] = (int32_t)_mm256_extract_epi32_rpl(vWH, 2);
-    }
-    if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
-        col[(i+5)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 2);
-    }
-    if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
-        row[j-6] = (int32_t)_mm256_extract_epi32_rpl(vWH, 1);
-    }
-    if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
-        col[(i+6)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 1);
-    }
-    if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
-        row[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWH, 0);
-    }
-    if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
-        col[(i+7)] = (int32_t)_mm256_extract_epi32_rpl(vWH, 0);
+        col[(i+3)] = (int64_t)_mm256_extract_epi64_rpl(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_diag_avx2_256_32
+#define FNAME parasail_sw_table_diag_avx2_256_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_diag_avx2_256_32
+#define FNAME parasail_sw_rowcol_diag_avx2_256_64
 #else
-#define FNAME parasail_sw_diag_avx2_256_32
+#define FNAME parasail_sw_diag_avx2_256_64
 #endif
 #endif
 
@@ -161,21 +175,21 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int32_t * restrict s1 = NULL;
-    int32_t * restrict s2B = NULL;
-    int32_t * restrict _H_pr = NULL;
-    int32_t * restrict _F_pr = NULL;
-    int32_t * restrict s2 = NULL;
-    int32_t * restrict H_pr = NULL;
-    int32_t * restrict F_pr = NULL;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict F_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
-    int32_t score = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -208,7 +222,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 8; /* number of values in vector */
+    N = 4; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -218,28 +232,28 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = 0;
     end_ref = 0;
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm256_set1_epi32(NEG_LIMIT);
-    vNegInf0 = _mm256_srli_si256_rpl(vNegInf, 4); /* shift in a 0 */
-    vOpen = _mm256_set1_epi32(open);
-    vGap  = _mm256_set1_epi32(gap);
-    vZero = _mm256_set1_epi32(0);
-    vOne = _mm256_set1_epi32(1);
-    vN = _mm256_set1_epi32(N);
-    vNegOne = _mm256_set1_epi32(-1);
-    vI = _mm256_set_epi32(0,1,2,3,4,5,6,7);
-    vJreset = _mm256_set_epi32(0,-1,-2,-3,-4,-5,-6,-7);
+    vNegInf = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vNegInf0 = _mm256_srli_si256_rpl(vNegInf, 8); /* shift in a 0 */
+    vOpen = _mm256_set1_epi64x_rpl(open);
+    vGap  = _mm256_set1_epi64x_rpl(gap);
+    vZero = _mm256_set1_epi64x_rpl(0);
+    vOne = _mm256_set1_epi64x_rpl(1);
+    vN = _mm256_set1_epi64x_rpl(N);
+    vNegOne = _mm256_set1_epi64x_rpl(-1);
+    vI = _mm256_set_epi64x_rpl(0,1,2,3);
+    vJreset = _mm256_set_epi64x_rpl(0,-1,-2,-3);
     vMaxH = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm256_set1_epi32(s1Len);
-    vJLimit = _mm256_set1_epi32(s2Len);
+    vILimit = _mm256_set1_epi64x_rpl(s1Len);
+    vJLimit = _mm256_set1_epi64x_rpl(s2Len);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -255,7 +269,7 @@ parasail_result_t* FNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -264,9 +278,9 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s2B= parasail_memalign_int32_t(32, s2Len+PAD2);
-    _H_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
-    _F_pr = parasail_memalign_int32_t(32, s2Len+PAD2);
+    s2B= parasail_memalign_int64_t(32, s2Len+PAD2);
+    _H_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _F_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     F_pr = _F_pr+PAD;
@@ -278,7 +292,7 @@ parasail_result_t* FNAME(
 
     /* convert _s1 from char to int in range 0-23 */
     if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
-        s1 = parasail_memalign_int32_t(32, s1Len+PAD);
+        s1 = parasail_memalign_int64_t(32, s1Len+PAD);
         if (!s1) return NULL;
         for (i=0; i<s1Len; ++i) {
             s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -329,51 +343,43 @@ parasail_result_t* FNAME(
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
         const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
         const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
-        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
-        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
-        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        __m256i vIltLimit = _mm256_cmplt_epi32_rpl(vI, vILimit);
+        __m256i vIltLimit = _mm256_cmplt_epi64_rpl(vI, vILimit);
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             __m256i vMat;
             __m256i vNWH = vNH;
-            vNH = _mm256_srli_si256_rpl(vWH, 4);
-            vNH = _mm256_insert_epi32_rpl(vNH, H_pr[j], 7);
-            vF = _mm256_srli_si256_rpl(vF, 4);
-            vF = _mm256_insert_epi32_rpl(vF, F_pr[j], 7);
-            vF = _mm256_max_epi32(
-                    _mm256_sub_epi32(vNH, vOpen),
-                    _mm256_sub_epi32(vF, vGap));
-            vE = _mm256_max_epi32(
-                    _mm256_sub_epi32(vWH, vOpen),
-                    _mm256_sub_epi32(vE, vGap));
-            vMat = _mm256_set_epi32(
+            vNH = _mm256_srli_si256_rpl(vWH, 8);
+            vNH = _mm256_insert_epi64_rpl(vNH, H_pr[j], 3);
+            vF = _mm256_srli_si256_rpl(vF, 8);
+            vF = _mm256_insert_epi64_rpl(vF, F_pr[j], 3);
+            vF = _mm256_max_epi64_rpl(
+                    _mm256_sub_epi64(vNH, vOpen),
+                    _mm256_sub_epi64(vF, vGap));
+            vE = _mm256_max_epi64_rpl(
+                    _mm256_sub_epi64(vWH, vOpen),
+                    _mm256_sub_epi64(vE, vGap));
+            vMat = _mm256_set_epi64x_rpl(
                     matrow0[s2[j-0]],
                     matrow1[s2[j-1]],
                     matrow2[s2[j-2]],
-                    matrow3[s2[j-3]],
-                    matrow4[s2[j-4]],
-                    matrow5[s2[j-5]],
-                    matrow6[s2[j-6]],
-                    matrow7[s2[j-7]]
+                    matrow3[s2[j-3]]
                     );
-            vNWH = _mm256_add_epi32(vNWH, vMat);
-            vWH = _mm256_max_epi32(vNWH, vE);
-            vWH = _mm256_max_epi32(vWH, vF);
-            vWH = _mm256_max_epi32(vWH, vZero);
+            vNWH = _mm256_add_epi64(vNWH, vMat);
+            vWH = _mm256_max_epi64_rpl(vNWH, vE);
+            vWH = _mm256_max_epi64_rpl(vWH, vF);
+            vWH = _mm256_max_epi64_rpl(vWH, vZero);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m256i cond = _mm256_cmpeq_epi32(vJ,vNegOne);
+                __m256i cond = _mm256_cmpeq_epi64(vJ,vNegOne);
                 vWH = _mm256_andnot_si256(cond, vWH);
                 vF = _mm256_blendv_epi8(vF, vNegInf, cond);
                 vE = _mm256_blendv_epi8(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWH);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -381,19 +387,19 @@ parasail_result_t* FNAME(
 #ifdef PARASAIL_ROWCOL
             arr_store_rowcol(result->rowcols->score_row, result->rowcols->score_col, vWH, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vWH,0);
-            F_pr[j-7] = (int32_t)_mm256_extract_epi32_rpl(vF,0);
+            H_pr[j-3] = (int64_t)_mm256_extract_epi64_rpl(vWH,0);
+            F_pr[j-3] = (int64_t)_mm256_extract_epi64_rpl(vF,0);
             /* as minor diagonal vector passes across table, extract
              * max values within the i,j bounds */
             {
                 __m256i cond_valid_J = _mm256_and_si256(
-                        _mm256_cmpgt_epi32(vJ, vNegOne),
-                        _mm256_cmplt_epi32_rpl(vJ, vJLimit));
+                        _mm256_cmpgt_epi64(vJ, vNegOne),
+                        _mm256_cmplt_epi64_rpl(vJ, vJLimit));
                 __m256i cond_valid_IJ = _mm256_and_si256(cond_valid_J, vIltLimit);
-                __m256i cond_eq = _mm256_cmpeq_epi32(vWH, vMaxH);
-                __m256i cond_max = _mm256_cmpgt_epi32(vWH, vMaxH);
+                __m256i cond_eq = _mm256_cmpeq_epi64(vWH, vMaxH);
+                __m256i cond_max = _mm256_cmpgt_epi64(vWH, vMaxH);
                 __m256i cond_all = _mm256_and_si256(cond_max, cond_valid_IJ);
-                __m256i cond_Jlt = _mm256_cmplt_epi32_rpl(vJ, vEndJ);
+                __m256i cond_Jlt = _mm256_cmplt_epi64_rpl(vJ, vEndJ);
                 vMaxH = _mm256_blendv_epi8(vMaxH, vWH, cond_all);
                 vEndI = _mm256_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm256_blendv_epi8(vEndJ, vJ, cond_all);
@@ -402,16 +408,16 @@ parasail_result_t* FNAME(
                 vEndI = _mm256_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm256_blendv_epi8(vEndJ, vJ, cond_all);
             }
-            vJ = _mm256_add_epi32(vJ, vOne);
+            vJ = _mm256_add_epi64(vJ, vOne);
         }
-        vI = _mm256_add_epi32(vI, vN);
+        vI = _mm256_add_epi64(vI, vN);
     }
 
     /* alignment ending position */
     {
-        int32_t *t = (int32_t*)&vMaxH;
-        int32_t *i = (int32_t*)&vEndI;
-        int32_t *j = (int32_t*)&vEndJ;
+        int64_t *t = (int64_t*)&vMaxH;
+        int64_t *i = (int64_t*)&vEndI;
+        int64_t *j = (int64_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++t, ++i, ++j) {
             if (*t > score) {
@@ -433,8 +439,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
