@@ -13,7 +13,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -21,6 +20,39 @@
 #include "parasail/internal_sse.h"
 
 
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
+static inline __m128i _mm_insert_epi32_rpl(__m128i a, int32_t i, const int imm) {
+    __m128i_32_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline int32_t _mm_extract_epi32_rpl(__m128i a, const int imm) {
+    __m128i_32_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_min_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(b, a);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -32,16 +64,16 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)_mm_extract_epi32_rpl(vWH, 3);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm_extract_epi32(vWH, 2);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)_mm_extract_epi32_rpl(vWH, 2);
     }
     if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm_extract_epi32(vWH, 1);
+        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)_mm_extract_epi32_rpl(vWH, 1);
     }
     if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)_mm_extract_epi32_rpl(vWH, 0);
     }
 }
 #endif
@@ -57,39 +89,39 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int32_t)_mm_extract_epi32(vWH, 3);
+        row[j-0] = (int32_t)_mm_extract_epi32_rpl(vWH, 3);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int32_t)_mm_extract_epi32(vWH, 3);
+        col[(i+0)] = (int32_t)_mm_extract_epi32_rpl(vWH, 3);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int32_t)_mm_extract_epi32(vWH, 2);
+        row[j-1] = (int32_t)_mm_extract_epi32_rpl(vWH, 2);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int32_t)_mm_extract_epi32(vWH, 2);
+        col[(i+1)] = (int32_t)_mm_extract_epi32_rpl(vWH, 2);
     }
     if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int32_t)_mm_extract_epi32(vWH, 1);
+        row[j-2] = (int32_t)_mm_extract_epi32_rpl(vWH, 1);
     }
     if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int32_t)_mm_extract_epi32(vWH, 1);
+        col[(i+2)] = (int32_t)_mm_extract_epi32_rpl(vWH, 1);
     }
     if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int32_t)_mm_extract_epi32(vWH, 0);
+        row[j-3] = (int32_t)_mm_extract_epi32_rpl(vWH, 0);
     }
     if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int32_t)_mm_extract_epi32(vWH, 0);
+        col[(i+3)] = (int32_t)_mm_extract_epi32_rpl(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_diag_sse41_128_32
+#define FNAME parasail_sw_table_diag_sse2_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_diag_sse41_128_32
+#define FNAME parasail_sw_rowcol_diag_sse2_128_32
 #else
-#define FNAME parasail_sw_diag_sse41_128_32
+#define FNAME parasail_sw_diag_sse2_128_32
 #endif
 #endif
 
@@ -279,13 +311,13 @@ parasail_result_t* FNAME(
             __m128i vMat;
             __m128i vNWH = vNH;
             vNH = _mm_srli_si128(vWH, 4);
-            vNH = _mm_insert_epi32(vNH, H_pr[j], 3);
+            vNH = _mm_insert_epi32_rpl(vNH, H_pr[j], 3);
             vF = _mm_srli_si128(vF, 4);
-            vF = _mm_insert_epi32(vF, F_pr[j], 3);
-            vF = _mm_max_epi32(
+            vF = _mm_insert_epi32_rpl(vF, F_pr[j], 3);
+            vF = _mm_max_epi32_rpl(
                     _mm_sub_epi32(vNH, vOpen),
                     _mm_sub_epi32(vF, vGap));
-            vE = _mm_max_epi32(
+            vE = _mm_max_epi32_rpl(
                     _mm_sub_epi32(vWH, vOpen),
                     _mm_sub_epi32(vE, vGap));
             vMat = _mm_set_epi32(
@@ -295,21 +327,21 @@ parasail_result_t* FNAME(
                     matrow3[s2[j-3]]
                     );
             vNWH = _mm_add_epi32(vNWH, vMat);
-            vWH = _mm_max_epi32(vNWH, vE);
-            vWH = _mm_max_epi32(vWH, vF);
-            vWH = _mm_max_epi32(vWH, vZero);
+            vWH = _mm_max_epi32_rpl(vNWH, vE);
+            vWH = _mm_max_epi32_rpl(vWH, vF);
+            vWH = _mm_max_epi32_rpl(vWH, vZero);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
                 __m128i cond = _mm_cmpeq_epi32(vJ,vNegOne);
                 vWH = _mm_andnot_si128(cond, vWH);
-                vF = _mm_blendv_epi8(vF, vNegInf, cond);
-                vE = _mm_blendv_epi8(vE, vNegInf, cond);
+                vF = _mm_blendv_epi8_rpl(vF, vNegInf, cond);
+                vE = _mm_blendv_epi8_rpl(vE, vNegInf, cond);
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMin = _mm_min_epi32_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vWH);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -317,8 +349,8 @@ parasail_result_t* FNAME(
 #ifdef PARASAIL_ROWCOL
             arr_store_rowcol(result->rowcols->score_row, result->rowcols->score_col, vWH, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-3] = (int32_t)_mm_extract_epi32(vWH,0);
-            F_pr[j-3] = (int32_t)_mm_extract_epi32(vF,0);
+            H_pr[j-3] = (int32_t)_mm_extract_epi32_rpl(vWH,0);
+            F_pr[j-3] = (int32_t)_mm_extract_epi32_rpl(vF,0);
             /* as minor diagonal vector passes across table, extract
              * max values within the i,j bounds */
             {
@@ -330,13 +362,13 @@ parasail_result_t* FNAME(
                 __m128i cond_max = _mm_cmpgt_epi32(vWH, vMaxH);
                 __m128i cond_all = _mm_and_si128(cond_max, cond_valid_IJ);
                 __m128i cond_Jlt = _mm_cmplt_epi32(vJ, vEndJ);
-                vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
-                vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
-                vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
+                vMaxH = _mm_blendv_epi8_rpl(vMaxH, vWH, cond_all);
+                vEndI = _mm_blendv_epi8_rpl(vEndI, vI, cond_all);
+                vEndJ = _mm_blendv_epi8_rpl(vEndJ, vJ, cond_all);
                 cond_all = _mm_and_si128(cond_Jlt, cond_eq);
                 cond_all = _mm_and_si128(cond_all, cond_valid_IJ);
-                vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
-                vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
+                vEndI = _mm_blendv_epi8_rpl(vEndI, vI, cond_all);
+                vEndJ = _mm_blendv_epi8_rpl(vEndJ, vJ, cond_all);
             }
             vJ = _mm_add_epi32(vJ, vOne);
         }
