@@ -28,10 +28,8 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
 }
 #endif
 
@@ -42,23 +40,21 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
+    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_scan_neon_128_32
-#define PNAME parasail_sw_table_scan_profile_neon_128_32
+#define FNAME parasail_sw_table_scan_neon_128_64
+#define PNAME parasail_sw_table_scan_profile_neon_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_scan_neon_128_32
-#define PNAME parasail_sw_rowcol_scan_profile_neon_128_32
+#define FNAME parasail_sw_rowcol_scan_neon_128_64
+#define PNAME parasail_sw_rowcol_scan_profile_neon_128_64
 #else
-#define FNAME parasail_sw_scan_neon_128_32
-#define PNAME parasail_sw_scan_profile_neon_128_32
+#define FNAME parasail_sw_scan_neon_128_64
+#define PNAME parasail_sw_scan_profile_neon_128_64
 #endif
 #endif
 
@@ -83,7 +79,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -118,10 +114,10 @@ parasail_result_t* PNAME(
     simde__m128i* restrict pvGapper = NULL;
     simde__m128i vGapO;
     simde__m128i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     simde__m128i vZero;
-    int32_t score = 0;
+    int64_t score = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -134,7 +130,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -149,29 +145,29 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    pvP = (simde__m128i*)profile->profile32.score;
-    vGapO = simde_mm_set1_epi32(open);
-    vGapE = simde_mm_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    pvP = (simde__m128i*)profile->profile64.score;
+    vGapO = simde_mm_set1_epi64x(open);
+    vGapE = simde_mm_set1_epi64x(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = simde_mm_setzero_si128();
     score = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHUnit = vNegLimit;
     vNegInfFront = vZero;
-    vNegInfFront = simde_mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = simde_mm_add_epi32(vNegInfFront,
-            simde_mm_slli_si128(simde_mm_set1_epi32(-segLen*gap), 4));
+    vNegInfFront = simde_mm_insert_epi64(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = simde_mm_add_epi64(vNegInfFront,
+            simde_mm_slli_si128(simde_mm_set1_epi64x(-segLen*gap), 8));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -187,7 +183,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -212,10 +208,10 @@ parasail_result_t* PNAME(
     parasail_memset_simde__m128i(pvH, vZero, segLen);
     parasail_memset_simde__m128i(pvE, vNegLimit, segLen);
     {
-        simde__m128i vGapper = simde_mm_sub_epi32(vZero,vGapO);
+        simde__m128i vGapper = simde_mm_sub_epi64(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             simde_mm_store_si128(pvGapper+i, vGapper);
-            vGapper = simde_mm_sub_epi32(vGapper, vGapE);
+            vGapper = simde_mm_sub_epi64(vGapper, vGapE);
         }
     }
 
@@ -233,7 +229,7 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = simde_mm_load_si128(pvH+(segLen-1));
-        vHp = simde_mm_slli_si128(vHp, 4);
+        vHp = simde_mm_slli_si128(vHp, 8);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         vHt = vZero;
         vF = vNegLimit;
@@ -241,52 +237,52 @@ parasail_result_t* PNAME(
             vH = simde_mm_load_si128(pvH+i);
             vE = simde_mm_load_si128(pvE+i);
             vW = simde_mm_load_si128(pvW+i);
-            vE = simde_mm_max_epi32(
-                    simde_mm_sub_epi32(vE, vGapE),
-                    simde_mm_sub_epi32(vH, vGapO));
-            vHp = simde_mm_add_epi32(vHp, vW);
-            vF = simde_mm_max_epi32(vF, simde_mm_add_epi32(vHt, pvGapper[i]));
-            vHt = simde_mm_max_epi32(vE, vHp);
+            vE = simde_mm_max_epi64(
+                    simde_mm_sub_epi64(vE, vGapE),
+                    simde_mm_sub_epi64(vH, vGapO));
+            vHp = simde_mm_add_epi64(vHp, vW);
+            vF = simde_mm_max_epi64(vF, simde_mm_add_epi64(vHt, pvGapper[i]));
+            vHt = simde_mm_max_epi64(vE, vHp);
             simde_mm_store_si128(pvE+i, vE);
             simde_mm_store_si128(pvHt+i, vHt);
             vHp = vH;
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = simde_mm_slli_si128(vHt, 4);
-        vF = simde_mm_max_epi32(vF, simde_mm_add_epi32(vHt, pvGapper[0]));
+        vHt = simde_mm_slli_si128(vHt, 8);
+        vF = simde_mm_max_epi64(vF, simde_mm_add_epi64(vHt, pvGapper[0]));
         for (i=0; i<segWidth-2; ++i) {
-            simde__m128i vFt = simde_mm_slli_si128(vF, 4);
-            vFt = simde_mm_add_epi32(vFt, vSegLenXgap);
-            vF = simde_mm_max_epi32(vF, vFt);
+            simde__m128i vFt = simde_mm_slli_si128(vF, 8);
+            vFt = simde_mm_add_epi64(vFt, vSegLenXgap);
+            vF = simde_mm_max_epi64(vF, vFt);
         }
 
         /* calculate final H */
-        vF = simde_mm_slli_si128(vF, 4);
-        vF = simde_mm_add_epi32(vF, vNegInfFront);
-        vH = simde_mm_max_epi32(vHt, vF);
-        vH = simde_mm_max_epi32(vH, vZero);
+        vF = simde_mm_slli_si128(vF, 8);
+        vF = simde_mm_add_epi64(vF, vNegInfFront);
+        vH = simde_mm_max_epi64(vHt, vF);
+        vH = simde_mm_max_epi64(vH, vZero);
         for (i=0; i<segLen; ++i) {
             vHt = simde_mm_load_si128(pvHt+i);
-            vF = simde_mm_max_epi32(
-                    simde_mm_sub_epi32(vF, vGapE),
-                    simde_mm_sub_epi32(vH, vGapO));
-            vH = simde_mm_max_epi32(vHt, vF);
-            vH = simde_mm_max_epi32(vH, vZero);
+            vF = simde_mm_max_epi64(
+                    simde_mm_sub_epi64(vF, vGapE),
+                    simde_mm_sub_epi64(vH, vGapO));
+            vH = simde_mm_max_epi64(vHt, vF);
+            vH = simde_mm_max_epi64(vH, vZero);
             simde_mm_store_si128(pvH+i, vH);
-            vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vMaxH = simde_mm_max_epi32(vH, vMaxH);
+            vMaxH = simde_mm_max_epi64(vH, vMaxH);
         } 
 
         {
-            simde__m128i vCompare = simde_mm_cmpgt_epi32(vMaxH, vMaxHUnit);
+            simde__m128i vCompare = simde_mm_cmpgt_epi64(vMaxH, vMaxHUnit);
             if (simde_mm_movemask_epi8(vCompare)) {
-                score = simde_mm_hmax_epi32(vMaxH);
-                vMaxHUnit = simde_mm_set1_epi32(score);
+                score = simde_mm_hmax_epi64(vMaxH);
+                vMaxHUnit = simde_mm_set1_epi64x(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(simde__m128i)*segLen);
             }
@@ -298,16 +294,16 @@ parasail_result_t* PNAME(
             int32_t k = 0;
             simde__m128i vH = simde_mm_load_si128(pvH + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 4);
+                vH = simde_mm_slli_si128(vH, 8);
             }
-            result->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            result->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
         }
 #endif
     }
 
     /* Trace the alignment ending position on read. */
     {
-        int32_t *t = (int32_t*)pvHMax;
+        int64_t *t = (int64_t*)pvHMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t) {
@@ -328,8 +324,8 @@ parasail_result_t* PNAME(
 #endif
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
