@@ -27,28 +27,10 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int16_t)_mm_extract_epi16(vWH, 7);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int64_t)_mm_extract_epi64(vWH, 1);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int16_t)_mm_extract_epi16(vWH, 6);
-    }
-    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
-        array[1LL*(i+2)*s2Len + (j-2)] = (int16_t)_mm_extract_epi16(vWH, 5);
-    }
-    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
-        array[1LL*(i+3)*s2Len + (j-3)] = (int16_t)_mm_extract_epi16(vWH, 4);
-    }
-    if (0 <= i+4 && i+4 < s1Len && 0 <= j-4 && j-4 < s2Len) {
-        array[1LL*(i+4)*s2Len + (j-4)] = (int16_t)_mm_extract_epi16(vWH, 3);
-    }
-    if (0 <= i+5 && i+5 < s1Len && 0 <= j-5 && j-5 < s2Len) {
-        array[1LL*(i+5)*s2Len + (j-5)] = (int16_t)_mm_extract_epi16(vWH, 2);
-    }
-    if (0 <= i+6 && i+6 < s1Len && 0 <= j-6 && j-6 < s2Len) {
-        array[1LL*(i+6)*s2Len + (j-6)] = (int16_t)_mm_extract_epi16(vWH, 1);
-    }
-    if (0 <= i+7 && i+7 < s1Len && 0 <= j-7 && j-7 < s2Len) {
-        array[1LL*(i+7)*s2Len + (j-7)] = (int16_t)_mm_extract_epi16(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int64_t)_mm_extract_epi64(vWH, 0);
     }
 }
 #endif
@@ -64,63 +46,27 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int16_t)_mm_extract_epi16(vWH, 7);
+        row[j-0] = (int64_t)_mm_extract_epi64(vWH, 1);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int16_t)_mm_extract_epi16(vWH, 7);
+        col[(i+0)] = (int64_t)_mm_extract_epi64(vWH, 1);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int16_t)_mm_extract_epi16(vWH, 6);
+        row[j-1] = (int64_t)_mm_extract_epi64(vWH, 0);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int16_t)_mm_extract_epi16(vWH, 6);
-    }
-    if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
-        row[j-2] = (int16_t)_mm_extract_epi16(vWH, 5);
-    }
-    if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
-        col[(i+2)] = (int16_t)_mm_extract_epi16(vWH, 5);
-    }
-    if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
-        row[j-3] = (int16_t)_mm_extract_epi16(vWH, 4);
-    }
-    if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
-        col[(i+3)] = (int16_t)_mm_extract_epi16(vWH, 4);
-    }
-    if (i+4 == s1Len-1 && 0 <= j-4 && j-4 < s2Len) {
-        row[j-4] = (int16_t)_mm_extract_epi16(vWH, 3);
-    }
-    if (j-4 == s2Len-1 && 0 <= i+4 && i+4 < s1Len) {
-        col[(i+4)] = (int16_t)_mm_extract_epi16(vWH, 3);
-    }
-    if (i+5 == s1Len-1 && 0 <= j-5 && j-5 < s2Len) {
-        row[j-5] = (int16_t)_mm_extract_epi16(vWH, 2);
-    }
-    if (j-5 == s2Len-1 && 0 <= i+5 && i+5 < s1Len) {
-        col[(i+5)] = (int16_t)_mm_extract_epi16(vWH, 2);
-    }
-    if (i+6 == s1Len-1 && 0 <= j-6 && j-6 < s2Len) {
-        row[j-6] = (int16_t)_mm_extract_epi16(vWH, 1);
-    }
-    if (j-6 == s2Len-1 && 0 <= i+6 && i+6 < s1Len) {
-        col[(i+6)] = (int16_t)_mm_extract_epi16(vWH, 1);
-    }
-    if (i+7 == s1Len-1 && 0 <= j-7 && j-7 < s2Len) {
-        row[j-7] = (int16_t)_mm_extract_epi16(vWH, 0);
-    }
-    if (j-7 == s2Len-1 && 0 <= i+7 && i+7 < s1Len) {
-        col[(i+7)] = (int16_t)_mm_extract_epi16(vWH, 0);
+        col[(i+1)] = (int64_t)_mm_extract_epi64(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_diag_altivec_128_16
+#define FNAME parasail_sw_stats_table_diag_altivec_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_diag_altivec_128_16
+#define FNAME parasail_sw_stats_rowcol_diag_altivec_128_64
 #else
-#define FNAME parasail_sw_stats_diag_altivec_128_16
+#define FNAME parasail_sw_stats_diag_altivec_128_64
 #endif
 #endif
 
@@ -136,36 +82,36 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int16_t * restrict s1 = NULL;
-    int16_t * restrict s2B = NULL;
-    int16_t * restrict _H_pr = NULL;
-    int16_t * restrict _HM_pr = NULL;
-    int16_t * restrict _HS_pr = NULL;
-    int16_t * restrict _HL_pr = NULL;
-    int16_t * restrict _F_pr = NULL;
-    int16_t * restrict _FM_pr = NULL;
-    int16_t * restrict _FS_pr = NULL;
-    int16_t * restrict _FL_pr = NULL;
-    int16_t * restrict s2 = NULL;
-    int16_t * restrict H_pr = NULL;
-    int16_t * restrict HM_pr = NULL;
-    int16_t * restrict HS_pr = NULL;
-    int16_t * restrict HL_pr = NULL;
-    int16_t * restrict F_pr = NULL;
-    int16_t * restrict FM_pr = NULL;
-    int16_t * restrict FS_pr = NULL;
-    int16_t * restrict FL_pr = NULL;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _HM_pr = NULL;
+    int64_t * restrict _HS_pr = NULL;
+    int64_t * restrict _HL_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict _FM_pr = NULL;
+    int64_t * restrict _FS_pr = NULL;
+    int64_t * restrict _FL_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict HM_pr = NULL;
+    int64_t * restrict HS_pr = NULL;
+    int64_t * restrict HL_pr = NULL;
+    int64_t * restrict F_pr = NULL;
+    int64_t * restrict FM_pr = NULL;
+    int64_t * restrict FS_pr = NULL;
+    int64_t * restrict FL_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int16_t NEG_LIMIT = 0;
-    int16_t POS_LIMIT = 0;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
     vec128i vNegLimit;
     vec128i vPosLimit;
     vec128i vSaturationCheckMin;
@@ -204,7 +150,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 8; /* number of values in vector */
+    N = 2; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -214,34 +160,34 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = 0;
     end_ref = 0;
-    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     score = NEG_LIMIT;
     matches = NEG_LIMIT;
     similar = NEG_LIMIT;
     length = NEG_LIMIT;
-    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vNegLimit = _mm_set1_epi64(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = _mm_set1_epi16(NEG_LIMIT);
-    vNegInf0 = _mm_srli_si128(vNegInf, 2); /* shift in a 0 */
-    vOpen = _mm_set1_epi16(open);
-    vGap  = _mm_set1_epi16(gap);
-    vZero = _mm_set1_epi16(0);
-    vOne = _mm_set1_epi16(1);
-    vN = _mm_set1_epi16(N);
-    vNegOne = _mm_set1_epi16(-1);
-    vI = _mm_set_epi16(0,1,2,3,4,5,6,7);
-    vJreset = _mm_set_epi16(0,-1,-2,-3,-4,-5,-6,-7);
+    vNegInf = _mm_set1_epi64(NEG_LIMIT);
+    vNegInf0 = _mm_srli_si128(vNegInf, 8); /* shift in a 0 */
+    vOpen = _mm_set1_epi64(open);
+    vGap  = _mm_set1_epi64(gap);
+    vZero = _mm_set1_epi64(0);
+    vOne = _mm_set1_epi64(1);
+    vN = _mm_set1_epi64(N);
+    vNegOne = _mm_set1_epi64(-1);
+    vI = _mm_set_epi64(0,1);
+    vJreset = _mm_set_epi64(0,-1);
     vMaxH = vNegInf;
     vMaxM = vNegInf;
     vMaxS = vNegInf;
     vMaxL = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = _mm_set1_epi16(s1Len);
-    vJLimit = _mm_set1_epi16(s2Len);
+    vILimit = _mm_set1_epi64(s1Len);
+    vJLimit = _mm_set1_epi64(s2Len);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -258,7 +204,7 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -267,16 +213,16 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s1     = parasail_memalign_int16_t(16, s1Len+PAD);
-    s2B    = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _H_pr  = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _HM_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _HS_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _HL_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _F_pr  = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _FM_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _FS_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
-    _FL_pr = parasail_memalign_int16_t(16, s2Len+PAD2);
+    s1     = parasail_memalign_int64_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     HM_pr = _HM_pr+PAD;
@@ -382,25 +328,13 @@ parasail_result_t* FNAME(
         vec128i vFS = vZero;
         vec128i vFL = vZero;
         vec128i vJ = vJreset;
-        vec128i vs1 = _mm_set_epi16(
+        vec128i vs1 = _mm_set_epi64(
                 s1[i+0],
-                s1[i+1],
-                s1[i+2],
-                s1[i+3],
-                s1[i+4],
-                s1[i+5],
-                s1[i+6],
-                s1[i+7]);
+                s1[i+1]);
         vec128i vs2 = vNegInf;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
-        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
-        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
-        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
-        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
-        vec128i vIltLimit = _mm_cmplt_epi16(vI, vILimit);
+        vec128i vIltLimit = _mm_cmplt_epi64(vI, vILimit);
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             vec128i vMat;
@@ -408,81 +342,75 @@ parasail_result_t* FNAME(
             vec128i vNWM = vNM;
             vec128i vNWS = vNS;
             vec128i vNWL = vNL;
-            vNH = _mm_srli_si128(vWH, 2);
-            vNH = _mm_insert_epi16(vNH, H_pr[j], 7);
-            vNM = _mm_srli_si128(vWM, 2);
-            vNM = _mm_insert_epi16(vNM, HM_pr[j], 7);
-            vNS = _mm_srli_si128(vWS, 2);
-            vNS = _mm_insert_epi16(vNS, HS_pr[j], 7);
-            vNL = _mm_srli_si128(vWL, 2);
-            vNL = _mm_insert_epi16(vNL, HL_pr[j], 7);
-            vF = _mm_srli_si128(vF, 2);
-            vF = _mm_insert_epi16(vF, F_pr[j], 7);
-            vFM = _mm_srli_si128(vFM, 2);
-            vFM = _mm_insert_epi16(vFM, FM_pr[j], 7);
-            vFS = _mm_srli_si128(vFS, 2);
-            vFS = _mm_insert_epi16(vFS, FS_pr[j], 7);
-            vFL = _mm_srli_si128(vFL, 2);
-            vFL = _mm_insert_epi16(vFL, FL_pr[j], 7);
-            vF_opn = _mm_subs_epi16(vNH, vOpen);
-            vF_ext = _mm_subs_epi16(vF, vGap);
-            vF = _mm_max_epi16(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
+            vNH = _mm_srli_si128(vWH, 8);
+            vNH = _mm_insert_epi64(vNH, H_pr[j], 1);
+            vNM = _mm_srli_si128(vWM, 8);
+            vNM = _mm_insert_epi64(vNM, HM_pr[j], 1);
+            vNS = _mm_srli_si128(vWS, 8);
+            vNS = _mm_insert_epi64(vNS, HS_pr[j], 1);
+            vNL = _mm_srli_si128(vWL, 8);
+            vNL = _mm_insert_epi64(vNL, HL_pr[j], 1);
+            vF = _mm_srli_si128(vF, 8);
+            vF = _mm_insert_epi64(vF, F_pr[j], 1);
+            vFM = _mm_srli_si128(vFM, 8);
+            vFM = _mm_insert_epi64(vFM, FM_pr[j], 1);
+            vFS = _mm_srli_si128(vFS, 8);
+            vFS = _mm_insert_epi64(vFS, FS_pr[j], 1);
+            vFL = _mm_srli_si128(vFL, 8);
+            vFL = _mm_insert_epi64(vFL, FL_pr[j], 1);
+            vF_opn = _mm_sub_epi64(vNH, vOpen);
+            vF_ext = _mm_sub_epi64(vF, vGap);
+            vF = _mm_max_epi64(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi64(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vNM, case1);
             vFS = _mm_blendv_epi8(vFS, vNS, case1);
             vFL = _mm_blendv_epi8(vFL, vNL, case1);
-            vFL = _mm_adds_epi16(vFL, vOne);
-            vE_opn = _mm_subs_epi16(vWH, vOpen);
-            vE_ext = _mm_subs_epi16(vE, vGap);
-            vE = _mm_max_epi16(vE_opn, vE_ext);
-            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
+            vFL = _mm_add_epi64(vFL, vOne);
+            vE_opn = _mm_sub_epi64(vWH, vOpen);
+            vE_ext = _mm_sub_epi64(vE, vGap);
+            vE = _mm_max_epi64(vE_opn, vE_ext);
+            case1 = _mm_cmpgt_epi64(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vWM, case1);
             vES = _mm_blendv_epi8(vES, vWS, case1);
             vEL = _mm_blendv_epi8(vEL, vWL, case1);
-            vEL = _mm_adds_epi16(vEL, vOne);
-            vs2 = _mm_srli_si128(vs2, 2);
-            vs2 = _mm_insert_epi16(vs2, s2[j], 7);
-            vMat = _mm_set_epi16(
+            vEL = _mm_add_epi64(vEL, vOne);
+            vs2 = _mm_srli_si128(vs2, 8);
+            vs2 = _mm_insert_epi64(vs2, s2[j], 1);
+            vMat = _mm_set_epi64(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]],
-                    matrow2[s2[j-2]],
-                    matrow3[s2[j-3]],
-                    matrow4[s2[j-4]],
-                    matrow5[s2[j-5]],
-                    matrow6[s2[j-6]],
-                    matrow7[s2[j-7]]
+                    matrow1[s2[j-1]]
                     );
-            vNWH = _mm_adds_epi16(vNWH, vMat);
-            vWH = _mm_max_epi16(vNWH, vE);
-            vWH = _mm_max_epi16(vWH, vF);
-            vWH = _mm_max_epi16(vWH, vZero);
-            case1 = _mm_cmpeq_epi16(vWH, vNWH);
-            case2 = _mm_cmpeq_epi16(vWH, vF);
-            case0 = _mm_cmpeq_epi16(vWH, vZero);
+            vNWH = _mm_add_epi64(vNWH, vMat);
+            vWH = _mm_max_epi64(vNWH, vE);
+            vWH = _mm_max_epi64(vWH, vF);
+            vWH = _mm_max_epi64(vWH, vZero);
+            case1 = _mm_cmpeq_epi64(vWH, vNWH);
+            case2 = _mm_cmpeq_epi64(vWH, vF);
+            case0 = _mm_cmpeq_epi64(vWH, vZero);
             vWM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_adds_epi16(vNWM,
+                    _mm_add_epi64(vNWM,
                         _mm_and_si128(
-                            _mm_cmpeq_epi16(vs1,vs2),
+                            _mm_cmpeq_epi64(vs1,vs2),
                             vOne)),
                     case1);
             vWM = _mm_blendv_epi8(vWM, vZero, case0);
             vWS = _mm_blendv_epi8(
                     _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_adds_epi16(vNWS,
+                    _mm_add_epi64(vNWS,
                         _mm_and_si128(
-                            _mm_cmpgt_epi16(vMat,vZero),
+                            _mm_cmpgt_epi64(vMat,vZero),
                             vOne)),
                     case1);
             vWS = _mm_blendv_epi8(vWS, vZero, case0);
             vWL = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_adds_epi16(vNWL, vOne), case1);
+                    _mm_add_epi64(vNWL, vOne), case1);
             vWL = _mm_blendv_epi8(vWL, vZero, case0);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                vec128i cond = _mm_cmpeq_epi16(vJ,vNegOne);
+                vec128i cond = _mm_cmpeq_epi64(vJ,vNegOne);
                 vWH = _mm_andnot_si128(cond, vWH);
                 vWM = _mm_andnot_si128(cond, vWM);
                 vWS = _mm_andnot_si128(cond, vWS);
@@ -494,11 +422,11 @@ parasail_result_t* FNAME(
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWM);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWS);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vWL);
+                vSaturationCheckMin = _mm_min_epi64(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm_max_epi64(vSaturationCheckMax, vWL);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -512,25 +440,25 @@ parasail_result_t* FNAME(
             arr_store_rowcol(result->stats->rowcols->similar_row, result->stats->rowcols->similar_col, vWS, i, s1Len, j, s2Len);
             arr_store_rowcol(result->stats->rowcols->length_row,  result->stats->rowcols->length_col, vWL, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-7] = (int16_t)_mm_extract_epi16(vWH,0);
-            HM_pr[j-7] = (int16_t)_mm_extract_epi16(vWM,0);
-            HS_pr[j-7] = (int16_t)_mm_extract_epi16(vWS,0);
-            HL_pr[j-7] = (int16_t)_mm_extract_epi16(vWL,0);
-            F_pr[j-7] = (int16_t)_mm_extract_epi16(vF,0);
-            FM_pr[j-7] = (int16_t)_mm_extract_epi16(vFM,0);
-            FS_pr[j-7] = (int16_t)_mm_extract_epi16(vFS,0);
-            FL_pr[j-7] = (int16_t)_mm_extract_epi16(vFL,0);
+            H_pr[j-1] = (int64_t)_mm_extract_epi64(vWH,0);
+            HM_pr[j-1] = (int64_t)_mm_extract_epi64(vWM,0);
+            HS_pr[j-1] = (int64_t)_mm_extract_epi64(vWS,0);
+            HL_pr[j-1] = (int64_t)_mm_extract_epi64(vWL,0);
+            F_pr[j-1] = (int64_t)_mm_extract_epi64(vF,0);
+            FM_pr[j-1] = (int64_t)_mm_extract_epi64(vFM,0);
+            FS_pr[j-1] = (int64_t)_mm_extract_epi64(vFS,0);
+            FL_pr[j-1] = (int64_t)_mm_extract_epi64(vFL,0);
             /* as minor diagonal vector passes across table, extract
              * max values within the i,j bounds */
             {
                 vec128i cond_valid_J = _mm_and_si128(
-                        _mm_cmpgt_epi16(vJ, vNegOne),
-                        _mm_cmplt_epi16(vJ, vJLimit));
+                        _mm_cmpgt_epi64(vJ, vNegOne),
+                        _mm_cmplt_epi64(vJ, vJLimit));
                 vec128i cond_valid_IJ = _mm_and_si128(cond_valid_J, vIltLimit);
-                vec128i cond_eq = _mm_cmpeq_epi16(vWH, vMaxH);
-                vec128i cond_max = _mm_cmpgt_epi16(vWH, vMaxH);
+                vec128i cond_eq = _mm_cmpeq_epi64(vWH, vMaxH);
+                vec128i cond_max = _mm_cmpgt_epi64(vWH, vMaxH);
                 vec128i cond_all = _mm_and_si128(cond_max, cond_valid_IJ);
-                vec128i cond_Jlt = _mm_cmplt_epi16(vJ, vEndJ);
+                vec128i cond_Jlt = _mm_cmplt_epi64(vJ, vEndJ);
                 vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
                 vMaxM = _mm_blendv_epi8(vMaxM, vWM, cond_all);
                 vMaxS = _mm_blendv_epi8(vMaxS, vWS, cond_all);
@@ -545,19 +473,19 @@ parasail_result_t* FNAME(
                 vEndI = _mm_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = _mm_blendv_epi8(vEndJ, vJ, cond_all);
             }
-            vJ = _mm_adds_epi16(vJ, vOne);
+            vJ = _mm_add_epi64(vJ, vOne);
         }
-        vI = _mm_adds_epi16(vI, vN);
+        vI = _mm_add_epi64(vI, vN);
     }
 
     /* alignment ending position */
     {
-        int16_t *t = (int16_t*)&vMaxH;
-        int16_t *m = (int16_t*)&vMaxM;
-        int16_t *s = (int16_t*)&vMaxS;
-        int16_t *l = (int16_t*)&vMaxL;
-        int16_t *i = (int16_t*)&vEndI;
-        int16_t *j = (int16_t*)&vEndJ;
+        int64_t *t = (int64_t*)&vMaxH;
+        int64_t *m = (int64_t*)&vMaxM;
+        int64_t *s = (int64_t*)&vMaxS;
+        int64_t *l = (int64_t*)&vMaxL;
+        int64_t *i = (int64_t*)&vEndI;
+        int64_t *j = (int64_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++t, ++m, ++s, ++l, ++i, ++j) {
             if (*t > score) {
@@ -588,8 +516,8 @@ parasail_result_t* FNAME(
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
