@@ -27,10 +27,16 @@ static inline void arr_store_si128(
         int32_t s2Len)
 {
     if (0 <= i+0 && i+0 < s1Len && 0 <= j-0 && j-0 < s2Len) {
-        array[1LL*(i+0)*s2Len + (j-0)] = (int64_t)simde_mm_extract_epi64(vWH, 1);
+        array[1LL*(i+0)*s2Len + (j-0)] = (int32_t)simde_mm_extract_epi32(vWH, 3);
     }
     if (0 <= i+1 && i+1 < s1Len && 0 <= j-1 && j-1 < s2Len) {
-        array[1LL*(i+1)*s2Len + (j-1)] = (int64_t)simde_mm_extract_epi64(vWH, 0);
+        array[1LL*(i+1)*s2Len + (j-1)] = (int32_t)simde_mm_extract_epi32(vWH, 2);
+    }
+    if (0 <= i+2 && i+2 < s1Len && 0 <= j-2 && j-2 < s2Len) {
+        array[1LL*(i+2)*s2Len + (j-2)] = (int32_t)simde_mm_extract_epi32(vWH, 1);
+    }
+    if (0 <= i+3 && i+3 < s1Len && 0 <= j-3 && j-3 < s2Len) {
+        array[1LL*(i+3)*s2Len + (j-3)] = (int32_t)simde_mm_extract_epi32(vWH, 0);
     }
 }
 #endif
@@ -46,27 +52,39 @@ static inline void arr_store_rowcol(
         int32_t s2Len)
 {
     if (i+0 == s1Len-1 && 0 <= j-0 && j-0 < s2Len) {
-        row[j-0] = (int64_t)simde_mm_extract_epi64(vWH, 1);
+        row[j-0] = (int32_t)simde_mm_extract_epi32(vWH, 3);
     }
     if (j-0 == s2Len-1 && 0 <= i+0 && i+0 < s1Len) {
-        col[(i+0)] = (int64_t)simde_mm_extract_epi64(vWH, 1);
+        col[(i+0)] = (int32_t)simde_mm_extract_epi32(vWH, 3);
     }
     if (i+1 == s1Len-1 && 0 <= j-1 && j-1 < s2Len) {
-        row[j-1] = (int64_t)simde_mm_extract_epi64(vWH, 0);
+        row[j-1] = (int32_t)simde_mm_extract_epi32(vWH, 2);
     }
     if (j-1 == s2Len-1 && 0 <= i+1 && i+1 < s1Len) {
-        col[(i+1)] = (int64_t)simde_mm_extract_epi64(vWH, 0);
+        col[(i+1)] = (int32_t)simde_mm_extract_epi32(vWH, 2);
+    }
+    if (i+2 == s1Len-1 && 0 <= j-2 && j-2 < s2Len) {
+        row[j-2] = (int32_t)simde_mm_extract_epi32(vWH, 1);
+    }
+    if (j-2 == s2Len-1 && 0 <= i+2 && i+2 < s1Len) {
+        col[(i+2)] = (int32_t)simde_mm_extract_epi32(vWH, 1);
+    }
+    if (i+3 == s1Len-1 && 0 <= j-3 && j-3 < s2Len) {
+        row[j-3] = (int32_t)simde_mm_extract_epi32(vWH, 0);
+    }
+    if (j-3 == s2Len-1 && 0 <= i+3 && i+3 < s1Len) {
+        col[(i+3)] = (int32_t)simde_mm_extract_epi32(vWH, 0);
     }
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_diag_neon_128_64
+#define FNAME parasail_sw_stats_table_diag_neon_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_diag_neon_128_64
+#define FNAME parasail_sw_stats_rowcol_diag_neon_128_32
 #else
-#define FNAME parasail_sw_stats_diag_neon_128_64
+#define FNAME parasail_sw_stats_diag_neon_128_32
 #endif
 #endif
 
@@ -82,36 +100,36 @@ parasail_result_t* FNAME(
     int32_t s1Len = 0;
     int32_t s1Len_PAD = 0;
     int32_t s2Len_PAD = 0;
-    int64_t * restrict s1 = NULL;
-    int64_t * restrict s2B = NULL;
-    int64_t * restrict _H_pr = NULL;
-    int64_t * restrict _HM_pr = NULL;
-    int64_t * restrict _HS_pr = NULL;
-    int64_t * restrict _HL_pr = NULL;
-    int64_t * restrict _F_pr = NULL;
-    int64_t * restrict _FM_pr = NULL;
-    int64_t * restrict _FS_pr = NULL;
-    int64_t * restrict _FL_pr = NULL;
-    int64_t * restrict s2 = NULL;
-    int64_t * restrict H_pr = NULL;
-    int64_t * restrict HM_pr = NULL;
-    int64_t * restrict HS_pr = NULL;
-    int64_t * restrict HL_pr = NULL;
-    int64_t * restrict F_pr = NULL;
-    int64_t * restrict FM_pr = NULL;
-    int64_t * restrict FS_pr = NULL;
-    int64_t * restrict FL_pr = NULL;
+    int32_t * restrict s1 = NULL;
+    int32_t * restrict s2B = NULL;
+    int32_t * restrict _H_pr = NULL;
+    int32_t * restrict _HM_pr = NULL;
+    int32_t * restrict _HS_pr = NULL;
+    int32_t * restrict _HL_pr = NULL;
+    int32_t * restrict _F_pr = NULL;
+    int32_t * restrict _FM_pr = NULL;
+    int32_t * restrict _FS_pr = NULL;
+    int32_t * restrict _FL_pr = NULL;
+    int32_t * restrict s2 = NULL;
+    int32_t * restrict H_pr = NULL;
+    int32_t * restrict HM_pr = NULL;
+    int32_t * restrict HS_pr = NULL;
+    int32_t * restrict HL_pr = NULL;
+    int32_t * restrict F_pr = NULL;
+    int32_t * restrict FM_pr = NULL;
+    int32_t * restrict FS_pr = NULL;
+    int32_t * restrict FL_pr = NULL;
     parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    int64_t NEG_LIMIT = 0;
-    int64_t POS_LIMIT = 0;
-    int64_t score = 0;
-    int64_t matches = 0;
-    int64_t similar = 0;
-    int64_t length = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
     simde__m128i vNegLimit;
     simde__m128i vPosLimit;
     simde__m128i vSaturationCheckMin;
@@ -150,7 +168,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize stack variables */
-    N = 2; /* number of values in vector */
+    N = 4; /* number of values in vector */
     PAD = N-1;
     PAD2 = PAD*2;
     s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
@@ -160,34 +178,34 @@ parasail_result_t* FNAME(
     j = 0;
     end_query = 0;
     end_ref = 0;
-    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     score = NEG_LIMIT;
     matches = NEG_LIMIT;
     similar = NEG_LIMIT;
     length = NEG_LIMIT;
-    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
-    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
+    vNegLimit = simde_mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
-    vNegInf = simde_mm_set1_epi64x(NEG_LIMIT);
-    vNegInf0 = simde_mm_srli_si128(vNegInf, 8); /* shift in a 0 */
-    vOpen = simde_mm_set1_epi64x(open);
-    vGap  = simde_mm_set1_epi64x(gap);
-    vZero = simde_mm_set1_epi64x(0);
-    vOne = simde_mm_set1_epi64x(1);
-    vN = simde_mm_set1_epi64x(N);
-    vNegOne = simde_mm_set1_epi64x(-1);
-    vI = simde_mm_set_epi64x(0,1);
-    vJreset = simde_mm_set_epi64x(0,-1);
+    vNegInf = simde_mm_set1_epi32(NEG_LIMIT);
+    vNegInf0 = simde_mm_srli_si128(vNegInf, 4); /* shift in a 0 */
+    vOpen = simde_mm_set1_epi32(open);
+    vGap  = simde_mm_set1_epi32(gap);
+    vZero = simde_mm_set1_epi32(0);
+    vOne = simde_mm_set1_epi32(1);
+    vN = simde_mm_set1_epi32(N);
+    vNegOne = simde_mm_set1_epi32(-1);
+    vI = simde_mm_set_epi32(0,1,2,3);
+    vJreset = simde_mm_set_epi32(0,-1,-2,-3);
     vMaxH = vNegInf;
     vMaxM = vNegInf;
     vMaxS = vNegInf;
     vMaxL = vNegInf;
     vEndI = vNegInf;
     vEndJ = vNegInf;
-    vILimit = simde_mm_set1_epi64x(s1Len);
-    vJLimit = simde_mm_set1_epi64x(s2Len);
+    vILimit = simde_mm_set1_epi32(s1Len);
+    vJLimit = simde_mm_set1_epi32(s2Len);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -204,7 +222,7 @@ parasail_result_t* FNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -213,16 +231,16 @@ parasail_result_t* FNAME(
 #endif
 
     /* initialize heap variables */
-    s1     = parasail_memalign_int64_t(16, s1Len+PAD);
-    s2B    = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _H_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _HM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _HS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _HL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _F_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _FM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _FS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    _FL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    s1     = parasail_memalign_int32_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
     s2 = s2B+PAD; /* will allow later for negative indices */
     H_pr = _H_pr+PAD;
     HM_pr = _HM_pr+PAD;
@@ -328,13 +346,17 @@ parasail_result_t* FNAME(
         simde__m128i vFS = vZero;
         simde__m128i vFL = vZero;
         simde__m128i vJ = vJreset;
-        simde__m128i vs1 = simde_mm_set_epi64x(
+        simde__m128i vs1 = simde_mm_set_epi32(
                 s1[i+0],
-                s1[i+1]);
+                s1[i+1],
+                s1[i+2],
+                s1[i+3]);
         simde__m128i vs2 = vNegInf;
         const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
         const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
-        simde__m128i vIltLimit = simde_mm_cmplt_epi64(vI, vILimit);
+        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
+        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
+        simde__m128i vIltLimit = simde_mm_cmplt_epi32(vI, vILimit);
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
             simde__m128i vMat;
@@ -342,75 +364,77 @@ parasail_result_t* FNAME(
             simde__m128i vNWM = vNM;
             simde__m128i vNWS = vNS;
             simde__m128i vNWL = vNL;
-            vNH = simde_mm_srli_si128(vWH, 8);
-            vNH = simde_mm_insert_epi64(vNH, H_pr[j], 1);
-            vNM = simde_mm_srli_si128(vWM, 8);
-            vNM = simde_mm_insert_epi64(vNM, HM_pr[j], 1);
-            vNS = simde_mm_srli_si128(vWS, 8);
-            vNS = simde_mm_insert_epi64(vNS, HS_pr[j], 1);
-            vNL = simde_mm_srli_si128(vWL, 8);
-            vNL = simde_mm_insert_epi64(vNL, HL_pr[j], 1);
-            vF = simde_mm_srli_si128(vF, 8);
-            vF = simde_mm_insert_epi64(vF, F_pr[j], 1);
-            vFM = simde_mm_srli_si128(vFM, 8);
-            vFM = simde_mm_insert_epi64(vFM, FM_pr[j], 1);
-            vFS = simde_mm_srli_si128(vFS, 8);
-            vFS = simde_mm_insert_epi64(vFS, FS_pr[j], 1);
-            vFL = simde_mm_srli_si128(vFL, 8);
-            vFL = simde_mm_insert_epi64(vFL, FL_pr[j], 1);
-            vF_opn = simde_mm_sub_epi64(vNH, vOpen);
-            vF_ext = simde_mm_sub_epi64(vF, vGap);
-            vF = simde_mm_max_epi64(vF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi64(vF_opn, vF_ext);
+            vNH = simde_mm_srli_si128(vWH, 4);
+            vNH = simde_mm_insert_epi32(vNH, H_pr[j], 3);
+            vNM = simde_mm_srli_si128(vWM, 4);
+            vNM = simde_mm_insert_epi32(vNM, HM_pr[j], 3);
+            vNS = simde_mm_srli_si128(vWS, 4);
+            vNS = simde_mm_insert_epi32(vNS, HS_pr[j], 3);
+            vNL = simde_mm_srli_si128(vWL, 4);
+            vNL = simde_mm_insert_epi32(vNL, HL_pr[j], 3);
+            vF = simde_mm_srli_si128(vF, 4);
+            vF = simde_mm_insert_epi32(vF, F_pr[j], 3);
+            vFM = simde_mm_srli_si128(vFM, 4);
+            vFM = simde_mm_insert_epi32(vFM, FM_pr[j], 3);
+            vFS = simde_mm_srli_si128(vFS, 4);
+            vFS = simde_mm_insert_epi32(vFS, FS_pr[j], 3);
+            vFL = simde_mm_srli_si128(vFL, 4);
+            vFL = simde_mm_insert_epi32(vFL, FL_pr[j], 3);
+            vF_opn = simde_mm_sub_epi32(vNH, vOpen);
+            vF_ext = simde_mm_sub_epi32(vF, vGap);
+            vF = simde_mm_max_epi32(vF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi32(vF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vNM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vNS, case1);
             vFL = simde_mm_blendv_epi8(vFL, vNL, case1);
-            vFL = simde_mm_add_epi64(vFL, vOne);
-            vE_opn = simde_mm_sub_epi64(vWH, vOpen);
-            vE_ext = simde_mm_sub_epi64(vE, vGap);
-            vE = simde_mm_max_epi64(vE_opn, vE_ext);
-            case1 = simde_mm_cmpgt_epi64(vE_opn, vE_ext);
+            vFL = simde_mm_add_epi32(vFL, vOne);
+            vE_opn = simde_mm_sub_epi32(vWH, vOpen);
+            vE_ext = simde_mm_sub_epi32(vE, vGap);
+            vE = simde_mm_max_epi32(vE_opn, vE_ext);
+            case1 = simde_mm_cmpgt_epi32(vE_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vWM, case1);
             vES = simde_mm_blendv_epi8(vES, vWS, case1);
             vEL = simde_mm_blendv_epi8(vEL, vWL, case1);
-            vEL = simde_mm_add_epi64(vEL, vOne);
-            vs2 = simde_mm_srli_si128(vs2, 8);
-            vs2 = simde_mm_insert_epi64(vs2, s2[j], 1);
-            vMat = simde_mm_set_epi64x(
+            vEL = simde_mm_add_epi32(vEL, vOne);
+            vs2 = simde_mm_srli_si128(vs2, 4);
+            vs2 = simde_mm_insert_epi32(vs2, s2[j], 3);
+            vMat = simde_mm_set_epi32(
                     matrow0[s2[j-0]],
-                    matrow1[s2[j-1]]
+                    matrow1[s2[j-1]],
+                    matrow2[s2[j-2]],
+                    matrow3[s2[j-3]]
                     );
-            vNWH = simde_mm_add_epi64(vNWH, vMat);
-            vWH = simde_mm_max_epi64(vNWH, vE);
-            vWH = simde_mm_max_epi64(vWH, vF);
-            vWH = simde_mm_max_epi64(vWH, vZero);
-            case1 = simde_mm_cmpeq_epi64(vWH, vNWH);
-            case2 = simde_mm_cmpeq_epi64(vWH, vF);
-            case0 = simde_mm_cmpeq_epi64(vWH, vZero);
+            vNWH = simde_mm_add_epi32(vNWH, vMat);
+            vWH = simde_mm_max_epi32(vNWH, vE);
+            vWH = simde_mm_max_epi32(vWH, vF);
+            vWH = simde_mm_max_epi32(vWH, vZero);
+            case1 = simde_mm_cmpeq_epi32(vWH, vNWH);
+            case2 = simde_mm_cmpeq_epi32(vWH, vF);
+            case0 = simde_mm_cmpeq_epi32(vWH, vZero);
             vWM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
-                    simde_mm_add_epi64(vNWM,
+                    simde_mm_add_epi32(vNWM,
                         simde_mm_and_si128(
-                            simde_mm_cmpeq_epi64(vs1,vs2),
+                            simde_mm_cmpeq_epi32(vs1,vs2),
                             vOne)),
                     case1);
             vWM = simde_mm_blendv_epi8(vWM, vZero, case0);
             vWS = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vES, vFS, case2),
-                    simde_mm_add_epi64(vNWS,
+                    simde_mm_add_epi32(vNWS,
                         simde_mm_and_si128(
-                            simde_mm_cmpgt_epi64(vMat,vZero),
+                            simde_mm_cmpgt_epi32(vMat,vZero),
                             vOne)),
                     case1);
             vWS = simde_mm_blendv_epi8(vWS, vZero, case0);
             vWL = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEL, vFL, case2),
-                    simde_mm_add_epi64(vNWL, vOne), case1);
+                    simde_mm_add_epi32(vNWL, vOne), case1);
             vWL = simde_mm_blendv_epi8(vWL, vZero, case0);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                simde__m128i cond = simde_mm_cmpeq_epi64(vJ,vNegOne);
+                simde__m128i cond = simde_mm_cmpeq_epi32(vJ,vNegOne);
                 vWH = simde_mm_andnot_si128(cond, vWH);
                 vWM = simde_mm_andnot_si128(cond, vWM);
                 vWS = simde_mm_andnot_si128(cond, vWS);
@@ -422,11 +446,11 @@ parasail_result_t* FNAME(
             }
             /* cannot start checking sat until after J clears boundary */
             if (j > PAD) {
-                vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vWH);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vWH);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vWM);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vWS);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vWL);
+                vSaturationCheckMin = simde_mm_min_epi32(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vWL);
             }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
@@ -440,25 +464,25 @@ parasail_result_t* FNAME(
             arr_store_rowcol(result->stats->rowcols->similar_row, result->stats->rowcols->similar_col, vWS, i, s1Len, j, s2Len);
             arr_store_rowcol(result->stats->rowcols->length_row,  result->stats->rowcols->length_col, vWL, i, s1Len, j, s2Len);
 #endif
-            H_pr[j-1] = (int64_t)simde_mm_extract_epi64(vWH,0);
-            HM_pr[j-1] = (int64_t)simde_mm_extract_epi64(vWM,0);
-            HS_pr[j-1] = (int64_t)simde_mm_extract_epi64(vWS,0);
-            HL_pr[j-1] = (int64_t)simde_mm_extract_epi64(vWL,0);
-            F_pr[j-1] = (int64_t)simde_mm_extract_epi64(vF,0);
-            FM_pr[j-1] = (int64_t)simde_mm_extract_epi64(vFM,0);
-            FS_pr[j-1] = (int64_t)simde_mm_extract_epi64(vFS,0);
-            FL_pr[j-1] = (int64_t)simde_mm_extract_epi64(vFL,0);
+            H_pr[j-3] = (int32_t)simde_mm_extract_epi32(vWH,0);
+            HM_pr[j-3] = (int32_t)simde_mm_extract_epi32(vWM,0);
+            HS_pr[j-3] = (int32_t)simde_mm_extract_epi32(vWS,0);
+            HL_pr[j-3] = (int32_t)simde_mm_extract_epi32(vWL,0);
+            F_pr[j-3] = (int32_t)simde_mm_extract_epi32(vF,0);
+            FM_pr[j-3] = (int32_t)simde_mm_extract_epi32(vFM,0);
+            FS_pr[j-3] = (int32_t)simde_mm_extract_epi32(vFS,0);
+            FL_pr[j-3] = (int32_t)simde_mm_extract_epi32(vFL,0);
             /* as minor diagonal vector passes across table, extract
              * max values within the i,j bounds */
             {
                 simde__m128i cond_valid_J = simde_mm_and_si128(
-                        simde_mm_cmpgt_epi64(vJ, vNegOne),
-                        simde_mm_cmplt_epi64(vJ, vJLimit));
+                        simde_mm_cmpgt_epi32(vJ, vNegOne),
+                        simde_mm_cmplt_epi32(vJ, vJLimit));
                 simde__m128i cond_valid_IJ = simde_mm_and_si128(cond_valid_J, vIltLimit);
-                simde__m128i cond_eq = simde_mm_cmpeq_epi64(vWH, vMaxH);
-                simde__m128i cond_max = simde_mm_cmpgt_epi64(vWH, vMaxH);
+                simde__m128i cond_eq = simde_mm_cmpeq_epi32(vWH, vMaxH);
+                simde__m128i cond_max = simde_mm_cmpgt_epi32(vWH, vMaxH);
                 simde__m128i cond_all = simde_mm_and_si128(cond_max, cond_valid_IJ);
-                simde__m128i cond_Jlt = simde_mm_cmplt_epi64(vJ, vEndJ);
+                simde__m128i cond_Jlt = simde_mm_cmplt_epi32(vJ, vEndJ);
                 vMaxH = simde_mm_blendv_epi8(vMaxH, vWH, cond_all);
                 vMaxM = simde_mm_blendv_epi8(vMaxM, vWM, cond_all);
                 vMaxS = simde_mm_blendv_epi8(vMaxS, vWS, cond_all);
@@ -473,19 +497,19 @@ parasail_result_t* FNAME(
                 vEndI = simde_mm_blendv_epi8(vEndI, vI, cond_all);
                 vEndJ = simde_mm_blendv_epi8(vEndJ, vJ, cond_all);
             }
-            vJ = simde_mm_add_epi64(vJ, vOne);
+            vJ = simde_mm_add_epi32(vJ, vOne);
         }
-        vI = simde_mm_add_epi64(vI, vN);
+        vI = simde_mm_add_epi32(vI, vN);
     }
 
     /* alignment ending position */
     {
-        int64_t *t = (int64_t*)&vMaxH;
-        int64_t *m = (int64_t*)&vMaxM;
-        int64_t *s = (int64_t*)&vMaxS;
-        int64_t *l = (int64_t*)&vMaxL;
-        int64_t *i = (int64_t*)&vEndI;
-        int64_t *j = (int64_t*)&vEndJ;
+        int32_t *t = (int32_t*)&vMaxH;
+        int32_t *m = (int32_t*)&vMaxM;
+        int32_t *s = (int32_t*)&vMaxS;
+        int32_t *l = (int32_t*)&vMaxL;
+        int32_t *i = (int32_t*)&vEndI;
+        int32_t *j = (int32_t*)&vEndJ;
         int32_t k;
         for (k=0; k<N; ++k, ++t, ++m, ++s, ++l, ++i, ++j) {
             if (*t > score) {
@@ -516,8 +540,8 @@ parasail_result_t* FNAME(
     }
 
     if (simde_mm_movemask_epi8(simde_mm_or_si128(
-            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
-            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+            simde_mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
