@@ -18,38 +18,36 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_INSERT_EPI8
-#define _mm256_insert_epi8_rpl _mm256_insert_epi8
+#if HAVE_AVX2_MM256_INSERT_EPI32
+#define _mm256_insert_epi32_rpl _mm256_insert_epi32
 #else
-static inline __m256i _mm256_insert_epi8_rpl(__m256i a, int8_t i, int imm) {
-    __m256i_8_t A;
+static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
+    __m256i_32_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI8
-#define _mm256_extract_epi8_rpl _mm256_extract_epi8
+#if HAVE_AVX2_MM256_EXTRACT_EPI32
+#define _mm256_extract_epi32_rpl _mm256_extract_epi32
 #else
-static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
-    __m256i_8_t A;
+static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
+    __m256i_32_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#define _mm256_cmplt_epi8_rpl(a,b) _mm256_cmpgt_epi8(b,a)
+#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
-static inline int8_t _mm256_hmax_epi8_rpl(__m256i a) {
-    a = _mm256_max_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 8));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 4));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 2));
-    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 1));
-    return _mm256_extract_epi8_rpl(a, 31);
+static inline int32_t _mm256_hmax_epi32_rpl(__m256i a) {
+    a = _mm256_max_epi32(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
+    a = _mm256_max_epi32(a, _mm256_slli_si256(a, 8));
+    a = _mm256_max_epi32(a, _mm256_slli_si256(a, 4));
+    return _mm256_extract_epi32_rpl(a, 7);
 }
 
 
@@ -62,38 +60,14 @@ static inline void arr_store_si256(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  0);
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  1);
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  2);
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  3);
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  4);
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  5);
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  6);
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  7);
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  8);
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  9);
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 10);
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 11);
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 12);
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 13);
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 14);
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 15);
-    array[1LL*(16*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 16);
-    array[1LL*(17*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 17);
-    array[1LL*(18*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 18);
-    array[1LL*(19*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 19);
-    array[1LL*(20*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 20);
-    array[1LL*(21*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 21);
-    array[1LL*(22*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 22);
-    array[1LL*(23*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 23);
-    array[1LL*(24*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 24);
-    array[1LL*(25*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 25);
-    array[1LL*(26*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 26);
-    array[1LL*(27*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 27);
-    array[1LL*(28*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 28);
-    array[1LL*(29*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 29);
-    array[1LL*(30*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 30);
-    array[1LL*(31*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 31);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
+    array[1LL*(4*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
+    array[1LL*(5*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
+    array[1LL*(6*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
+    array[1LL*(7*seglen+t)*dlen + d] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
 }
 #endif
 
@@ -104,51 +78,27 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  0);
-    col[ 1*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  1);
-    col[ 2*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  2);
-    col[ 3*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  3);
-    col[ 4*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  4);
-    col[ 5*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  5);
-    col[ 6*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  6);
-    col[ 7*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  7);
-    col[ 8*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  8);
-    col[ 9*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  9);
-    col[10*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 10);
-    col[11*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 11);
-    col[12*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 12);
-    col[13*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 13);
-    col[14*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 14);
-    col[15*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 15);
-    col[16*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 16);
-    col[17*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 17);
-    col[18*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 18);
-    col[19*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 19);
-    col[20*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 20);
-    col[21*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 21);
-    col[22*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 22);
-    col[23*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 23);
-    col[24*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 24);
-    col[25*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 25);
-    col[26*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 26);
-    col[27*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 27);
-    col[28*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 28);
-    col[29*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 29);
-    col[30*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 30);
-    col[31*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 31);
+    col[0*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 0);
+    col[1*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 1);
+    col[2*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 2);
+    col[3*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 3);
+    col[4*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 4);
+    col[5*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 5);
+    col[6*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 6);
+    col[7*seglen+t] = (int32_t)_mm256_extract_epi32_rpl(vH, 7);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_scan_avx2_256_8
-#define PNAME parasail_sw_stats_table_scan_profile_avx2_256_8
+#define FNAME parasail_sw_stats_table_scan_avx2_256_32
+#define PNAME parasail_sw_stats_table_scan_profile_avx2_256_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_scan_avx2_256_8
-#define PNAME parasail_sw_stats_rowcol_scan_profile_avx2_256_8
+#define FNAME parasail_sw_stats_rowcol_scan_avx2_256_32
+#define PNAME parasail_sw_stats_rowcol_scan_profile_avx2_256_32
 #else
-#define FNAME parasail_sw_stats_scan_avx2_256_8
-#define PNAME parasail_sw_stats_scan_profile_avx2_256_8
+#define FNAME parasail_sw_stats_scan_avx2_256_32
+#define PNAME parasail_sw_stats_scan_profile_avx2_256_32
 #endif
 #endif
 
@@ -176,7 +126,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_avx_256_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_avx_256_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -222,14 +172,14 @@ parasail_result_t* PNAME(
     __m256i* restrict pvGapperL = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int8_t NEG_LIMIT = 0;
-    int8_t POS_LIMIT = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
     __m256i vZero;
     __m256i vOne;
-    int8_t score = 0;
-    int8_t matches = 0;
-    int8_t similar = 0;
-    int8_t length = 0;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -246,7 +196,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -261,27 +211,27 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 32; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    pvP  = (__m256i*)profile->profile8.score;
-    pvPm = (__m256i*)profile->profile8.matches;
-    pvPs = (__m256i*)profile->profile8.similar;
-    vGapO = _mm256_set1_epi8(open);
-    vGapE = _mm256_set1_epi8(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT8_MIN + open : INT8_MIN - matrix->min) + 1;
-    POS_LIMIT = INT8_MAX - matrix->max - 1;
+    pvP  = (__m256i*)profile->profile32.score;
+    pvPm = (__m256i*)profile->profile32.matches;
+    pvPs = (__m256i*)profile->profile32.similar;
+    vGapO = _mm256_set1_epi32(open);
+    vGapE = _mm256_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
-    vOne = _mm256_set1_epi8(1);
+    vOne = _mm256_set1_epi32(1);
     score = NEG_LIMIT;
     matches = 0;
     similar = 0;
     length = 0;
-    vNegLimit = _mm256_set1_epi8(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi8(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
@@ -289,11 +239,11 @@ parasail_result_t* PNAME(
     vMaxS = vNegLimit;
     vMaxL = vNegLimit;
     vMaxHUnit = vNegLimit;
-    vSegLen = _mm256_slli_si256_rpl(_mm256_set1_epi8(segLen), 1);
+    vSegLen = _mm256_slli_si256_rpl(_mm256_set1_epi32(segLen), 4);
     vNegInfFront = vZero;
-    vNegInfFront = _mm256_insert_epi8_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm256_adds_epi8(vNegInfFront,
-            _mm256_slli_si256_rpl(_mm256_set1_epi8(-segLen*gap), 1));
+    vNegInfFront = _mm256_insert_epi32_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm256_add_epi32(vNegInfFront,
+            _mm256_slli_si256_rpl(_mm256_set1_epi32(-segLen*gap), 4));
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -310,7 +260,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -359,13 +309,13 @@ parasail_result_t* PNAME(
     parasail_memset___m256i(pvES, vZero, segLen);
     parasail_memset___m256i(pvEL, vZero, segLen);
     {
-        __m256i vGapper = _mm256_subs_epi8(vZero,vGapO);
+        __m256i vGapper = _mm256_sub_epi32(vZero,vGapO);
         __m256i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm256_store_si256(pvGapper+i, vGapper);
             _mm256_store_si256(pvGapperL+i, vGapperL);
-            vGapper = _mm256_subs_epi8(vGapper, vGapE);
-            vGapperL = _mm256_adds_epi8(vGapperL, vOne);
+            vGapper = _mm256_sub_epi32(vGapper, vGapE);
+            vGapperL = _mm256_add_epi32(vGapperL, vOne);
         }
     }
 
@@ -414,10 +364,10 @@ parasail_result_t* PNAME(
         vHpM = _mm256_load_si256(pvHM+(segLen-1));
         vHpS = _mm256_load_si256(pvHS+(segLen-1));
         vHpL = _mm256_load_si256(pvHL+(segLen-1));
-        vHp = _mm256_slli_si256_rpl(vHp, 1);
-        vHpM = _mm256_slli_si256_rpl(vHpM, 1);
-        vHpS = _mm256_slli_si256_rpl(vHpS, 1);
-        vHpL = _mm256_slli_si256_rpl(vHpL, 1);
+        vHp = _mm256_slli_si256_rpl(vHp, 4);
+        vHpM = _mm256_slli_si256_rpl(vHpM, 4);
+        vHpS = _mm256_slli_si256_rpl(vHpS, 4);
+        vHpL = _mm256_slli_si256_rpl(vHpL, 4);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
@@ -440,30 +390,30 @@ parasail_result_t* PNAME(
             vWS = _mm256_load_si256(pvWS+i);
             vGapper = _mm256_load_si256(pvGapper+i);
             vGapperL = _mm256_load_si256(pvGapperL+i);
-            vE_opn = _mm256_subs_epi8(vH, vGapO);
-            vE_ext = _mm256_subs_epi8(vE, vGapE);
-            case1 = _mm256_cmpgt_epi8(vE_opn, vE_ext);
-            vE = _mm256_max_epi8(vE_opn, vE_ext);
+            vE_opn = _mm256_sub_epi32(vH, vGapO);
+            vE_ext = _mm256_sub_epi32(vE, vGapE);
+            case1 = _mm256_cmpgt_epi32(vE_opn, vE_ext);
+            vE = _mm256_max_epi32(vE_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vHM, case1);
             vES = _mm256_blendv_epi8(vES, vHS, case1);
             vEL = _mm256_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm256_adds_epi8(vEL, vOne);
-            vGapper = _mm256_adds_epi8(vHt, vGapper);
+            vEL = _mm256_add_epi32(vEL, vOne);
+            vGapper = _mm256_add_epi32(vHt, vGapper);
             case1 = _mm256_or_si256(
-                    _mm256_cmpgt_epi8(vF, vGapper),
-                    _mm256_cmpeq_epi8(vF, vGapper));
-            vF = _mm256_max_epi8(vF, vGapper);
+                    _mm256_cmpgt_epi32(vF, vGapper),
+                    _mm256_cmpeq_epi32(vF, vGapper));
+            vF = _mm256_max_epi32(vF, vGapper);
             vFM = _mm256_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm256_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm256_blendv_epi8(
-                    _mm256_adds_epi8(vHtL, vGapperL),
+                    _mm256_add_epi32(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm256_adds_epi8(vHp, vW);
-            vHpM = _mm256_adds_epi8(vHpM, vWM);
-            vHpS = _mm256_adds_epi8(vHpS, vWS);
-            vHpL = _mm256_adds_epi8(vHpL, vOne);
-            case1 = _mm256_cmpgt_epi8(vE, vHp);
-            vHt = _mm256_max_epi8(vE, vHp);
+            vHp = _mm256_add_epi32(vHp, vW);
+            vHpM = _mm256_add_epi32(vHpM, vWM);
+            vHpS = _mm256_add_epi32(vHpS, vWS);
+            vHpL = _mm256_add_epi32(vHpL, vOne);
+            case1 = _mm256_cmpgt_epi32(vE, vHp);
+            vHt = _mm256_max_epi32(vE, vHp);
             vHtM = _mm256_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm256_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm256_blendv_epi8(vHpL, vEL, case1);
@@ -482,49 +432,49 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm256_slli_si256_rpl(vHt, 1);
-        vHtM = _mm256_slli_si256_rpl(vHtM, 1);
-        vHtS = _mm256_slli_si256_rpl(vHtS, 1);
-        vHtL = _mm256_slli_si256_rpl(vHtL, 1);
+        vHt = _mm256_slli_si256_rpl(vHt, 4);
+        vHtM = _mm256_slli_si256_rpl(vHtM, 4);
+        vHtS = _mm256_slli_si256_rpl(vHtS, 4);
+        vHtL = _mm256_slli_si256_rpl(vHtL, 4);
         vGapper = _mm256_load_si256(pvGapper);
         vGapperL = _mm256_load_si256(pvGapperL);
-        vGapper = _mm256_adds_epi8(vHt, vGapper);
+        vGapper = _mm256_add_epi32(vHt, vGapper);
         case1 = _mm256_or_si256(
-                _mm256_cmpgt_epi8(vGapper, vF),
-                _mm256_cmpeq_epi8(vGapper, vF));
-        vF = _mm256_max_epi8(vF, vGapper);
+                _mm256_cmpgt_epi32(vGapper, vF),
+                _mm256_cmpeq_epi32(vGapper, vF));
+        vF = _mm256_max_epi32(vF, vGapper);
         vFM = _mm256_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm256_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm256_blendv_epi8(
                 vFL,
-                _mm256_adds_epi8(vHtL, vGapperL),
+                _mm256_add_epi32(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            __m256i vFt = _mm256_slli_si256_rpl(vF, 1);
-            __m256i vFtM = _mm256_slli_si256_rpl(vFM, 1);
-            __m256i vFtS = _mm256_slli_si256_rpl(vFS, 1);
-            __m256i vFtL = _mm256_slli_si256_rpl(vFL, 1);
-            vFt = _mm256_adds_epi8(vFt, vSegLenXgap);
+            __m256i vFt = _mm256_slli_si256_rpl(vF, 4);
+            __m256i vFtM = _mm256_slli_si256_rpl(vFM, 4);
+            __m256i vFtS = _mm256_slli_si256_rpl(vFS, 4);
+            __m256i vFtL = _mm256_slli_si256_rpl(vFL, 4);
+            vFt = _mm256_add_epi32(vFt, vSegLenXgap);
             case1 = _mm256_or_si256(
-                    _mm256_cmpgt_epi8(vFt, vF),
-                    _mm256_cmpeq_epi8(vFt, vF));
-            vF = _mm256_max_epi8(vF, vFt);
+                    _mm256_cmpgt_epi32(vFt, vF),
+                    _mm256_cmpeq_epi32(vFt, vF));
+            vF = _mm256_max_epi32(vF, vFt);
             vFM = _mm256_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm256_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm256_blendv_epi8(
                     vFL,
-                    _mm256_adds_epi8(vFtL, vSegLen),
+                    _mm256_add_epi32(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm256_slli_si256_rpl(vF, 1);
-        vFM = _mm256_slli_si256_rpl(vFM, 1);
-        vFS = _mm256_slli_si256_rpl(vFS, 1);
-        vFL = _mm256_slli_si256_rpl(vFL, 1);
-        vF = _mm256_adds_epi8(vF, vNegInfFront);
-        case1 = _mm256_cmpgt_epi8(vF, vHt);
-        vH = _mm256_max_epi8(vF, vHt);
+        vF = _mm256_slli_si256_rpl(vF, 4);
+        vFM = _mm256_slli_si256_rpl(vFM, 4);
+        vFS = _mm256_slli_si256_rpl(vFS, 4);
+        vFL = _mm256_slli_si256_rpl(vFL, 4);
+        vF = _mm256_add_epi32(vF, vNegInfFront);
+        case1 = _mm256_cmpgt_epi32(vF, vHt);
+        vH = _mm256_max_epi32(vF, vHt);
         vHM = _mm256_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm256_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm256_blendv_epi8(vHtL, vFL, case1);
@@ -537,20 +487,20 @@ parasail_result_t* PNAME(
             vEM = _mm256_load_si256(pvEM+i);
             vES = _mm256_load_si256(pvES+i);
             vEL = _mm256_load_si256(pvEL+i);
-            vF_opn = _mm256_subs_epi8(vH, vGapO);
-            vF_ext = _mm256_subs_epi8(vF, vGapE);
-            vF = _mm256_max_epi8(vF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi8(vF_opn, vF_ext);
+            vF_opn = _mm256_sub_epi32(vH, vGapO);
+            vF_ext = _mm256_sub_epi32(vF, vGapE);
+            vF = _mm256_max_epi32(vF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi32(vF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vHM, case1);
             vFS = _mm256_blendv_epi8(vFS, vHS, case1);
             vFL = _mm256_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm256_adds_epi8(vFL, vOne);
-            vH = _mm256_max_epi8(vHp, vE);
-            vH = _mm256_max_epi8(vH, vF);
-            vH = _mm256_max_epi8(vH, vZero);
-            case1 = _mm256_cmpeq_epi8(vH, vHp);
-            case2 = _mm256_cmpeq_epi8(vH, vF);
-            case0 = _mm256_cmpeq_epi8(vH, vZero);
+            vFL = _mm256_add_epi32(vFL, vOne);
+            vH = _mm256_max_epi32(vHp, vE);
+            vH = _mm256_max_epi32(vH, vF);
+            vH = _mm256_max_epi32(vH, vZero);
+            case1 = _mm256_cmpeq_epi32(vH, vHp);
+            case2 = _mm256_cmpeq_epi32(vH, vF);
+            case0 = _mm256_cmpeq_epi32(vH, vZero);
             vHM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -567,11 +517,11 @@ parasail_result_t* PNAME(
             _mm256_store_si256(pvHM+i, vHM);
             _mm256_store_si256(pvHS+i, vHS);
             _mm256_store_si256(pvHL+i, vHL);
-            vSaturationCheckMin = _mm256_min_epi8(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si256(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -579,8 +529,8 @@ parasail_result_t* PNAME(
             arr_store_si256(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
 #endif
             {
-                __m256i cond_max = _mm256_cmpgt_epi8(vH, vMaxH);
-                vMaxH = _mm256_max_epi8(vH, vMaxH);
+                __m256i cond_max = _mm256_cmpgt_epi32(vH, vMaxH);
+                vMaxH = _mm256_max_epi32(vH, vMaxH);
                 vMaxM = _mm256_blendv_epi8(vMaxM, vHM, cond_max);
                 vMaxS = _mm256_blendv_epi8(vMaxS, vHS, cond_max);
                 vMaxL = _mm256_blendv_epi8(vMaxL, vHL, cond_max);
@@ -588,10 +538,10 @@ parasail_result_t* PNAME(
         } 
 
         {
-            __m256i vCompare = _mm256_cmpgt_epi8(vMaxH, vMaxHUnit);
+            __m256i vCompare = _mm256_cmpgt_epi32(vMaxH, vMaxHUnit);
             if (_mm256_movemask_epi8(vCompare)) {
-                score = _mm256_hmax_epi8_rpl(vMaxH);
-                vMaxHUnit = _mm256_set1_epi8(score);
+                score = _mm256_hmax_epi32_rpl(vMaxH);
+                vMaxHUnit = _mm256_set1_epi32(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(__m256i)*segLen);
                 (void)memcpy(pvHMMax, pvHM, sizeof(__m256i)*segLen);
@@ -609,25 +559,25 @@ parasail_result_t* PNAME(
             __m256i vHS = _mm256_load_si256(pvHS + offset);
             __m256i vHL = _mm256_load_si256(pvHL + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 1);
-                vHM = _mm256_slli_si256_rpl(vHM, 1);
-                vHS = _mm256_slli_si256_rpl(vHS, 1);
-                vHL = _mm256_slli_si256_rpl(vHL, 1);
+                vH = _mm256_slli_si256_rpl(vH, 4);
+                vHM = _mm256_slli_si256_rpl(vHM, 4);
+                vHS = _mm256_slli_si256_rpl(vHS, 4);
+                vHL = _mm256_slli_si256_rpl(vHL, 4);
             }
-            result->stats->rowcols->score_row[j] = (int8_t) _mm256_extract_epi8_rpl (vH, 31);
-            result->stats->rowcols->matches_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHM, 31);
-            result->stats->rowcols->similar_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHS, 31);
-            result->stats->rowcols->length_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHL, 31);
+            result->stats->rowcols->score_row[j] = (int32_t) _mm256_extract_epi32_rpl (vH, 7);
+            result->stats->rowcols->matches_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHM, 7);
+            result->stats->rowcols->similar_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHS, 7);
+            result->stats->rowcols->length_row[j] = (int32_t) _mm256_extract_epi32_rpl (vHL, 7);
         }
 #endif
     }
 
     /* Trace the alignment ending position on read. */
     {
-        int8_t *t = (int8_t*)pvHMax;
-        int8_t *m = (int8_t*)pvHMMax;
-        int8_t *s = (int8_t*)pvHSMax;
-        int8_t *l = (int8_t*)pvHLMax;
+        int32_t *t = (int32_t*)pvHMax;
+        int32_t *m = (int32_t*)pvHMMax;
+        int32_t *s = (int32_t*)pvHSMax;
+        int32_t *l = (int32_t*)pvHLMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -657,8 +607,8 @@ parasail_result_t* PNAME(
 #endif
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi8_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
