@@ -21,7 +21,7 @@
 #define SWAP(A,B) { vec128i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { vec128i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF INT16_MIN
+#define NEG_INF (INT32_MIN/(int32_t)(2))
 
 
 #ifdef PARASAIL_TABLE
@@ -31,17 +31,12 @@ static inline void arr_store(
         int32_t t,
         int32_t seglen,
         int32_t d,
-        int32_t dlen,
-        int32_t bias)
+        int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0) - bias;
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1) - bias;
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2) - bias;
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3) - bias;
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4) - bias;
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5) - bias;
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6) - bias;
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7) - bias;
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
 }
 #endif
 
@@ -50,39 +45,34 @@ static inline void arr_store_col(
         int *col,
         vec128i vH,
         int32_t t,
-        int32_t seglen,
-        int32_t bias)
+        int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0) - bias;
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1) - bias;
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2) - bias;
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3) - bias;
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4) - bias;
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5) - bias;
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6) - bias;
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7) - bias;
+    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
+    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
+    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
+    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_striped_altivec_128_16
-#define PNAME parasail_sw_stats_table_striped_profile_altivec_128_16
+#define FNAME parasail_sw_stats_table_striped_altivec_128_32
+#define PNAME parasail_sw_stats_table_striped_profile_altivec_128_32
 #define INAME PNAME
 #define STATIC
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_striped_altivec_128_16
-#define PNAME parasail_sw_stats_rowcol_striped_profile_altivec_128_16
+#define FNAME parasail_sw_stats_rowcol_striped_altivec_128_32
+#define PNAME parasail_sw_stats_rowcol_striped_profile_altivec_128_32
 #define INAME PNAME
 #define STATIC
 #else
-#define FNAME parasail_sw_stats_striped_altivec_128_16
+#define FNAME parasail_sw_stats_striped_altivec_128_32
 #ifdef FASTSTATS
-#define PNAME parasail_sw_stats_striped_profile_altivec_128_16_internal
-#define INAME parasail_sw_stats_striped_profile_altivec_128_16
+#define PNAME parasail_sw_stats_striped_profile_altivec_128_32_internal
+#define INAME parasail_sw_stats_striped_profile_altivec_128_32
 #define STATIC static
 #else
-#define PNAME parasail_sw_stats_striped_profile_altivec_128_16
+#define PNAME parasail_sw_stats_striped_profile_altivec_128_32
 #define INAME PNAME
 #define STATIC
 #endif
@@ -113,7 +103,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_altivec_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_altivec_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = INAME(profile, s2, s2Len, open, gap);
 
@@ -164,24 +154,20 @@ STATIC parasail_result_t* PNAME(
     vec128i vGapE;
     vec128i vZero;
     vec128i vOne;
-    int16_t bias = 0;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
-    vec128i vBias;
-    vec128i vBias1;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
     vec128i vMaxH;
     vec128i vMaxHUnit;
-    vec128i insert_mask;
     vec128i vSaturationCheckMax;
     vec128i vPosLimit;
-    int16_t maxp = 0;
+    int32_t maxp = 0;
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -197,34 +183,28 @@ STATIC parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile  = (vec128i*)profile->profile16.score;
-    vProfileM = (vec128i*)profile->profile16.matches;
-    vProfileS = (vec128i*)profile->profile16.similar;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
+    vProfile  = (vec128i*)profile->profile32.score;
+    vProfileM = (vec128i*)profile->profile32.matches;
+    vProfileS = (vec128i*)profile->profile32.similar;
+    vGapO = _mm_set1_epi32(open);
+    vGapE = _mm_set1_epi32(gap);
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi16(1);
-    bias = INT16_MIN;
-    score = bias;
-    matches = bias;
-    similar = bias;
-    length = bias;
-    vBias = _mm_set1_epi16(bias);
-    vBias1 = _mm_adds_epi16(vBias,vOne);
-    vMaxH = vBias;
-    vMaxHUnit = vBias;
-    insert_mask = _mm_cmpgt_epi16(
-            _mm_set_epi16(0,0,0,0,0,0,0,1),
-            vZero);
-    vSaturationCheckMax = vBias;
-    vPosLimit = _mm_set1_epi16(INT16_MAX);
-    maxp = INT16_MAX - (int16_t)(matrix->max+1);
+    vOne = _mm_set1_epi32(1);
+    score = NEG_INF;
+    matches = NEG_INF;
+    similar = NEG_INF;
+    length = NEG_INF;
+    vMaxH = vZero;
+    vMaxHUnit = vZero;
+    vSaturationCheckMax = vZero;
+    vPosLimit = _mm_set1_epi32(INT32_MAX);
+    maxp = INT32_MAX - (int32_t)(matrix->max+1);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -241,7 +221,7 @@ STATIC parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -285,14 +265,14 @@ STATIC parasail_result_t* PNAME(
     if (!pvHSMax) return NULL;
     if (!pvHLMax) return NULL;
 
-    parasail_memset_vec128i(pvHStore, vBias, segLen);
-    parasail_memset_vec128i(pvHMStore, vBias, segLen);
-    parasail_memset_vec128i(pvHSStore, vBias, segLen);
-    parasail_memset_vec128i(pvHLStore, vBias, segLen);
-    parasail_memset_vec128i(pvE, vBias, segLen);
-    parasail_memset_vec128i(pvEM, vBias, segLen);
-    parasail_memset_vec128i(pvES, vBias, segLen);
-    parasail_memset_vec128i(pvEL, vBias1, segLen);
+    parasail_memset_vec128i(pvHStore, vZero, segLen);
+    parasail_memset_vec128i(pvHMStore, vZero, segLen);
+    parasail_memset_vec128i(pvHSStore, vZero, segLen);
+    parasail_memset_vec128i(pvHLStore, vZero, segLen);
+    parasail_memset_vec128i(pvE, _mm_set1_epi32(-open), segLen);
+    parasail_memset_vec128i(pvEM, vZero, segLen);
+    parasail_memset_vec128i(pvES, vZero, segLen);
+    parasail_memset_vec128i(pvEL, vOne, segLen);
 
     /* outer loop over database sequence */
     for (j=0; j<s2Len; ++j) {
@@ -318,24 +298,20 @@ STATIC parasail_result_t* PNAME(
 
         /* Initialize F value to 0.  Any errors to vH values will be
          * corrected in the Lazy_F loop. */
-        vF = vBias;
-        vFM = vBias;
-        vFS = vBias;
-        vFL = vBias1;
+        vF = vZero;
+        vFM = vZero;
+        vFS = vZero;
+        vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 4 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
         vHM = _mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = _mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = _mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 2);
-        vHM = _mm_slli_si128(vHM, 2);
-        vHS = _mm_slli_si128(vHS, 2);
-        vHL = _mm_slli_si128(vHL, 2);
-        vH = _mm_blendv_epi8(vH, vBias, insert_mask);
-        vHM = _mm_blendv_epi8(vHM, vBias, insert_mask);
-        vHS = _mm_blendv_epi8(vHS, vBias, insert_mask);
-        vHL = _mm_blendv_epi8(vHL, vBias, insert_mask);
+        vH = _mm_slli_si128(vH, 4);
+        vHM = _mm_slli_si128(vHM, 4);
+        vHS = _mm_slli_si128(vHS, 4);
+        vHL = _mm_slli_si128(vHL, 4);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -369,59 +345,59 @@ STATIC parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
-            /*vH = _mm_max_epi16(vH_dag, vBias);*/
-            vH = _mm_max_epi16(vH_dag, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH_dag = _mm_add_epi32(vH, _mm_load_si128(vP + i));
+            vH_dag = _mm_max_epi32(vH_dag, vZero);
+            vH = _mm_max_epi32(vH_dag, vE);
+            vH = _mm_max_epi32(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            cond_zero = _mm_cmpeq_epi16(vH, vBias);
+            cond_zero = _mm_cmpeq_epi32(vH, vZero);
 
-            case1 = _mm_cmpeq_epi16(vH, vH_dag);
-            case2 = _mm_cmpeq_epi16(vH, vF);
+            case1 = _mm_cmpeq_epi32(vH, vH_dag);
+            case2 = _mm_cmpeq_epi32(vH, vF);
 
             /* calculate vM */
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_adds_epi16(vHM, _mm_load_si128(vPM + i)), case1);
-            vHM = _mm_blendv_epi8(vHM, vBias, cond_zero);
+                    _mm_add_epi32(vHM, _mm_load_si128(vPM + i)), case1);
+            vHM = _mm_andnot_si128(cond_zero, vHM);
             _mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm_blendv_epi8(
                     _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_adds_epi16(vHS, _mm_load_si128(vPS + i)), case1);
-            vHS = _mm_blendv_epi8(vHS, vBias, cond_zero);
+                    _mm_add_epi32(vHS, _mm_load_si128(vPS + i)), case1);
+            vHS = _mm_andnot_si128(cond_zero, vHS);
             _mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_adds_epi16(vHL, vOne), case1);
-            vHL = _mm_blendv_epi8(vHL, vBias, cond_zero);
+                    _mm_add_epi32(vHL, vOne), case1);
+            vHL = _mm_andnot_si128(cond_zero, vHL);
             _mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
-            arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
-            arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
-            arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
-            arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
+            arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
+            arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
+            arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
+            arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vMaxH = _mm_max_epi16(vH, vMaxH);
-            vEF_opn = _mm_subs_epi16(vH, vGapO);
+            vMaxH = _mm_max_epi32(vH, vMaxH);
+            vEF_opn = _mm_sub_epi32(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_subs_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vEF_opn, vE_ext);
-            case1 = _mm_cmpgt_epi16(vEF_opn, vE_ext);
+            vE_ext = _mm_sub_epi32(vE, vGapE);
+            vE = _mm_max_epi32(vEF_opn, vE_ext);
+            case1 = _mm_cmpgt_epi32(vEF_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(
-                    _mm_adds_epi16(vEL, vOne),
-                    _mm_adds_epi16(vHL, vOne),
+                    _mm_add_epi32(vEL, vOne),
+                    _mm_add_epi32(vHL, vOne),
                     case1);
             _mm_store_si128(pvE + i, vE);
             _mm_store_si128(pvEM + i, vEM);
@@ -429,14 +405,14 @@ STATIC parasail_result_t* PNAME(
             _mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vEF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+            vF_ext = _mm_sub_epi32(vF, vGapE);
+            vF = _mm_max_epi32(vEF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi32(vEF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_adds_epi16(vFL, vOne),
-                    _mm_adds_epi16(vHL, vOne),
+                    _mm_add_epi32(vFL, vOne),
+                    _mm_add_epi32(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -450,27 +426,25 @@ STATIC parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             vec128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = _mm_slli_si128(vHp, 2);
-            vF = _mm_slli_si128(vF, 2);
-            vFM = _mm_slli_si128(vFM, 2);
-            vFS = _mm_slli_si128(vFS, 2);
-            vFL = _mm_slli_si128(vFL, 2);
-            vHp = _mm_blendv_epi8(vHp, vBias, insert_mask);
-            vF = _mm_blendv_epi8(vF, vBias, insert_mask);
-            vFM = _mm_blendv_epi8(vFM, vBias, insert_mask);
-            vFS = _mm_blendv_epi8(vFS, vBias, insert_mask);
-            vFL = _mm_blendv_epi8(vFL, vBias1, insert_mask);
+            vHp = _mm_slli_si128(vHp, 4);
+            vF = _mm_slli_si128(vF, 4);
+            vF = _mm_insert_epi32(vF, -open, 0);
+            vFM = _mm_slli_si128(vFM, 4);
+            vFS = _mm_slli_si128(vFS, 4);
+            vFL = _mm_slli_si128(vFL, 4);
+            vFL = _mm_insert_epi32(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 vec128i case1;
                 vec128i case2;
                 vec128i cond;
 
-                vHp = _mm_adds_epi16(vHp, _mm_load_si128(vP + i));
+                vHp = _mm_add_epi32(vHp, _mm_load_si128(vP + i));
+                vHp = _mm_max_epi32(vHp, vZero);
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH, vF);
+                vH = _mm_max_epi32(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                case1 = _mm_cmpeq_epi16(vH, vHp);
-                case2 = _mm_cmpeq_epi16(vH, vF);
+                case1 = _mm_cmpeq_epi32(vH, vHp);
+                case2 = _mm_cmpeq_epi32(vH, vF);
                 cond = _mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -488,34 +462,34 @@ STATIC parasail_result_t* PNAME(
                 vHL = _mm_blendv_epi8(vHL, vFL, cond);
                 _mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
-                arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
-                arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
-                arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
-                arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
+                arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
+                arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
+                arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
+                arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vMaxH = _mm_max_epi16(vH, vMaxH);
+                vMaxH = _mm_max_epi32(vH, vMaxH);
                 /* Update vF value. */
-                vEF_opn = _mm_subs_epi16(vH, vGapO);
-                vF_ext = _mm_subs_epi16(vF, vGapE);
+                vEF_opn = _mm_sub_epi32(vH, vGapO);
+                vF_ext = _mm_sub_epi32(vF, vGapE);
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm_cmpgt_epi32(vF_ext, vEF_opn),
                                 _mm_and_si128(
-                                    _mm_cmpeq_epi16(vF_ext, vEF_opn),
-                                    _mm_cmpgt_epi16(vF_ext, vBias)))))
+                                    _mm_cmpeq_epi32(vF_ext, vEF_opn),
+                                    _mm_cmpgt_epi32(vF_ext, vZero)))))
                     goto end;
-                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi32(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+                cond = _mm_cmpgt_epi32(vEF_opn, vF_ext);
                 vFM = _mm_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm_blendv_epi8(
-                        _mm_adds_epi16(vFL, vOne),
-                        _mm_adds_epi16(vHL, vOne),
+                        _mm_add_epi32(vFL, vOne),
+                        _mm_add_epi32(vHL, vOne),
                         cond);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
@@ -532,28 +506,28 @@ end:
             vHS = _mm_load_si128(pvHSStore + offset);
             vHL = _mm_load_si128(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
-                vHM = _mm_slli_si128(vHM, 2);
-                vHS = _mm_slli_si128(vHS, 2);
-                vHL = _mm_slli_si128(vHL, 2);
+                vH = _mm_slli_si128(vH, 4);
+                vHM = _mm_slli_si128(vHM, 4);
+                vHS = _mm_slli_si128(vHS, 4);
+                vHL = _mm_slli_si128(vHL, 4);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7) - bias;
-            result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7) - bias;
-            result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7) - bias;
-            result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7) - bias;
+            result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
+            result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32 (vHM, 3);
+            result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32 (vHS, 3);
+            result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32 (vHL, 3);
         }
 #endif
 
         {
-            vec128i vCompare = _mm_cmpgt_epi16(vMaxH, vMaxHUnit);
+            vec128i vCompare = _mm_cmpgt_epi32(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi16(vMaxH);
+                score = _mm_hmax_epi32(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm_set1_epi16(score);
+                vMaxHUnit = _mm_set1_epi32(score);
                 end_ref = j;
             }
         }
@@ -565,15 +539,15 @@ end:
         vec128i vHM = _mm_load_si128(pvHMStore+i);
         vec128i vHS = _mm_load_si128(pvHSStore+i);
         vec128i vHL = _mm_load_si128(pvHLStore+i);
-        arr_store_col(result->stats->rowcols->score_col, vH, i, segLen, bias);
-        arr_store_col(result->stats->rowcols->matches_col, vHM, i, segLen, bias);
-        arr_store_col(result->stats->rowcols->similar_col, vHS, i, segLen, bias);
-        arr_store_col(result->stats->rowcols->length_col, vHL, i, segLen, bias);
+        arr_store_col(result->stats->rowcols->score_col, vH, i, segLen);
+        arr_store_col(result->stats->rowcols->matches_col, vHM, i, segLen);
+        arr_store_col(result->stats->rowcols->similar_col, vHS, i, segLen);
+        arr_store_col(result->stats->rowcols->length_col, vHL, i, segLen);
     }
 #endif
 
-    if (score == INT16_MAX
-            || _mm_movemask_epi8(_mm_cmpeq_epi16(vSaturationCheckMax,vPosLimit))) {
+    if (score == INT32_MAX
+            || _mm_movemask_epi8(_mm_cmpeq_epi32(vSaturationCheckMax,vPosLimit))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -602,10 +576,10 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int16_t *t = (int16_t*)pvHMax;
-            int16_t *m = (int16_t*)pvHMMax;
-            int16_t *s = (int16_t*)pvHSMax;
-            int16_t *l = (int16_t*)pvHLMax;
+            int32_t *t = (int32_t*)pvHMax;
+            int32_t *m = (int32_t*)pvHMMax;
+            int32_t *s = (int32_t*)pvHSMax;
+            int32_t *l = (int32_t*)pvHLMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len;
             for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -622,12 +596,12 @@ end:
         }
     }
 
-    result->score = score - bias;
+    result->score = score;
     result->end_query = end_query;
     result->end_ref = end_ref;
-    result->stats->matches = matches - bias;
-    result->stats->similar = similar - bias;
-    result->stats->length = length - bias;
+    result->stats->matches = matches;
+    result->stats->similar = similar;
+    result->stats->length = length;
 
     parasail_free(pvHLMax);
     parasail_free(pvHSMax);
@@ -664,7 +638,7 @@ parasail_result_t* INAME(
     const parasail_matrix_t *matrix = profile->matrix;
 
     /* find the end loc first with the faster implementation */
-    parasail_result_t *result = parasail_sw_striped_profile_altivec_128_16(profile, s2, s2Len, open, gap);
+    parasail_result_t *result = parasail_sw_striped_profile_altivec_128_32(profile, s2, s2Len, open, gap);
     if (!parasail_result_is_saturated(result)) {
 #if 0
         int s1Len_new = 0;
@@ -686,10 +660,10 @@ parasail_result_t* INAME(
         s2Len_new = result->end_ref+1;
         s1_new = parasail_reverse(s1, s1Len_new);
         s2_new = parasail_reverse(s2, s2Len_new);
-        profile_new = parasail_profile_create_altivec_128_16(
+        profile_new = parasail_profile_create_altivec_128_32(
                 s1_new, s1Len_new, matrix);
         profile_new->stop = result->score;
-        result_new = parasail_sw_striped_profile_altivec_128_16(
+        result_new = parasail_sw_striped_profile_altivec_128_32(
                 profile_new, s2_new, s2Len_new, open, gap);
 
         /* using both the beg and end loc, call the original stats func */
@@ -701,7 +675,7 @@ parasail_result_t* INAME(
         assert(s2_begin >= 0);
         assert(s1Len_new > s1_begin);
         assert(s2Len_new > s2_begin);
-        profile_final = parasail_profile_create_stats_altivec_128_16(
+        profile_final = parasail_profile_create_stats_altivec_128_32(
                 &s1[s1_begin], s1Len_final, matrix);
         result_final = PNAME(
                 profile_final, &s2[s2_begin], s2Len_final, open, gap);
@@ -727,7 +701,7 @@ parasail_result_t* INAME(
         /* using the end loc, call the original stats function */
         s1Len_new = result->end_query+1;
         s2Len_new = result->end_ref+1;
-        profile_final = parasail_profile_create_stats_altivec_128_16(
+        profile_final = parasail_profile_create_stats_altivec_128_32(
                 s1, s1Len_new, matrix);
         result_final = PNAME(
                 profile_final, s2, s2Len_new, open, gap);
