@@ -21,13 +21,13 @@
 #define SWAP(A,B) { __m256i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { __m256i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF INT16_MIN
+#define NEG_INF INT8_MIN
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI16
-#define _mm256_extract_epi16_rpl _mm256_extract_epi16
+#if HAVE_AVX2_MM256_EXTRACT_EPI8
+#define _mm256_extract_epi8_rpl _mm256_extract_epi8
 #else
-static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
-    __m256i_16_t A;
+static inline int8_t _mm256_extract_epi8_rpl(__m256i a, int imm) {
+    __m256i_8_t A;
     A.m = a;
     return A.v[imm];
 }
@@ -35,12 +35,13 @@ static inline int16_t _mm256_extract_epi16_rpl(__m256i a, int imm) {
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
-static inline int16_t _mm256_hmax_epi16_rpl(__m256i a) {
-    a = _mm256_max_epi16(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
-    a = _mm256_max_epi16(a, _mm256_slli_si256(a, 8));
-    a = _mm256_max_epi16(a, _mm256_slli_si256(a, 4));
-    a = _mm256_max_epi16(a, _mm256_slli_si256(a, 2));
-    return _mm256_extract_epi16_rpl(a, 15);
+static inline int8_t _mm256_hmax_epi8_rpl(__m256i a) {
+    a = _mm256_max_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
+    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 8));
+    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 4));
+    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 2));
+    a = _mm256_max_epi8(a, _mm256_slli_si256(a, 1));
+    return _mm256_extract_epi8_rpl(a, 31);
 }
 
 
@@ -54,22 +55,38 @@ static inline void arr_store(
         int32_t dlen,
         int32_t bias)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  0) - bias;
-    array[1LL*( 1*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  1) - bias;
-    array[1LL*( 2*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  2) - bias;
-    array[1LL*( 3*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  3) - bias;
-    array[1LL*( 4*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  4) - bias;
-    array[1LL*( 5*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  5) - bias;
-    array[1LL*( 6*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  6) - bias;
-    array[1LL*( 7*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  7) - bias;
-    array[1LL*( 8*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  8) - bias;
-    array[1LL*( 9*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH,  9) - bias;
-    array[1LL*(10*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 10) - bias;
-    array[1LL*(11*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 11) - bias;
-    array[1LL*(12*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 12) - bias;
-    array[1LL*(13*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 13) - bias;
-    array[1LL*(14*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 14) - bias;
-    array[1LL*(15*seglen+t)*dlen + d] = (int16_t)_mm256_extract_epi16_rpl(vH, 15) - bias;
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  0) - bias;
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  1) - bias;
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  2) - bias;
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  3) - bias;
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  4) - bias;
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  5) - bias;
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  6) - bias;
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  7) - bias;
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  8) - bias;
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH,  9) - bias;
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 10) - bias;
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 11) - bias;
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 12) - bias;
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 13) - bias;
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 14) - bias;
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 15) - bias;
+    array[1LL*(16*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 16) - bias;
+    array[1LL*(17*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 17) - bias;
+    array[1LL*(18*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 18) - bias;
+    array[1LL*(19*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 19) - bias;
+    array[1LL*(20*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 20) - bias;
+    array[1LL*(21*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 21) - bias;
+    array[1LL*(22*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 22) - bias;
+    array[1LL*(23*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 23) - bias;
+    array[1LL*(24*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 24) - bias;
+    array[1LL*(25*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 25) - bias;
+    array[1LL*(26*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 26) - bias;
+    array[1LL*(27*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 27) - bias;
+    array[1LL*(28*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 28) - bias;
+    array[1LL*(29*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 29) - bias;
+    array[1LL*(30*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 30) - bias;
+    array[1LL*(31*seglen+t)*dlen + d] = (int8_t)_mm256_extract_epi8_rpl(vH, 31) - bias;
 }
 #endif
 
@@ -81,44 +98,60 @@ static inline void arr_store_col(
         int32_t seglen,
         int32_t bias)
 {
-    col[ 0*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  0) - bias;
-    col[ 1*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  1) - bias;
-    col[ 2*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  2) - bias;
-    col[ 3*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  3) - bias;
-    col[ 4*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  4) - bias;
-    col[ 5*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  5) - bias;
-    col[ 6*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  6) - bias;
-    col[ 7*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  7) - bias;
-    col[ 8*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  8) - bias;
-    col[ 9*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH,  9) - bias;
-    col[10*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 10) - bias;
-    col[11*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 11) - bias;
-    col[12*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 12) - bias;
-    col[13*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 13) - bias;
-    col[14*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 14) - bias;
-    col[15*seglen+t] = (int16_t)_mm256_extract_epi16_rpl(vH, 15) - bias;
+    col[ 0*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  0) - bias;
+    col[ 1*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  1) - bias;
+    col[ 2*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  2) - bias;
+    col[ 3*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  3) - bias;
+    col[ 4*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  4) - bias;
+    col[ 5*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  5) - bias;
+    col[ 6*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  6) - bias;
+    col[ 7*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  7) - bias;
+    col[ 8*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  8) - bias;
+    col[ 9*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH,  9) - bias;
+    col[10*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 10) - bias;
+    col[11*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 11) - bias;
+    col[12*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 12) - bias;
+    col[13*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 13) - bias;
+    col[14*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 14) - bias;
+    col[15*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 15) - bias;
+    col[16*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 16) - bias;
+    col[17*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 17) - bias;
+    col[18*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 18) - bias;
+    col[19*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 19) - bias;
+    col[20*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 20) - bias;
+    col[21*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 21) - bias;
+    col[22*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 22) - bias;
+    col[23*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 23) - bias;
+    col[24*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 24) - bias;
+    col[25*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 25) - bias;
+    col[26*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 26) - bias;
+    col[27*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 27) - bias;
+    col[28*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 28) - bias;
+    col[29*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 29) - bias;
+    col[30*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 30) - bias;
+    col[31*seglen+t] = (int8_t)_mm256_extract_epi8_rpl(vH, 31) - bias;
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_striped_avx2_256_16
-#define PNAME parasail_sw_stats_table_striped_profile_avx2_256_16
+#define FNAME parasail_sw_stats_table_striped_avx2_256_8
+#define PNAME parasail_sw_stats_table_striped_profile_avx2_256_8
 #define INAME PNAME
 #define STATIC
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_striped_avx2_256_16
-#define PNAME parasail_sw_stats_rowcol_striped_profile_avx2_256_16
+#define FNAME parasail_sw_stats_rowcol_striped_avx2_256_8
+#define PNAME parasail_sw_stats_rowcol_striped_profile_avx2_256_8
 #define INAME PNAME
 #define STATIC
 #else
-#define FNAME parasail_sw_stats_striped_avx2_256_16
+#define FNAME parasail_sw_stats_striped_avx2_256_8
 #ifdef FASTSTATS
-#define PNAME parasail_sw_stats_striped_profile_avx2_256_16_internal
-#define INAME parasail_sw_stats_striped_profile_avx2_256_16
+#define PNAME parasail_sw_stats_striped_profile_avx2_256_8_internal
+#define INAME parasail_sw_stats_striped_profile_avx2_256_8
 #define STATIC static
 #else
-#define PNAME parasail_sw_stats_striped_profile_avx2_256_16
+#define PNAME parasail_sw_stats_striped_profile_avx2_256_8
 #define INAME PNAME
 #define STATIC
 #endif
@@ -149,7 +182,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_avx_256_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_avx_256_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = INAME(profile, s2, s2Len, open, gap);
 
@@ -200,11 +233,11 @@ STATIC parasail_result_t* PNAME(
     __m256i vGapE;
     __m256i vZero;
     __m256i vOne;
-    int16_t bias = 0;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int8_t bias = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
     __m256i vBias;
     __m256i vBias1;
     __m256i vMaxH;
@@ -212,12 +245,12 @@ STATIC parasail_result_t* PNAME(
     __m256i insert_mask;
     __m256i vSaturationCheckMax;
     __m256i vPosLimit;
-    int16_t maxp = 0;
+    int8_t maxp = 0;
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -233,34 +266,34 @@ STATIC parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 32; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile  = (__m256i*)profile->profile16.score;
-    vProfileM = (__m256i*)profile->profile16.matches;
-    vProfileS = (__m256i*)profile->profile16.similar;
-    vGapO = _mm256_set1_epi16(open);
-    vGapE = _mm256_set1_epi16(gap);
+    vProfile  = (__m256i*)profile->profile8.score;
+    vProfileM = (__m256i*)profile->profile8.matches;
+    vProfileS = (__m256i*)profile->profile8.similar;
+    vGapO = _mm256_set1_epi8(open);
+    vGapE = _mm256_set1_epi8(gap);
     vZero = _mm256_setzero_si256();
-    vOne = _mm256_set1_epi16(1);
-    bias = INT16_MIN;
+    vOne = _mm256_set1_epi8(1);
+    bias = INT8_MIN;
     score = bias;
     matches = bias;
     similar = bias;
     length = bias;
-    vBias = _mm256_set1_epi16(bias);
-    vBias1 = _mm256_adds_epi16(vBias,vOne);
+    vBias = _mm256_set1_epi8(bias);
+    vBias1 = _mm256_adds_epi8(vBias,vOne);
     vMaxH = vBias;
     vMaxHUnit = vBias;
-    insert_mask = _mm256_cmpgt_epi16(
-            _mm256_set_epi16(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
+    insert_mask = _mm256_cmpgt_epi8(
+            _mm256_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
             vZero);
     vSaturationCheckMax = vBias;
-    vPosLimit = _mm256_set1_epi16(INT16_MAX);
-    maxp = INT16_MAX - (int16_t)(matrix->max+1);
+    vPosLimit = _mm256_set1_epi8(INT8_MAX);
+    maxp = INT8_MAX - (int8_t)(matrix->max+1);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -277,7 +310,7 @@ STATIC parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_32;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -359,15 +392,15 @@ STATIC parasail_result_t* PNAME(
         vFS = vBias;
         vFL = vBias1;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = _mm256_load_si256(&pvHStore[segLen - 1]);
         vHM = _mm256_load_si256(&pvHMStore[segLen - 1]);
         vHS = _mm256_load_si256(&pvHSStore[segLen - 1]);
         vHL = _mm256_load_si256(&pvHLStore[segLen - 1]);
-        vH = _mm256_slli_si256_rpl(vH, 2);
-        vHM = _mm256_slli_si256_rpl(vHM, 2);
-        vHS = _mm256_slli_si256_rpl(vHS, 2);
-        vHL = _mm256_slli_si256_rpl(vHL, 2);
+        vH = _mm256_slli_si256_rpl(vH, 1);
+        vHM = _mm256_slli_si256_rpl(vHM, 1);
+        vHS = _mm256_slli_si256_rpl(vHS, 1);
+        vHL = _mm256_slli_si256_rpl(vHL, 1);
         vH = _mm256_blendv_epi8(vH, vBias, insert_mask);
         vHM = _mm256_blendv_epi8(vHM, vBias, insert_mask);
         vHS = _mm256_blendv_epi8(vHS, vBias, insert_mask);
@@ -405,59 +438,59 @@ STATIC parasail_result_t* PNAME(
             vEL = _mm256_load_si256(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm256_adds_epi16(vH, _mm256_load_si256(vP + i));
-            /*vH = _mm256_max_epi16(vH_dag, vBias);*/
-            vH = _mm256_max_epi16(vH_dag, vE);
-            vH = _mm256_max_epi16(vH, vF);
+            vH_dag = _mm256_adds_epi8(vH, _mm256_load_si256(vP + i));
+            /*vH = _mm256_max_epi8(vH_dag, vBias);*/
+            vH = _mm256_max_epi8(vH_dag, vE);
+            vH = _mm256_max_epi8(vH, vF);
             /* Save vH values. */
             _mm256_store_si256(pvHStore + i, vH);
-            cond_zero = _mm256_cmpeq_epi16(vH, vBias);
+            cond_zero = _mm256_cmpeq_epi8(vH, vBias);
 
-            case1 = _mm256_cmpeq_epi16(vH, vH_dag);
-            case2 = _mm256_cmpeq_epi16(vH, vF);
+            case1 = _mm256_cmpeq_epi8(vH, vH_dag);
+            case2 = _mm256_cmpeq_epi8(vH, vF);
 
             /* calculate vM */
             vHM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
-                    _mm256_adds_epi16(vHM, _mm256_load_si256(vPM + i)), case1);
+                    _mm256_adds_epi8(vHM, _mm256_load_si256(vPM + i)), case1);
             vHM = _mm256_blendv_epi8(vHM, vBias, cond_zero);
             _mm256_store_si256(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vES, vFS, case2),
-                    _mm256_adds_epi16(vHS, _mm256_load_si256(vPS + i)), case1);
+                    _mm256_adds_epi8(vHS, _mm256_load_si256(vPS + i)), case1);
             vHS = _mm256_blendv_epi8(vHS, vBias, cond_zero);
             _mm256_store_si256(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEL, vFL, case2),
-                    _mm256_adds_epi16(vHL, vOne), case1);
+                    _mm256_adds_epi8(vHL, vOne), case1);
             vHL = _mm256_blendv_epi8(vHL, vBias, cond_zero);
             _mm256_store_si256(pvHLStore + i, vHL);
 
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
             arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
             arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
             arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-            vMaxH = _mm256_max_epi16(vH, vMaxH);
-            vEF_opn = _mm256_subs_epi16(vH, vGapO);
+            vMaxH = _mm256_max_epi8(vH, vMaxH);
+            vEF_opn = _mm256_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm256_subs_epi16(vE, vGapE);
-            vE = _mm256_max_epi16(vEF_opn, vE_ext);
-            case1 = _mm256_cmpgt_epi16(vEF_opn, vE_ext);
+            vE_ext = _mm256_subs_epi8(vE, vGapE);
+            vE = _mm256_max_epi8(vEF_opn, vE_ext);
+            case1 = _mm256_cmpgt_epi8(vEF_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vHM, case1);
             vES = _mm256_blendv_epi8(vES, vHS, case1);
             vEL = _mm256_blendv_epi8(
-                    _mm256_adds_epi16(vEL, vOne),
-                    _mm256_adds_epi16(vHL, vOne),
+                    _mm256_adds_epi8(vEL, vOne),
+                    _mm256_adds_epi8(vHL, vOne),
                     case1);
             _mm256_store_si256(pvE + i, vE);
             _mm256_store_si256(pvEM + i, vEM);
@@ -465,14 +498,14 @@ STATIC parasail_result_t* PNAME(
             _mm256_store_si256(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm256_subs_epi16(vF, vGapE);
-            vF = _mm256_max_epi16(vEF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi16(vEF_opn, vF_ext);
+            vF_ext = _mm256_subs_epi8(vF, vGapE);
+            vF = _mm256_max_epi8(vEF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi8(vEF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vHM, case1);
             vFS = _mm256_blendv_epi8(vFS, vHS, case1);
             vFL = _mm256_blendv_epi8(
-                    _mm256_adds_epi16(vFL, vOne),
-                    _mm256_adds_epi16(vHL, vOne),
+                    _mm256_adds_epi8(vFL, vOne),
+                    _mm256_adds_epi8(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -486,11 +519,11 @@ STATIC parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             __m256i vHp = _mm256_load_si256(&pvHLoad[segLen - 1]);
-            vHp = _mm256_slli_si256_rpl(vHp, 2);
-            vF = _mm256_slli_si256_rpl(vF, 2);
-            vFM = _mm256_slli_si256_rpl(vFM, 2);
-            vFS = _mm256_slli_si256_rpl(vFS, 2);
-            vFL = _mm256_slli_si256_rpl(vFL, 2);
+            vHp = _mm256_slli_si256_rpl(vHp, 1);
+            vF = _mm256_slli_si256_rpl(vF, 1);
+            vFM = _mm256_slli_si256_rpl(vFM, 1);
+            vFS = _mm256_slli_si256_rpl(vFS, 1);
+            vFL = _mm256_slli_si256_rpl(vFL, 1);
             vHp = _mm256_blendv_epi8(vHp, vBias, insert_mask);
             vF = _mm256_blendv_epi8(vF, vBias, insert_mask);
             vFM = _mm256_blendv_epi8(vFM, vBias, insert_mask);
@@ -501,12 +534,12 @@ STATIC parasail_result_t* PNAME(
                 __m256i case2;
                 __m256i cond;
 
-                vHp = _mm256_adds_epi16(vHp, _mm256_load_si256(vP + i));
+                vHp = _mm256_adds_epi8(vHp, _mm256_load_si256(vP + i));
                 vH = _mm256_load_si256(pvHStore + i);
-                vH = _mm256_max_epi16(vH, vF);
+                vH = _mm256_max_epi8(vH, vF);
                 _mm256_store_si256(pvHStore + i, vH);
-                case1 = _mm256_cmpeq_epi16(vH, vHp);
-                case2 = _mm256_cmpeq_epi16(vH, vF);
+                case1 = _mm256_cmpeq_epi8(vH, vHp);
+                case2 = _mm256_cmpeq_epi8(vH, vF);
                 cond = _mm256_andnot_si256(case1, case2);
 
                 /* calculate vM */
@@ -524,34 +557,34 @@ STATIC parasail_result_t* PNAME(
                 vHL = _mm256_blendv_epi8(vHL, vFL, cond);
                 _mm256_store_si256(pvHLStore + i, vHL);
 
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vHL);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm256_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
                 arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
                 arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
                 arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-                vMaxH = _mm256_max_epi16(vH, vMaxH);
+                vMaxH = _mm256_max_epi8(vH, vMaxH);
                 /* Update vF value. */
-                vEF_opn = _mm256_subs_epi16(vH, vGapO);
-                vF_ext = _mm256_subs_epi16(vF, vGapE);
+                vEF_opn = _mm256_subs_epi8(vH, vGapO);
+                vF_ext = _mm256_subs_epi8(vF, vGapE);
                 if (! _mm256_movemask_epi8(
                             _mm256_or_si256(
-                                _mm256_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm256_cmpgt_epi8(vF_ext, vEF_opn),
                                 _mm256_and_si256(
-                                    _mm256_cmpeq_epi16(vF_ext, vEF_opn),
-                                    _mm256_cmpgt_epi16(vF_ext, vBias)))))
+                                    _mm256_cmpeq_epi8(vF_ext, vEF_opn),
+                                    _mm256_cmpgt_epi8(vF_ext, vBias)))))
                     goto end;
-                /*vF = _mm256_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm256_max_epi8(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm256_cmpgt_epi16(vEF_opn, vF_ext);
+                cond = _mm256_cmpgt_epi8(vEF_opn, vF_ext);
                 vFM = _mm256_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm256_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm256_blendv_epi8(
-                        _mm256_adds_epi16(vFL, vOne),
-                        _mm256_adds_epi16(vHL, vOne),
+                        _mm256_adds_epi8(vFL, vOne),
+                        _mm256_adds_epi8(vHL, vOne),
                         cond);
                 vHp = _mm256_load_si256(pvHLoad + i);
             }
@@ -568,28 +601,28 @@ end:
             vHS = _mm256_load_si256(pvHSStore + offset);
             vHL = _mm256_load_si256(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm256_slli_si256_rpl(vH, 2);
-                vHM = _mm256_slli_si256_rpl(vHM, 2);
-                vHS = _mm256_slli_si256_rpl(vHS, 2);
-                vHL = _mm256_slli_si256_rpl(vHL, 2);
+                vH = _mm256_slli_si256_rpl(vH, 1);
+                vHM = _mm256_slli_si256_rpl(vHM, 1);
+                vHS = _mm256_slli_si256_rpl(vHS, 1);
+                vHL = _mm256_slli_si256_rpl(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) _mm256_extract_epi16_rpl (vH, 15) - bias;
-            result->stats->rowcols->matches_row[j] = (int16_t) _mm256_extract_epi16_rpl (vHM, 15) - bias;
-            result->stats->rowcols->similar_row[j] = (int16_t) _mm256_extract_epi16_rpl (vHS, 15) - bias;
-            result->stats->rowcols->length_row[j] = (int16_t) _mm256_extract_epi16_rpl (vHL, 15) - bias;
+            result->stats->rowcols->score_row[j] = (int8_t) _mm256_extract_epi8_rpl (vH, 31) - bias;
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHM, 31) - bias;
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHS, 31) - bias;
+            result->stats->rowcols->length_row[j] = (int8_t) _mm256_extract_epi8_rpl (vHL, 31) - bias;
         }
 #endif
 
         {
-            __m256i vCompare = _mm256_cmpgt_epi16(vMaxH, vMaxHUnit);
+            __m256i vCompare = _mm256_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (_mm256_movemask_epi8(vCompare)) {
-                score = _mm256_hmax_epi16_rpl(vMaxH);
+                score = _mm256_hmax_epi8_rpl(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm256_set1_epi16(score);
+                vMaxHUnit = _mm256_set1_epi8(score);
                 end_ref = j;
             }
         }
@@ -608,8 +641,8 @@ end:
     }
 #endif
 
-    if (score == INT16_MAX
-            || _mm256_movemask_epi8(_mm256_cmpeq_epi16(vSaturationCheckMax,vPosLimit))) {
+    if (score == INT8_MAX
+            || _mm256_movemask_epi8(_mm256_cmpeq_epi8(vSaturationCheckMax,vPosLimit))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -638,10 +671,10 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int16_t *t = (int16_t*)pvHMax;
-            int16_t *m = (int16_t*)pvHMMax;
-            int16_t *s = (int16_t*)pvHSMax;
-            int16_t *l = (int16_t*)pvHLMax;
+            int8_t *t = (int8_t*)pvHMax;
+            int8_t *m = (int8_t*)pvHMMax;
+            int8_t *s = (int8_t*)pvHSMax;
+            int8_t *l = (int8_t*)pvHLMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len;
             for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -700,7 +733,7 @@ parasail_result_t* INAME(
     const parasail_matrix_t *matrix = profile->matrix;
 
     /* find the end loc first with the faster implementation */
-    parasail_result_t *result = parasail_sw_striped_profile_avx2_256_16(profile, s2, s2Len, open, gap);
+    parasail_result_t *result = parasail_sw_striped_profile_avx2_256_8(profile, s2, s2Len, open, gap);
     if (!parasail_result_is_saturated(result)) {
 #if 0
         int s1Len_new = 0;
@@ -722,10 +755,10 @@ parasail_result_t* INAME(
         s2Len_new = result->end_ref+1;
         s1_new = parasail_reverse(s1, s1Len_new);
         s2_new = parasail_reverse(s2, s2Len_new);
-        profile_new = parasail_profile_create_avx_256_16(
+        profile_new = parasail_profile_create_avx_256_8(
                 s1_new, s1Len_new, matrix);
         profile_new->stop = result->score;
-        result_new = parasail_sw_striped_profile_avx2_256_16(
+        result_new = parasail_sw_striped_profile_avx2_256_8(
                 profile_new, s2_new, s2Len_new, open, gap);
 
         /* using both the beg and end loc, call the original stats func */
@@ -737,7 +770,7 @@ parasail_result_t* INAME(
         assert(s2_begin >= 0);
         assert(s1Len_new > s1_begin);
         assert(s2Len_new > s2_begin);
-        profile_final = parasail_profile_create_stats_avx_256_16(
+        profile_final = parasail_profile_create_stats_avx_256_8(
                 &s1[s1_begin], s1Len_final, matrix);
         result_final = PNAME(
                 profile_final, &s2[s2_begin], s2Len_final, open, gap);
@@ -763,7 +796,7 @@ parasail_result_t* INAME(
         /* using the end loc, call the original stats function */
         s1Len_new = result->end_query+1;
         s2Len_new = result->end_ref+1;
-        profile_final = parasail_profile_create_stats_avx_256_16(
+        profile_final = parasail_profile_create_stats_avx_256_8(
                 s1, s1Len_new, matrix);
         result_final = PNAME(
                 profile_final, s2, s2Len_new, open, gap);
