@@ -21,7 +21,7 @@
 #define SWAP(A,B) { simde__m128i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { simde__m128i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF (INT64_MIN/(int64_t)(2))
+#define NEG_INF (INT32_MIN/(int32_t)(2))
 
 
 #ifdef PARASAIL_TABLE
@@ -33,8 +33,10 @@ static inline void arr_store(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
 }
 #endif
 
@@ -45,30 +47,32 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
+    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
+    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
+    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_striped_neon_128_64
-#define PNAME parasail_sw_stats_table_striped_profile_neon_128_64
+#define FNAME parasail_sw_stats_table_striped_neon_128_32
+#define PNAME parasail_sw_stats_table_striped_profile_neon_128_32
 #define INAME PNAME
 #define STATIC
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_striped_neon_128_64
-#define PNAME parasail_sw_stats_rowcol_striped_profile_neon_128_64
+#define FNAME parasail_sw_stats_rowcol_striped_neon_128_32
+#define PNAME parasail_sw_stats_rowcol_striped_profile_neon_128_32
 #define INAME PNAME
 #define STATIC
 #else
-#define FNAME parasail_sw_stats_striped_neon_128_64
+#define FNAME parasail_sw_stats_striped_neon_128_32
 #ifdef FASTSTATS
-#define PNAME parasail_sw_stats_striped_profile_neon_128_64_internal
-#define INAME parasail_sw_stats_striped_profile_neon_128_64
+#define PNAME parasail_sw_stats_striped_profile_neon_128_32_internal
+#define INAME parasail_sw_stats_striped_profile_neon_128_32
 #define STATIC static
 #else
-#define PNAME parasail_sw_stats_striped_profile_neon_128_64
+#define PNAME parasail_sw_stats_striped_profile_neon_128_32
 #define INAME PNAME
 #define STATIC
 #endif
@@ -99,7 +103,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_neon_128_64(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_neon_128_32(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = INAME(profile, s2, s2Len, open, gap);
 
@@ -150,20 +154,20 @@ STATIC parasail_result_t* PNAME(
     simde__m128i vGapE;
     simde__m128i vZero;
     simde__m128i vOne;
-    int64_t score = 0;
-    int64_t matches = 0;
-    int64_t similar = 0;
-    int64_t length = 0;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
     simde__m128i vMaxH;
     simde__m128i vMaxHUnit;
     simde__m128i vSaturationCheckMax;
     simde__m128i vPosLimit;
-    int64_t maxp = 0;
+    int32_t maxp = 0;
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -179,19 +183,19 @@ STATIC parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 2; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile  = (simde__m128i*)profile->profile64.score;
-    vProfileM = (simde__m128i*)profile->profile64.matches;
-    vProfileS = (simde__m128i*)profile->profile64.similar;
-    vGapO = simde_mm_set1_epi64x(open);
-    vGapE = simde_mm_set1_epi64x(gap);
+    vProfile  = (simde__m128i*)profile->profile32.score;
+    vProfileM = (simde__m128i*)profile->profile32.matches;
+    vProfileS = (simde__m128i*)profile->profile32.similar;
+    vGapO = simde_mm_set1_epi32(open);
+    vGapE = simde_mm_set1_epi32(gap);
     vZero = simde_mm_setzero_si128();
-    vOne = simde_mm_set1_epi64x(1);
+    vOne = simde_mm_set1_epi32(1);
     score = NEG_INF;
     matches = NEG_INF;
     similar = NEG_INF;
@@ -199,8 +203,8 @@ STATIC parasail_result_t* PNAME(
     vMaxH = vZero;
     vMaxHUnit = vZero;
     vSaturationCheckMax = vZero;
-    vPosLimit = simde_mm_set1_epi64x(INT64_MAX);
-    maxp = INT64_MAX - (int64_t)(matrix->max+1);
+    vPosLimit = simde_mm_set1_epi32(INT32_MAX);
+    maxp = INT32_MAX - (int32_t)(matrix->max+1);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -217,7 +221,7 @@ STATIC parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -265,7 +269,7 @@ STATIC parasail_result_t* PNAME(
     parasail_memset_simde__m128i(pvHMStore, vZero, segLen);
     parasail_memset_simde__m128i(pvHSStore, vZero, segLen);
     parasail_memset_simde__m128i(pvHLStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvE, simde_mm_set1_epi64x(-open), segLen);
+    parasail_memset_simde__m128i(pvE, simde_mm_set1_epi32(-open), segLen);
     parasail_memset_simde__m128i(pvEM, vZero, segLen);
     parasail_memset_simde__m128i(pvES, vZero, segLen);
     parasail_memset_simde__m128i(pvEL, vOne, segLen);
@@ -299,15 +303,15 @@ STATIC parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 8 bytes */
+        /* load final segment of pvHStore and shift left by 4 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
         vHM = simde_mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = simde_mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = simde_mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 8);
-        vHM = simde_mm_slli_si128(vHM, 8);
-        vHS = simde_mm_slli_si128(vHS, 8);
-        vHL = simde_mm_slli_si128(vHL, 8);
+        vH = simde_mm_slli_si128(vH, 4);
+        vHM = simde_mm_slli_si128(vHM, 4);
+        vHS = simde_mm_slli_si128(vHS, 4);
+        vHL = simde_mm_slli_si128(vHL, 4);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -341,59 +345,59 @@ STATIC parasail_result_t* PNAME(
             vEL = simde_mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_add_epi64(vH, simde_mm_load_si128(vP + i));
-            vH_dag = simde_mm_max_epi64(vH_dag, vZero);
-            vH = simde_mm_max_epi64(vH_dag, vE);
-            vH = simde_mm_max_epi64(vH, vF);
+            vH_dag = simde_mm_add_epi32(vH, simde_mm_load_si128(vP + i));
+            vH_dag = simde_mm_max_epi32(vH_dag, vZero);
+            vH = simde_mm_max_epi32(vH_dag, vE);
+            vH = simde_mm_max_epi32(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
-            cond_zero = simde_mm_cmpeq_epi64(vH, vZero);
+            cond_zero = simde_mm_cmpeq_epi32(vH, vZero);
 
-            case1 = simde_mm_cmpeq_epi64(vH, vH_dag);
-            case2 = simde_mm_cmpeq_epi64(vH, vF);
+            case1 = simde_mm_cmpeq_epi32(vH, vH_dag);
+            case2 = simde_mm_cmpeq_epi32(vH, vF);
 
             /* calculate vM */
             vHM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
-                    simde_mm_add_epi64(vHM, simde_mm_load_si128(vPM + i)), case1);
+                    simde_mm_add_epi32(vHM, simde_mm_load_si128(vPM + i)), case1);
             vHM = simde_mm_andnot_si128(cond_zero, vHM);
             simde_mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vES, vFS, case2),
-                    simde_mm_add_epi64(vHS, simde_mm_load_si128(vPS + i)), case1);
+                    simde_mm_add_epi32(vHS, simde_mm_load_si128(vPS + i)), case1);
             vHS = simde_mm_andnot_si128(cond_zero, vHS);
             simde_mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEL, vFL, case2),
-                    simde_mm_add_epi64(vHL, vOne), case1);
+                    simde_mm_add_epi32(vHL, vOne), case1);
             vHL = simde_mm_andnot_si128(cond_zero, vHL);
             simde_mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vMaxH = simde_mm_max_epi64(vH, vMaxH);
-            vEF_opn = simde_mm_sub_epi64(vH, vGapO);
+            vMaxH = simde_mm_max_epi32(vH, vMaxH);
+            vEF_opn = simde_mm_sub_epi32(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_sub_epi64(vE, vGapE);
-            vE = simde_mm_max_epi64(vEF_opn, vE_ext);
-            case1 = simde_mm_cmpgt_epi64(vEF_opn, vE_ext);
+            vE_ext = simde_mm_sub_epi32(vE, vGapE);
+            vE = simde_mm_max_epi32(vEF_opn, vE_ext);
+            case1 = simde_mm_cmpgt_epi32(vEF_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vHM, case1);
             vES = simde_mm_blendv_epi8(vES, vHS, case1);
             vEL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi64(vEL, vOne),
-                    simde_mm_add_epi64(vHL, vOne),
+                    simde_mm_add_epi32(vEL, vOne),
+                    simde_mm_add_epi32(vHL, vOne),
                     case1);
             simde_mm_store_si128(pvE + i, vE);
             simde_mm_store_si128(pvEM + i, vEM);
@@ -401,14 +405,14 @@ STATIC parasail_result_t* PNAME(
             simde_mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = simde_mm_sub_epi64(vF, vGapE);
-            vF = simde_mm_max_epi64(vEF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
+            vF_ext = simde_mm_sub_epi32(vF, vGapE);
+            vF = simde_mm_max_epi32(vEF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi32(vEF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vHM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vHS, case1);
             vFL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi64(vFL, vOne),
-                    simde_mm_add_epi64(vHL, vOne),
+                    simde_mm_add_epi32(vFL, vOne),
+                    simde_mm_add_epi32(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -422,25 +426,25 @@ STATIC parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = simde_mm_slli_si128(vHp, 8);
-            vF = simde_mm_slli_si128(vF, 8);
-            vF = simde_mm_insert_epi64(vF, -open, 0);
-            vFM = simde_mm_slli_si128(vFM, 8);
-            vFS = simde_mm_slli_si128(vFS, 8);
-            vFL = simde_mm_slli_si128(vFL, 8);
-            vFL = simde_mm_insert_epi64(vFL, 1, 0);
+            vHp = simde_mm_slli_si128(vHp, 4);
+            vF = simde_mm_slli_si128(vF, 4);
+            vF = simde_mm_insert_epi32(vF, -open, 0);
+            vFM = simde_mm_slli_si128(vFM, 4);
+            vFS = simde_mm_slli_si128(vFS, 4);
+            vFL = simde_mm_slli_si128(vFL, 4);
+            vFL = simde_mm_insert_epi32(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 simde__m128i case1;
                 simde__m128i case2;
                 simde__m128i cond;
 
-                vHp = simde_mm_add_epi64(vHp, simde_mm_load_si128(vP + i));
-                vHp = simde_mm_max_epi64(vHp, vZero);
+                vHp = simde_mm_add_epi32(vHp, simde_mm_load_si128(vP + i));
+                vHp = simde_mm_max_epi32(vHp, vZero);
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi64(vH,vF);
+                vH = simde_mm_max_epi32(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                case1 = simde_mm_cmpeq_epi64(vH, vHp);
-                case2 = simde_mm_cmpeq_epi64(vH, vF);
+                case1 = simde_mm_cmpeq_epi32(vH, vHp);
+                case2 = simde_mm_cmpeq_epi32(vH, vF);
                 cond = simde_mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -458,34 +462,34 @@ STATIC parasail_result_t* PNAME(
                 vHL = simde_mm_blendv_epi8(vHL, vFL, cond);
                 simde_mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = simde_mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
                 arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
                 arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vMaxH = simde_mm_max_epi64(vH, vMaxH);
+                vMaxH = simde_mm_max_epi32(vH, vMaxH);
                 /* Update vF value. */
-                vEF_opn = simde_mm_sub_epi64(vH, vGapO);
-                vF_ext = simde_mm_sub_epi64(vF, vGapE);
+                vEF_opn = simde_mm_sub_epi32(vH, vGapO);
+                vF_ext = simde_mm_sub_epi32(vF, vGapE);
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi64(vF_ext, vEF_opn),
+                                simde_mm_cmpgt_epi32(vF_ext, vEF_opn),
                                 simde_mm_and_si128(
-                                    simde_mm_cmpeq_epi64(vF_ext, vEF_opn),
-                                    simde_mm_cmpgt_epi64(vF_ext, vZero)))))
+                                    simde_mm_cmpeq_epi32(vF_ext, vEF_opn),
+                                    simde_mm_cmpgt_epi32(vF_ext, vZero)))))
                     goto end;
-                /*vF = simde_mm_max_epi64(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi32(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
+                cond = simde_mm_cmpgt_epi32(vEF_opn, vF_ext);
                 vFM = simde_mm_blendv_epi8(vFM, vHM, cond);
                 vFS = simde_mm_blendv_epi8(vFS, vHS, cond);
                 vFL = simde_mm_blendv_epi8(
-                        simde_mm_add_epi64(vFL, vOne),
-                        simde_mm_add_epi64(vHL, vOne),
+                        simde_mm_add_epi32(vFL, vOne),
+                        simde_mm_add_epi32(vHL, vOne),
                         cond);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
@@ -502,28 +506,28 @@ end:
             vHS = simde_mm_load_si128(pvHSStore + offset);
             vHL = simde_mm_load_si128(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 8);
-                vHM = simde_mm_slli_si128(vHM, 8);
-                vHS = simde_mm_slli_si128(vHS, 8);
-                vHL = simde_mm_slli_si128(vHL, 8);
+                vH = simde_mm_slli_si128(vH, 4);
+                vHM = simde_mm_slli_si128(vHM, 4);
+                vHS = simde_mm_slli_si128(vHS, 4);
+                vHL = simde_mm_slli_si128(vHL, 4);
             }
-            result->stats->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
-            result->stats->rowcols->matches_row[j] = (int64_t) simde_mm_extract_epi64 (vHM, 1);
-            result->stats->rowcols->similar_row[j] = (int64_t) simde_mm_extract_epi64 (vHS, 1);
-            result->stats->rowcols->length_row[j] = (int64_t) simde_mm_extract_epi64 (vHL, 1);
+            result->stats->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            result->stats->rowcols->matches_row[j] = (int32_t) simde_mm_extract_epi32 (vHM, 3);
+            result->stats->rowcols->similar_row[j] = (int32_t) simde_mm_extract_epi32 (vHS, 3);
+            result->stats->rowcols->length_row[j] = (int32_t) simde_mm_extract_epi32 (vHL, 3);
         }
 #endif
 
         {
-            simde__m128i vCompare = simde_mm_cmpgt_epi64(vMaxH, vMaxHUnit);
+            simde__m128i vCompare = simde_mm_cmpgt_epi32(vMaxH, vMaxHUnit);
             if (simde_mm_movemask_epi8(vCompare)) {
-                score = simde_mm_hmax_epi64(vMaxH);
+                score = simde_mm_hmax_epi32(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = simde_mm_set1_epi64x(score);
+                vMaxHUnit = simde_mm_set1_epi32(score);
                 end_ref = j;
             }
         }
@@ -542,8 +546,8 @@ end:
     }
 #endif
 
-    if (score == INT64_MAX
-            || simde_mm_movemask_epi8(simde_mm_cmpeq_epi64(vSaturationCheckMax,vPosLimit))) {
+    if (score == INT32_MAX
+            || simde_mm_movemask_epi8(simde_mm_cmpeq_epi32(vSaturationCheckMax,vPosLimit))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -572,10 +576,10 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int64_t *t = (int64_t*)pvHMax;
-            int64_t *m = (int64_t*)pvHMMax;
-            int64_t *s = (int64_t*)pvHSMax;
-            int64_t *l = (int64_t*)pvHLMax;
+            int32_t *t = (int32_t*)pvHMax;
+            int32_t *m = (int32_t*)pvHMMax;
+            int32_t *s = (int32_t*)pvHSMax;
+            int32_t *l = (int32_t*)pvHLMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len;
             for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -634,7 +638,7 @@ parasail_result_t* INAME(
     const parasail_matrix_t *matrix = profile->matrix;
 
     /* find the end loc first with the faster implementation */
-    parasail_result_t *result = parasail_sw_striped_profile_neon_128_64(profile, s2, s2Len, open, gap);
+    parasail_result_t *result = parasail_sw_striped_profile_neon_128_32(profile, s2, s2Len, open, gap);
     if (!parasail_result_is_saturated(result)) {
 #if 0
         int s1Len_new = 0;
@@ -656,10 +660,10 @@ parasail_result_t* INAME(
         s2Len_new = result->end_ref+1;
         s1_new = parasail_reverse(s1, s1Len_new);
         s2_new = parasail_reverse(s2, s2Len_new);
-        profile_new = parasail_profile_create_neon_128_64(
+        profile_new = parasail_profile_create_neon_128_32(
                 s1_new, s1Len_new, matrix);
         profile_new->stop = result->score;
-        result_new = parasail_sw_striped_profile_neon_128_64(
+        result_new = parasail_sw_striped_profile_neon_128_32(
                 profile_new, s2_new, s2Len_new, open, gap);
 
         /* using both the beg and end loc, call the original stats func */
@@ -671,7 +675,7 @@ parasail_result_t* INAME(
         assert(s2_begin >= 0);
         assert(s1Len_new > s1_begin);
         assert(s2Len_new > s2_begin);
-        profile_final = parasail_profile_create_stats_neon_128_64(
+        profile_final = parasail_profile_create_stats_neon_128_32(
                 &s1[s1_begin], s1Len_final, matrix);
         result_final = PNAME(
                 profile_final, &s2[s2_begin], s2Len_final, open, gap);
@@ -697,7 +701,7 @@ parasail_result_t* INAME(
         /* using the end loc, call the original stats function */
         s1Len_new = result->end_query+1;
         s2Len_new = result->end_ref+1;
-        profile_final = parasail_profile_create_stats_neon_128_64(
+        profile_final = parasail_profile_create_stats_neon_128_32(
                 s1, s1Len_new, matrix);
         result_final = PNAME(
                 profile_final, s2, s2Len_new, open, gap);
