@@ -14,7 +14,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -26,13 +25,33 @@
 #define SWAP(A,B) { __m128i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { __m128i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF INT16_MIN
+#define NEG_INF INT8_MIN
 
-static inline int16_t _mm_hmax_epi16_rpl(__m128i a) {
-    a = _mm_max_epi16(a, _mm_srli_si128(a, 8));
-    a = _mm_max_epi16(a, _mm_srli_si128(a, 4));
-    a = _mm_max_epi16(a, _mm_srli_si128(a, 2));
-    return _mm_extract_epi16(a, 0);
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
+static inline __m128i _mm_max_epi8_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi8(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline int8_t _mm_extract_epi8_rpl(__m128i a, const int imm) {
+    __m128i_8_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline int8_t _mm_hmax_epi8_rpl(__m128i a) {
+    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 8));
+    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 4));
+    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 2));
+    a = _mm_max_epi8_rpl(a, _mm_srli_si128(a, 1));
+    return _mm_extract_epi8_rpl(a, 0);
 }
 
 
@@ -46,14 +65,22 @@ static inline void arr_store(
         int32_t dlen,
         int32_t bias)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0) - bias;
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1) - bias;
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2) - bias;
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3) - bias;
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4) - bias;
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5) - bias;
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6) - bias;
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7) - bias;
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  0) - bias;
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  1) - bias;
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  2) - bias;
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  3) - bias;
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  4) - bias;
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  5) - bias;
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  6) - bias;
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  7) - bias;
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  8) - bias;
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH,  9) - bias;
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 10) - bias;
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 11) - bias;
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 12) - bias;
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 13) - bias;
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 14) - bias;
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8_rpl(vH, 15) - bias;
 }
 #endif
 
@@ -65,36 +92,44 @@ static inline void arr_store_col(
         int32_t seglen,
         int32_t bias)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0) - bias;
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1) - bias;
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2) - bias;
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3) - bias;
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4) - bias;
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5) - bias;
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6) - bias;
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7) - bias;
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  0) - bias;
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  1) - bias;
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  2) - bias;
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  3) - bias;
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  4) - bias;
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  5) - bias;
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  6) - bias;
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  7) - bias;
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  8) - bias;
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH,  9) - bias;
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 10) - bias;
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 11) - bias;
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 12) - bias;
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 13) - bias;
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 14) - bias;
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8_rpl(vH, 15) - bias;
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_striped_sse41_128_16
-#define PNAME parasail_sw_stats_table_striped_profile_sse41_128_16
+#define FNAME parasail_sw_stats_table_striped_sse2_128_8
+#define PNAME parasail_sw_stats_table_striped_profile_sse2_128_8
 #define INAME PNAME
 #define STATIC
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_striped_sse41_128_16
-#define PNAME parasail_sw_stats_rowcol_striped_profile_sse41_128_16
+#define FNAME parasail_sw_stats_rowcol_striped_sse2_128_8
+#define PNAME parasail_sw_stats_rowcol_striped_profile_sse2_128_8
 #define INAME PNAME
 #define STATIC
 #else
-#define FNAME parasail_sw_stats_striped_sse41_128_16
+#define FNAME parasail_sw_stats_striped_sse2_128_8
 #ifdef FASTSTATS
-#define PNAME parasail_sw_stats_striped_profile_sse41_128_16_internal
-#define INAME parasail_sw_stats_striped_profile_sse41_128_16
+#define PNAME parasail_sw_stats_striped_profile_sse2_128_8_internal
+#define INAME parasail_sw_stats_striped_profile_sse2_128_8
 #define STATIC static
 #else
-#define PNAME parasail_sw_stats_striped_profile_sse41_128_16
+#define PNAME parasail_sw_stats_striped_profile_sse2_128_8
 #define INAME PNAME
 #define STATIC
 #endif
@@ -125,7 +160,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_stats_sse_128_16(s1, s1Len, matrix);
+    profile = parasail_profile_create_stats_sse_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = INAME(profile, s2, s2Len, open, gap);
 
@@ -176,11 +211,11 @@ STATIC parasail_result_t* PNAME(
     __m128i vGapE;
     __m128i vZero;
     __m128i vOne;
-    int16_t bias = 0;
-    int16_t score = 0;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
+    int8_t bias = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
     __m128i vBias;
     __m128i vBias1;
     __m128i vMaxH;
@@ -188,12 +223,12 @@ STATIC parasail_result_t* PNAME(
     __m128i insert_mask;
     __m128i vSaturationCheckMax;
     __m128i vPosLimit;
-    int16_t maxp = 0;
+    int8_t maxp = 0;
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -209,34 +244,34 @@ STATIC parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile  = (__m128i*)profile->profile16.score;
-    vProfileM = (__m128i*)profile->profile16.matches;
-    vProfileS = (__m128i*)profile->profile16.similar;
-    vGapO = _mm_set1_epi16(open);
-    vGapE = _mm_set1_epi16(gap);
+    vProfile  = (__m128i*)profile->profile8.score;
+    vProfileM = (__m128i*)profile->profile8.matches;
+    vProfileS = (__m128i*)profile->profile8.similar;
+    vGapO = _mm_set1_epi8(open);
+    vGapE = _mm_set1_epi8(gap);
     vZero = _mm_setzero_si128();
-    vOne = _mm_set1_epi16(1);
-    bias = INT16_MIN;
+    vOne = _mm_set1_epi8(1);
+    bias = INT8_MIN;
     score = bias;
     matches = bias;
     similar = bias;
     length = bias;
-    vBias = _mm_set1_epi16(bias);
-    vBias1 = _mm_adds_epi16(vBias,vOne);
+    vBias = _mm_set1_epi8(bias);
+    vBias1 = _mm_adds_epi8(vBias,vOne);
     vMaxH = vBias;
     vMaxHUnit = vBias;
-    insert_mask = _mm_cmpgt_epi16(
-            _mm_set_epi16(0,0,0,0,0,0,0,1),
+    insert_mask = _mm_cmpgt_epi8(
+            _mm_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
             vZero);
     vSaturationCheckMax = vBias;
-    vPosLimit = _mm_set1_epi16(INT16_MAX);
-    maxp = INT16_MAX - (int16_t)(matrix->max+1);
+    vPosLimit = _mm_set1_epi8(INT8_MAX);
+    maxp = INT8_MAX - (int8_t)(matrix->max+1);
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -253,7 +288,7 @@ STATIC parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -335,19 +370,19 @@ STATIC parasail_result_t* PNAME(
         vFS = vBias;
         vFL = vBias1;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
         vHM = _mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = _mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = _mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 2);
-        vHM = _mm_slli_si128(vHM, 2);
-        vHS = _mm_slli_si128(vHS, 2);
-        vHL = _mm_slli_si128(vHL, 2);
-        vH = _mm_blendv_epi8(vH, vBias, insert_mask);
-        vHM = _mm_blendv_epi8(vHM, vBias, insert_mask);
-        vHS = _mm_blendv_epi8(vHS, vBias, insert_mask);
-        vHL = _mm_blendv_epi8(vHL, vBias, insert_mask);
+        vH = _mm_slli_si128(vH, 1);
+        vHM = _mm_slli_si128(vHM, 1);
+        vHS = _mm_slli_si128(vHS, 1);
+        vHL = _mm_slli_si128(vHL, 1);
+        vH = _mm_blendv_epi8_rpl(vH, vBias, insert_mask);
+        vHM = _mm_blendv_epi8_rpl(vHM, vBias, insert_mask);
+        vHS = _mm_blendv_epi8_rpl(vHS, vBias, insert_mask);
+        vHL = _mm_blendv_epi8_rpl(vHL, vBias, insert_mask);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -381,59 +416,59 @@ STATIC parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
-            /*vH = _mm_max_epi16(vH_dag, vBias);*/
-            vH = _mm_max_epi16(vH_dag, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH_dag = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
+            /*vH = _mm_max_epi8_rpl(vH_dag, vBias);*/
+            vH = _mm_max_epi8_rpl(vH_dag, vE);
+            vH = _mm_max_epi8_rpl(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
-            cond_zero = _mm_cmpeq_epi16(vH, vBias);
+            cond_zero = _mm_cmpeq_epi8(vH, vBias);
 
-            case1 = _mm_cmpeq_epi16(vH, vH_dag);
-            case2 = _mm_cmpeq_epi16(vH, vF);
+            case1 = _mm_cmpeq_epi8(vH, vH_dag);
+            case2 = _mm_cmpeq_epi8(vH, vF);
 
             /* calculate vM */
-            vHM = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_adds_epi16(vHM, _mm_load_si128(vPM + i)), case1);
-            vHM = _mm_blendv_epi8(vHM, vBias, cond_zero);
+            vHM = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
+                    _mm_adds_epi8(vHM, _mm_load_si128(vPM + i)), case1);
+            vHM = _mm_blendv_epi8_rpl(vHM, vBias, cond_zero);
             _mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
-            vHS = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_adds_epi16(vHS, _mm_load_si128(vPS + i)), case1);
-            vHS = _mm_blendv_epi8(vHS, vBias, cond_zero);
+            vHS = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vES, vFS, case2),
+                    _mm_adds_epi8(vHS, _mm_load_si128(vPS + i)), case1);
+            vHS = _mm_blendv_epi8_rpl(vHS, vBias, cond_zero);
             _mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
-            vHL = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_adds_epi16(vHL, vOne), case1);
-            vHL = _mm_blendv_epi8(vHL, vBias, cond_zero);
+            vHL = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
+                    _mm_adds_epi8(vHL, vOne), case1);
+            vHL = _mm_blendv_epi8_rpl(vHL, vBias, cond_zero);
             _mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
             arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
             arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
             arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-            vMaxH = _mm_max_epi16(vH, vMaxH);
-            vEF_opn = _mm_subs_epi16(vH, vGapO);
+            vMaxH = _mm_max_epi8_rpl(vH, vMaxH);
+            vEF_opn = _mm_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_subs_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vEF_opn, vE_ext);
-            case1 = _mm_cmpgt_epi16(vEF_opn, vE_ext);
-            vEM = _mm_blendv_epi8(vEM, vHM, case1);
-            vES = _mm_blendv_epi8(vES, vHS, case1);
-            vEL = _mm_blendv_epi8(
-                    _mm_adds_epi16(vEL, vOne),
-                    _mm_adds_epi16(vHL, vOne),
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            vE = _mm_max_epi8_rpl(vEF_opn, vE_ext);
+            case1 = _mm_cmpgt_epi8(vEF_opn, vE_ext);
+            vEM = _mm_blendv_epi8_rpl(vEM, vHM, case1);
+            vES = _mm_blendv_epi8_rpl(vES, vHS, case1);
+            vEL = _mm_blendv_epi8_rpl(
+                    _mm_adds_epi8(vEL, vOne),
+                    _mm_adds_epi8(vHL, vOne),
                     case1);
             _mm_store_si128(pvE + i, vE);
             _mm_store_si128(pvEM + i, vEM);
@@ -441,14 +476,14 @@ STATIC parasail_result_t* PNAME(
             _mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm_subs_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vEF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vEF_opn, vF_ext);
-            vFM = _mm_blendv_epi8(vFM, vHM, case1);
-            vFS = _mm_blendv_epi8(vFS, vHS, case1);
-            vFL = _mm_blendv_epi8(
-                    _mm_adds_epi16(vFL, vOne),
-                    _mm_adds_epi16(vHL, vOne),
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8_rpl(vEF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi8(vEF_opn, vF_ext);
+            vFM = _mm_blendv_epi8_rpl(vFM, vHM, case1);
+            vFS = _mm_blendv_epi8_rpl(vFS, vHS, case1);
+            vFL = _mm_blendv_epi8_rpl(
+                    _mm_adds_epi8(vFL, vOne),
+                    _mm_adds_epi8(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -462,72 +497,72 @@ STATIC parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             __m128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = _mm_slli_si128(vHp, 2);
-            vF = _mm_slli_si128(vF, 2);
-            vFM = _mm_slli_si128(vFM, 2);
-            vFS = _mm_slli_si128(vFS, 2);
-            vFL = _mm_slli_si128(vFL, 2);
-            vHp = _mm_blendv_epi8(vHp, vBias, insert_mask);
-            vF = _mm_blendv_epi8(vF, vBias, insert_mask);
-            vFM = _mm_blendv_epi8(vFM, vBias, insert_mask);
-            vFS = _mm_blendv_epi8(vFS, vBias, insert_mask);
-            vFL = _mm_blendv_epi8(vFL, vBias1, insert_mask);
+            vHp = _mm_slli_si128(vHp, 1);
+            vF = _mm_slli_si128(vF, 1);
+            vFM = _mm_slli_si128(vFM, 1);
+            vFS = _mm_slli_si128(vFS, 1);
+            vFL = _mm_slli_si128(vFL, 1);
+            vHp = _mm_blendv_epi8_rpl(vHp, vBias, insert_mask);
+            vF = _mm_blendv_epi8_rpl(vF, vBias, insert_mask);
+            vFM = _mm_blendv_epi8_rpl(vFM, vBias, insert_mask);
+            vFS = _mm_blendv_epi8_rpl(vFS, vBias, insert_mask);
+            vFL = _mm_blendv_epi8_rpl(vFL, vBias1, insert_mask);
             for (i=0; i<segLen; ++i) {
                 __m128i case1;
                 __m128i case2;
                 __m128i cond;
 
-                vHp = _mm_adds_epi16(vHp, _mm_load_si128(vP + i));
+                vHp = _mm_adds_epi8(vHp, _mm_load_si128(vP + i));
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH, vF);
+                vH = _mm_max_epi8_rpl(vH, vF);
                 _mm_store_si128(pvHStore + i, vH);
-                case1 = _mm_cmpeq_epi16(vH, vHp);
-                case2 = _mm_cmpeq_epi16(vH, vF);
+                case1 = _mm_cmpeq_epi8(vH, vHp);
+                case2 = _mm_cmpeq_epi8(vH, vF);
                 cond = _mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
                 vHM = _mm_load_si128(pvHMStore + i);
-                vHM = _mm_blendv_epi8(vHM, vFM, cond);
+                vHM = _mm_blendv_epi8_rpl(vHM, vFM, cond);
                 _mm_store_si128(pvHMStore + i, vHM);
 
                 /* calculate vS */
                 vHS = _mm_load_si128(pvHSStore + i);
-                vHS = _mm_blendv_epi8(vHS, vFS, cond);
+                vHS = _mm_blendv_epi8_rpl(vHS, vFS, cond);
                 _mm_store_si128(pvHSStore + i, vHS);
 
                 /* calculate vL */
                 vHL = _mm_load_si128(pvHLStore + i);
-                vHL = _mm_blendv_epi8(vHL, vFL, cond);
+                vHL = _mm_blendv_epi8_rpl(vHL, vFL, cond);
                 _mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+                vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm_max_epi8_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
                 arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
                 arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
                 arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-                vMaxH = _mm_max_epi16(vH, vMaxH);
+                vMaxH = _mm_max_epi8_rpl(vH, vMaxH);
                 /* Update vF value. */
-                vEF_opn = _mm_subs_epi16(vH, vGapO);
-                vF_ext = _mm_subs_epi16(vF, vGapE);
+                vEF_opn = _mm_subs_epi8(vH, vGapO);
+                vF_ext = _mm_subs_epi8(vF, vGapE);
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm_cmpgt_epi8(vF_ext, vEF_opn),
                                 _mm_and_si128(
-                                    _mm_cmpeq_epi16(vF_ext, vEF_opn),
-                                    _mm_cmpgt_epi16(vF_ext, vBias)))))
+                                    _mm_cmpeq_epi8(vF_ext, vEF_opn),
+                                    _mm_cmpgt_epi8(vF_ext, vBias)))))
                     goto end;
-                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi8_rpl(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
-                vFM = _mm_blendv_epi8(vFM, vHM, cond);
-                vFS = _mm_blendv_epi8(vFS, vHS, cond);
-                vFL = _mm_blendv_epi8(
-                        _mm_adds_epi16(vFL, vOne),
-                        _mm_adds_epi16(vHL, vOne),
+                cond = _mm_cmpgt_epi8(vEF_opn, vF_ext);
+                vFM = _mm_blendv_epi8_rpl(vFM, vHM, cond);
+                vFS = _mm_blendv_epi8_rpl(vFS, vHS, cond);
+                vFL = _mm_blendv_epi8_rpl(
+                        _mm_adds_epi8(vFL, vOne),
+                        _mm_adds_epi8(vHL, vOne),
                         cond);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
@@ -544,28 +579,28 @@ end:
             vHS = _mm_load_si128(pvHSStore + offset);
             vHL = _mm_load_si128(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
-                vHM = _mm_slli_si128(vHM, 2);
-                vHS = _mm_slli_si128(vHS, 2);
-                vHL = _mm_slli_si128(vHL, 2);
+                vH = _mm_slli_si128(vH, 1);
+                vHM = _mm_slli_si128(vHM, 1);
+                vHS = _mm_slli_si128(vHS, 1);
+                vHL = _mm_slli_si128(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7) - bias;
-            result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7) - bias;
-            result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7) - bias;
-            result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7) - bias;
+            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8_rpl (vH, 15) - bias;
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8_rpl (vHM, 15) - bias;
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8_rpl (vHS, 15) - bias;
+            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8_rpl (vHL, 15) - bias;
         }
 #endif
 
         {
-            __m128i vCompare = _mm_cmpgt_epi16(vMaxH, vMaxHUnit);
+            __m128i vCompare = _mm_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi16_rpl(vMaxH);
+                score = _mm_hmax_epi8_rpl(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm_set1_epi16(score);
+                vMaxHUnit = _mm_set1_epi8(score);
                 end_ref = j;
             }
         }
@@ -584,8 +619,8 @@ end:
     }
 #endif
 
-    if (score == INT16_MAX
-            || _mm_movemask_epi8(_mm_cmpeq_epi16(vSaturationCheckMax,vPosLimit))) {
+    if (score == INT8_MAX
+            || _mm_movemask_epi8(_mm_cmpeq_epi8(vSaturationCheckMax,vPosLimit))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -614,10 +649,10 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int16_t *t = (int16_t*)pvHMax;
-            int16_t *m = (int16_t*)pvHMMax;
-            int16_t *s = (int16_t*)pvHSMax;
-            int16_t *l = (int16_t*)pvHLMax;
+            int8_t *t = (int8_t*)pvHMax;
+            int8_t *m = (int8_t*)pvHMMax;
+            int8_t *s = (int8_t*)pvHSMax;
+            int8_t *l = (int8_t*)pvHLMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len;
             for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -676,7 +711,7 @@ parasail_result_t* INAME(
     const parasail_matrix_t *matrix = profile->matrix;
 
     /* find the end loc first with the faster implementation */
-    parasail_result_t *result = parasail_sw_striped_profile_sse41_128_16(profile, s2, s2Len, open, gap);
+    parasail_result_t *result = parasail_sw_striped_profile_sse2_128_8(profile, s2, s2Len, open, gap);
     if (!parasail_result_is_saturated(result)) {
 #if 0
         int s1Len_new = 0;
@@ -698,10 +733,10 @@ parasail_result_t* INAME(
         s2Len_new = result->end_ref+1;
         s1_new = parasail_reverse(s1, s1Len_new);
         s2_new = parasail_reverse(s2, s2Len_new);
-        profile_new = parasail_profile_create_sse_128_16(
+        profile_new = parasail_profile_create_sse_128_8(
                 s1_new, s1Len_new, matrix);
         profile_new->stop = result->score;
-        result_new = parasail_sw_striped_profile_sse41_128_16(
+        result_new = parasail_sw_striped_profile_sse2_128_8(
                 profile_new, s2_new, s2Len_new, open, gap);
 
         /* using both the beg and end loc, call the original stats func */
@@ -713,7 +748,7 @@ parasail_result_t* INAME(
         assert(s2_begin >= 0);
         assert(s1Len_new > s1_begin);
         assert(s2Len_new > s2_begin);
-        profile_final = parasail_profile_create_stats_sse_128_16(
+        profile_final = parasail_profile_create_stats_sse_128_8(
                 &s1[s1_begin], s1Len_final, matrix);
         result_final = PNAME(
                 profile_final, &s2[s2_begin], s2Len_final, open, gap);
@@ -739,7 +774,7 @@ parasail_result_t* INAME(
         /* using the end loc, call the original stats function */
         s1Len_new = result->end_query+1;
         s2Len_new = result->end_ref+1;
-        profile_final = parasail_profile_create_stats_sse_128_16(
+        profile_final = parasail_profile_create_stats_sse_128_8(
                 s1, s1Len_new, matrix);
         result_final = PNAME(
                 profile_final, s2, s2Len_new, open, gap);
