@@ -16,7 +16,6 @@
 #include "parasail/memory.h"
 #include "parasail/internal_neon.h"
 
-#define NEG_INF (INT32_MIN/(int32_t)(2))
 
 
 #ifdef PARASAIL_TABLE
@@ -26,12 +25,25 @@ static inline void arr_store_si128(
         int32_t t,
         int32_t seglen,
         int32_t d,
-        int32_t dlen)
+        int32_t dlen,
+        int32_t bias)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  0) - bias;
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  1) - bias;
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  2) - bias;
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  3) - bias;
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  4) - bias;
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  5) - bias;
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  6) - bias;
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  7) - bias;
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  8) - bias;
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  9) - bias;
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 10) - bias;
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 11) - bias;
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 12) - bias;
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 13) - bias;
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 14) - bias;
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 15) - bias;
 }
 #endif
 
@@ -40,25 +52,38 @@ static inline void arr_store_col(
         int *col,
         simde__m128i vH,
         int32_t t,
-        int32_t seglen)
+        int32_t seglen,
+        int32_t bias)
 {
-    col[0*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)simde_mm_extract_epi32(vH, 3);
+    col[ 0*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  0) - bias;
+    col[ 1*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  1) - bias;
+    col[ 2*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  2) - bias;
+    col[ 3*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  3) - bias;
+    col[ 4*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  4) - bias;
+    col[ 5*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  5) - bias;
+    col[ 6*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  6) - bias;
+    col[ 7*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  7) - bias;
+    col[ 8*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  8) - bias;
+    col[ 9*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  9) - bias;
+    col[10*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 10) - bias;
+    col[11*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 11) - bias;
+    col[12*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 12) - bias;
+    col[13*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 13) - bias;
+    col[14*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 14) - bias;
+    col[15*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 15) - bias;
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_striped_neon_128_32
-#define PNAME parasail_sw_table_striped_profile_neon_128_32
+#define FNAME parasail_sw_table_striped_neon_128_8
+#define PNAME parasail_sw_table_striped_profile_neon_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_striped_neon_128_32
-#define PNAME parasail_sw_rowcol_striped_profile_neon_128_32
+#define FNAME parasail_sw_rowcol_striped_neon_128_8
+#define PNAME parasail_sw_rowcol_striped_profile_neon_128_8
 #else
-#define FNAME parasail_sw_striped_neon_128_32
-#define PNAME parasail_sw_striped_profile_neon_128_32
+#define FNAME parasail_sw_striped_neon_128_8
+#define PNAME parasail_sw_striped_profile_neon_128_8
 #endif
 #endif
 
@@ -83,7 +108,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -119,17 +144,19 @@ parasail_result_t* PNAME(
     simde__m128i vGapO;
     simde__m128i vGapE;
     simde__m128i vZero;
-    simde__m128i vNegInf;
-    int32_t score = 0;
+    int8_t bias = 0;
+    int8_t score = 0;
+    simde__m128i vBias;
     simde__m128i vMaxH;
     simde__m128i vMaxHUnit;
-    int32_t maxp = 0;
-    /*int32_t stop = 0;*/
+    int8_t maxp = 0;
+    simde__m128i insert_mask;
+    /*int8_t stop = 0;*/
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -145,22 +172,26 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile = (simde__m128i*)profile->profile32.score;
-    vGapO = simde_mm_set1_epi32(open);
-    vGapE = simde_mm_set1_epi32(gap);
-    vZero = simde_mm_setzero_si128();
-    vNegInf = simde_mm_set1_epi32(NEG_INF);
-    score = NEG_INF;
-    vMaxH = vNegInf;
-    vMaxHUnit = vNegInf;
-    maxp = INT32_MAX - (int32_t)(matrix->max+1);
-    /*stop = profile->stop == INT32_MAX ?  INT32_MAX : (int32_t)profile->stop;*/
+    vProfile = (simde__m128i*)profile->profile8.score;
+    vGapO = simde_mm_set1_epi8(open);
+    vGapE = simde_mm_set1_epi8(gap);
+    vZero = simde_mm_set1_epi8(0);
+    bias = INT8_MIN;
+    score = bias;
+    vBias = simde_mm_set1_epi8(bias);
+    vMaxH = vBias;
+    vMaxHUnit = vBias;
+    maxp = INT8_MAX - (int8_t)(matrix->max+1);
+    insert_mask = simde_mm_cmpgt_epi8(
+            simde_mm_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
+            vZero);
+    /*stop = profile->stop == INT32_MAX ?  INT8_MAX : (int8_t)profile->stop-bias;*/
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -176,7 +207,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -186,7 +217,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvHStore = parasail_memalign_simde__m128i(16, segLen);
-    pvHLoad =  parasail_memalign_simde__m128i(16, segLen);
+    pvHLoad = parasail_memalign_simde__m128i(16, segLen);
     pvHMax = parasail_memalign_simde__m128i(16, segLen);
     pvE = parasail_memalign_simde__m128i(16, segLen);
 
@@ -197,8 +228,8 @@ parasail_result_t* PNAME(
     if (!pvE) return NULL;
 
     /* initialize H and E */
-    parasail_memset_simde__m128i(pvHStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvE, simde_mm_set1_epi32(-open), segLen);
+    parasail_memset_simde__m128i(pvHStore, vBias, segLen);
+    parasail_memset_simde__m128i(pvE, vBias, segLen);
 
     /* outer loop over database sequence */
     for (j=0; j<s2Len; ++j) {
@@ -209,12 +240,12 @@ parasail_result_t* PNAME(
         simde__m128i* pv = NULL;
 
         /* Initialize F value to 0.  Any errors to vH values will be
-         * corrected in the Lazy_F loop. */
-        vF = vZero;
+         * corrected in the Lazy_F loop.  */
+        vF = vBias;
 
-        /* load final segment of pvHStore and shift left by 4 bytes */
-        vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 4);
+        /* load final segment of pvHStore and shift left by 1 bytes */
+        vH = simde_mm_slli_si128(pvHStore[segLen - 1], 1);
+        vH = simde_mm_blendv_epi8(vH, vBias, insert_mask);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -235,29 +266,28 @@ parasail_result_t* PNAME(
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = simde_mm_add_epi32(vH, simde_mm_load_si128(vP + i));
+            vH = simde_mm_adds_epi8(vH, simde_mm_load_si128(vP + i));
             vE = simde_mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = simde_mm_max_epi32(vH, vE);
-            vH = simde_mm_max_epi32(vH, vF);
-            vH = simde_mm_max_epi32(vH, vZero);
+            vH = simde_mm_max_epi8(vH, vE);
+            vH = simde_mm_max_epi8(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
 #ifdef PARASAIL_TABLE
-            arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
+            arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-            vMaxH = simde_mm_max_epi32(vH, vMaxH);
+            vMaxH = simde_mm_max_epi8(vH, vMaxH);
 
             /* Update vE value. */
-            vH = simde_mm_sub_epi32(vH, vGapO);
-            vE = simde_mm_sub_epi32(vE, vGapE);
-            vE = simde_mm_max_epi32(vE, vH);
+            vH = simde_mm_subs_epi8(vH, vGapO);
+            vE = simde_mm_subs_epi8(vE, vGapE);
+            vE = simde_mm_max_epi8(vE, vH);
             simde_mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = simde_mm_sub_epi32(vF, vGapE);
-            vF = simde_mm_max_epi32(vF, vH);
+            vF = simde_mm_subs_epi8(vF, vGapE);
+            vF = simde_mm_max_epi8(vF, vH);
 
             /* Load the next vH. */
             vH = simde_mm_load_si128(pvHLoad + i);
@@ -266,19 +296,20 @@ parasail_result_t* PNAME(
         /* Lazy_F loop: has been revised to disallow adjecent insertion and
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
-            vF = simde_mm_slli_si128(vF, 4);
+            vF = simde_mm_slli_si128(vF, 1);
+            vF = simde_mm_blendv_epi8(vF, vBias, insert_mask);
             for (i=0; i<segLen; ++i) {
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi32(vH,vF);
+                vH = simde_mm_max_epi8(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
 #ifdef PARASAIL_TABLE
-                arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
+                arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-                vMaxH = simde_mm_max_epi32(vH, vMaxH);
-                vH = simde_mm_sub_epi32(vH, vGapO);
-                vF = simde_mm_sub_epi32(vF, vGapE);
-                if (! simde_mm_movemask_epi8(simde_mm_cmpgt_epi32(vF, vH))) goto end;
-                /*vF = simde_mm_max_epi32(vF, vH);*/
+                vMaxH = simde_mm_max_epi8(vH, vMaxH);
+                vH = simde_mm_subs_epi8(vH, vGapO);
+                vF = simde_mm_subs_epi8(vF, vGapE);
+                if (! simde_mm_movemask_epi8(simde_mm_cmpgt_epi8(vF, vH))) goto end;
+                /*vF = simde_mm_max_epi8(vF, vH);*/
             }
         }
 end:
@@ -290,22 +321,22 @@ end:
         {
             vH = simde_mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 4);
+                vH = simde_mm_slli_si128(vH, 1);
             }
-            result->rowcols->score_row[j] = (int32_t) simde_mm_extract_epi32 (vH, 3);
+            result->rowcols->score_row[j] = (int8_t) simde_mm_extract_epi8 (vH, 15) - bias;
         }
 #endif
 
         {
-            simde__m128i vCompare = simde_mm_cmpgt_epi32(vMaxH, vMaxHUnit);
+            simde__m128i vCompare = simde_mm_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (simde_mm_movemask_epi8(vCompare)) {
-                score = simde_mm_hmax_epi32(vMaxH);
+                score = simde_mm_hmax_epi8(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = simde_mm_set1_epi32(score);
+                vMaxHUnit = simde_mm_set1_epi8(score);
                 end_ref = j;
             }
         }
@@ -316,16 +347,16 @@ end:
 #ifdef PARASAIL_ROWCOL
     for (i=0; i<segLen; ++i) {
         simde__m128i vH = simde_mm_load_si128(pvHStore+i);
-        arr_store_col(result->rowcols->score_col, vH, i, segLen);
+        arr_store_col(result->rowcols->score_col, vH, i, segLen, bias);
     }
 #endif
 
-    if (score == INT32_MAX) {
+    if (score == INT8_MAX) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
     if (parasail_result_is_saturated(result)) {
-        score = INT32_MAX;
+        score = INT8_MAX;
         end_query = 0;
         end_ref = 0;
     }
@@ -344,7 +375,7 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int32_t *t = (int32_t*)pvHMax;
+            int8_t *t = (int8_t*)pvHMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len - 1;
             for (i = 0; i<column_len; ++i, ++t) {
@@ -358,7 +389,7 @@ end:
         }
     }
 
-    result->score = score;
+    result->score = score - bias;
     result->end_query = end_query;
     result->end_ref = end_ref;
 
@@ -370,4 +401,3 @@ end:
     return result;
 }
 
-
