@@ -21,13 +21,52 @@
 #include "parasail/memory.h"
 #include "parasail/internal_sse.h"
 
+#define NEG_INF (INT64_MIN/(int64_t)(2))
 
-static inline int8_t _mm_hmax_epi8_rpl(__m128i a) {
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 8));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 4));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 2));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 1));
-    return _mm_extract_epi8(a, 0);
+static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]>B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    return A.m;
+}
+
+#if HAVE_SSE41_MM_EXTRACT_EPI64
+#define _mm_extract_epi64_rpl _mm_extract_epi64
+#else
+static inline int64_t _mm_extract_epi64_rpl(__m128i a, int imm) {
+    __m128i_64_t A;
+    A.m = a;
+    return A.v[imm];
+}
+#endif
+
+#if HAVE_SSE2_MM_SET1_EPI64X
+#define _mm_set1_epi64x_rpl _mm_set1_epi64x
+#else
+static inline __m128i _mm_set1_epi64x_rpl(int64_t i) {
+    __m128i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    return A.m;
+}
+#endif
+
+static inline int64_t _mm_hmax_epi64_rpl(__m128i a) {
+    a = _mm_max_epi64_rpl(a, _mm_srli_si128(a, 8));
+    return _mm_extract_epi64_rpl(a, 0);
 }
 
 
@@ -38,25 +77,10 @@ static inline void arr_store_si128(
         int32_t t,
         int32_t seglen,
         int32_t d,
-        int32_t dlen,
-        int32_t bias)
+        int32_t dlen)
 {
-    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0) - bias;
-    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1) - bias;
-    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2) - bias;
-    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3) - bias;
-    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4) - bias;
-    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5) - bias;
-    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6) - bias;
-    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7) - bias;
-    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8) - bias;
-    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9) - bias;
-    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10) - bias;
-    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11) - bias;
-    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12) - bias;
-    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13) - bias;
-    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14) - bias;
-    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15) - bias;
+    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
 }
 #endif
 
@@ -65,38 +89,23 @@ static inline void arr_store_col(
         int *col,
         __m128i vH,
         int32_t t,
-        int32_t seglen,
-        int32_t bias)
+        int32_t seglen)
 {
-    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0) - bias;
-    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1) - bias;
-    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2) - bias;
-    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3) - bias;
-    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4) - bias;
-    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5) - bias;
-    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6) - bias;
-    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7) - bias;
-    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8) - bias;
-    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9) - bias;
-    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10) - bias;
-    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11) - bias;
-    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12) - bias;
-    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13) - bias;
-    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14) - bias;
-    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15) - bias;
+    col[0*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 0);
+    col[1*seglen+t] = (int64_t)_mm_extract_epi64_rpl(vH, 1);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_table_striped_sse41_128_8
-#define PNAME parasail_sw_table_striped_profile_sse41_128_8
+#define FNAME parasail_sw_table_striped_sse41_128_64
+#define PNAME parasail_sw_table_striped_profile_sse41_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_rowcol_striped_sse41_128_8
-#define PNAME parasail_sw_rowcol_striped_profile_sse41_128_8
+#define FNAME parasail_sw_rowcol_striped_sse41_128_64
+#define PNAME parasail_sw_rowcol_striped_profile_sse41_128_64
 #else
-#define FNAME parasail_sw_striped_sse41_128_8
-#define PNAME parasail_sw_striped_profile_sse41_128_8
+#define FNAME parasail_sw_striped_sse41_128_64
+#define PNAME parasail_sw_striped_profile_sse41_128_64
 #endif
 #endif
 
@@ -121,7 +130,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -157,19 +166,17 @@ parasail_result_t* PNAME(
     __m128i vGapO;
     __m128i vGapE;
     __m128i vZero;
-    int8_t bias = 0;
-    int8_t score = 0;
-    __m128i vBias;
+    __m128i vNegInf;
+    int64_t score = 0;
     __m128i vMaxH;
     __m128i vMaxHUnit;
-    int8_t maxp = 0;
-    __m128i insert_mask;
-    /*int8_t stop = 0;*/
+    int64_t maxp = 0;
+    /*int64_t stop = 0;*/
     parasail_result_t *result = NULL;
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -185,26 +192,22 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 2; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
 #ifdef PARASAIL_ROWCOL
     offset = (s1Len - 1) % segLen;
     position = (segWidth - 1) - (s1Len - 1) / segLen;
 #endif
-    vProfile = (__m128i*)profile->profile8.score;
-    vGapO = _mm_set1_epi8(open);
-    vGapE = _mm_set1_epi8(gap);
-    vZero = _mm_set1_epi8(0);
-    bias = INT8_MIN;
-    score = bias;
-    vBias = _mm_set1_epi8(bias);
-    vMaxH = vBias;
-    vMaxHUnit = vBias;
-    maxp = INT8_MAX - (int8_t)(matrix->max+1);
-    insert_mask = _mm_cmpgt_epi8(
-            _mm_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
-            vZero);
-    /*stop = profile->stop == INT32_MAX ?  INT8_MAX : (int8_t)profile->stop-bias;*/
+    vProfile = (__m128i*)profile->profile64.score;
+    vGapO = _mm_set1_epi64x_rpl(open);
+    vGapE = _mm_set1_epi64x_rpl(gap);
+    vZero = _mm_setzero_si128();
+    vNegInf = _mm_set1_epi64x_rpl(NEG_INF);
+    score = NEG_INF;
+    vMaxH = vNegInf;
+    vMaxHUnit = vNegInf;
+    maxp = INT64_MAX - (int64_t)(matrix->max+1);
+    /*stop = profile->stop == INT32_MAX ?  INT64_MAX : (int64_t)profile->stop;*/
 
     /* initialize result */
 #ifdef PARASAIL_TABLE
@@ -220,7 +223,7 @@ parasail_result_t* PNAME(
 
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -230,7 +233,7 @@ parasail_result_t* PNAME(
 
     /* initialize heap variables */
     pvHStore = parasail_memalign___m128i(16, segLen);
-    pvHLoad = parasail_memalign___m128i(16, segLen);
+    pvHLoad =  parasail_memalign___m128i(16, segLen);
     pvHMax = parasail_memalign___m128i(16, segLen);
     pvE = parasail_memalign___m128i(16, segLen);
 
@@ -241,8 +244,8 @@ parasail_result_t* PNAME(
     if (!pvE) return NULL;
 
     /* initialize H and E */
-    parasail_memset___m128i(pvHStore, vBias, segLen);
-    parasail_memset___m128i(pvE, vBias, segLen);
+    parasail_memset___m128i(pvHStore, vZero, segLen);
+    parasail_memset___m128i(pvE, _mm_set1_epi64x_rpl(-open), segLen);
 
     /* outer loop over database sequence */
     for (j=0; j<s2Len; ++j) {
@@ -253,12 +256,12 @@ parasail_result_t* PNAME(
         __m128i* pv = NULL;
 
         /* Initialize F value to 0.  Any errors to vH values will be
-         * corrected in the Lazy_F loop.  */
-        vF = vBias;
+         * corrected in the Lazy_F loop. */
+        vF = vZero;
 
-        /* load final segment of pvHStore and shift left by 1 bytes */
-        vH = _mm_slli_si128(pvHStore[segLen - 1], 1);
-        vH = _mm_blendv_epi8(vH, vBias, insert_mask);
+        /* load final segment of pvHStore and shift left by 8 bytes */
+        vH = _mm_load_si128(&pvHStore[segLen - 1]);
+        vH = _mm_slli_si128(vH, 8);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -279,28 +282,29 @@ parasail_result_t* PNAME(
 
         /* inner loop to process the query sequence */
         for (i=0; i<segLen; ++i) {
-            vH = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
+            vH = _mm_add_epi64(vH, _mm_load_si128(vP + i));
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH = _mm_max_epi8(vH, vE);
-            vH = _mm_max_epi8(vH, vF);
+            vH = _mm_max_epi64_rpl(vH, vE);
+            vH = _mm_max_epi64_rpl(vH, vF);
+            vH = _mm_max_epi64_rpl(vH, vZero);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
 #ifdef PARASAIL_TABLE
-            arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len, bias);
+            arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vMaxH = _mm_max_epi8(vH, vMaxH);
+            vMaxH = _mm_max_epi64_rpl(vH, vMaxH);
 
             /* Update vE value. */
-            vH = _mm_subs_epi8(vH, vGapO);
-            vE = _mm_subs_epi8(vE, vGapE);
-            vE = _mm_max_epi8(vE, vH);
+            vH = _mm_sub_epi64(vH, vGapO);
+            vE = _mm_sub_epi64(vE, vGapE);
+            vE = _mm_max_epi64_rpl(vE, vH);
             _mm_store_si128(pvE + i, vE);
 
             /* Update vF value. */
-            vF = _mm_subs_epi8(vF, vGapE);
-            vF = _mm_max_epi8(vF, vH);
+            vF = _mm_sub_epi64(vF, vGapE);
+            vF = _mm_max_epi64_rpl(vF, vH);
 
             /* Load the next vH. */
             vH = _mm_load_si128(pvHLoad + i);
@@ -309,20 +313,19 @@ parasail_result_t* PNAME(
         /* Lazy_F loop: has been revised to disallow adjecent insertion and
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
-            vF = _mm_slli_si128(vF, 1);
-            vF = _mm_blendv_epi8(vF, vBias, insert_mask);
+            vF = _mm_slli_si128(vF, 8);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi8(vH,vF);
+                vH = _mm_max_epi64_rpl(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
 #ifdef PARASAIL_TABLE
-                arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len, bias);
+                arr_store_si128(result->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-                vMaxH = _mm_max_epi8(vH, vMaxH);
-                vH = _mm_subs_epi8(vH, vGapO);
-                vF = _mm_subs_epi8(vF, vGapE);
-                if (! _mm_movemask_epi8(_mm_cmpgt_epi8(vF, vH))) goto end;
-                /*vF = _mm_max_epi8(vF, vH);*/
+                vMaxH = _mm_max_epi64_rpl(vH, vMaxH);
+                vH = _mm_sub_epi64(vH, vGapO);
+                vF = _mm_sub_epi64(vF, vGapE);
+                if (! _mm_movemask_epi8(_mm_cmpgt_epi64_rpl(vF, vH))) goto end;
+                /*vF = _mm_max_epi64_rpl(vF, vH);*/
             }
         }
 end:
@@ -334,22 +337,22 @@ end:
         {
             vH = _mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 1);
+                vH = _mm_slli_si128(vH, 8);
             }
-            result->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15) - bias;
+            result->rowcols->score_row[j] = (int64_t) _mm_extract_epi64_rpl (vH, 1);
         }
 #endif
 
         {
-            __m128i vCompare = _mm_cmpgt_epi8(vMaxH, vMaxHUnit);
+            __m128i vCompare = _mm_cmpgt_epi64_rpl(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi8_rpl(vMaxH);
+                score = _mm_hmax_epi64_rpl(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm_set1_epi8(score);
+                vMaxHUnit = _mm_set1_epi64x_rpl(score);
                 end_ref = j;
             }
         }
@@ -360,16 +363,16 @@ end:
 #ifdef PARASAIL_ROWCOL
     for (i=0; i<segLen; ++i) {
         __m128i vH = _mm_load_si128(pvHStore+i);
-        arr_store_col(result->rowcols->score_col, vH, i, segLen, bias);
+        arr_store_col(result->rowcols->score_col, vH, i, segLen);
     }
 #endif
 
-    if (score == INT8_MAX) {
+    if (score == INT64_MAX) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
     if (parasail_result_is_saturated(result)) {
-        score = INT8_MAX;
+        score = INT64_MAX;
         end_query = 0;
         end_ref = 0;
     }
@@ -388,7 +391,7 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int8_t *t = (int8_t*)pvHMax;
+            int64_t *t = (int64_t*)pvHMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len - 1;
             for (i = 0; i<column_len; ++i, ++t) {
@@ -402,7 +405,7 @@ end:
         }
     }
 
-    result->score = score - bias;
+    result->score = score;
     result->end_query = end_query;
     result->end_ref = end_ref;
 
@@ -414,3 +417,4 @@ end:
     return result;
 }
 
+
