@@ -18,36 +18,72 @@
 #include "parasail/internal_avx.h"
 
 
-#if HAVE_AVX2_MM256_EXTRACT_EPI32
-#define _mm256_extract_epi32_rpl _mm256_extract_epi32
+#if HAVE_AVX2_MM256_EXTRACT_EPI64
+#define _mm256_extract_epi64_rpl _mm256_extract_epi64
 #else
-static inline int32_t _mm256_extract_epi32_rpl(__m256i a, int imm) {
-    __m256i_32_t A;
+static inline int64_t _mm256_extract_epi64_rpl(__m256i a, int imm) {
+    __m256i_64_t A;
     A.m = a;
     return A.v[imm];
 }
 #endif
 
-#if HAVE_AVX2_MM256_INSERT_EPI32
-#define _mm256_insert_epi32_rpl _mm256_insert_epi32
+#if HAVE_AVX2_MM256_INSERT_EPI64
+#define _mm256_insert_epi64_rpl _mm256_insert_epi64
 #else
-static inline __m256i _mm256_insert_epi32_rpl(__m256i a, int32_t i, int imm) {
-    __m256i_32_t A;
+static inline __m256i _mm256_insert_epi64_rpl(__m256i a, int64_t i, int imm) {
+    __m256i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
 #endif
 
-#define _mm256_cmplt_epi32_rpl(a,b) _mm256_cmpgt_epi32(b,a)
+#if HAVE_AVX2_MM256_SET1_EPI64X
+#define _mm256_set1_epi64x_rpl _mm256_set1_epi64x
+#else
+static inline __m256i _mm256_set1_epi64x_rpl(int64_t i) {
+    __m256i_64_t A;
+    A.v[0] = i;
+    A.v[1] = i;
+    A.v[2] = i;
+    A.v[3] = i;
+    return A.m;
+}
+#endif
+
+static inline __m256i _mm256_max_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]>B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]>B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]>B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]>B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+static inline __m256i _mm256_min_epi64_rpl(__m256i a, __m256i b) {
+    __m256i_64_t A;
+    __m256i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]<B.v[0]) ? A.v[0] : B.v[0];
+    A.v[1] = (A.v[1]<B.v[1]) ? A.v[1] : B.v[1];
+    A.v[2] = (A.v[2]<B.v[2]) ? A.v[2] : B.v[2];
+    A.v[3] = (A.v[3]<B.v[3]) ? A.v[3] : B.v[3];
+    return A.m;
+}
+
+#define _mm256_cmplt_epi64_rpl(a,b) _mm256_cmpgt_epi64(b,a)
 
 #define _mm256_slli_si256_rpl(a,imm) _mm256_alignr_epi8(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,3,0)), 16-imm)
 
-static inline int32_t _mm256_hmax_epi32_rpl(__m256i a) {
-    a = _mm256_max_epi32(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
-    a = _mm256_max_epi32(a, _mm256_slli_si256(a, 8));
-    a = _mm256_max_epi32(a, _mm256_slli_si256(a, 4));
-    return _mm256_extract_epi32_rpl(a, 7);
+static inline int64_t _mm256_hmax_epi64_rpl(__m256i a) {
+    a = _mm256_max_epi64_rpl(a, _mm256_permute2x128_si256(a, a, _MM_SHUFFLE(0,0,0,0)));
+    a = _mm256_max_epi64_rpl(a, _mm256_slli_si256(a, 8));
+    return _mm256_extract_epi64_rpl(a, 3);
 }
 
 
@@ -70,8 +106,8 @@ static inline __m256i arr_load(
     return _mm256_load_si256(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sw_trace_scan_avx2_256_32
-#define PNAME parasail_sw_trace_scan_profile_avx2_256_32
+#define FNAME parasail_sw_trace_scan_avx2_256_64
+#define PNAME parasail_sw_trace_scan_profile_avx2_256_64
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -94,7 +130,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_avx_256_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_avx_256_64(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -125,10 +161,10 @@ parasail_result_t* PNAME(
     __m256i* restrict pvGapper = NULL;
     __m256i vGapO;
     __m256i vGapE;
-    int32_t NEG_LIMIT = 0;
-    int32_t POS_LIMIT = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
     __m256i vZero;
-    int32_t score = 0;
+    int64_t score = 0;
     __m256i vNegLimit;
     __m256i vPosLimit;
     __m256i vSaturationCheckMin;
@@ -149,7 +185,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -164,33 +200,33 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 8; /* number of values in vector unit */
+    segWidth = 4; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
-    pvP = (__m256i*)profile->profile32.score;
-    vGapO = _mm256_set1_epi32(open);
-    vGapE = _mm256_set1_epi32(gap);
-    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    pvP = (__m256i*)profile->profile64.score;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
     vZero = _mm256_setzero_si256();
     score = NEG_LIMIT;
-    vNegLimit = _mm256_set1_epi32(NEG_LIMIT);
-    vPosLimit = _mm256_set1_epi32(POS_LIMIT);
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
     vSaturationCheckMin = vPosLimit;
     vSaturationCheckMax = vNegLimit;
     vMaxH = vNegLimit;
     vMaxHUnit = vNegLimit;
     vNegInfFront = vZero;
-    vNegInfFront = _mm256_insert_epi32_rpl(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm256_add_epi32(vNegInfFront,
-            _mm256_slli_si256_rpl(_mm256_set1_epi32(-segLen*gap), 4));
-    vTZero = _mm256_set1_epi32(PARASAIL_ZERO);
-    vTIns  = _mm256_set1_epi32(PARASAIL_INS);
-    vTDel  = _mm256_set1_epi32(PARASAIL_DEL);
-    vTDiag = _mm256_set1_epi32(PARASAIL_DIAG);
-    vTDiagE = _mm256_set1_epi32(PARASAIL_DIAG_E);
-    vTInsE = _mm256_set1_epi32(PARASAIL_INS_E);
-    vTDiagF = _mm256_set1_epi32(PARASAIL_DIAG_F);
-    vTDelF = _mm256_set1_epi32(PARASAIL_DEL_F);
+    vNegInfFront = _mm256_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm256_add_epi64(vNegInfFront,
+            _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(-segLen*gap), 8));
+    vTZero = _mm256_set1_epi64x_rpl(PARASAIL_ZERO);
+    vTIns  = _mm256_set1_epi64x_rpl(PARASAIL_INS);
+    vTDel  = _mm256_set1_epi64x_rpl(PARASAIL_DEL);
+    vTDiag = _mm256_set1_epi64x_rpl(PARASAIL_DIAG);
+    vTDiagE = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_E);
+    vTInsE = _mm256_set1_epi64x_rpl(PARASAIL_INS_E);
+    vTDiagF = _mm256_set1_epi64x_rpl(PARASAIL_DIAG_F);
+    vTDelF = _mm256_set1_epi64x_rpl(PARASAIL_DEL_F);
 
 
     /* initialize result */
@@ -200,7 +236,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
 
     /* initialize heap variables */
     pvE    = parasail_memalign___m256i(32, segLen);
@@ -219,10 +255,10 @@ parasail_result_t* PNAME(
     parasail_memset___m256i(pvH, vZero, segLen);
     parasail_memset___m256i(pvE, vNegLimit, segLen);
     {
-        __m256i vGapper = _mm256_sub_epi32(vZero,vGapO);
+        __m256i vGapper = _mm256_sub_epi64(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm256_store_si256(pvGapper+i, vGapper);
-            vGapper = _mm256_sub_epi32(vGapper, vGapE);
+            vGapper = _mm256_sub_epi64(vGapper, vGapE);
         }
     }
 
@@ -251,7 +287,7 @@ parasail_result_t* PNAME(
         /* calculate Ht */
         /* calculate F and H first pass */
         vHp = _mm256_load_si256(pvH+(segLen-1));
-        vHp = _mm256_slli_si256_rpl(vHp, 4);
+        vHp = _mm256_slli_si256_rpl(vHp, 8);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         vHt = vZero;
         vF = vNegLimit;
@@ -260,16 +296,16 @@ parasail_result_t* PNAME(
             vE = _mm256_load_si256(pvE+i);
             vW = _mm256_load_si256(pvW+i);
             vGapper = _mm256_load_si256(pvGapper+i);
-            vE_opn = _mm256_sub_epi32(vH, vGapO);
-            vE_ext = _mm256_sub_epi32(vE, vGapE);
-            case1 = _mm256_cmpgt_epi32(vE_opn, vE_ext);
+            vE_opn = _mm256_sub_epi64(vH, vGapO);
+            vE_ext = _mm256_sub_epi64(vE, vGapE);
+            case1 = _mm256_cmpgt_epi64(vE_opn, vE_ext);
             vET = _mm256_blendv_epi8(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
-            vE = _mm256_max_epi32(vE_opn, vE_ext);
-            vGapper = _mm256_add_epi32(vHt, vGapper);
-            vF = _mm256_max_epi32(vF, vGapper);
-            vHp = _mm256_add_epi32(vHp, vW);
-            vHt = _mm256_max_epi32(vE, vHp);
+            vE = _mm256_max_epi64_rpl(vE_opn, vE_ext);
+            vGapper = _mm256_add_epi64(vHt, vGapper);
+            vF = _mm256_max_epi64_rpl(vF, vGapper);
+            vHp = _mm256_add_epi64(vHp, vW);
+            vHt = _mm256_max_epi64_rpl(vE, vHp);
             _mm256_store_si256(pvE+i, vE);
             _mm256_store_si256(pvHt+i, vHt);
             _mm256_store_si256(pvH+i, vHp);
@@ -277,35 +313,35 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm256_slli_si256_rpl(vHt, 4);
+        vHt = _mm256_slli_si256_rpl(vHt, 8);
         vGapper = _mm256_load_si256(pvGapper);
-        vGapper = _mm256_add_epi32(vHt, vGapper);
-        vF = _mm256_max_epi32(vF, vGapper);
+        vGapper = _mm256_add_epi64(vHt, vGapper);
+        vF = _mm256_max_epi64_rpl(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
-            __m256i vFt = _mm256_slli_si256_rpl(vF, 4);
-            vFt = _mm256_add_epi32(vFt, vSegLenXgap);
-            vF = _mm256_max_epi32(vF, vFt);
+            __m256i vFt = _mm256_slli_si256_rpl(vF, 8);
+            vFt = _mm256_add_epi64(vFt, vSegLenXgap);
+            vF = _mm256_max_epi64_rpl(vF, vFt);
         }
 
         /* calculate final H */
-        vF = _mm256_slli_si256_rpl(vF, 4);
-        vF = _mm256_add_epi32(vF, vNegInfFront);
-        vH = _mm256_max_epi32(vF, vHt);
-        vH = _mm256_max_epi32(vH, vZero);
+        vF = _mm256_slli_si256_rpl(vF, 8);
+        vF = _mm256_add_epi64(vF, vNegInfFront);
+        vH = _mm256_max_epi64_rpl(vF, vHt);
+        vH = _mm256_max_epi64_rpl(vH, vZero);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm256_load_si256(pvH+i);
             vHt = _mm256_load_si256(pvHt+i);
-            vF_opn = _mm256_sub_epi32(vH, vGapO);
-            vF_ext = _mm256_sub_epi32(vF, vGapE);
-            vF = _mm256_max_epi32(vF_opn, vF_ext);
-            case1 = _mm256_cmpgt_epi32(vF_opn, vF_ext);
+            vF_opn = _mm256_sub_epi64(vH, vGapO);
+            vF_ext = _mm256_sub_epi64(vF, vGapE);
+            vF = _mm256_max_epi64_rpl(vF_opn, vF_ext);
+            case1 = _mm256_cmpgt_epi64(vF_opn, vF_ext);
             vFT = _mm256_blendv_epi8(vTDelF, vTDiagF, case1);
-            vH = _mm256_max_epi32(vHt, vF);
-            vH = _mm256_max_epi32(vH, vZero);
-            case0 = _mm256_cmpeq_epi32(vH, vZero);
-            case1 = _mm256_cmpeq_epi32(vH, vHp);
-            case2 = _mm256_cmpeq_epi32(vH, vF);
+            vH = _mm256_max_epi64_rpl(vHt, vF);
+            vH = _mm256_max_epi64_rpl(vH, vZero);
+            case0 = _mm256_cmpeq_epi64(vH, vZero);
+            case1 = _mm256_cmpeq_epi64(vH, vHp);
+            case2 = _mm256_cmpeq_epi64(vH, vF);
             vT = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vTIns, vTDel, case2),
                     vTDiag, case1);
@@ -314,18 +350,18 @@ parasail_result_t* PNAME(
             vT = _mm256_or_si256(vT, vFT);
             arr_store(result->trace->trace_table, vT, i, segLen, j);
             _mm256_store_si256(pvH+i, vH);
-            vSaturationCheckMin = _mm256_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm256_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vH);
             {
-                vMaxH = _mm256_max_epi32(vH, vMaxH);
+                vMaxH = _mm256_max_epi64_rpl(vH, vMaxH);
             }
         } 
 
         {
-            __m256i vCompare = _mm256_cmpgt_epi32(vMaxH, vMaxHUnit);
+            __m256i vCompare = _mm256_cmpgt_epi64(vMaxH, vMaxHUnit);
             if (_mm256_movemask_epi8(vCompare)) {
-                score = _mm256_hmax_epi32_rpl(vMaxH);
-                vMaxHUnit = _mm256_set1_epi32(score);
+                score = _mm256_hmax_epi64_rpl(vMaxH);
+                vMaxHUnit = _mm256_set1_epi64x_rpl(score);
                 end_ref = j;
                 (void)memcpy(pvHMax, pvH, sizeof(__m256i)*segLen);
             }
@@ -334,7 +370,7 @@ parasail_result_t* PNAME(
 
     /* Trace the alignment ending position on read. */
     {
-        int32_t *t = (int32_t*)pvHMax;
+        int64_t *t = (int64_t*)pvHMax;
         int32_t column_len = segLen * segWidth;
         end_query = s1Len;
         for (i = 0; i<column_len; ++i, ++t) {
@@ -348,8 +384,8 @@ parasail_result_t* PNAME(
     }
 
     if (_mm256_movemask_epi8(_mm256_or_si256(
-            _mm256_cmplt_epi32_rpl(vSaturationCheckMin, vNegLimit),
-            _mm256_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
+            _mm256_cmplt_epi64_rpl(vSaturationCheckMin, vNegLimit),
+            _mm256_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         end_query = 0;
