@@ -19,7 +19,7 @@
 #define SWAP(A,B) { simde__m128i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { simde__m128i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF (INT32_MIN/(int32_t)(2))
+#define NEG_INF INT8_MIN
 
 
 static inline void arr_store(
@@ -41,8 +41,8 @@ static inline simde__m128i arr_load(
     return simde_mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sw_trace_striped_neon_128_32
-#define PNAME parasail_sw_trace_striped_profile_neon_128_32
+#define FNAME parasail_sw_trace_striped_neon_128_8
+#define PNAME parasail_sw_trace_striped_profile_neon_128_8
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -65,7 +65,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_neon_128_32(s1, s1Len, matrix);
+    profile = parasail_profile_create_neon_128_8(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -100,10 +100,10 @@ parasail_result_t* PNAME(
     simde__m128i vGapO;
     simde__m128i vGapE;
     simde__m128i vZero;
-    int32_t score = 0;
+    int8_t score = 0;
     simde__m128i vMaxH;
     simde__m128i vMaxHUnit;
-    int32_t maxp = 0;
+    int8_t maxp = 0;
     parasail_result_t *result = NULL;
     simde__m128i vTZero;
     simde__m128i vTIns;
@@ -118,7 +118,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -134,26 +134,26 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 4; /* number of values in vector unit */
+    segWidth = 16; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
-    vProfile = (simde__m128i*)profile->profile32.score;
-    vGapO = simde_mm_set1_epi32(open);
-    vGapE = simde_mm_set1_epi32(gap);
+    vProfile = (simde__m128i*)profile->profile8.score;
+    vGapO = simde_mm_set1_epi8(open);
+    vGapE = simde_mm_set1_epi8(gap);
     vZero = simde_mm_setzero_si128();
     score = NEG_INF;
     vMaxH = vZero;
     vMaxHUnit = vZero;
-    maxp = INT32_MAX - (int32_t)(matrix->max+1);
-    vTZero = simde_mm_set1_epi32(PARASAIL_ZERO);
-    vTIns  = simde_mm_set1_epi32(PARASAIL_INS);
-    vTDel  = simde_mm_set1_epi32(PARASAIL_DEL);
-    vTDiag = simde_mm_set1_epi32(PARASAIL_DIAG);
-    vTDiagE= simde_mm_set1_epi32(PARASAIL_DIAG_E);
-    vTInsE = simde_mm_set1_epi32(PARASAIL_INS_E);
-    vTDiagF= simde_mm_set1_epi32(PARASAIL_DIAG_F);
-    vTDelF = simde_mm_set1_epi32(PARASAIL_DEL_F);
-    vTMask = simde_mm_set1_epi32(PARASAIL_ZERO_MASK);
-    vFTMask= simde_mm_set1_epi32(PARASAIL_F_MASK);
+    maxp = INT8_MAX - (int8_t)(matrix->max+1);
+    vTZero = simde_mm_set1_epi8(PARASAIL_ZERO);
+    vTIns  = simde_mm_set1_epi8(PARASAIL_INS);
+    vTDel  = simde_mm_set1_epi8(PARASAIL_DEL);
+    vTDiag = simde_mm_set1_epi8(PARASAIL_DIAG);
+    vTDiagE= simde_mm_set1_epi8(PARASAIL_DIAG_E);
+    vTInsE = simde_mm_set1_epi8(PARASAIL_INS_E);
+    vTDiagF= simde_mm_set1_epi8(PARASAIL_DIAG_F);
+    vTDelF = simde_mm_set1_epi8(PARASAIL_DEL_F);
+    vTMask = simde_mm_set1_epi8(PARASAIL_ZERO_MASK);
+    vFTMask= simde_mm_set1_epi8(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(simde__m128i));
@@ -162,7 +162,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 
     /* initialize heap variables */
     pvHStore = parasail_memalign_simde__m128i(16, segLen);
@@ -184,8 +184,8 @@ parasail_result_t* PNAME(
 
     /* initialize H and E */
     parasail_memset_simde__m128i(pvHStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvE, simde_mm_set1_epi32(-open), segLen);
-    parasail_memset_simde__m128i(pvEaStore, simde_mm_set1_epi32(-open), segLen);
+    parasail_memset_simde__m128i(pvE, simde_mm_set1_epi8(-open), segLen);
+    parasail_memset_simde__m128i(pvEaStore, simde_mm_set1_epi8(-open), segLen);
 
     for (i=0; i<segLen; ++i) {
         arr_store(result->trace->trace_table, vTDiagE, i, segLen, 0);
@@ -206,11 +206,11 @@ parasail_result_t* PNAME(
 
         /* Initialize F value to 0.  Any errors to vH values will be
          * corrected in the Lazy_F loop. */
-        vF = simde_mm_sub_epi32(vZero,vGapO);
+        vF = simde_mm_subs_epi8(vZero,vGapO);
 
-        /* load final segment of pvHStore and shift left by 4 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 4);
+        vH = simde_mm_slli_si128(vH, 1);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -231,18 +231,18 @@ parasail_result_t* PNAME(
             vE = simde_mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_add_epi32(vH, simde_mm_load_si128(vP + i));
-            vH_dag = simde_mm_max_epi32(vH_dag, vZero);
-            vH = simde_mm_max_epi32(vH_dag, vE);
-            vH = simde_mm_max_epi32(vH, vF);
+            vH_dag = simde_mm_adds_epi8(vH, simde_mm_load_si128(vP + i));
+            vH_dag = simde_mm_max_epi8(vH_dag, vZero);
+            vH = simde_mm_max_epi8(vH_dag, vE);
+            vH = simde_mm_max_epi8(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
 
             {
                 simde__m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                simde__m128i cond_zero = simde_mm_cmpeq_epi32(vH, vZero);
-                simde__m128i case1 = simde_mm_cmpeq_epi32(vH, vH_dag);
-                simde__m128i case2 = simde_mm_cmpeq_epi32(vH, vF);
+                simde__m128i cond_zero = simde_mm_cmpeq_epi8(vH, vZero);
+                simde__m128i case1 = simde_mm_cmpeq_epi8(vH, vH_dag);
+                simde__m128i case2 = simde_mm_cmpeq_epi8(vH, vF);
                 simde__m128i vT = simde_mm_blendv_epi8(
                         simde_mm_blendv_epi8(vTIns, vTDel, case2),
                         simde_mm_blendv_epi8(vTDiag, vTZero, cond_zero),
@@ -251,31 +251,31 @@ parasail_result_t* PNAME(
                 vT = simde_mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
-            vMaxH = simde_mm_max_epi32(vH, vMaxH);
-            vEF_opn = simde_mm_sub_epi32(vH, vGapO);
+            vMaxH = simde_mm_max_epi8(vH, vMaxH);
+            vEF_opn = simde_mm_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_sub_epi32(vE, vGapE);
-            vE = simde_mm_max_epi32(vEF_opn, vE_ext);
+            vE_ext = simde_mm_subs_epi8(vE, vGapE);
+            vE = simde_mm_max_epi8(vEF_opn, vE_ext);
             simde_mm_store_si128(pvE + i, vE);
             {
                 simde__m128i vEa = simde_mm_load_si128(pvEaLoad + i);
-                simde__m128i vEa_ext = simde_mm_sub_epi32(vEa, vGapE);
-                vEa = simde_mm_max_epi32(vEF_opn, vEa_ext);
+                simde__m128i vEa_ext = simde_mm_subs_epi8(vEa, vGapE);
+                vEa = simde_mm_max_epi8(vEF_opn, vEa_ext);
                 simde_mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vEa_ext);
+                    simde__m128i cond = simde_mm_cmpgt_epi8(vEF_opn, vEa_ext);
                     simde__m128i vT = simde_mm_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = simde_mm_sub_epi32(vF, vGapE);
-            vF = simde_mm_max_epi32(vEF_opn, vF_ext);
+            vF_ext = simde_mm_subs_epi8(vF, vGapE);
+            vF = simde_mm_max_epi8(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 simde__m128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vF_ext);
+                simde__m128i cond = simde_mm_cmpgt_epi8(vEF_opn, vF_ext);
                 simde__m128i vT = simde_mm_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = simde_mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -291,20 +291,20 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = simde_mm_slli_si128(vHp, 4);
-            vEF_opn = simde_mm_slli_si128(vEF_opn, 4);
-            vEF_opn = simde_mm_insert_epi32(vEF_opn, -open, 0);
-            vF_ext = simde_mm_slli_si128(vF_ext, 4);
-            vF_ext = simde_mm_insert_epi32(vF_ext, NEG_INF, 0);
-            vF = simde_mm_slli_si128(vF, 4);
-            vF = simde_mm_insert_epi32(vF, -open, 0);
-            vFa_ext = simde_mm_slli_si128(vFa_ext, 4);
-            vFa_ext = simde_mm_insert_epi32(vFa_ext, NEG_INF, 0);
-            vFa = simde_mm_slli_si128(vFa, 4);
-            vFa = simde_mm_insert_epi32(vFa, -open, 0);
+            vHp = simde_mm_slli_si128(vHp, 1);
+            vEF_opn = simde_mm_slli_si128(vEF_opn, 1);
+            vEF_opn = simde_mm_insert_epi8(vEF_opn, -open, 0);
+            vF_ext = simde_mm_slli_si128(vF_ext, 1);
+            vF_ext = simde_mm_insert_epi8(vF_ext, NEG_INF, 0);
+            vF = simde_mm_slli_si128(vF, 1);
+            vF = simde_mm_insert_epi8(vF, -open, 0);
+            vFa_ext = simde_mm_slli_si128(vFa_ext, 1);
+            vFa_ext = simde_mm_insert_epi8(vFa_ext, NEG_INF, 0);
+            vFa = simde_mm_slli_si128(vFa, 1);
+            vFa = simde_mm_insert_epi8(vFa, -open, 0);
             for (i=0; i<segLen; ++i) {
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi32(vH,vF);
+                vH = simde_mm_max_epi8(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
                 {
                     simde__m128i vTAll;
@@ -312,10 +312,10 @@ parasail_result_t* PNAME(
                     simde__m128i case1;
                     simde__m128i case2;
                     simde__m128i cond;
-                    vHp = simde_mm_add_epi32(vHp, simde_mm_load_si128(vP + i));
-                    vHp = simde_mm_max_epi32(vHp, vZero);
-                    case1 = simde_mm_cmpeq_epi32(vH, vHp);
-                    case2 = simde_mm_cmpeq_epi32(vH, vF);
+                    vHp = simde_mm_adds_epi8(vHp, simde_mm_load_si128(vP + i));
+                    vHp = simde_mm_max_epi8(vHp, vZero);
+                    case1 = simde_mm_cmpeq_epi8(vH, vHp);
+                    case2 = simde_mm_cmpeq_epi8(vH, vF);
                     cond = simde_mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = simde_mm_load_si128(pvHT + i);
@@ -325,38 +325,38 @@ parasail_result_t* PNAME(
                     vTAll = simde_mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vMaxH = simde_mm_max_epi32(vH, vMaxH);
+                vMaxH = simde_mm_max_epi8(vH, vMaxH);
                 /* Update vF value. */
                 {
                     simde__m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vFa_ext);
+                    simde__m128i cond = simde_mm_cmpgt_epi8(vEF_opn, vFa_ext);
                     simde__m128i vT = simde_mm_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = simde_mm_and_si128(vTAll, vFTMask);
                     vTAll = simde_mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = simde_mm_sub_epi32(vH, vGapO);
-                vF_ext = simde_mm_sub_epi32(vF, vGapE);
+                vEF_opn = simde_mm_subs_epi8(vH, vGapO);
+                vF_ext = simde_mm_subs_epi8(vF, vGapE);
                 {
                     simde__m128i vEa = simde_mm_load_si128(pvEaLoad + i);
-                    simde__m128i vEa_ext = simde_mm_sub_epi32(vEa, vGapE);
-                    vEa = simde_mm_max_epi32(vEF_opn, vEa_ext);
+                    simde__m128i vEa_ext = simde_mm_subs_epi8(vEa, vGapE);
+                    vEa = simde_mm_max_epi8(vEF_opn, vEa_ext);
                     simde_mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        simde__m128i cond = simde_mm_cmpgt_epi32(vEF_opn, vEa_ext);
+                        simde__m128i cond = simde_mm_cmpgt_epi8(vEF_opn, vEa_ext);
                         simde__m128i vT = simde_mm_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi32(vF_ext, vEF_opn),
-                                simde_mm_cmpeq_epi32(vF_ext, vEF_opn))))
+                                simde_mm_cmpgt_epi8(vF_ext, vEF_opn),
+                                simde_mm_cmpeq_epi8(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = simde_mm_max_epi32(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi8(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = simde_mm_sub_epi32(vFa, vGapE);
-                vFa = simde_mm_max_epi32(vEF_opn, vFa_ext);
+                vFa_ext = simde_mm_subs_epi8(vFa, vGapE);
+                vFa = simde_mm_max_epi8(vEF_opn, vFa_ext);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
         }
@@ -365,15 +365,15 @@ end:
         }
 
         {
-            simde__m128i vCompare = simde_mm_cmpgt_epi32(vMaxH, vMaxHUnit);
+            simde__m128i vCompare = simde_mm_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (simde_mm_movemask_epi8(vCompare)) {
-                score = simde_mm_hmax_epi32(vMaxH);
+                score = simde_mm_hmax_epi8(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = simde_mm_set1_epi32(score);
+                vMaxHUnit = simde_mm_set1_epi8(score);
                 end_ref = j;
             }
         }
@@ -381,7 +381,7 @@ end:
         /*if (score == stop) break;*/
     }
 
-    if (score == INT32_MAX) {
+    if (score == INT8_MAX) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -401,7 +401,7 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int32_t *t = (int32_t*)pvHMax;
+            int8_t *t = (int8_t*)pvHMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len - 1;
             for (i = 0; i<column_len; ++i, ++t) {
