@@ -14,7 +14,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -24,14 +23,19 @@
 #define SWAP(A,B) { __m128i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { __m128i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF INT8_MIN
+#define NEG_INF INT16_MIN
 
-static inline int8_t _mm_hmax_epi8_rpl(__m128i a) {
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 8));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 4));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 2));
-    a = _mm_max_epi8(a, _mm_srli_si128(a, 1));
-    return _mm_extract_epi8(a, 0);
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
+static inline int16_t _mm_hmax_epi16_rpl(__m128i a) {
+    a = _mm_max_epi16(a, _mm_srli_si128(a, 8));
+    a = _mm_max_epi16(a, _mm_srli_si128(a, 4));
+    a = _mm_max_epi16(a, _mm_srli_si128(a, 2));
+    return _mm_extract_epi16(a, 0);
 }
 
 
@@ -54,8 +58,8 @@ static inline __m128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sw_trace_striped_sse41_128_8
-#define PNAME parasail_sw_trace_striped_profile_sse41_128_8
+#define FNAME parasail_sw_trace_striped_sse2_128_16
+#define PNAME parasail_sw_trace_striped_profile_sse2_128_16
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -78,7 +82,7 @@ parasail_result_t* FNAME(
     }
 
     /* initialize local variables */
-    profile = parasail_profile_create_sse_128_8(s1, s1Len, matrix);
+    profile = parasail_profile_create_sse_128_16(s1, s1Len, matrix);
     if (!profile) return NULL;
     result = PNAME(profile, s2, s2Len, open, gap);
 
@@ -113,10 +117,10 @@ parasail_result_t* PNAME(
     __m128i vGapO;
     __m128i vGapE;
     __m128i vZero;
-    int8_t score = 0;
+    int16_t score = 0;
     __m128i vMaxH;
     __m128i vMaxHUnit;
-    int8_t maxp = 0;
+    int16_t maxp = 0;
     parasail_result_t *result = NULL;
     __m128i vTZero;
     __m128i vTIns;
@@ -131,7 +135,7 @@ parasail_result_t* PNAME(
 
     /* validate inputs */
     PARASAIL_CHECK_NULL(profile);
-    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
     PARASAIL_CHECK_NULL(profile->matrix);
     PARASAIL_CHECK_GT0(profile->s1Len);
     PARASAIL_CHECK_NULL(s2);
@@ -147,26 +151,26 @@ parasail_result_t* PNAME(
     end_ref = 0;
     s1Len = profile->s1Len;
     matrix = profile->matrix;
-    segWidth = 16; /* number of values in vector unit */
+    segWidth = 8; /* number of values in vector unit */
     segLen = (s1Len + segWidth - 1) / segWidth;
-    vProfile = (__m128i*)profile->profile8.score;
-    vGapO = _mm_set1_epi8(open);
-    vGapE = _mm_set1_epi8(gap);
+    vProfile = (__m128i*)profile->profile16.score;
+    vGapO = _mm_set1_epi16(open);
+    vGapE = _mm_set1_epi16(gap);
     vZero = _mm_setzero_si128();
     score = NEG_INF;
     vMaxH = vZero;
     vMaxHUnit = vZero;
-    maxp = INT8_MAX - (int8_t)(matrix->max+1);
-    vTZero = _mm_set1_epi8(PARASAIL_ZERO);
-    vTIns  = _mm_set1_epi8(PARASAIL_INS);
-    vTDel  = _mm_set1_epi8(PARASAIL_DEL);
-    vTDiag = _mm_set1_epi8(PARASAIL_DIAG);
-    vTDiagE= _mm_set1_epi8(PARASAIL_DIAG_E);
-    vTInsE = _mm_set1_epi8(PARASAIL_INS_E);
-    vTDiagF= _mm_set1_epi8(PARASAIL_DIAG_F);
-    vTDelF = _mm_set1_epi8(PARASAIL_DEL_F);
-    vTMask = _mm_set1_epi8(PARASAIL_ZERO_MASK);
-    vFTMask= _mm_set1_epi8(PARASAIL_F_MASK);
+    maxp = INT16_MAX - (int16_t)(matrix->max+1);
+    vTZero = _mm_set1_epi16(PARASAIL_ZERO);
+    vTIns  = _mm_set1_epi16(PARASAIL_INS);
+    vTDel  = _mm_set1_epi16(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi16(PARASAIL_DIAG);
+    vTDiagE= _mm_set1_epi16(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi16(PARASAIL_INS_E);
+    vTDiagF= _mm_set1_epi16(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi16(PARASAIL_DEL_F);
+    vTMask = _mm_set1_epi16(PARASAIL_ZERO_MASK);
+    vFTMask= _mm_set1_epi16(PARASAIL_F_MASK);
 
     /* initialize result */
     result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(__m128i));
@@ -175,7 +179,7 @@ parasail_result_t* PNAME(
     /* set known flags */
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
 
     /* initialize heap variables */
     pvHStore = parasail_memalign___m128i(16, segLen);
@@ -197,8 +201,8 @@ parasail_result_t* PNAME(
 
     /* initialize H and E */
     parasail_memset___m128i(pvHStore, vZero, segLen);
-    parasail_memset___m128i(pvE, _mm_set1_epi8(-open), segLen);
-    parasail_memset___m128i(pvEaStore, _mm_set1_epi8(-open), segLen);
+    parasail_memset___m128i(pvE, _mm_set1_epi16(-open), segLen);
+    parasail_memset___m128i(pvEaStore, _mm_set1_epi16(-open), segLen);
 
     for (i=0; i<segLen; ++i) {
         arr_store(result->trace->trace_table, vTDiagE, i, segLen, 0);
@@ -219,11 +223,11 @@ parasail_result_t* PNAME(
 
         /* Initialize F value to 0.  Any errors to vH values will be
          * corrected in the Lazy_F loop. */
-        vF = _mm_subs_epi8(vZero,vGapO);
+        vF = _mm_subs_epi16(vZero,vGapO);
 
-        /* load final segment of pvHStore and shift left by 1 bytes */
+        /* load final segment of pvHStore and shift left by 2 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 1);
+        vH = _mm_slli_si128(vH, 2);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -244,52 +248,52 @@ parasail_result_t* PNAME(
             vE = _mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_adds_epi8(vH, _mm_load_si128(vP + i));
-            vH_dag = _mm_max_epi8(vH_dag, vZero);
-            vH = _mm_max_epi8(vH_dag, vE);
-            vH = _mm_max_epi8(vH, vF);
+            vH_dag = _mm_adds_epi16(vH, _mm_load_si128(vP + i));
+            vH_dag = _mm_max_epi16(vH_dag, vZero);
+            vH = _mm_max_epi16(vH_dag, vE);
+            vH = _mm_max_epi16(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
 
             {
                 __m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                __m128i cond_zero = _mm_cmpeq_epi8(vH, vZero);
-                __m128i case1 = _mm_cmpeq_epi8(vH, vH_dag);
-                __m128i case2 = _mm_cmpeq_epi8(vH, vF);
-                __m128i vT = _mm_blendv_epi8(
-                        _mm_blendv_epi8(vTIns, vTDel, case2),
-                        _mm_blendv_epi8(vTDiag, vTZero, cond_zero),
+                __m128i cond_zero = _mm_cmpeq_epi16(vH, vZero);
+                __m128i case1 = _mm_cmpeq_epi16(vH, vH_dag);
+                __m128i case2 = _mm_cmpeq_epi16(vH, vF);
+                __m128i vT = _mm_blendv_epi8_rpl(
+                        _mm_blendv_epi8_rpl(vTIns, vTDel, case2),
+                        _mm_blendv_epi8_rpl(vTDiag, vTZero, cond_zero),
                         case1);
                 _mm_store_si128(pvHT + i, vT);
                 vT = _mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
-            vMaxH = _mm_max_epi8(vH, vMaxH);
-            vEF_opn = _mm_subs_epi8(vH, vGapO);
+            vMaxH = _mm_max_epi16(vH, vMaxH);
+            vEF_opn = _mm_subs_epi16(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_subs_epi8(vE, vGapE);
-            vE = _mm_max_epi8(vEF_opn, vE_ext);
+            vE_ext = _mm_subs_epi16(vE, vGapE);
+            vE = _mm_max_epi16(vEF_opn, vE_ext);
             _mm_store_si128(pvE + i, vE);
             {
                 __m128i vEa = _mm_load_si128(pvEaLoad + i);
-                __m128i vEa_ext = _mm_subs_epi8(vEa, vGapE);
-                vEa = _mm_max_epi8(vEF_opn, vEa_ext);
+                __m128i vEa_ext = _mm_subs_epi16(vEa, vGapE);
+                vEa = _mm_max_epi16(vEF_opn, vEa_ext);
                 _mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    __m128i cond = _mm_cmpgt_epi8(vEF_opn, vEa_ext);
-                    __m128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
+                    __m128i cond = _mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                    __m128i vT = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = _mm_subs_epi8(vF, vGapE);
-            vF = _mm_max_epi8(vEF_opn, vF_ext);
+            vF_ext = _mm_subs_epi16(vF, vGapE);
+            vF = _mm_max_epi16(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 __m128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                __m128i cond = _mm_cmpgt_epi8(vEF_opn, vF_ext);
-                __m128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
+                __m128i cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+                __m128i vT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, cond);
                 vT = _mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
             }
@@ -304,20 +308,20 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             __m128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = _mm_slli_si128(vHp, 1);
-            vEF_opn = _mm_slli_si128(vEF_opn, 1);
-            vEF_opn = _mm_insert_epi8(vEF_opn, -open, 0);
-            vF_ext = _mm_slli_si128(vF_ext, 1);
-            vF_ext = _mm_insert_epi8(vF_ext, NEG_INF, 0);
-            vF = _mm_slli_si128(vF, 1);
-            vF = _mm_insert_epi8(vF, -open, 0);
-            vFa_ext = _mm_slli_si128(vFa_ext, 1);
-            vFa_ext = _mm_insert_epi8(vFa_ext, NEG_INF, 0);
-            vFa = _mm_slli_si128(vFa, 1);
-            vFa = _mm_insert_epi8(vFa, -open, 0);
+            vHp = _mm_slli_si128(vHp, 2);
+            vEF_opn = _mm_slli_si128(vEF_opn, 2);
+            vEF_opn = _mm_insert_epi16(vEF_opn, -open, 0);
+            vF_ext = _mm_slli_si128(vF_ext, 2);
+            vF_ext = _mm_insert_epi16(vF_ext, NEG_INF, 0);
+            vF = _mm_slli_si128(vF, 2);
+            vF = _mm_insert_epi16(vF, -open, 0);
+            vFa_ext = _mm_slli_si128(vFa_ext, 2);
+            vFa_ext = _mm_insert_epi16(vFa_ext, NEG_INF, 0);
+            vFa = _mm_slli_si128(vFa, 2);
+            vFa = _mm_insert_epi16(vFa, -open, 0);
             for (i=0; i<segLen; ++i) {
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi8(vH,vF);
+                vH = _mm_max_epi16(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
                 {
                     __m128i vTAll;
@@ -325,51 +329,51 @@ parasail_result_t* PNAME(
                     __m128i case1;
                     __m128i case2;
                     __m128i cond;
-                    vHp = _mm_adds_epi8(vHp, _mm_load_si128(vP + i));
-                    vHp = _mm_max_epi8(vHp, vZero);
-                    case1 = _mm_cmpeq_epi8(vH, vHp);
-                    case2 = _mm_cmpeq_epi8(vH, vF);
+                    vHp = _mm_adds_epi16(vHp, _mm_load_si128(vP + i));
+                    vHp = _mm_max_epi16(vHp, vZero);
+                    case1 = _mm_cmpeq_epi16(vH, vHp);
+                    case2 = _mm_cmpeq_epi16(vH, vF);
                     cond = _mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = _mm_load_si128(pvHT + i);
-                    vT = _mm_blendv_epi8(vT, vTDel, cond);
+                    vT = _mm_blendv_epi8_rpl(vT, vTDel, cond);
                     _mm_store_si128(pvHT + i, vT);
                     vTAll = _mm_and_si128(vTAll, vTMask);
                     vTAll = _mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vMaxH = _mm_max_epi8(vH, vMaxH);
+                vMaxH = _mm_max_epi16(vH, vMaxH);
                 /* Update vF value. */
                 {
                     __m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    __m128i cond = _mm_cmpgt_epi8(vEF_opn, vFa_ext);
-                    __m128i vT = _mm_blendv_epi8(vTDelF, vTDiagF, cond);
+                    __m128i cond = _mm_cmpgt_epi16(vEF_opn, vFa_ext);
+                    __m128i vT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, cond);
                     vTAll = _mm_and_si128(vTAll, vFTMask);
                     vTAll = _mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = _mm_subs_epi8(vH, vGapO);
-                vF_ext = _mm_subs_epi8(vF, vGapE);
+                vEF_opn = _mm_subs_epi16(vH, vGapO);
+                vF_ext = _mm_subs_epi16(vF, vGapE);
                 {
                     __m128i vEa = _mm_load_si128(pvEaLoad + i);
-                    __m128i vEa_ext = _mm_subs_epi8(vEa, vGapE);
-                    vEa = _mm_max_epi8(vEF_opn, vEa_ext);
+                    __m128i vEa_ext = _mm_subs_epi16(vEa, vGapE);
+                    vEa = _mm_max_epi16(vEF_opn, vEa_ext);
                     _mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        __m128i cond = _mm_cmpgt_epi8(vEF_opn, vEa_ext);
-                        __m128i vT = _mm_blendv_epi8(vTInsE, vTDiagE, cond);
+                        __m128i cond = _mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                        __m128i vT = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi8(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi8(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi16(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi8(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = _mm_subs_epi8(vFa, vGapE);
-                vFa = _mm_max_epi8(vEF_opn, vFa_ext);
+                vFa_ext = _mm_subs_epi16(vFa, vGapE);
+                vFa = _mm_max_epi16(vEF_opn, vFa_ext);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
         }
@@ -378,15 +382,15 @@ end:
         }
 
         {
-            __m128i vCompare = _mm_cmpgt_epi8(vMaxH, vMaxHUnit);
+            __m128i vCompare = _mm_cmpgt_epi16(vMaxH, vMaxHUnit);
             if (_mm_movemask_epi8(vCompare)) {
-                score = _mm_hmax_epi8_rpl(vMaxH);
+                score = _mm_hmax_epi16_rpl(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = _mm_set1_epi8(score);
+                vMaxHUnit = _mm_set1_epi16(score);
                 end_ref = j;
             }
         }
@@ -394,7 +398,7 @@ end:
         /*if (score == stop) break;*/
     }
 
-    if (score == INT8_MAX) {
+    if (score == INT16_MAX) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -414,7 +418,7 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int8_t *t = (int8_t*)pvHMax;
+            int16_t *t = (int16_t*)pvHMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len - 1;
             for (i = 0; i<column_len; ++i, ++t) {
